<!doctype html>
<html lang="en">

<head>
    <title>Giskard</title>
    {% block head %}{% endblock %}
</head>

<body>
    {% block content %}{% endblock %}
    {% block javascript %}{% endblock %}
</body>

</html>