# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ml-worker.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fml-worker.proto\x12\x06worker\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1bgoogle/protobuf/empty.proto\",\n\x13MLWorkerInfoRequest\x12\x15\n\rlist_packages\x18\x01 \x01(\x08\"\xe6\x02\n\x0cMLWorkerInfo\x12&\n\x08platform\x18\x01 \x01(\x0b\x32\x14.worker.PlatformInfo\x12\x13\n\x0binterpreter\x18\x02 \x01(\t\x12\x1b\n\x13interpreter_version\x18\x03 \x01(\t\x12G\n\x12installed_packages\x18\x04 \x03(\x0b\x32+.worker.MLWorkerInfo.InstalledPackagesEntry\x12\x1d\n\x15internal_grpc_address\x18\x05 \x01(\t\x12\x11\n\tis_remote\x18\x06 \x01(\x08\x12\x0b\n\x03pid\x18\x07 \x01(\r\x12\x1a\n\x12process_start_time\x18\x08 \x01(\x04\x12\x1e\n\x16giskard_client_version\x18\t \x01(\t\x1a\x38\n\x16InstalledPackagesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"r\n\x0cPlatformInfo\x12\x0f\n\x07machine\x18\x01 \x01(\t\x12\x0c\n\x04node\x18\x02 \x01(\t\x12\x11\n\tprocessor\x18\x03 \x01(\t\x12\x0f\n\x07release\x18\x04 \x01(\t\x12\x0e\n\x06system\x18\x05 \x01(\t\x12\x0f\n\x07version\x18\x06 \x01(\t\"n\n\x11\x46ileUploadRequest\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1a.worker.FileUploadMetadataH\x00\x12\x1e\n\x05\x63hunk\x18\x02 \x01(\x0b\x32\r.worker.ChunkH\x00\x42\t\n\x07request\"h\n\x12\x46ileUploadMetadata\x12\n\n\x02id\x18\x01 \x01(\x04\x12#\n\tfile_type\x18\x02 \x01(\x0e\x32\x10.worker.FileType\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0bproject_key\x18\x04 \x01(\t\"\x16\n\x07\x45\x63hoMsg\x12\x0b\n\x03msg\x18\x01 \x01(\t\"\xc0\x01\n\x0e\x45xplainRequest\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.worker.ArtifactRef\x12$\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x13.worker.ArtifactRef\x12\x34\n\x07\x63olumns\x18\x03 \x03(\x0b\x32#.worker.ExplainRequest.ColumnsEntry\x1a.\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xc2\x02\n\x12\x45xplainTextRequest\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.worker.ArtifactRef\x12\x14\n\x0c\x66\x65\x61ture_name\x18\x02 \x01(\t\x12\x38\n\x07\x63olumns\x18\x03 \x03(\x0b\x32\'.worker.ExplainTextRequest.ColumnsEntry\x12\x41\n\x0c\x63olumn_types\x18\x04 \x03(\x0b\x32+.worker.ExplainTextRequest.ColumnTypesEntry\x12\x11\n\tn_samples\x18\x05 \x01(\r\x1a.\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x43olumnTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb9\x02\n\x0f\x45xplainResponse\x12?\n\x0c\x65xplanations\x18\x01 \x03(\x0b\x32).worker.ExplainResponse.ExplanationsEntry\x1a\x8a\x01\n\x0b\x45xplanation\x12H\n\x0bper_feature\x18\x01 \x03(\x0b\x32\x33.worker.ExplainResponse.Explanation.PerFeatureEntry\x1a\x31\n\x0fPerFeatureEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1aX\n\x11\x45xplanationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.worker.ExplainResponse.Explanation:\x02\x38\x01\"\xe4\x01\n\x13\x45xplainTextResponse\x12\r\n\x05words\x18\x01 \x03(\t\x12\x39\n\x07weights\x18\x02 \x03(\x0b\x32(.worker.ExplainTextResponse.WeightsEntry\x1a$\n\x11WeightsPerFeature\x12\x0f\n\x07weights\x18\x01 \x03(\x02\x1a]\n\x0cWeightsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12<\n\x05value\x18\x02 \x01(\x0b\x32-.worker.ExplainTextResponse.WeightsPerFeature:\x02\x38\x01\"\x8d\x01\n\x1cRunModelForDataFrameResponse\x12*\n\x0f\x61ll_predictions\x18\x01 \x01(\x0b\x32\x11.worker.DataFrame\x12\x12\n\nprediction\x18\x02 \x03(\t\x12\x15\n\rprobabilities\x18\x03 \x03(\x02\x12\x16\n\x0eraw_prediction\x18\x04 \x03(\x02\"h\n\x07\x44\x61taRow\x12-\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x1c.worker.DataRow.ColumnsEntry\x1a.\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"*\n\tDataFrame\x12\x1d\n\x04rows\x18\x01 \x03(\x0b\x32\x0f.worker.DataRow\"\xfa\x02\n\x1bRunModelForDataFrameRequest\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.worker.ArtifactRef\x12$\n\tdataframe\x18\x02 \x01(\x0b\x32\x11.worker.DataFrame\x12\x0e\n\x06target\x18\x03 \x01(\t\x12J\n\x0c\x63olumn_types\x18\x04 \x03(\x0b\x32\x34.worker.RunModelForDataFrameRequest.ColumnTypesEntry\x12L\n\rcolumn_dtypes\x18\x05 \x03(\x0b\x32\x35.worker.RunModelForDataFrameRequest.ColumnDtypesEntry\x1a\x32\n\x10\x43olumnTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11\x43olumnDtypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x86\x01\n\x0fRunModelRequest\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.worker.ArtifactRef\x12$\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x13.worker.ArtifactRef\x12\x14\n\x0cinspectionId\x18\x03 \x01(\x04\x12\x13\n\x0bproject_key\x18\x04 \x01(\t\"?\n\x10RunModelResponse\x12\x13\n\x0bresults_csv\x18\x01 \x01(\t\x12\x16\n\x0e\x63\x61lculated_csv\x18\x02 \x01(\t\"P\n\x13RunAdHocTestRequest\x12\x10\n\x08testUuid\x18\x01 \x01(\t\x12\'\n\targuments\x18\x02 \x03(\x0b\x32\x14.worker.FuncArgument\"v\n\x18\x44\x61tasetProcessingRequest\x12$\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x13.worker.ArtifactRef\x12\x34\n\tfunctions\x18\x02 \x03(\x0b\x32!.worker.DatasetProcessingFunction\"\xb7\x01\n\x19\x44\x61tasetProcessingFunction\x12.\n\x0fslicingFunction\x18\x01 \x01(\x0b\x32\x13.worker.ArtifactRefH\x00\x12\x35\n\x16transformationFunction\x18\x02 \x01(\x0b\x32\x13.worker.ArtifactRefH\x00\x12\'\n\targuments\x18\x03 \x03(\x0b\x32\x14.worker.FuncArgumentB\n\n\x08\x66unction\"Z\n\x11SuiteTestArgument\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x10\n\x08testUuid\x18\x02 \x01(\t\x12\'\n\targuments\x18\x03 \x03(\x0b\x32\x14.worker.FuncArgument\"n\n\x13RunTestSuiteRequest\x12(\n\x05tests\x18\x01 \x03(\x0b\x32\x19.worker.SuiteTestArgument\x12-\n\x0fglobalArguments\x18\x02 \x03(\x0b\x32\x14.worker.FuncArgument\"\xe0\x02\n\x0c\x46uncArgument\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x05model\x18\x02 \x01(\x0b\x32\x13.worker.ArtifactRefH\x00\x12&\n\x07\x64\x61taset\x18\x03 \x01(\x0b\x32\x13.worker.ArtifactRefH\x00\x12\x0f\n\x05\x66loat\x18\x04 \x01(\x02H\x00\x12\r\n\x03int\x18\x05 \x01(\x05H\x00\x12\r\n\x03str\x18\x06 \x01(\tH\x00\x12\x0e\n\x04\x62ool\x18\x07 \x01(\x08H\x00\x12.\n\x0fslicingFunction\x18\x08 \x01(\x0b\x32\x13.worker.ArtifactRefH\x00\x12\x35\n\x16transformationFunction\x18\t \x01(\x0b\x32\x13.worker.ArtifactRefH\x00\x12\x10\n\x06kwargs\x18\n \x01(\tH\x00\x12\"\n\x04\x61rgs\x18\x0b \x03(\x0b\x32\x14.worker.FuncArgument\x12\x0c\n\x04none\x18\x0c \x01(\x08\x42\n\n\x08\x61rgument\"\x1f\n\x0fRunTestResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\"9\n\x19Partial_unexpected_counts\x12\r\n\x05value\x18\x01 \x03(\r\x12\r\n\x05\x63ount\x18\x02 \x01(\r\"S\n\x15NamedSingleTestResult\x12\x10\n\x08testUuid\x18\x01 \x01(\t\x12(\n\x06result\x18\x02 \x01(\x0b\x32\x18.worker.SingleTestResult\"R\n\x1aIdentifierSingleTestResult\x12\n\n\x02id\x18\x01 \x01(\x03\x12(\n\x06result\x18\x02 \x01(\x0b\x32\x18.worker.SingleTestResult\"B\n\x0bTestMessage\x12%\n\x04type\x18\x01 \x01(\x0e\x32\x17.worker.TestMessageType\x12\x0c\n\x04text\x18\x02 \x01(\t\"\xf8\x04\n\x10SingleTestResult\x12\x0e\n\x06passed\x18\r \x01(\x08\x12\x10\n\x08is_error\x18\x17 \x01(\x08\x12%\n\x08messages\x18\x10 \x03(\x0b\x32\x13.worker.TestMessage\x12\x32\n\x05props\x18\x0e \x03(\x0b\x32#.worker.SingleTestResult.PropsEntry\x12\x0e\n\x06metric\x18\x0f \x01(\x02\x12\x15\n\rmissing_count\x18\x02 \x01(\x05\x12\x17\n\x0fmissing_percent\x18\x03 \x01(\x01\x12\x18\n\x10unexpected_count\x18\x04 \x01(\x05\x12\x1a\n\x12unexpected_percent\x18\x05 \x01(\x01\x12 \n\x18unexpected_percent_total\x18\x06 \x01(\x01\x12%\n\x1dunexpected_percent_nonmissing\x18\x07 \x01(\x01\x12%\n\x1dpartial_unexpected_index_list\x18\t \x03(\r\x12\x44\n\x19partial_unexpected_counts\x18\n \x03(\x0b\x32!.worker.Partial_unexpected_counts\x12\x1d\n\x15unexpected_index_list\x18\x0c \x03(\r\x12\x11\n\toutput_df\x18\x12 \x01(\x0c\x12 \n\x18number_of_perturbed_rows\x18\x14 \x01(\r\x12\x1a\n\x12\x61\x63tual_slices_size\x18\x15 \x03(\r\x12\x1d\n\x15reference_slices_size\x18\x16 \x03(\r\x1a,\n\nPropsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"C\n\x11TestResultMessage\x12.\n\x07results\x18\x01 \x03(\x0b\x32\x1d.worker.NamedSingleTestResult\"\x99\x01\n\x1e\x44\x61tasetProcessingResultMessage\x12\x11\n\tdatasetId\x18\x01 \x01(\t\x12\x11\n\ttotalRows\x18\x02 \x01(\r\x12\x14\n\x0c\x66ilteredRows\x18\x03 \x03(\r\x12;\n\rmodifications\x18\x04 \x03(\x0b\x32$.worker.DatasetRowModificationResult\"\xb3\x01\n\x1c\x44\x61tasetRowModificationResult\x12\r\n\x05rowId\x18\x01 \x01(\r\x12N\n\rmodifications\x18\x02 \x03(\x0b\x32\x37.worker.DatasetRowModificationResult.ModificationsEntry\x1a\x34\n\x12ModificationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"~\n\x16TestSuiteResultMessage\x12\x10\n\x08is_error\x18\x01 \x01(\x08\x12\x0f\n\x07is_pass\x18\x02 \x01(\x08\x12\x33\n\x07results\x18\x03 \x03(\x0b\x32\".worker.IdentifierSingleTestResult\x12\x0c\n\x04logs\x18\x04 \x01(\t\"N\n\x0b\x41rtifactRef\x12\x13\n\x0bproject_key\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x13\n\x06sample\x18\x03 \x01(\x08H\x00\x88\x01\x01\x42\t\n\x07_sample\"\x18\n\x05\x43hunk\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"0\n\x0cUploadStatus\x12 \n\x04\x63ode\x18\x01 \x01(\x0e\x32\x12.worker.StatusCode\"1\n\x11MLWorkerErrorInfo\x12\r\n\x05\x65rror\x18\x01 \x01(\t\x12\r\n\x05stack\x18\x02 \x01(\t\"\xa4\x01\n\nSuiteInput\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12*\n\nmodel_meta\x18\x03 \x01(\x0b\x32\x11.worker.ModelMetaH\x00\x88\x01\x01\x12.\n\x0c\x64\x61taset_meta\x18\x04 \x01(\x0b\x32\x13.worker.DatasetMetaH\x01\x88\x01\x01\x42\r\n\x0b_model_metaB\x0f\n\r_dataset_meta\"3\n\tModelMeta\x12\x17\n\nmodel_type\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_model_type\"-\n\x0b\x44\x61tasetMeta\x12\x13\n\x06target\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\t\n\x07_target\"S\n\x18GenerateTestSuiteRequest\x12\x13\n\x0bproject_key\x18\x01 \x01(\t\x12\"\n\x06inputs\x18\x02 \x03(\x0b\x32\x12.worker.SuiteInput\"A\n\x19GenerateTestSuiteResponse\x12$\n\x05tests\x18\x01 \x03(\x0b\x32\x15.worker.GeneratedTest\"N\n\rGeneratedTest\x12\x11\n\ttest_uuid\x18\x01 \x01(\t\x12*\n\x06inputs\x18\x02 \x03(\x0b\x32\x1a.worker.GeneratedTestInput\"C\n\x12GeneratedTestInput\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x10\n\x08is_alias\x18\x03 \x01(\x08\"\xb3\x03\n\x0f\x43\x61talogResponse\x12\x31\n\x05tests\x18\x01 \x03(\x0b\x32\".worker.CatalogResponse.TestsEntry\x12\x33\n\x06slices\x18\x02 \x03(\x0b\x32#.worker.CatalogResponse.SlicesEntry\x12\x45\n\x0ftransformations\x18\x03 \x03(\x0b\x32,.worker.CatalogResponse.TransformationsEntry\x1a\x42\n\nTestsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.worker.FunctionMeta:\x02\x38\x01\x1aQ\n\x0bSlicesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\".worker.DatasetProcessFunctionMeta:\x02\x38\x01\x1aZ\n\x14TransformationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\".worker.DatasetProcessFunctionMeta:\x02\x38\x01\"\xd6\x01\n\x0c\x46unctionMeta\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64isplayName\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\r\x12\x0e\n\x06module\x18\x05 \x01(\t\x12\x0b\n\x03\x64oc\x18\x06 \x01(\t\x12\x11\n\tmoduleDoc\x18\x07 \x01(\t\x12*\n\x04\x61rgs\x18\x08 \x03(\x0b\x32\x1c.worker.TestFunctionArgument\x12\x0c\n\x04tags\x18\t \x03(\t\x12\x0c\n\x04\x63ode\x18\n \x01(\t\x12\x0c\n\x04type\x18\x0b \x01(\t\"\xb4\x02\n\x1a\x44\x61tasetProcessFunctionMeta\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64isplayName\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\r\x12\x0e\n\x06module\x18\x05 \x01(\t\x12\x0b\n\x03\x64oc\x18\x06 \x01(\t\x12\x11\n\tmoduleDoc\x18\x07 \x01(\t\x12*\n\x04\x61rgs\x18\x08 \x03(\x0b\x32\x1c.worker.TestFunctionArgument\x12\x0c\n\x04tags\x18\t \x03(\t\x12\x0c\n\x04\x63ode\x18\n \x01(\t\x12\x0c\n\x04type\x18\x0b \x01(\t\x12\x11\n\tcellLevel\x18\x0c \x01(\x08\x12\x17\n\ncolumnType\x18\r \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bprocessType\x18\x0e \x01(\tB\r\n\x0b_columnType\"g\n\x14TestFunctionArgument\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x10\n\x08optional\x18\x03 \x01(\x08\x12\x0f\n\x07\x64\x65\x66\x61ult\x18\x04 \x01(\t\x12\x10\n\x08\x61rgOrder\x18\x05 \x01(\r\"\x8c\x01\n\x10\x43omparisonClause\x12\x13\n\x0b\x63olumn_name\x18\x01 \x01(\t\x12/\n\x0f\x63omparison_type\x18\x02 \x01(\x0e\x32\x16.worker.ComparisonType\x12\x14\n\x0c\x63olumn_dtype\x18\x03 \x01(\t\x12\x12\n\x05value\x18\x04 \x01(\tH\x00\x88\x01\x01\x42\x08\n\x06_value\"V\n\x1eGenerateQueryBasedSliceRequest\x12\x34\n\x12\x63omparison_clauses\x18\x01 \x03(\x0b\x32\x18.worker.ComparisonClause\"-\n\x1fGenerateQueryBasedSliceResponse\x12\n\n\x02id\x18\x01 \x01(\t*\"\n\x08\x46ileType\x12\t\n\x05MODEL\x10\x00\x12\x0b\n\x07\x44\x41TASET\x10\x01*&\n\x0fTestMessageType\x12\t\n\x05\x45RROR\x10\x00\x12\x08\n\x04INFO\x10\x01*Q\n\nStatusCode\x12\x0b\n\x07Unknown\x10\x00\x12\x06\n\x02Ok\x10\x01\x12\n\n\x06\x46\x61iled\x10\x02\x12\r\n\tCacheMiss\x10\x03\x12\t\n\x05Ready\x10\x04\x12\x08\n\x04Next\x10\x05*\x89\x01\n\x0e\x43omparisonType\x12\x06\n\x02IS\x10\x00\x12\n\n\x06IS_NOT\x10\x01\x12\x0c\n\x08\x43ONTAINS\x10\x02\x12\x15\n\x11\x44OES_NOT_CONTAINS\x10\x03\x12\x0f\n\x0bSTARTS_WITH\x10\x04\x12\r\n\tENDS_WITH\x10\x05\x12\x0c\n\x08IS_EMPTY\x10\x06\x12\x10\n\x0cIS_NOT_EMPTY\x10\x07\x32\xf1\x07\n\x08MLWorker\x12>\n\x07getInfo\x12\x1b.worker.MLWorkerInfoRequest\x1a\x14.worker.MLWorkerInfo\"\x00\x12H\n\x0crunAdHocTest\x12\x1b.worker.RunAdHocTestRequest\x1a\x19.worker.TestResultMessage\"\x00\x12_\n\x11\x64\x61tasetProcessing\x12 .worker.DatasetProcessingRequest\x1a&.worker.DatasetProcessingResultMessage\"\x00\x12M\n\x0crunTestSuite\x12\x1b.worker.RunTestSuiteRequest\x1a\x1e.worker.TestSuiteResultMessage\"\x00\x12=\n\x08runModel\x12\x17.worker.RunModelRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x63\n\x14runModelForDataFrame\x12#.worker.RunModelForDataFrameRequest\x1a$.worker.RunModelForDataFrameResponse\"\x00\x12<\n\x07\x65xplain\x12\x16.worker.ExplainRequest\x1a\x17.worker.ExplainResponse\"\x00\x12H\n\x0b\x65xplainText\x12\x1a.worker.ExplainTextRequest\x1a\x1b.worker.ExplainTextResponse\"\x00\x12*\n\x04\x65\x63ho\x12\x0f.worker.EchoMsg\x1a\x0f.worker.EchoMsg\"\x00\x12Z\n\x11generateTestSuite\x12 .worker.GenerateTestSuiteRequest\x1a!.worker.GenerateTestSuiteResponse\"\x00\x12>\n\nstopWorker\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"\x00\x12?\n\ngetCatalog\x12\x16.google.protobuf.Empty\x1a\x17.worker.CatalogResponse\"\x00\x12v\n!generateQueryBasedSlicingFunction\x12&.worker.GenerateQueryBasedSliceRequest\x1a\'.worker.GenerateQueryBasedSliceResponse\"\x00\x42(\n\x11\x61i.giskard.workerB\x0bWorkerProtoP\x01\xa2\x02\x03WRKb\x06proto3')

_FILETYPE = DESCRIPTOR.enum_types_by_name['FileType']
FileType = enum_type_wrapper.EnumTypeWrapper(_FILETYPE)
_TESTMESSAGETYPE = DESCRIPTOR.enum_types_by_name['TestMessageType']
TestMessageType = enum_type_wrapper.EnumTypeWrapper(_TESTMESSAGETYPE)
_STATUSCODE = DESCRIPTOR.enum_types_by_name['StatusCode']
StatusCode = enum_type_wrapper.EnumTypeWrapper(_STATUSCODE)
_COMPARISONTYPE = DESCRIPTOR.enum_types_by_name['ComparisonType']
ComparisonType = enum_type_wrapper.EnumTypeWrapper(_COMPARISONTYPE)
MODEL = 0
DATASET = 1
ERROR = 0
INFO = 1
Unknown = 0
Ok = 1
Failed = 2
CacheMiss = 3
Ready = 4
Next = 5
IS = 0
IS_NOT = 1
CONTAINS = 2
DOES_NOT_CONTAINS = 3
STARTS_WITH = 4
ENDS_WITH = 5
IS_EMPTY = 6
IS_NOT_EMPTY = 7


_MLWORKERINFOREQUEST = DESCRIPTOR.message_types_by_name['MLWorkerInfoRequest']
_MLWORKERINFO = DESCRIPTOR.message_types_by_name['MLWorkerInfo']
_MLWORKERINFO_INSTALLEDPACKAGESENTRY = _MLWORKERINFO.nested_types_by_name['InstalledPackagesEntry']
_PLATFORMINFO = DESCRIPTOR.message_types_by_name['PlatformInfo']
_FILEUPLOADREQUEST = DESCRIPTOR.message_types_by_name['FileUploadRequest']
_FILEUPLOADMETADATA = DESCRIPTOR.message_types_by_name['FileUploadMetadata']
_ECHOMSG = DESCRIPTOR.message_types_by_name['EchoMsg']
_EXPLAINREQUEST = DESCRIPTOR.message_types_by_name['ExplainRequest']
_EXPLAINREQUEST_COLUMNSENTRY = _EXPLAINREQUEST.nested_types_by_name['ColumnsEntry']
_EXPLAINTEXTREQUEST = DESCRIPTOR.message_types_by_name['ExplainTextRequest']
_EXPLAINTEXTREQUEST_COLUMNSENTRY = _EXPLAINTEXTREQUEST.nested_types_by_name['ColumnsEntry']
_EXPLAINTEXTREQUEST_COLUMNTYPESENTRY = _EXPLAINTEXTREQUEST.nested_types_by_name['ColumnTypesEntry']
_EXPLAINRESPONSE = DESCRIPTOR.message_types_by_name['ExplainResponse']
_EXPLAINRESPONSE_EXPLANATION = _EXPLAINRESPONSE.nested_types_by_name['Explanation']
_EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY = _EXPLAINRESPONSE_EXPLANATION.nested_types_by_name['PerFeatureEntry']
_EXPLAINRESPONSE_EXPLANATIONSENTRY = _EXPLAINRESPONSE.nested_types_by_name['ExplanationsEntry']
_EXPLAINTEXTRESPONSE = DESCRIPTOR.message_types_by_name['ExplainTextResponse']
_EXPLAINTEXTRESPONSE_WEIGHTSPERFEATURE = _EXPLAINTEXTRESPONSE.nested_types_by_name['WeightsPerFeature']
_EXPLAINTEXTRESPONSE_WEIGHTSENTRY = _EXPLAINTEXTRESPONSE.nested_types_by_name['WeightsEntry']
_RUNMODELFORDATAFRAMERESPONSE = DESCRIPTOR.message_types_by_name['RunModelForDataFrameResponse']
_DATAROW = DESCRIPTOR.message_types_by_name['DataRow']
_DATAROW_COLUMNSENTRY = _DATAROW.nested_types_by_name['ColumnsEntry']
_DATAFRAME = DESCRIPTOR.message_types_by_name['DataFrame']
_RUNMODELFORDATAFRAMEREQUEST = DESCRIPTOR.message_types_by_name['RunModelForDataFrameRequest']
_RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY = _RUNMODELFORDATAFRAMEREQUEST.nested_types_by_name['ColumnTypesEntry']
_RUNMODELFORDATAFRAMEREQUEST_COLUMNDTYPESENTRY = _RUNMODELFORDATAFRAMEREQUEST.nested_types_by_name['ColumnDtypesEntry']
_RUNMODELREQUEST = DESCRIPTOR.message_types_by_name['RunModelRequest']
_RUNMODELRESPONSE = DESCRIPTOR.message_types_by_name['RunModelResponse']
_RUNADHOCTESTREQUEST = DESCRIPTOR.message_types_by_name['RunAdHocTestRequest']
_DATASETPROCESSINGREQUEST = DESCRIPTOR.message_types_by_name['DatasetProcessingRequest']
_DATASETPROCESSINGFUNCTION = DESCRIPTOR.message_types_by_name['DatasetProcessingFunction']
_SUITETESTARGUMENT = DESCRIPTOR.message_types_by_name['SuiteTestArgument']
_RUNTESTSUITEREQUEST = DESCRIPTOR.message_types_by_name['RunTestSuiteRequest']
_FUNCARGUMENT = DESCRIPTOR.message_types_by_name['FuncArgument']
_RUNTESTRESPONSE = DESCRIPTOR.message_types_by_name['RunTestResponse']
_PARTIAL_UNEXPECTED_COUNTS = DESCRIPTOR.message_types_by_name['Partial_unexpected_counts']
_NAMEDSINGLETESTRESULT = DESCRIPTOR.message_types_by_name['NamedSingleTestResult']
_IDENTIFIERSINGLETESTRESULT = DESCRIPTOR.message_types_by_name['IdentifierSingleTestResult']
_TESTMESSAGE = DESCRIPTOR.message_types_by_name['TestMessage']
_SINGLETESTRESULT = DESCRIPTOR.message_types_by_name['SingleTestResult']
_SINGLETESTRESULT_PROPSENTRY = _SINGLETESTRESULT.nested_types_by_name['PropsEntry']
_TESTRESULTMESSAGE = DESCRIPTOR.message_types_by_name['TestResultMessage']
_DATASETPROCESSINGRESULTMESSAGE = DESCRIPTOR.message_types_by_name['DatasetProcessingResultMessage']
_DATASETROWMODIFICATIONRESULT = DESCRIPTOR.message_types_by_name['DatasetRowModificationResult']
_DATASETROWMODIFICATIONRESULT_MODIFICATIONSENTRY = _DATASETROWMODIFICATIONRESULT.nested_types_by_name['ModificationsEntry']
_TESTSUITERESULTMESSAGE = DESCRIPTOR.message_types_by_name['TestSuiteResultMessage']
_ARTIFACTREF = DESCRIPTOR.message_types_by_name['ArtifactRef']
_CHUNK = DESCRIPTOR.message_types_by_name['Chunk']
_UPLOADSTATUS = DESCRIPTOR.message_types_by_name['UploadStatus']
_MLWORKERERRORINFO = DESCRIPTOR.message_types_by_name['MLWorkerErrorInfo']
_SUITEINPUT = DESCRIPTOR.message_types_by_name['SuiteInput']
_MODELMETA = DESCRIPTOR.message_types_by_name['ModelMeta']
_DATASETMETA = DESCRIPTOR.message_types_by_name['DatasetMeta']
_GENERATETESTSUITEREQUEST = DESCRIPTOR.message_types_by_name['GenerateTestSuiteRequest']
_GENERATETESTSUITERESPONSE = DESCRIPTOR.message_types_by_name['GenerateTestSuiteResponse']
_GENERATEDTEST = DESCRIPTOR.message_types_by_name['GeneratedTest']
_GENERATEDTESTINPUT = DESCRIPTOR.message_types_by_name['GeneratedTestInput']
_CATALOGRESPONSE = DESCRIPTOR.message_types_by_name['CatalogResponse']
_CATALOGRESPONSE_TESTSENTRY = _CATALOGRESPONSE.nested_types_by_name['TestsEntry']
_CATALOGRESPONSE_SLICESENTRY = _CATALOGRESPONSE.nested_types_by_name['SlicesEntry']
_CATALOGRESPONSE_TRANSFORMATIONSENTRY = _CATALOGRESPONSE.nested_types_by_name['TransformationsEntry']
_FUNCTIONMETA = DESCRIPTOR.message_types_by_name['FunctionMeta']
_DATASETPROCESSFUNCTIONMETA = DESCRIPTOR.message_types_by_name['DatasetProcessFunctionMeta']
_TESTFUNCTIONARGUMENT = DESCRIPTOR.message_types_by_name['TestFunctionArgument']
_COMPARISONCLAUSE = DESCRIPTOR.message_types_by_name['ComparisonClause']
_GENERATEQUERYBASEDSLICEREQUEST = DESCRIPTOR.message_types_by_name['GenerateQueryBasedSliceRequest']
_GENERATEQUERYBASEDSLICERESPONSE = DESCRIPTOR.message_types_by_name['GenerateQueryBasedSliceResponse']
MLWorkerInfoRequest = _reflection.GeneratedProtocolMessageType('MLWorkerInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _MLWORKERINFOREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.MLWorkerInfoRequest)
  })
_sym_db.RegisterMessage(MLWorkerInfoRequest)

MLWorkerInfo = _reflection.GeneratedProtocolMessageType('MLWorkerInfo', (_message.Message,), {

  'InstalledPackagesEntry' : _reflection.GeneratedProtocolMessageType('InstalledPackagesEntry', (_message.Message,), {
    'DESCRIPTOR' : _MLWORKERINFO_INSTALLEDPACKAGESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.MLWorkerInfo.InstalledPackagesEntry)
    })
  ,
  'DESCRIPTOR' : _MLWORKERINFO,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.MLWorkerInfo)
  })
_sym_db.RegisterMessage(MLWorkerInfo)
_sym_db.RegisterMessage(MLWorkerInfo.InstalledPackagesEntry)

PlatformInfo = _reflection.GeneratedProtocolMessageType('PlatformInfo', (_message.Message,), {
  'DESCRIPTOR' : _PLATFORMINFO,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.PlatformInfo)
  })
_sym_db.RegisterMessage(PlatformInfo)

FileUploadRequest = _reflection.GeneratedProtocolMessageType('FileUploadRequest', (_message.Message,), {
  'DESCRIPTOR' : _FILEUPLOADREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.FileUploadRequest)
  })
_sym_db.RegisterMessage(FileUploadRequest)

FileUploadMetadata = _reflection.GeneratedProtocolMessageType('FileUploadMetadata', (_message.Message,), {
  'DESCRIPTOR' : _FILEUPLOADMETADATA,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.FileUploadMetadata)
  })
_sym_db.RegisterMessage(FileUploadMetadata)

EchoMsg = _reflection.GeneratedProtocolMessageType('EchoMsg', (_message.Message,), {
  'DESCRIPTOR' : _ECHOMSG,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.EchoMsg)
  })
_sym_db.RegisterMessage(EchoMsg)

ExplainRequest = _reflection.GeneratedProtocolMessageType('ExplainRequest', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINREQUEST_COLUMNSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainRequest.ColumnsEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainRequest)
  })
_sym_db.RegisterMessage(ExplainRequest)
_sym_db.RegisterMessage(ExplainRequest.ColumnsEntry)

ExplainTextRequest = _reflection.GeneratedProtocolMessageType('ExplainTextRequest', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTREQUEST_COLUMNSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextRequest.ColumnsEntry)
    })
  ,

  'ColumnTypesEntry' : _reflection.GeneratedProtocolMessageType('ColumnTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTREQUEST_COLUMNTYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextRequest.ColumnTypesEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINTEXTREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainTextRequest)
  })
_sym_db.RegisterMessage(ExplainTextRequest)
_sym_db.RegisterMessage(ExplainTextRequest.ColumnsEntry)
_sym_db.RegisterMessage(ExplainTextRequest.ColumnTypesEntry)

ExplainResponse = _reflection.GeneratedProtocolMessageType('ExplainResponse', (_message.Message,), {

  'Explanation' : _reflection.GeneratedProtocolMessageType('Explanation', (_message.Message,), {

    'PerFeatureEntry' : _reflection.GeneratedProtocolMessageType('PerFeatureEntry', (_message.Message,), {
      'DESCRIPTOR' : _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY,
      '__module__' : 'ml_worker_pb2'
      # @@protoc_insertion_point(class_scope:worker.ExplainResponse.Explanation.PerFeatureEntry)
      })
    ,
    'DESCRIPTOR' : _EXPLAINRESPONSE_EXPLANATION,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainResponse.Explanation)
    })
  ,

  'ExplanationsEntry' : _reflection.GeneratedProtocolMessageType('ExplanationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINRESPONSE_EXPLANATIONSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainResponse.ExplanationsEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainResponse)
  })
_sym_db.RegisterMessage(ExplainResponse)
_sym_db.RegisterMessage(ExplainResponse.Explanation)
_sym_db.RegisterMessage(ExplainResponse.Explanation.PerFeatureEntry)
_sym_db.RegisterMessage(ExplainResponse.ExplanationsEntry)

ExplainTextResponse = _reflection.GeneratedProtocolMessageType('ExplainTextResponse', (_message.Message,), {

  'WeightsPerFeature' : _reflection.GeneratedProtocolMessageType('WeightsPerFeature', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTRESPONSE_WEIGHTSPERFEATURE,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextResponse.WeightsPerFeature)
    })
  ,

  'WeightsEntry' : _reflection.GeneratedProtocolMessageType('WeightsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTRESPONSE_WEIGHTSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextResponse.WeightsEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINTEXTRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainTextResponse)
  })
_sym_db.RegisterMessage(ExplainTextResponse)
_sym_db.RegisterMessage(ExplainTextResponse.WeightsPerFeature)
_sym_db.RegisterMessage(ExplainTextResponse.WeightsEntry)

RunModelForDataFrameResponse = _reflection.GeneratedProtocolMessageType('RunModelForDataFrameResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNMODELFORDATAFRAMERESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameResponse)
  })
_sym_db.RegisterMessage(RunModelForDataFrameResponse)

DataRow = _reflection.GeneratedProtocolMessageType('DataRow', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATAROW_COLUMNSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.DataRow.ColumnsEntry)
    })
  ,
  'DESCRIPTOR' : _DATAROW,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DataRow)
  })
_sym_db.RegisterMessage(DataRow)
_sym_db.RegisterMessage(DataRow.ColumnsEntry)

DataFrame = _reflection.GeneratedProtocolMessageType('DataFrame', (_message.Message,), {
  'DESCRIPTOR' : _DATAFRAME,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DataFrame)
  })
_sym_db.RegisterMessage(DataFrame)

RunModelForDataFrameRequest = _reflection.GeneratedProtocolMessageType('RunModelForDataFrameRequest', (_message.Message,), {

  'ColumnTypesEntry' : _reflection.GeneratedProtocolMessageType('ColumnTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameRequest.ColumnTypesEntry)
    })
  ,

  'ColumnDtypesEntry' : _reflection.GeneratedProtocolMessageType('ColumnDtypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _RUNMODELFORDATAFRAMEREQUEST_COLUMNDTYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameRequest.ColumnDtypesEntry)
    })
  ,
  'DESCRIPTOR' : _RUNMODELFORDATAFRAMEREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameRequest)
  })
_sym_db.RegisterMessage(RunModelForDataFrameRequest)
_sym_db.RegisterMessage(RunModelForDataFrameRequest.ColumnTypesEntry)
_sym_db.RegisterMessage(RunModelForDataFrameRequest.ColumnDtypesEntry)

RunModelRequest = _reflection.GeneratedProtocolMessageType('RunModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNMODELREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelRequest)
  })
_sym_db.RegisterMessage(RunModelRequest)

RunModelResponse = _reflection.GeneratedProtocolMessageType('RunModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNMODELRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelResponse)
  })
_sym_db.RegisterMessage(RunModelResponse)

RunAdHocTestRequest = _reflection.GeneratedProtocolMessageType('RunAdHocTestRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNADHOCTESTREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunAdHocTestRequest)
  })
_sym_db.RegisterMessage(RunAdHocTestRequest)

DatasetProcessingRequest = _reflection.GeneratedProtocolMessageType('DatasetProcessingRequest', (_message.Message,), {
  'DESCRIPTOR' : _DATASETPROCESSINGREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DatasetProcessingRequest)
  })
_sym_db.RegisterMessage(DatasetProcessingRequest)

DatasetProcessingFunction = _reflection.GeneratedProtocolMessageType('DatasetProcessingFunction', (_message.Message,), {
  'DESCRIPTOR' : _DATASETPROCESSINGFUNCTION,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DatasetProcessingFunction)
  })
_sym_db.RegisterMessage(DatasetProcessingFunction)

SuiteTestArgument = _reflection.GeneratedProtocolMessageType('SuiteTestArgument', (_message.Message,), {
  'DESCRIPTOR' : _SUITETESTARGUMENT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.SuiteTestArgument)
  })
_sym_db.RegisterMessage(SuiteTestArgument)

RunTestSuiteRequest = _reflection.GeneratedProtocolMessageType('RunTestSuiteRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNTESTSUITEREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunTestSuiteRequest)
  })
_sym_db.RegisterMessage(RunTestSuiteRequest)

FuncArgument = _reflection.GeneratedProtocolMessageType('FuncArgument', (_message.Message,), {
  'DESCRIPTOR' : _FUNCARGUMENT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.FuncArgument)
  })
_sym_db.RegisterMessage(FuncArgument)

RunTestResponse = _reflection.GeneratedProtocolMessageType('RunTestResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNTESTRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunTestResponse)
  })
_sym_db.RegisterMessage(RunTestResponse)

Partial_unexpected_counts = _reflection.GeneratedProtocolMessageType('Partial_unexpected_counts', (_message.Message,), {
  'DESCRIPTOR' : _PARTIAL_UNEXPECTED_COUNTS,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.Partial_unexpected_counts)
  })
_sym_db.RegisterMessage(Partial_unexpected_counts)

NamedSingleTestResult = _reflection.GeneratedProtocolMessageType('NamedSingleTestResult', (_message.Message,), {
  'DESCRIPTOR' : _NAMEDSINGLETESTRESULT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.NamedSingleTestResult)
  })
_sym_db.RegisterMessage(NamedSingleTestResult)

IdentifierSingleTestResult = _reflection.GeneratedProtocolMessageType('IdentifierSingleTestResult', (_message.Message,), {
  'DESCRIPTOR' : _IDENTIFIERSINGLETESTRESULT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.IdentifierSingleTestResult)
  })
_sym_db.RegisterMessage(IdentifierSingleTestResult)

TestMessage = _reflection.GeneratedProtocolMessageType('TestMessage', (_message.Message,), {
  'DESCRIPTOR' : _TESTMESSAGE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.TestMessage)
  })
_sym_db.RegisterMessage(TestMessage)

SingleTestResult = _reflection.GeneratedProtocolMessageType('SingleTestResult', (_message.Message,), {

  'PropsEntry' : _reflection.GeneratedProtocolMessageType('PropsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SINGLETESTRESULT_PROPSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.SingleTestResult.PropsEntry)
    })
  ,
  'DESCRIPTOR' : _SINGLETESTRESULT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.SingleTestResult)
  })
_sym_db.RegisterMessage(SingleTestResult)
_sym_db.RegisterMessage(SingleTestResult.PropsEntry)

TestResultMessage = _reflection.GeneratedProtocolMessageType('TestResultMessage', (_message.Message,), {
  'DESCRIPTOR' : _TESTRESULTMESSAGE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.TestResultMessage)
  })
_sym_db.RegisterMessage(TestResultMessage)

DatasetProcessingResultMessage = _reflection.GeneratedProtocolMessageType('DatasetProcessingResultMessage', (_message.Message,), {
  'DESCRIPTOR' : _DATASETPROCESSINGRESULTMESSAGE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DatasetProcessingResultMessage)
  })
_sym_db.RegisterMessage(DatasetProcessingResultMessage)

DatasetRowModificationResult = _reflection.GeneratedProtocolMessageType('DatasetRowModificationResult', (_message.Message,), {

  'ModificationsEntry' : _reflection.GeneratedProtocolMessageType('ModificationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETROWMODIFICATIONRESULT_MODIFICATIONSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.DatasetRowModificationResult.ModificationsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETROWMODIFICATIONRESULT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DatasetRowModificationResult)
  })
_sym_db.RegisterMessage(DatasetRowModificationResult)
_sym_db.RegisterMessage(DatasetRowModificationResult.ModificationsEntry)

TestSuiteResultMessage = _reflection.GeneratedProtocolMessageType('TestSuiteResultMessage', (_message.Message,), {
  'DESCRIPTOR' : _TESTSUITERESULTMESSAGE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.TestSuiteResultMessage)
  })
_sym_db.RegisterMessage(TestSuiteResultMessage)

ArtifactRef = _reflection.GeneratedProtocolMessageType('ArtifactRef', (_message.Message,), {
  'DESCRIPTOR' : _ARTIFACTREF,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ArtifactRef)
  })
_sym_db.RegisterMessage(ArtifactRef)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), {
  'DESCRIPTOR' : _CHUNK,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.Chunk)
  })
_sym_db.RegisterMessage(Chunk)

UploadStatus = _reflection.GeneratedProtocolMessageType('UploadStatus', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADSTATUS,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.UploadStatus)
  })
_sym_db.RegisterMessage(UploadStatus)

MLWorkerErrorInfo = _reflection.GeneratedProtocolMessageType('MLWorkerErrorInfo', (_message.Message,), {
  'DESCRIPTOR' : _MLWORKERERRORINFO,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.MLWorkerErrorInfo)
  })
_sym_db.RegisterMessage(MLWorkerErrorInfo)

SuiteInput = _reflection.GeneratedProtocolMessageType('SuiteInput', (_message.Message,), {
  'DESCRIPTOR' : _SUITEINPUT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.SuiteInput)
  })
_sym_db.RegisterMessage(SuiteInput)

ModelMeta = _reflection.GeneratedProtocolMessageType('ModelMeta', (_message.Message,), {
  'DESCRIPTOR' : _MODELMETA,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ModelMeta)
  })
_sym_db.RegisterMessage(ModelMeta)

DatasetMeta = _reflection.GeneratedProtocolMessageType('DatasetMeta', (_message.Message,), {
  'DESCRIPTOR' : _DATASETMETA,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DatasetMeta)
  })
_sym_db.RegisterMessage(DatasetMeta)

GenerateTestSuiteRequest = _reflection.GeneratedProtocolMessageType('GenerateTestSuiteRequest', (_message.Message,), {
  'DESCRIPTOR' : _GENERATETESTSUITEREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.GenerateTestSuiteRequest)
  })
_sym_db.RegisterMessage(GenerateTestSuiteRequest)

GenerateTestSuiteResponse = _reflection.GeneratedProtocolMessageType('GenerateTestSuiteResponse', (_message.Message,), {
  'DESCRIPTOR' : _GENERATETESTSUITERESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.GenerateTestSuiteResponse)
  })
_sym_db.RegisterMessage(GenerateTestSuiteResponse)

GeneratedTest = _reflection.GeneratedProtocolMessageType('GeneratedTest', (_message.Message,), {
  'DESCRIPTOR' : _GENERATEDTEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.GeneratedTest)
  })
_sym_db.RegisterMessage(GeneratedTest)

GeneratedTestInput = _reflection.GeneratedProtocolMessageType('GeneratedTestInput', (_message.Message,), {
  'DESCRIPTOR' : _GENERATEDTESTINPUT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.GeneratedTestInput)
  })
_sym_db.RegisterMessage(GeneratedTestInput)

CatalogResponse = _reflection.GeneratedProtocolMessageType('CatalogResponse', (_message.Message,), {

  'TestsEntry' : _reflection.GeneratedProtocolMessageType('TestsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CATALOGRESPONSE_TESTSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.CatalogResponse.TestsEntry)
    })
  ,

  'SlicesEntry' : _reflection.GeneratedProtocolMessageType('SlicesEntry', (_message.Message,), {
    'DESCRIPTOR' : _CATALOGRESPONSE_SLICESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.CatalogResponse.SlicesEntry)
    })
  ,

  'TransformationsEntry' : _reflection.GeneratedProtocolMessageType('TransformationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CATALOGRESPONSE_TRANSFORMATIONSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.CatalogResponse.TransformationsEntry)
    })
  ,
  'DESCRIPTOR' : _CATALOGRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.CatalogResponse)
  })
_sym_db.RegisterMessage(CatalogResponse)
_sym_db.RegisterMessage(CatalogResponse.TestsEntry)
_sym_db.RegisterMessage(CatalogResponse.SlicesEntry)
_sym_db.RegisterMessage(CatalogResponse.TransformationsEntry)

FunctionMeta = _reflection.GeneratedProtocolMessageType('FunctionMeta', (_message.Message,), {
  'DESCRIPTOR' : _FUNCTIONMETA,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.FunctionMeta)
  })
_sym_db.RegisterMessage(FunctionMeta)

DatasetProcessFunctionMeta = _reflection.GeneratedProtocolMessageType('DatasetProcessFunctionMeta', (_message.Message,), {
  'DESCRIPTOR' : _DATASETPROCESSFUNCTIONMETA,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DatasetProcessFunctionMeta)
  })
_sym_db.RegisterMessage(DatasetProcessFunctionMeta)

TestFunctionArgument = _reflection.GeneratedProtocolMessageType('TestFunctionArgument', (_message.Message,), {
  'DESCRIPTOR' : _TESTFUNCTIONARGUMENT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.TestFunctionArgument)
  })
_sym_db.RegisterMessage(TestFunctionArgument)

ComparisonClause = _reflection.GeneratedProtocolMessageType('ComparisonClause', (_message.Message,), {
  'DESCRIPTOR' : _COMPARISONCLAUSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ComparisonClause)
  })
_sym_db.RegisterMessage(ComparisonClause)

GenerateQueryBasedSliceRequest = _reflection.GeneratedProtocolMessageType('GenerateQueryBasedSliceRequest', (_message.Message,), {
  'DESCRIPTOR' : _GENERATEQUERYBASEDSLICEREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.GenerateQueryBasedSliceRequest)
  })
_sym_db.RegisterMessage(GenerateQueryBasedSliceRequest)

GenerateQueryBasedSliceResponse = _reflection.GeneratedProtocolMessageType('GenerateQueryBasedSliceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GENERATEQUERYBASEDSLICERESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.GenerateQueryBasedSliceResponse)
  })
_sym_db.RegisterMessage(GenerateQueryBasedSliceResponse)

_MLWORKER = DESCRIPTOR.services_by_name['MLWorker']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021ai.giskard.workerB\013WorkerProtoP\001\242\002\003WRK'
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._options = None
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._serialized_options = b'8\001'
  _EXPLAINREQUEST_COLUMNSENTRY._options = None
  _EXPLAINREQUEST_COLUMNSENTRY._serialized_options = b'8\001'
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._options = None
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._serialized_options = b'8\001'
  _EXPLAINTEXTREQUEST_COLUMNTYPESENTRY._options = None
  _EXPLAINTEXTREQUEST_COLUMNTYPESENTRY._serialized_options = b'8\001'
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._options = None
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._serialized_options = b'8\001'
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._options = None
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._serialized_options = b'8\001'
  _EXPLAINTEXTRESPONSE_WEIGHTSENTRY._options = None
  _EXPLAINTEXTRESPONSE_WEIGHTSENTRY._serialized_options = b'8\001'
  _DATAROW_COLUMNSENTRY._options = None
  _DATAROW_COLUMNSENTRY._serialized_options = b'8\001'
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._options = None
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._serialized_options = b'8\001'
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNDTYPESENTRY._options = None
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNDTYPESENTRY._serialized_options = b'8\001'
  _SINGLETESTRESULT_PROPSENTRY._options = None
  _SINGLETESTRESULT_PROPSENTRY._serialized_options = b'8\001'
  _DATASETROWMODIFICATIONRESULT_MODIFICATIONSENTRY._options = None
  _DATASETROWMODIFICATIONRESULT_MODIFICATIONSENTRY._serialized_options = b'8\001'
  _CATALOGRESPONSE_TESTSENTRY._options = None
  _CATALOGRESPONSE_TESTSENTRY._serialized_options = b'8\001'
  _CATALOGRESPONSE_SLICESENTRY._options = None
  _CATALOGRESPONSE_SLICESENTRY._serialized_options = b'8\001'
  _CATALOGRESPONSE_TRANSFORMATIONSENTRY._options = None
  _CATALOGRESPONSE_TRANSFORMATIONSENTRY._serialized_options = b'8\001'
  _FILETYPE._serialized_start=7394
  _FILETYPE._serialized_end=7428
  _TESTMESSAGETYPE._serialized_start=7430
  _TESTMESSAGETYPE._serialized_end=7468
  _STATUSCODE._serialized_start=7470
  _STATUSCODE._serialized_end=7551
  _COMPARISONTYPE._serialized_start=7554
  _COMPARISONTYPE._serialized_end=7691
  _MLWORKERINFOREQUEST._serialized_start=115
  _MLWORKERINFOREQUEST._serialized_end=159
  _MLWORKERINFO._serialized_start=162
  _MLWORKERINFO._serialized_end=520
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._serialized_start=464
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._serialized_end=520
  _PLATFORMINFO._serialized_start=522
  _PLATFORMINFO._serialized_end=636
  _FILEUPLOADREQUEST._serialized_start=638
  _FILEUPLOADREQUEST._serialized_end=748
  _FILEUPLOADMETADATA._serialized_start=750
  _FILEUPLOADMETADATA._serialized_end=854
  _ECHOMSG._serialized_start=856
  _ECHOMSG._serialized_end=878
  _EXPLAINREQUEST._serialized_start=881
  _EXPLAINREQUEST._serialized_end=1073
  _EXPLAINREQUEST_COLUMNSENTRY._serialized_start=1027
  _EXPLAINREQUEST_COLUMNSENTRY._serialized_end=1073
  _EXPLAINTEXTREQUEST._serialized_start=1076
  _EXPLAINTEXTREQUEST._serialized_end=1398
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._serialized_start=1027
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._serialized_end=1073
  _EXPLAINTEXTREQUEST_COLUMNTYPESENTRY._serialized_start=1348
  _EXPLAINTEXTREQUEST_COLUMNTYPESENTRY._serialized_end=1398
  _EXPLAINRESPONSE._serialized_start=1401
  _EXPLAINRESPONSE._serialized_end=1714
  _EXPLAINRESPONSE_EXPLANATION._serialized_start=1486
  _EXPLAINRESPONSE_EXPLANATION._serialized_end=1624
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._serialized_start=1575
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._serialized_end=1624
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._serialized_start=1626
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._serialized_end=1714
  _EXPLAINTEXTRESPONSE._serialized_start=1717
  _EXPLAINTEXTRESPONSE._serialized_end=1945
  _EXPLAINTEXTRESPONSE_WEIGHTSPERFEATURE._serialized_start=1814
  _EXPLAINTEXTRESPONSE_WEIGHTSPERFEATURE._serialized_end=1850
  _EXPLAINTEXTRESPONSE_WEIGHTSENTRY._serialized_start=1852
  _EXPLAINTEXTRESPONSE_WEIGHTSENTRY._serialized_end=1945
  _RUNMODELFORDATAFRAMERESPONSE._serialized_start=1948
  _RUNMODELFORDATAFRAMERESPONSE._serialized_end=2089
  _DATAROW._serialized_start=2091
  _DATAROW._serialized_end=2195
  _DATAROW_COLUMNSENTRY._serialized_start=1027
  _DATAROW_COLUMNSENTRY._serialized_end=1073
  _DATAFRAME._serialized_start=2197
  _DATAFRAME._serialized_end=2239
  _RUNMODELFORDATAFRAMEREQUEST._serialized_start=2242
  _RUNMODELFORDATAFRAMEREQUEST._serialized_end=2620
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._serialized_start=1348
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._serialized_end=1398
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNDTYPESENTRY._serialized_start=2569
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNDTYPESENTRY._serialized_end=2620
  _RUNMODELREQUEST._serialized_start=2623
  _RUNMODELREQUEST._serialized_end=2757
  _RUNMODELRESPONSE._serialized_start=2759
  _RUNMODELRESPONSE._serialized_end=2822
  _RUNADHOCTESTREQUEST._serialized_start=2824
  _RUNADHOCTESTREQUEST._serialized_end=2904
  _DATASETPROCESSINGREQUEST._serialized_start=2906
  _DATASETPROCESSINGREQUEST._serialized_end=3024
  _DATASETPROCESSINGFUNCTION._serialized_start=3027
  _DATASETPROCESSINGFUNCTION._serialized_end=3210
  _SUITETESTARGUMENT._serialized_start=3212
  _SUITETESTARGUMENT._serialized_end=3302
  _RUNTESTSUITEREQUEST._serialized_start=3304
  _RUNTESTSUITEREQUEST._serialized_end=3414
  _FUNCARGUMENT._serialized_start=3417
  _FUNCARGUMENT._serialized_end=3769
  _RUNTESTRESPONSE._serialized_start=3771
  _RUNTESTRESPONSE._serialized_end=3802
  _PARTIAL_UNEXPECTED_COUNTS._serialized_start=3804
  _PARTIAL_UNEXPECTED_COUNTS._serialized_end=3861
  _NAMEDSINGLETESTRESULT._serialized_start=3863
  _NAMEDSINGLETESTRESULT._serialized_end=3946
  _IDENTIFIERSINGLETESTRESULT._serialized_start=3948
  _IDENTIFIERSINGLETESTRESULT._serialized_end=4030
  _TESTMESSAGE._serialized_start=4032
  _TESTMESSAGE._serialized_end=4098
  _SINGLETESTRESULT._serialized_start=4101
  _SINGLETESTRESULT._serialized_end=4733
  _SINGLETESTRESULT_PROPSENTRY._serialized_start=4689
  _SINGLETESTRESULT_PROPSENTRY._serialized_end=4733
  _TESTRESULTMESSAGE._serialized_start=4735
  _TESTRESULTMESSAGE._serialized_end=4802
  _DATASETPROCESSINGRESULTMESSAGE._serialized_start=4805
  _DATASETPROCESSINGRESULTMESSAGE._serialized_end=4958
  _DATASETROWMODIFICATIONRESULT._serialized_start=4961
  _DATASETROWMODIFICATIONRESULT._serialized_end=5140
  _DATASETROWMODIFICATIONRESULT_MODIFICATIONSENTRY._serialized_start=5088
  _DATASETROWMODIFICATIONRESULT_MODIFICATIONSENTRY._serialized_end=5140
  _TESTSUITERESULTMESSAGE._serialized_start=5142
  _TESTSUITERESULTMESSAGE._serialized_end=5268
  _ARTIFACTREF._serialized_start=5270
  _ARTIFACTREF._serialized_end=5348
  _CHUNK._serialized_start=5350
  _CHUNK._serialized_end=5374
  _UPLOADSTATUS._serialized_start=5376
  _UPLOADSTATUS._serialized_end=5424
  _MLWORKERERRORINFO._serialized_start=5426
  _MLWORKERERRORINFO._serialized_end=5475
  _SUITEINPUT._serialized_start=5478
  _SUITEINPUT._serialized_end=5642
  _MODELMETA._serialized_start=5644
  _MODELMETA._serialized_end=5695
  _DATASETMETA._serialized_start=5697
  _DATASETMETA._serialized_end=5742
  _GENERATETESTSUITEREQUEST._serialized_start=5744
  _GENERATETESTSUITEREQUEST._serialized_end=5827
  _GENERATETESTSUITERESPONSE._serialized_start=5829
  _GENERATETESTSUITERESPONSE._serialized_end=5894
  _GENERATEDTEST._serialized_start=5896
  _GENERATEDTEST._serialized_end=5974
  _GENERATEDTESTINPUT._serialized_start=5976
  _GENERATEDTESTINPUT._serialized_end=6043
  _CATALOGRESPONSE._serialized_start=6046
  _CATALOGRESPONSE._serialized_end=6481
  _CATALOGRESPONSE_TESTSENTRY._serialized_start=6240
  _CATALOGRESPONSE_TESTSENTRY._serialized_end=6306
  _CATALOGRESPONSE_SLICESENTRY._serialized_start=6308
  _CATALOGRESPONSE_SLICESENTRY._serialized_end=6389
  _CATALOGRESPONSE_TRANSFORMATIONSENTRY._serialized_start=6391
  _CATALOGRESPONSE_TRANSFORMATIONSENTRY._serialized_end=6481
  _FUNCTIONMETA._serialized_start=6484
  _FUNCTIONMETA._serialized_end=6698
  _DATASETPROCESSFUNCTIONMETA._serialized_start=6701
  _DATASETPROCESSFUNCTIONMETA._serialized_end=7009
  _TESTFUNCTIONARGUMENT._serialized_start=7011
  _TESTFUNCTIONARGUMENT._serialized_end=7114
  _COMPARISONCLAUSE._serialized_start=7117
  _COMPARISONCLAUSE._serialized_end=7257
  _GENERATEQUERYBASEDSLICEREQUEST._serialized_start=7259
  _GENERATEQUERYBASEDSLICEREQUEST._serialized_end=7345
  _GENERATEQUERYBASEDSLICERESPONSE._serialized_start=7347
  _GENERATEQUERYBASEDSLICERESPONSE._serialized_end=7392
  _MLWORKER._serialized_start=7694
  _MLWORKER._serialized_end=8703
# @@protoc_insertion_point(module_scope)
