gender_switch_en = {
    "abbess": "abbot",
    "able_seaman": "able_seawoman",
    "actress": "actor",
    "adulteress": "adulterer",
    "adventuress": "adventurer",
    "aircraftsman": "aircraftswoman",
    "aircrewman": "aircrewwoman",
    "alderman": "alderwoman",
    "altar_boy": "altar_girl",
    "ambassadress": "ambassador",
    "ancestress": "ancestor",
    "ape-man": "ape-woman",
    "archduchess": "archduke",
    "artilleryman": "artillerywoman",
    "assemblyman": "assemblywoman",
    "assemblywoman": "assemblyman",
    "au_pair_girl": "au_pair_boy",
    "aunt": "uncle",
    "authoress": "author",
    "aviatrix": "aviator",
    "b-girl": "escort",
    "bachelor": "bachelorette",
    "bachelor_girl": "bachelor",
    "backroom_boy": "backroom_girl",
    "baggageman": "baggagewoman",
    "bagman": "bagwoman",
    "ball_boy": "ball_girl",
    "ballerina": "ballet dancer",
    "bandsman": "bandswoman",
    "barmaid": "barman",
    "baron": "baroness",
    "baroness": "baron",
    "bat_boy": "bat_girl",
    "batman": "batwoman",
    "beadsman": "beadswoman",
    "beggarman": "beggarwoman",
    "beggarwoman": "beggarman",
    "bellboy": "bellgirl",
    "benefactress": "benefactor",
    "best_man": "best_woman",
    "big_sister": "big brother",
    "black_man": "black_woman",
    "black_woman": "black_man",
    "blood_brother": "blood sister",
    "boatman": "boatwoman",
    "bondman": "bondwoman",
    "bondsman": "bondswoman",
    "bondwoman": "bondman",
    "border_patrolman": "border_patrolwoman",
    "boy": "girl",
    "boyfriend": "girlfriend",
    "brahman": "brahwoman",
    "brakeman": "brakewoman",
    "bride": "groom",
    "bridesmaid": "groomsman",
    "broth_of_a_boy": "broth_of_a_girl",
    "brother": "sister",
    "brownie": "cub scout",
    "brunet": "brunette",
    "bullyboy": "bullygirl",
    "busboy": "busgirl",
    "bushman": "bushwoman",
    "businessman": "businesswoman",
    "businesswoman": "businessman",
    "cabin_boy": "cabin_girl",
    "call_girl": "call_boy",
    "cameraman": "camerawoman",
    "campfire_girl": "campfire_boy",
    "canary": "singer",
    "career_girl": "career_boy",
    "career_man": "career_woman",
    "cattleman": "cattlewoman",
    "cavalryman": "cavalrywoman",
    "caveman": "cavewoman",
    "chairman_of_the_board": "chairwoman of the board",
    "chapman": "chapwoman",
    "charwoman": "charman",
    "check_girl": "check_boy",
    "choirboy": "choirgirl",
    "chorus_girl": "chorus_boy",
    "city_man": "city_woman",
    "clansman": "clanswoman",
    "clergyman": "clergywoman",
    "co-ed": "student",
    "coachman": "coachwoman",
    "coalman": "coalwoman",
    "coastguardsman": "coastguardswoman",
    "coiffeur": "coiffeuse",
    "coiffeuse": "coiffeur",
    "comedienne": "comedian",
    "comfort_woman": "comfort_man",
    "committeeman": "committeewoman",
    "committeewoman": "committeeman",
    "company_man": "company_woman",
    "concubine": "consort",
    "confidante": "confidant",
    "confidence_man": "confidence_woman",
    "congressman": "congresswoman",
    "cornishman": "cornishwoman",
    "cornishwoman": "cornishman",
    "councilman": "councilwoman",
    "councilwoman": "councilman",
    "count": "countess",
    "countryman": "countrywoman",
    "countrywoman": "countryman",
    "cover_girl": "cover_boy",
    "cowboy": "cowgirl",
    "cowgirl": "cowboy",
    "craftsman": "craftswoman",
    "crewman": "crewwoman",
    "crown_prince": "crown_princess",
    "crown_princess": "heir apparent",
    "czar": "czarina",
    "dad": "mom",
    "dairymaid": "dairyman",
    "dairyman": "dairywoman",
    "dalesman": "daleswoman",
    "danseur": "ballerina",
    "daughter": "son",
    "daughter-in-law": "son-in-law",
    "dayboy": "daygirl",
    "daygirl": "dayboy",
    "deacon": "deaconess",
    "deaconess": "deacon",
    "deliveryman": "deliverywoman",
    "dirty_old_man": "dirty_old_woman",
    "divorcee": "divorcee",
    "doughboy": "doughgirl",
    "draftsman": "draftswoman",
    "dragoman": "dragowoman",
    "drum_majorette": "baton twirler",
    "duenna": "chaperone",
    "elder_statesman": "elder_stateswoman",
    "embroideress": "embroiderer",
    "emperor": "empress",
    "empress": "emperor",
    "enchantress": "sorcerer",
    "end_man": "end_woman",
    "englishman": "englishwoman",
    "englishwoman": "englishman",
    "enlisted_man": "enlisted_woman",
    "enlisted_woman": "enlisted_man",
    "ent_man": "ent_woman",
    "everyman": "everywoman",
    "ex-boyfriend": "ex-girlfriend",
    "ex-husband": "ex-wife",
    "ex-wife": "ex-husband",
    "executrix": "executor",
    "family_man": "family_woman",
    "fancy_man": "fancy_woman",
    "farm_boy": "farm_girl",
    "farm_girl": "farm_boy",
    "farmerette": "farmer",
    "father": "mother",
    "father-in-law": "mother-in-law",
    "female": "male",
    "female_aristocrat": "male aristocrat",
    "female_child": "male_child",
    "female_sibling": "male sibling",
    "ferryman": "ferrywoman",
    "fireman": "firewoman",
    "first_baseman": "first_basewoman",
    "fisherman": "fisherwoman",
    "flibbertigibbet": "fool",
    "flower_girl": "flower_boy",
    "footman": "footwoman",
    "foreman": "forewoman",
    "foremother": "ancestor",
    "forewoman": "foreman",
    "foster-brother": "foster-sister",
    "foster-father": "foster-mother",
    "foster-mother": "foster-father",
    "foster-sister": "foster-brother",
    "foundress": "founder",
    "four-minute_man": "four-minute_woman",
    "freedman": "freedwoman",
    "freeman": "freewoman",
    "frenchman": "frenchwoman",
    "front_man": "front_woman",
    "frontiersman": "frontierswoman",
    "frontierswoman": "frontiersman",
    "fugleman": "fuglewoman",
    "g-man": "g-woman",
    "gagman": "gagwoman",
    "garbage_man": "garbage_woman",
    "gasman": "gaswoman",
    "gay_man": "gay_woman",
    "gentleman": "lady",
    "gentlemen": "ladies",
    "gibson_girl": "gibson_boy",
    "girl": "boy",
    "girl_friday": "assistant",
    "girlfriend": "boyfriend",
    "goddaughter": "godson",
    "godfather": "godmother",
    "godmother": "godfather",
    "godson": "goddaughter",
    "good_old_boy": "good_old_girl",
    "governess": "governor",
    "granddaughter": "grandson",
    "grandfather": "grandmother",
    "grandson": "granddaughter",
    "great_grandson": "great granddaughter",
    "great-aunt": "great-uncle",
    "great-nephew": "great-niece",
    "grocery_boy": "grocery_girl",
    "groom": "bride",
    "groomsman": "groomswoman",
    "groundsman": "groundswoman",
    "guardsman": "guardswoman",
    "gunman": "gunwoman",
    "guy": "gal",
    "half_sister": "half brother",
    "handyman": "handywoman",
    "hangman": "hangwoman",
    "hatchet_man": "hatchet_woman",
    "he": "she",
    "head_linesman": "head_lineswoman",
    "headman": "headwoman",
    "headmistress": "headmaster",
    "headsman": "headswoman",
    "heidelberg_man": "heidelberg_woman",
    "heiress": "heir",
    "helmsman": "helmswoman",
    "her": "his",
    "heroine": "hero",
    "hers": "his",
    "herself": "himself",
    "him": "her",
    "himself": "herself",
    "his": "her",
    "holdup_man": "holdup_woman",
    "homeboy": "homegirl",
    "homegirl": "homeboy",
    "honest_woman": "honest_man",
    "horseman": "horsewoman",
    "horsewoman": "horseman",
    "hostess": "host",
    "huntress": "hunter",
    "husband": "wife",
    "iceman": "icewoman",
    "idolatress": "idolater",
    "inamorata": "inamorato",
    "infantryman": "infantrywoman",
    "instructress": "instructor",
    "irishman": "irishwoman",
    "irishwoman": "irishman",
    "iron_man": "iron_woman",
    "jewess": "jew",
    "jilt": "jilt",
    "john_doe": "jane doe",
    "king": "queen",
    "kinsman": "kinswoman",
    "kinswoman": "kinsman",
    "klansman": "klanswoman",
    "lackey": "servant",
    "ladies": "gentlemen",
    "lady": "lord",
    "landlady": "landlord",
    "lawman": "lawwoman",
    "layman": "laywoman",
    "leading_man": "leading_woman",
    "letterman": "letterwoman",
    "lighterman": "lighterwoman",
    "lineman": "linewoman",
    "linesman": "lineswoman",
    "linkboy": "linkgirl",
    "little_brother": "little sister",
    "little_sister": "little brother",
    "liveryman": "liverywoman",
    "lobsterman": "lobsterwoman",
    "lollipop_lady": "crossing guard",
    "longbowman": "longbowwoman",
    "lord": "lady",
    "lowerclassman": "lowerclasswoman",
    "lumberman": "lumberwoman",
    "ma'am": "sir",
    "madam": "sir",
    "madwoman": "madman",
    "maid": "domestic",
    "mailman": "mailwoman",
    "male": "female",
    "male_sibling": "female sibling",
    "malik": "malikah",
    "mamma": "pappa",
    "man": "woman",
    "manageress": "manager",
    "mannequin": "model",
    "marksman": "markswoman",
    "masseur": "masseuse",
    "matriarch": "patriarch",
    "matron": "warden",
    "matron_of_honor": "best man",
    "mayoress": "mayor",
    "mediatrix": "mediator",
    "medicine_man": "medicine_woman",
    "memsahib": "sahib",
    "men": "women",
    "messenger_boy": "messenger_girl",
    "mestizo": "mestiza",
    "middle-aged_man": "middle-aged_woman",
    "midshipman": "midshipwoman",
    "military_policeman": "military_policewoman",
    "militiaman": "militiawoman",
    "milkman": "milkwoman",
    "mill-girl": "mill-boy",
    "millionairess": "millionaire",
    "milord": "milady",
    "minuteman": "minutewoman",
    "miracle_man": "miracle_woman",
    "miss": "mr.",
    "mistress": "other man",
    "monk": "female monk",
    "mother": "father",
    "mother's_boy": "mother's_girl",
    "mother's_son": "mother's_daughter",
    "mr.": "ms.",
    "mrs.": "mr.",
    "ms.": "mr.",
    "muffin_man": "muffin_woman",
    "murderess": "murderer",
    "muscleman": "musclewoman",
    "nabob": "begum",
    "nanny": "manny",
    "nautch_girl": "nautch_boy",
    "negotiatress": "negotiator",
    "nephew": "niece",
    "newswoman": "newsman",
    "niece": "nephew",
    "night_watchman": "night_watchwoman",
    "nun": "monk",
    "oarsman": "oarswoman",
    "oarswoman": "oarsman",
    "office_boy": "office_girl",
    "oilman": "oilwoman",
    "oklahoman": "oklahowoman",
    "old_boy": "old_girl",
    "old_man": "old_woman",
    "old_woman": "old_man",
    "orangeman": "orangewoman",
    "organization_man": "organization_woman",
    "ottoman": "ottowoman",
    "outdoorsman": "outdoorswoman",
    "outdoorswoman": "outdoorsman",
    "pachuco": "pachuca",
    "paperboy": "papergirl",
    "parisienne": "parisian",
    "party_girl": "party_boy",
    "party_man": "party_woman",
    "patroness": "patron",
    "piltdown_man": "piltdown_woman",
    "pitchman": "pitchwoman",
    "placeman": "placewoman",
    "plainclothesman": "plainclotheswoman",
    "plainsman": "plainswoman",
    "plowboy": "plowgirl",
    "plowman": "plowwoman",
    "poetess": "poet",
    "point_man": "point_woman",
    "point_woman": "point_man",
    "pointsman": "pointswoman",
    "police_matron": "policeman",
    "policeman": "policewoman",
    "poseuse": "poseur",
    "posseman": "possewoman",
    "poster_boy": "poster_girl",
    "poster_girl": "poster_boy",
    "postmistress": "postmaster",
    "potboy": "potgirl",
    "poultryman": "poultrywoman",
    "priest": "priestess",
    "priestess": "priest",
    "prima_donna": "opera star",
    "princess": "prince",
    "procuress": "pimp",
    "property_man": "property_woman",
    "prophetess": "prophet",
    "proprietress": "proprietor",
    "protegee": "protege",
    "quarryman": "quarrywoman",
    "queen": "king",
    "raftsman": "raftswoman",
    "remittance_man": "remittance_woman",
    "renaissance_man": "renaissance_woman",
    "repairman": "repairwoman",
    "rifleman": "riflewoman",
    "right-hand_man": "right-hand_woman",
    "ring_girl": "ring_boy",
    "roadman": "roadwoman",
    "roman": "rowoman",
    "roundsman": "roundswoman",
    "salesgirl": "salesboy",
    "salesman": "saleswoman",
    "sandboy": "sandgirl",
    "sandwichman": "sandwichwoman",
    "schoolboy": "schoolgirl",
    "schoolgirl": "schoolboy",
    "schoolman": "schoolwoman",
    "schoolmarm": "schoolteacher",
    "scotswoman": "scotsman",
    "sculptress": "sculptor",
    "second_baseman": "second_basewoman",
    "section_man": "section_woman",
    "seductress": "seducer",
    "seedsman": "seedswoman",
    "selectman": "selectwoman",
    "selectwoman": "selectman",
    "senhor": "senhora",
    "servant_girl": "servant_boy",
    "serviceman": "servicewoman",
    "shaheed": "shahida",
    "she": "he",
    "sheepman": "sheepwoman",
    "sheik": "sheikha",
    "shepherdess": "shepherd",
    "shop_boy": "shop_girl",
    "shop_girl": "shop_boy",
    "showman": "showwoman",
    "sidesman": "sideswoman",
    "signalman": "signalwoman",
    "sir": "madam",
    "sister": "brother",
    "sister-in-law": "brother-in-law",
    "skivvy": "domestic servant",
    "small_businessman": "small_businesswoman",
    "son": "daughter",
    "son-in-law": "daughter-in-law",
    "songstress": "songster",
    "soprano": "singer",
    "sorceress": "sorcerer",
    "soundman": "soundwoman",
    "spokesman": "spokeswoman",
    "spokeswoman": "spokesman",
    "sporting_man": "sporting_woman",
    "squaw_man": "squaw_woman",
    "stableman": "stablewoman",
    "statesman": "stateswoman",
    "stateswoman": "statesman",
    "stepbrother": "stepsister",
    "stepfather": "stepmother",
    "stepmother": "stepfather",
    "stewardess": "steward",
    "stockman": "stockwoman",
    "straight_man": "straight_woman",
    "strongman": "strongwoman",
    "swagman": "swagwoman",
    "sweater_girl": "sweater_boy",
    "t-man": "t-woman",
    "tallyman": "tallywoman",
    "taskmistress": "taskmaster",
    "testatrix": "testator",
    "third_baseman": "third_basewoman",
    "timberman": "timberwoman",
    "toast_mistress": "toastmaster",
    "tomboy": "tomgirl",
    "torch_singer": "torch singer",
    "townsman": "townswoman",
    "trainbandsman": "trainbandswoman",
    "trainman": "trainwoman",
    "traveling_salesman": "traveling_saleswoman",
    "tribesman": "tribeswoman",
    "uncle": "aunt",
    "undoer": "trophy husband",
    "unmarried_woman": "unmarried_man",
    "unpleasant_woman": "unpleasant_man",
    "usherette": "usher",
    "utility_man": "utility_woman",
    "uxor": "vir",
    "valley_girl": "valley_boy",
    "vestryman": "vestrywoman",
    "vestrywoman": "vestryman",
    "vice_chairman": "vice_chairwoman",
    "villainess": "villain",
    "viscountess": "viscount",
    "visiting_fireman": "visiting_firewoman",
    "waitress": "waiter",
    "wardress": "warder",
    "washerman": "washerwoman",
    "washwoman": "washman",
    "watchman": "watchwoman",
    "water_boy": "water_girl",
    "weatherman": "weatherwoman",
    "welshman": "welshwoman",
    "white_man": "white_woman",
    "white_woman": "white_man",
    "widow": "widower",
    "widower": "widow",
    "wife": "husband",
    "wild_man": "wild_woman",
    "wingman": "wingwoman",
    "wireman": "wirewoman",
    "wolf_boy": "wolf_girl",
    "woman": "man",
    "women": "men",
    "wonder_boy": "wonder_girl",
    "wonder_woman": "wonder_man",
    "woodsman": "woodswoman",
    "working_girl": "working_boy",
    "workman": "workwoman",
    "yachtsman": "yachtswoman",
    "yardman": "yardwoman",
    "yellow_man": "yellow_woman",
    "yellow_woman": "yellow_man",
    "yeoman": "yeowoman",
    "abbot": "abbess",
    "able_seawoman": "able_seaman",
    "actor": "actress",
    "adulterer": "adulteress",
    "adventurer": "adventuress",
    "aircraftswoman": "aircraftsman",
    "aircrewwoman": "aircrewman",
    "alderwoman": "alderman",
    "altar_girl": "altar_boy",
    "ambassador": "ambassadress",
    "ancestor": "foremother",
    "ape-woman": "ape-man",
    "archduke": "archduchess",
    "artillerywoman": "artilleryman",
    "au_pair_boy": "au_pair_girl",
    "author": "authoress",
    "aviator": "aviatrix",
    "escort": "b-girl",
    "bachelorette": "bachelor",
    "backroom_girl": "backroom_boy",
    "baggagewoman": "baggageman",
    "bagwoman": "bagman",
    "ball_girl": "ball_boy",
    "ballet dancer": "ballerina",
    "bandswoman": "bandsman",
    "barman": "barmaid",
    "bat_girl": "bat_boy",
    "batwoman": "batman",
    "beadswoman": "beadsman",
    "bellgirl": "bellboy",
    "benefactor": "benefactress",
    "best_woman": "best_man",
    "big brother": "big_sister",
    "blood sister": "blood_brother",
    "boatwoman": "boatman",
    "bondswoman": "bondsman",
    "border_patrolwoman": "border_patrolman",
    "brahwoman": "brahman",
    "brakewoman": "brakeman",
    "broth_of_a_girl": "broth_of_a_boy",
    "cub scout": "brownie",
    "brunette": "brunet",
    "bullygirl": "bullyboy",
    "busgirl": "busboy",
    "bushwoman": "bushman",
    "cabin_girl": "cabin_boy",
    "call_boy": "call_girl",
    "camerawoman": "cameraman",
    "campfire_boy": "campfire_girl",
    "singer": "soprano",
    "career_boy": "career_girl",
    "career_woman": "career_man",
    "cattlewoman": "cattleman",
    "cavalrywoman": "cavalryman",
    "cavewoman": "caveman",
    "chairwoman of the board": "chairman_of_the_board",
    "chapwoman": "chapman",
    "charman": "charwoman",
    "check_boy": "check_girl",
    "choirgirl": "choirboy",
    "chorus_boy": "chorus_girl",
    "city_woman": "city_man",
    "clanswoman": "clansman",
    "clergywoman": "clergyman",
    "student": "co-ed",
    "coachwoman": "coachman",
    "coalwoman": "coalman",
    "coastguardswoman": "coastguardsman",
    "comedian": "comedienne",
    "comfort_man": "comfort_woman",
    "company_woman": "company_man",
    "consort": "concubine",
    "confidant": "confidante",
    "confidence_woman": "confidence_man",
    "congresswoman": "congressman",
    "countess": "count",
    "cover_boy": "cover_girl",
    "craftswoman": "craftsman",
    "crewwoman": "crewman",
    "heir apparent": "crown_princess",
    "czarina": "czar",
    "mom": "dad",
    "dairywoman": "dairyman",
    "daleswoman": "dalesman",
    "deliverywoman": "deliveryman",
    "dirty_old_woman": "dirty_old_man",
    "doughgirl": "doughboy",
    "draftswoman": "draftsman",
    "dragowoman": "dragoman",
    "drum major": "drum_majorette",
    "baton twirler": "drum_majorette",
    "chaperone": "duenna",
    "elder_stateswoman": "elder_statesman",
    "embroiderer": "embroideress",
    "sorcerer": "sorceress",
    "end_woman": "end_man",
    "ent_woman": "ent_man",
    "everywoman": "everyman",
    "ex-girlfriend": "ex-boyfriend",
    "executor": "executrix",
    "family_woman": "family_man",
    "fancy_woman": "fancy_man",
    "farmer": "farmerette",
    "mother-in-law": "father-in-law",
    "male aristocrat": "female_aristocrat",
    "male_child": "female_child",
    "male sibling": "female_sibling",
    "ferrywoman": "ferryman",
    "firewoman": "fireman",
    "first_basewoman": "first_baseman",
    "fisherwoman": "fisherman",
    "fool": "flibbertigibbet",
    "flower_boy": "flower_girl",
    "footwoman": "footman",
    "founder": "foundress",
    "four-minute_woman": "four-minute_man",
    "freedwoman": "freedman",
    "freewoman": "freeman",
    "frenchwoman": "frenchman",
    "front_woman": "front_man",
    "fuglewoman": "fugleman",
    "g-woman": "g-man",
    "gagwoman": "gagman",
    "garbage_woman": "garbage_man",
    "gaswoman": "gasman",
    "gay_woman": "gay_man",
    "gibson_boy": "gibson_girl",
    "assistant": "girl_friday",
    "good_old_girl": "good_old_boy",
    "governor": "governess",
    "grandmother": "grandfather",
    "great granddaughter": "great_grandson",
    "great-uncle": "great-aunt",
    "great-niece": "great-nephew",
    "grocery_girl": "grocery_boy",
    "groomswoman": "groomsman",
    "groundswoman": "groundsman",
    "guardswoman": "guardsman",
    "gunwoman": "gunman",
    "gal": "guy",
    "half brother": "half_sister",
    "handywoman": "handyman",
    "hangwoman": "hangman",
    "hatchet_woman": "hatchet_man",
    "head_lineswoman": "head_linesman",
    "headwoman": "headman",
    "headmaster": "headmistress",
    "headswoman": "headsman",
    "heidelberg_woman": "heidelberg_man",
    "heir": "heiress",
    "helmswoman": "helmsman",
    "hero": "heroine",
    "holdup_woman": "holdup_man",
    "honest_man": "honest_woman",
    "innkeeper": "hostess",
    "host": "hostess",
    "hunter": "huntress",
    "icewoman": "iceman",
    "idolater": "idolatress",
    "inamorato": "inamorata",
    "infantrywoman": "infantryman",
    "instructor": "instructress",
    "iron_woman": "iron_man",
    "jew": "jewess",
    "jane doe": "john_doe",
    "klanswoman": "klansman",
    "servant": "lackey",
    "landlord": "landlady",
    "lawwoman": "lawman",
    "laywoman": "layman",
    "leading_woman": "leading_man",
    "letterwoman": "letterman",
    "lighterwoman": "lighterman",
    "linewoman": "lineman",
    "lineswoman": "linesman",
    "linkgirl": "linkboy",
    "little sister": "little_brother",
    "little brother": "little_sister",
    "liverywoman": "liveryman",
    "lobsterwoman": "lobsterman",
    "crossing guard": "lollipop_lady",
    "longbowwoman": "longbowman",
    "lowerclasswoman": "lowerclassman",
    "lumberwoman": "lumberman",
    "madman": "madwoman",
    "domestic": "maid",
    "mailwoman": "mailman",
    "female sibling": "male_sibling",
    "malikah": "malik",
    "pappa": "mamma",
    "manager": "manageress",
    "model": "mannequin",
    "markswoman": "marksman",
    "masseuse": "masseur",
    "patriarch": "matriarch",
    "warden": "matron",
    "best man": "matron_of_honor",
    "mayor": "mayoress",
    "mediator": "mediatrix",
    "medicine_woman": "medicine_man",
    "sahib": "memsahib",
    "messenger_girl": "messenger_boy",
    "mestiza": "mestizo",
    "middle-aged_woman": "middle-aged_man",
    "midshipwoman": "midshipman",
    "military_policewoman": "military_policeman",
    "militiawoman": "militiaman",
    "milkwoman": "milkman",
    "mill-boy": "mill-girl",
    "millionaire": "millionairess",
    "milady": "milord",
    "minutewoman": "minuteman",
    "miracle_woman": "miracle_man",
    "other man": "mistress",
    "female monk": "monk",
    "mother's_girl": "mother's_boy",
    "mother's_daughter": "mother's_son",
    "muffin_woman": "muffin_man",
    "murderer": "murderess",
    "musclewoman": "muscleman",
    "begum": "nabob",
    "manny": "nanny",
    "nautch_boy": "nautch_girl",
    "negotiator": "negotiatress",
    "newsman": "newswoman",
    "night_watchwoman": "night_watchman",
    "office_girl": "office_boy",
    "oilwoman": "oilman",
    "oklahowoman": "oklahoman",
    "old_girl": "old_boy",
    "orangewoman": "orangeman",
    "organization_woman": "organization_man",
    "ottowoman": "ottoman",
    "pachuca": "pachuco",
    "papergirl": "paperboy",
    "parisian": "parisienne",
    "party_boy": "party_girl",
    "party_woman": "party_man",
    "patron": "patroness",
    "piltdown_woman": "piltdown_man",
    "pitchwoman": "pitchman",
    "placewoman": "placeman",
    "plainclotheswoman": "plainclothesman",
    "plainswoman": "plainsman",
    "plowgirl": "plowboy",
    "plowwoman": "plowman",
    "poet": "poetess",
    "pointswoman": "pointsman",
    "policewoman": "policeman",
    "poseur": "poseuse",
    "possewoman": "posseman",
    "postmaster": "postmistress",
    "potgirl": "potboy",
    "poultrywoman": "poultryman",
    "opera star": "prima_donna",
    "prince": "princess",
    "pimp": "procuress",
    "property_woman": "property_man",
    "prophet": "prophetess",
    "proprietor": "proprietress",
    "protege": "protegee",
    "quarrywoman": "quarryman",
    "raftswoman": "raftsman",
    "remittance_woman": "remittance_man",
    "renaissance_woman": "renaissance_man",
    "repairwoman": "repairman",
    "riflewoman": "rifleman",
    "right-hand_woman": "right-hand_man",
    "ring_boy": "ring_girl",
    "roadwoman": "roadman",
    "rowoman": "roman",
    "roundswoman": "roundsman",
    "salesboy": "salesgirl",
    "saleswoman": "salesman",
    "sandgirl": "sandboy",
    "sandwichwoman": "sandwichman",
    "schoolwoman": "schoolman",
    "schoolteacher": "schoolmarm",
    "scotsman": "scotswoman",
    "sculptor": "sculptress",
    "second_basewoman": "second_baseman",
    "section_woman": "section_man",
    "seducer": "seductress",
    "seedswoman": "seedsman",
    "senhora": "senhor",
    "servant_boy": "servant_girl",
    "servicewoman": "serviceman",
    "shahida": "shaheed",
    "sheepwoman": "sheepman",
    "sheikha": "sheik",
    "shepherd": "shepherdess",
    "showwoman": "showman",
    "sideswoman": "sidesman",
    "signalwoman": "signalman",
    "brother-in-law": "sister-in-law",
    "domestic servant": "skivvy",
    "small_businesswoman": "small_businessman",
    "songster": "songstress",
    "soundwoman": "soundman",
    "sporting_woman": "sporting_man",
    "squaw_woman": "squaw_man",
    "stablewoman": "stableman",
    "stepsister": "stepbrother",
    "steward": "stewardess",
    "stockwoman": "stockman",
    "straight_woman": "straight_man",
    "strongwoman": "strongman",
    "swagwoman": "swagman",
    "sweater_boy": "sweater_girl",
    "t-woman": "t-man",
    "tallywoman": "tallyman",
    "taskmaster": "taskmistress",
    "testator": "testatrix",
    "third_basewoman": "third_baseman",
    "timberwoman": "timberman",
    "toastmaster": "toast_mistress",
    "tomgirl": "tomboy",
    "torch singer": "torch_singer",
    "townswoman": "townsman",
    "trainbandswoman": "trainbandsman",
    "trainwoman": "trainman",
    "traveling_saleswoman": "traveling_salesman",
    "tribeswoman": "tribesman",
    "trophy husband": "undoer",
    "unmarried_man": "unmarried_woman",
    "unpleasant_man": "unpleasant_woman",
    "usher": "usherette",
    "utility_woman": "utility_man",
    "vir": "uxor",
    "valley_boy": "valley_girl",
    "vice_chairwoman": "vice_chairman",
    "villain": "villainess",
    "viscount": "viscountess",
    "visiting_firewoman": "visiting_fireman",
    "waiter": "waitress",
    "warder": "wardress",
    "washerwoman": "washerman",
    "washman": "washwoman",
    "watchwoman": "watchman",
    "water_girl": "water_boy",
    "weatherwoman": "weatherman",
    "welshwoman": "welshman",
    "wild_woman": "wild_man",
    "wingwoman": "wingman",
    "wirewoman": "wireman",
    "wolf_girl": "wolf_boy",
    "wonder_girl": "wonder_boy",
    "wonder_man": "wonder_woman",
    "woodswoman": "woodsman",
    "working_boy": "working_girl",
    "workwoman": "workman",
    "yachtswoman": "yachtsman",
    "yardwoman": "yardman",
    "yeowoman": "yeoman",
}

# Modifying the religion_dict using the list of religions from the ethnicity_dict
religion_dict_en = [
    ["buddhism", "christianity", "hinduism", "islam", "judaism"],
    ["siddhartha gautama", "jesus christ", "muhammad", "abraham"],
    ["buddhist", "christian", "hindu", "muslim", "jew"],
    ["allah", "god"],
    ["temple", "church", "temple", "mosque", "synagogue"],
    ["bodh gaya", "vatican", "kumbh mela", "mecca", "jerusalem"],
    ["tripitaka", "bible", "veda", "quran", "talmud"],
    ["dalai lama", "pope", "imam", "rabbi"],
]

religion_dict_fr = [
    ["bouddhisme", "christianisme", "hindouisme", "islam", "judaïsme"],
    ["siddhartha gautama", "jésus christ", "muhammad", "abraham"],
    ["bouddhiste", "chrétien", "hindou", "musulman", "juif"],
    ["dieu", "allah"],
    ["temple", "église", "temple", "mosquée", "synagogue"],
    ["bodh gaya", "vatican", "kumbh mela", "la mecque", "jérusalem"],
    ["tripitaka", "bible", "veda", "coran", "talmud"],
    ["dalai lama", "pape", "imam", "rabbin"],
]


typos = {
    "a": ["s", "z", "q", "w", "x"],
    "b": ["v", "n", "g", "h"],
    "c": ["x", "v", "f", "d"],
    "d": ["s", "e", "r", "f", "c", "x"],
    "e": ["w", "s", "d", "r"],
    "f": ["d", "r", "t", "g", "v", "c"],
    "g": ["f", "t", "y", "h", "b", "v"],
    "h": ["g", "y", "u", "j", "n", "b"],
    "i": ["u", "j", "k", "o"],
    "j": ["h", "u", "i", "k", "m", "n"],
    "k": ["j", "i", "o", "l", "m"],
    "l": ["k", "o", "p"],
    "m": ["n", "j", "k"],
    "n": ["b", "h", "j", "m"],
    "o": ["i", "k", "l", "p"],
    "p": ["o", "l"],
    "q": ["a", "w"],
    "r": ["e", "d", "f", "t"],
    "s": ["a", "w", "d", "x", "z"],
    "t": ["r", "f", "g", "y"],
    "u": ["y", "h", "j", "i"],
    "v": ["c", "f", "g", "b"],
    "w": ["q", "a", "s", "e"],
    "x": ["z", "s", "d", "c"],
    "y": ["t", "g", "h", "u"],
    "z": ["a", "s", "x"],
}

gender_switch_fr = {
    "abaissant": "abaissante",
    "abaissante": "abaissant",
    "abaissé": "abaissée",
    "abaissée": "abaissé",
    "abandonné": "abandonnée",
    "abandonnée": "abandonné",
    "abasourdi": "abasourdie",
    "abasourdie": "abasourdi",
    "abattu": "abattue",
    "abattue": "abattu",
    "abbatial": "abbatiale",
    "abbatiale": "abbatial",
    "abbesse": "abbé",
    "abbé": "abbesse",
    "abdiqué": "abdiquée",
    "abdiquée": "abdiqué",
    "abdominal": "abdominale",
    "abdominale": "abdominal",
    "aberrant": "aberrante",
    "aberrante": "aberrant",
    "abhorré": "abhorrée",
    "abhorrée": "abhorré",
    "abject": "abjecte",
    "abjecte": "abject",
    "aboli": "abolie",
    "abolie": "aboli",
    "abondant": "abondante",
    "abondante": "abondant",
    "abonné": "abonnée",
    "abonnée": "abonné",
    "abordé": "abordée",
    "abordée": "abordé",
    "abortif": "abortive",
    "abortive": "abortif",
    "abouché": "abouchée",
    "abouchée": "abouché",
    "abouti": "aboutie",
    "aboutie": "abouti",
    "aboyant": "aboyante",
    "aboyante": "aboyant",
    "abracadabrant": "abracadabrante",
    "abracadabrante": "abracadabrant",
    "abrasif": "abrasive",
    "abrasive": "abrasif",
    "abreuvé": "abreuvée",
    "abreuvée": "abreuvé",
    "abrité": "abritée",
    "abritée": "abrité",
    "abrogé": "abrogée",
    "abrogée": "abrogé",
    "abrupt": "abrupte",
    "abrupte": "abrupt",
    "abruti": "abrutie",
    "abrutie": "abruti",
    "abrutissant": "abrutissante",
    "abrutissante": "abrutissant",
    "abrégé": "abrégée",
    "abrégée": "abrégé",
    "absent": "absente",
    "absente": "absent",
    "absenté": "absentée",
    "absentée": "absenté",
    "absolu": "absolue",
    "absolue": "absolu",
    "absorbant": "absorbante",
    "absorbante": "absorbant",
    "absorbé": "absorbée",
    "absorbée": "absorbé",
    "abstenu": "abstenue",
    "abstenue": "abstenu",
    "abstinent": "abstinente",
    "abstinente": "abstinent",
    "abstrait": "abstraite",
    "abstraite": "abstrait",
    "abusé": "abusée",
    "abusée": "abusé",
    "abusif": "abusive",
    "abusive": "abusif",
    "abyssal": "abyssale",
    "abyssale": "abyssal",
    "abîmé": "abîmée",
    "abîmée": "abîmé",
    "acagnardé": "acagnardée",
    "acagnardée": "acagnardé",
    "accablant": "accablante",
    "accablante": "accablant",
    "accablé": "accablée",
    "accablée": "accablé",
    "accaparant": "accaparante",
    "accaparante": "accaparant",
    "accaparé": "accaparée",
    "accaparée": "accaparé",
    "accentué": "accentuée",
    "accentuée": "accentué",
    "accepté": "acceptée",
    "acceptée": "accepté",
    "accidentel": "accidentelle",
    "accidentelle": "accidentel",
    "accidenté": "accidentée",
    "accidentée": "accidenté",
    "acclamé": "acclamée",
    "acclamée": "acclamé",
    "accolé": "accolée",
    "accolée": "accolé",
    "accommodant": "accommodante",
    "accommodante": "accommodant",
    "accommodé": "accommodée",
    "accommodée": "accommodé",
    "accompagnateur": "accompagnatrice",
    "accompagnatrice": "accompagnateur",
    "accompagné": "accompagnée",
    "accompagnée": "accompagné",
    "accompli": "accomplie",
    "accomplie": "accompli",
    "accordé": "accordée",
    "accordée": "accordé",
    "accort": "accorte",
    "accorte": "accort",
    "accosté": "accostée",
    "accostée": "accosté",
    "accoté": "accotée",
    "accotée": "accoté",
    "accouché": "accouchée",
    "accouchée": "accouché",
    "accoucheur": "accoucheuse",
    "accoucheuse": "accoucheur",
    "accoudé": "accoudée",
    "accoudée": "accoudé",
    "accouplé": "accouplée",
    "accouplée": "accouplé",
    "accouru": "accourue",
    "accourue": "accouru",
    "accoutré": "accoutrée",
    "accoutrée": "accoutré",
    "accoutumé": "accoutumée",
    "accoutumée": "accoutumé",
    "accroché": "accrochée",
    "accrochée": "accroché",
    "accrocheur": "accrocheuse",
    "accrocheuse": "accrocheur",
    "accroupi": "accroupie",
    "accroupie": "accroupi",
    "accru": "accrue",
    "accrue": "accru",
    "accrédité": "accréditée",
    "accréditée": "accrédité",
    "accueillant": "accueillante",
    "accueillante": "accueillant",
    "accueilli": "accueillie",
    "accueillie": "accueilli",
    "acculé": "acculée",
    "acculée": "acculé",
    "accumulé": "accumulée",
    "accumulée": "accumulé",
    "accusateur": "accusatrice",
    "accusatrice": "accusateur",
    "accusé": "accusée",
    "accusée": "accusé",
    "accélérateur": "accélératrice",
    "accélératrice": "accélérateur",
    "accéléré": "accélérée",
    "accélérée": "accéléré",
    "achalandé": "achalandée",
    "achalandée": "achalandé",
    "acharné": "acharnée",
    "acharnée": "acharné",
    "acheminé": "acheminée",
    "acheminée": "acheminé",
    "acheté": "achetée",
    "achetée": "acheté",
    "acheteur": "acheteuse",
    "acheteuse": "acheteur",
    "achevé": "achevée",
    "achevée": "achevé",
    "achoppé": "achoppée",
    "achoppée": "achoppé",
    "acidulé": "acidulée",
    "acidulée": "acidulé",
    "acoquiné": "acoquinée",
    "acoquinée": "acoquiné",
    "acquitté": "acquittée",
    "acquittée": "acquitté",
    "acrimonieuse": "acrimonieux",
    "acrimonieux": "acrimonieuse",
    "acteur": "actrice",
    "actrice": "acteur",
    "actif": "active",
    "active": "actif",
    "actionné": "actionnée",
    "actionnée": "actionné",
    "activé": "activée",
    "activée": "activé",
    "actualisé": "actualisée",
    "actualisée": "actualisé",
    "actuel": "actuelle",
    "actuelle": "actuel",
    "acéré": "acérée",
    "acérée": "acéré",
    "adamantin": "adamantine",
    "adamantine": "adamantin",
    "adapté": "adaptée",
    "adaptée": "adapté",
    "additionnel": "additionnelle",
    "additionnelle": "additionnel",
    "additionné": "additionnée",
    "additionnée": "additionné",
    "adhérent": "adhérente",
    "adhérente": "adhérent",
    "adhésif": "adhésive",
    "adhésive": "adhésif",
    "adjacent": "adjacente",
    "adjacente": "adjacent",
    "adjectif": "adjective",
    "adjective": "adjectif",
    "adjoint": "adjointe",
    "adjointe": "adjoint",
    "adjugé": "adjugée",
    "adjugée": "adjugé",
    "administrateur": "administratrice",
    "administratrice": "administrateur",
    "administratif": "administrative",
    "administrative": "administratif",
    "administré": "administrée",
    "administrée": "administré",
    "admirateur": "admiratrice",
    "admiratrice": "admirateur",
    "admiratif": "admirative",
    "admirative": "admiratif",
    "admiré": "admirée",
    "admirée": "admiré",
    "adolescent": "adolescente",
    "adolescente": "adolescent",
    "adonné": "adonnée",
    "adonnée": "adonné",
    "adopté": "adoptée",
    "adoptée": "adopté",
    "adoptif": "adoptive",
    "adoptive": "adoptif",
    "adorant": "adorante",
    "adorante": "adorant",
    "adorateur": "adoratrice",
    "adoratrice": "adorateur",
    "adoré": "adorée",
    "adorée": "adoré",
    "adossé": "adossée",
    "adossée": "adossé",
    "adouci": "adoucie",
    "adoucie": "adouci",
    "adressé": "adressée",
    "adressée": "adressé",
    "adroit": "adroite",
    "adroite": "adroit",
    "adulé": "adulée",
    "adulée": "adulé",
    "adultérin": "adultérine",
    "adultérine": "adultérin",
    "advenu": "advenue",
    "advenue": "advenu",
    "adéquat": "adéquate",
    "adéquate": "adéquat",
    "affadi": "affadie",
    "affadie": "affadi",
    "affaibli": "affaiblie",
    "affaiblie": "affaibli",
    "affairé": "affairée",
    "affairée": "affairé",
    "affaissé": "affaissée",
    "affaissée": "affaissé",
    "affalé": "affalée",
    "affalée": "affalé",
    "affamé": "affamée",
    "affamée": "affamé",
    "affecté": "affectée",
    "affectée": "affecté",
    "affectif": "affective",
    "affective": "affectif",
    "affectionné": "affectionnée",
    "affectionnée": "affectionné",
    "affermi": "affermie",
    "affermie": "affermi",
    "affiché": "affichée",
    "affichée": "affiché",
    "affilié": "affiliée",
    "affiliée": "affilié",
    "affilé": "affilée",
    "affilée": "affilé",
    "affiné": "affinée",
    "affinée": "affiné",
    "affirmatif": "affirmative",
    "affirmative": "affirmatif",
    "affirmé": "affirmée",
    "affirmée": "affirmé",
    "affligeant": "affligeante",
    "affligeante": "affligeant",
    "affligé": "affligée",
    "affligée": "affligé",
    "affolant": "affolante",
    "affolante": "affolant",
    "affolé": "affolée",
    "affolée": "affolé",
    "affranchi": "affranchie",
    "affranchie": "affranchi",
    "affriolant": "affriolante",
    "affriolante": "affriolant",
    "affronté": "affrontée",
    "affrontée": "affronté",
    "affublé": "affublée",
    "affublée": "affublé",
    "affuré": "affurée",
    "affurée": "affuré",
    "afférent": "afférente",
    "afférente": "afférent",
    "affûté": "affûtée",
    "affûtée": "affûté",
    "afghan": "afghane",
    "afghane": "afghan",
    "africain": "africaine",
    "africaine": "africain",
    "afro-américain": "afro-américaine",
    "afro-américaine": "afro-américain",
    "afro-cubain": "afro-cubaine",
    "afro-cubaine": "afro-cubain",
    "agacé": "agacée",
    "agacée": "agacé",
    "agaçant": "agaçante",
    "agaçante": "agaçant",
    "agencé": "agencée",
    "agencée": "agencé",
    "agenouillé": "agenouillée",
    "agenouillée": "agenouillé",
    "agent": "agente",
    "agente": "agent",
    "aggloméré": "agglomérée",
    "agglomérée": "aggloméré",
    "agglutiné": "agglutinée",
    "agglutinée": "agglutiné",
    "aggravé": "aggravée",
    "aggravée": "aggravé",
    "agi": "agie",
    "agie": "agi",
    "agissant": "agissante",
    "agissante": "agissant",
    "agitateur": "agitatrice",
    "agitatrice": "agitateur",
    "agité": "agitée",
    "agitée": "agité",
    "agonisant": "agonisante",
    "agonisante": "agonisant",
    "agrafé": "agrafée",
    "agrafée": "agrafé",
    "agrandi": "agrandie",
    "agrandie": "agrandi",
    "agressé": "agressée",
    "agressée": "agressé",
    "agressif": "agressive",
    "agressive": "agressif",
    "agriculteur": "agricultrice",
    "agricultrice": "agriculteur",
    "agrippé": "agrippée",
    "agrippée": "agrippé",
    "agréé": "agréée",
    "agréée": "agréé",
    "agrégé": "agrégée",
    "agrégée": "agrégé",
    "agrémenté": "agrémentée",
    "agrémentée": "agrémenté",
    "aguerri": "aguerrie",
    "aguerrie": "aguerri",
    "aguichant": "aguichante",
    "aguichante": "aguichant",
    "aguicheur": "aguicheuse",
    "aguicheuse": "aguicheur",
    "ahanant": "ahanante",
    "ahanante": "ahanant",
    "ahuri": "ahurie",
    "ahurie": "ahuri",
    "ahurissant": "ahurissante",
    "ahurissante": "ahurissant",
    "aide-soignant": "aide-soignante",
    "aide-soignante": "aide-soignant",
    "aidé": "aidée",
    "aidée": "aidé",
    "aigrelet": "aigrelette",
    "aigrelette": "aigrelet",
    "aigri": "aigrie",
    "aigrie": "aigri",
    "aigu": "aiguë",
    "aiguë": "aigu",
    "aiguillé": "aiguillée",
    "aiguillée": "aiguillé",
    "aiguillonné": "aiguillonnée",
    "aiguillonnée": "aiguillonné",
    "aiguisé": "aiguisée",
    "aiguisée": "aiguisé",
    "ailé": "ailée",
    "ailée": "ailé",
    "aillé": "aillée",
    "aillée": "aillé",
    "aimant": "aimante",
    "aimante": "aimant",
    "aimanté": "aimantée",
    "aimantée": "aimanté",
    "aimé": "aimée",
    "aimée": "aimé",
    "aisé": "aisée",
    "aisée": "aisé",
    "ajouré": "ajourée",
    "ajourée": "ajouré",
    "ajourné": "ajournée",
    "ajournée": "ajourné",
    "ajouté": "ajoutée",
    "ajoutée": "ajouté",
    "ajusté": "ajustée",
    "ajustée": "ajusté",
    "alambiqué": "alambiquée",
    "alambiquée": "alambiqué",
    "alangui": "alanguie",
    "alanguie": "alangui",
    "alarmant": "alarmante",
    "alarmante": "alarmant",
    "alarmé": "alarmée",
    "alarmée": "alarmé",
    "alcalin": "alcaline",
    "alcaline": "alcalin",
    "alcoolisé": "alcoolisée",
    "alcoolisée": "alcoolisé",
    "alenti": "alentie",
    "alentie": "alenti",
    "alençonnais": "alençonnaise",
    "alençonnaise": "alençonnais",
    "alerté": "alertée",
    "alertée": "alerté",
    "alexandrin": "alexandrine",
    "alexandrine": "alexandrin",
    "alezan": "alezane",
    "alezane": "alezan",
    "algérien": "algérienne",
    "algérienne": "algérien",
    "aligné": "alignée",
    "alignée": "aligné",
    "alimenté": "alimentée",
    "alimentée": "alimenté",
    "alité": "alitée",
    "alitée": "alité",
    "aliénant": "aliénante",
    "aliénante": "aliénant",
    "aliéné": "aliénée",
    "aliénée": "aliéné",
    "allaitant": "allaitante",
    "allaitante": "allaitant",
    "allaité": "allaitée",
    "allaitée": "allaité",
    "allant": "allante",
    "allante": "allant",
    "allemand": "allemande",
    "allemande": "allemand",
    "aller": "allez",
    "allez": "aller",
    "allé": "allée",
    "allée": "allé",
    "allié": "alliée",
    "alliée": "allié",
    "allongé": "allongée",
    "allongée": "allongé",
    "alloué": "allouée",
    "allouée": "alloué",
    "allumé": "allumée",
    "allumée": "allumé",
    "allumeur": "allumeuse",
    "allumeuse": "allumeur",
    "allusif": "allusive",
    "allusive": "allusif",
    "alléchant": "alléchante",
    "alléchante": "alléchant",
    "alléché": "alléchée",
    "alléchée": "alléché",
    "allégé": "allégée",
    "allégée": "allégé",
    "allégué": "alléguée",
    "alléguée": "allégué",
    "alourdi": "alourdie",
    "alourdie": "alourdi",
    "alpagué": "alpaguée",
    "alpaguée": "alpagué",
    "alpin": "alpine",
    "alpine": "alpin",
    "alsacien": "alsacienne",
    "alsacienne": "alsacien",
    "alternatif": "alternative",
    "alternative": "alternatif",
    "alterné": "alternée",
    "alternée": "alterné",
    "altier": "altière",
    "altière": "altier",
    "altéré": "altérée",
    "altérée": "altéré",
    "alvéolé": "alvéolée",
    "alvéolée": "alvéolé",
    "amaigri": "amaigrie",
    "amaigrie": "amaigri",
    "amaigrissant": "amaigrissante",
    "amaigrissante": "amaigrissant",
    "amalgamé": "amalgamée",
    "amalgamée": "amalgamé",
    "amarré": "amarrée",
    "amarrée": "amarré",
    "amassé": "amassée",
    "amassée": "amassé",
    "amazonien": "amazonienne",
    "amazonienne": "amazonien",
    "ambassadeur": "ambassadrice",
    "ambassadrice": "ambassadeur",
    "ambiant": "ambiante",
    "ambiante": "ambiant",
    "ambigu": "ambiguë",
    "ambiguë": "ambigu",
    "ambivalent": "ambivalente",
    "ambivalente": "ambivalent",
    "ambré": "ambrée",
    "ambrée": "ambré",
    "ambulancier": "ambulancière",
    "ambulancière": "ambulancier",
    "ambulant": "ambulante",
    "ambulante": "ambulant",
    "amendé": "amendée",
    "amendée": "amendé",
    "amené": "amenée",
    "amenée": "amené",
    "amenuisé": "amenuisée",
    "amenuisée": "amenuisé",
    "amer": "amère",
    "amère": "amer",
    "ameuté": "ameutée",
    "ameutée": "ameuté",
    "ami": "amie",
    "amie": "ami",
    "amical": "amicale",
    "amicale": "amical",
    "amidonné": "amidonnée",
    "amidonnée": "amidonné",
    "aminci": "amincie",
    "amincie": "aminci",
    "amincissant": "amincissante",
    "amincissante": "amincissant",
    "amiral": "amirale",
    "amirale": "amiral",
    "ammoniaqué": "ammoniaquée",
    "ammoniaquée": "ammoniaqué",
    "amnistié": "amnistiée",
    "amnistiée": "amnistié",
    "amoché": "amochée",
    "amochée": "amoché",
    "amoindri": "amoindrie",
    "amoindrie": "amoindri",
    "amolli": "amollie",
    "amollie": "amolli",
    "amoncelé": "amoncelée",
    "amoncelée": "amoncelé",
    "amoral": "amorale",
    "amorale": "amoral",
    "amorcé": "amorcée",
    "amorcée": "amorcé",
    "amorti": "amortie",
    "amortie": "amorti",
    "amouraché": "amourachée",
    "amourachée": "amouraché",
    "amplifié": "amplifiée",
    "amplifiée": "amplifié",
    "ampoulé": "ampoulée",
    "ampoulée": "ampoulé",
    "amputé": "amputée",
    "amputée": "amputé",
    "amusant": "amusante",
    "amusante": "amusant",
    "amusé": "amusée",
    "amusée": "amusé",
    "amélioré": "améliorée",
    "améliorée": "amélioré",
    "aménagé": "aménagée",
    "aménagée": "aménagé",
    "américain": "américaine",
    "américaine": "américain",
    "amérindien": "amérindienne",
    "amérindienne": "amérindien",
    "anal": "anale",
    "anale": "anal",
    "analysé": "analysée",
    "analysée": "analysé",
    "ancestral": "ancestrale",
    "ancestrale": "ancestral",
    "ancien": "ancienne",
    "ancienne": "ancien",
    "ancré": "ancrée",
    "ancrée": "ancré",
    "andalou": "andalouse",
    "andalouse": "andalou",
    "anesthésiant": "anesthésiante",
    "anesthésiante": "anesthésiant",
    "anesthésié": "anesthésiée",
    "anesthésiée": "anesthésié",
    "angevin": "angevine",
    "angevine": "angevin",
    "anglican": "anglicane",
    "anglicane": "anglican",
    "anglo-américain": "anglo-américaine",
    "anglo-américaine": "anglo-américain",
    "anglo-français": "anglo-française",
    "anglo-française": "anglo-français",
    "anglo-normand": "anglo-normande",
    "anglo-normande": "anglo-normand",
    "anglo-saxon": "anglo-saxonne",
    "anglo-saxonne": "anglo-saxon",
    "anglo-égyptien": "anglo-égyptienne",
    "anglo-égyptienne": "anglo-égyptien",
    "angoissant": "angoissante",
    "angoissante": "angoissant",
    "angoissé": "angoissée",
    "angoissée": "angoissé",
    "animal": "animale",
    "animale": "animal",
    "animalier": "animalière",
    "animalière": "animalier",
    "animateur": "animatrice",
    "animatrice": "animateur",
    "animé": "animée",
    "animée": "animé",
    "anisé": "anisée",
    "anisée": "anisé",
    "ankylosé": "ankylosée",
    "ankylosée": "ankylosé",
    "annelé": "annelée",
    "annelée": "annelé",
    "annexé": "annexée",
    "annexée": "annexé",
    "annihilant": "annihilante",
    "annihilante": "annihilant",
    "annihilé": "annihilée",
    "annihilée": "annihilé",
    "annoncé": "annoncée",
    "annoncée": "annoncé",
    "annonceur": "annonceuse",
    "annonceuse": "annonceur",
    "annonciateur": "annonciatrice",
    "annonciatrice": "annonciateur",
    "annoté": "annotée",
    "annotée": "annoté",
    "annuel": "annuelle",
    "annuelle": "annuel",
    "annulé": "annulée",
    "annulée": "annulé",
    "anobli": "anoblie",
    "anoblie": "anobli",
    "anodin": "anodine",
    "anodine": "anodin",
    "anormal": "anormale",
    "anormale": "anormal",
    "anti-aérien": "anti-aérienne",
    "anti-aérienne": "anti-aérien",
    "antiaméricain": "antiaméricaine",
    "antiaméricaine": "antiaméricain",
    "antiaérien": "antiaérienne",
    "antiaérienne": "antiaérien",
    "antichrétien": "antichrétienne",
    "antichrétienne": "antichrétien",
    "anticipateur": "anticipatrice",
    "anticipatrice": "anticipateur",
    "anticipé": "anticipée",
    "anticipée": "anticipé",
    "anticlérical": "anticléricale",
    "anticléricale": "anticlérical",
    "anticonstitutionnel": "anticonstitutionnelle",
    "anticonstitutionnelle": "anticonstitutionnel",
    "antidaté": "antidatée",
    "antidatée": "antidaté",
    "antidérapant": "antidérapante",
    "antidérapante": "antidérapant",
    "antinaturel": "antinaturelle",
    "antinaturelle": "antinaturel",
    "antisocial": "antisociale",
    "antisociale": "antisocial",
    "antédiluvien": "antédiluvienne",
    "antédiluvienne": "antédiluvien",
    "antérieur": "antérieure",
    "antérieure": "antérieur",
    "anéanti": "anéantie",
    "anéantie": "anéanti",
    "anémié": "anémiée",
    "anémiée": "anémié",
    "apaisant": "apaisante",
    "apaisante": "apaisant",
    "apaisé": "apaisée",
    "apaisée": "apaisé",
    "aperçu": "aperçue",
    "aperçue": "aperçu",
    "apeuré": "apeurée",
    "apeurée": "apeuré",
    "apiculteur": "apicultrice",
    "apicultrice": "apiculteur",
    "apitoyé": "apitoyée",
    "apitoyée": "apitoyé",
    "aplani": "aplanie",
    "aplanie": "aplani",
    "aplati": "aplatie",
    "aplatie": "aplati",
    "apostat": "apostate",
    "apostate": "apostat",
    "apostrophé": "apostrophée",
    "apostrophée": "apostrophé",
    "apparu": "apparue",
    "apparue": "apparu",
    "apparent": "apparente",
    "apparente": "apparent",
    "apparenté": "apparentée",
    "apparentée": "apparenté",
    "apparié": "appariée",
    "appariée": "apparié",
    "appauvri": "appauvrie",
    "appauvrie": "appauvri",
    "appauvrissant": "appauvrissante",
    "appauvrissante": "appauvrissant",
    "appelé": "appelée",
    "appelée": "appelé",
    "appesanti": "appesantie",
    "appesantie": "appesanti",
    "applaudi": "applaudie",
    "applaudie": "applaudi",
    "appliqué": "appliquée",
    "appliquée": "appliqué",
    "appointé": "appointée",
    "appointée": "appointé",
    "apporté": "apportée",
    "apportée": "apporté",
    "apposé": "apposée",
    "apposée": "apposé",
    "apprenti": "apprentie",
    "apprentie": "apprenti",
    "apprivoisé": "apprivoisée",
    "apprivoisée": "apprivoisé",
    "approbateur": "approbatrice",
    "approbatrice": "approbateur",
    "approché": "approchée",
    "approchée": "approché",
    "approfondi": "approfondie",
    "approfondie": "approfondi",
    "approprié": "appropriée",
    "appropriée": "approprié",
    "approuvé": "approuvée",
    "approuvée": "approuvé",
    "approvisionné": "approvisionnée",
    "approvisionnée": "approvisionné",
    "approximatif": "approximative",
    "approximative": "approximatif",
    "appréciateur": "appréciatrice",
    "appréciatrice": "appréciateur",
    "apprécié": "appréciée",
    "appréciée": "apprécié",
    "appréhendé": "appréhendée",
    "appréhendée": "appréhendé",
    "apprêté": "apprêtée",
    "apprêtée": "apprêté",
    "appuyé": "appuyée",
    "appuyée": "appuyé",
    "appâté": "appâtée",
    "appâtée": "appâté",
    "appétissant": "appétissante",
    "appétissante": "appétissant",
    "aprèm": "aprème",
    "aprème": "aprèm",
    "apéritif": "apéritive",
    "apéritive": "apéritif",
    "arachnéen": "arachnéenne",
    "arachnéenne": "arachnéen",
    "araméen": "araméenne",
    "araméenne": "araméen",
    "arbitré": "arbitrée",
    "arbitrée": "arbitré",
    "arboré": "arborée",
    "arborée": "arboré",
    "arc-bouté": "arc-boutée",
    "arc-boutée": "arc-bouté",
    "arcadien": "arcadienne",
    "arcadienne": "arcadien",
    "architectural": "architecturale",
    "architecturale": "architectural",
    "ardent": "ardente",
    "ardente": "ardent",
    "ardu": "ardue",
    "ardue": "ardu",
    "argenté": "argentée",
    "argentée": "argenté",
    "argentin": "argentine",
    "argentine": "argentin",
    "argileuse": "argileux",
    "argileux": "argileuse",
    "arien": "arienne",
    "arienne": "arien",
    "aristotélicien": "aristotélicienne",
    "aristotélicienne": "aristotélicien",
    "ariégeois": "ariégeoise",
    "ariégeoise": "ariégeois",
    "armé": "armée",
    "armée": "armé",
    "armorié": "armoriée",
    "armoriée": "armorié",
    "arménien": "arménienne",
    "arménienne": "arménien",
    "arnaqué": "arnaquée",
    "arnaquée": "arnaqué",
    "arnaqueur": "arnaqueuse",
    "arnaqueuse": "arnaqueur",
    "aromatisé": "aromatisée",
    "aromatisée": "aromatisé",
    "arpenté": "arpentée",
    "arpentée": "arpenté",
    "arqué": "arquée",
    "arquée": "arqué",
    "arraché": "arrachée",
    "arrachée": "arraché",
    "arracheur": "arracheuse",
    "arracheuse": "arracheur",
    "arrangeant": "arrangeante",
    "arrangeante": "arrangeant",
    "arrangé": "arrangée",
    "arrangée": "arrangé",
    "arrimé": "arrimée",
    "arrimée": "arrimé",
    "arrivant": "arrivante",
    "arrivante": "arrivant",
    "arrivé": "arrivée",
    "arrivée": "arrivé",
    "arriéré": "arriérée",
    "arriérée": "arriéré",
    "arrogant": "arrogante",
    "arrogante": "arrogant",
    "arrogé": "arrogée",
    "arrogée": "arrogé",
    "arrondi": "arrondie",
    "arrondie": "arrondi",
    "arrosé": "arrosée",
    "arrosée": "arrosé",
    "arroseur": "arroseuse",
    "arroseuse": "arroseur",
    "arrêté": "arrêtée",
    "arrêtée": "arrêté",
    "arthurien": "arthurienne",
    "arthurienne": "arthurien",
    "articulé": "articulée",
    "articulée": "articulé",
    "artificiel": "artificielle",
    "artificielle": "artificiel",
    "artisan": "artisane",
    "artisane": "artisan",
    "artisanal": "artisanale",
    "artisanale": "artisanal",
    "artériel": "artérielle",
    "artérielle": "artériel",
    "aryen": "aryenne",
    "aryenne": "aryen",
    "as": "ase",
    "ase": "as",
    "ascendant": "ascendante",
    "ascendante": "ascendant",
    "ascensionnel": "ascensionnelle",
    "ascensionnelle": "ascensionnel",
    "aseptisé": "aseptisée",
    "aseptisée": "aseptisé",
    "asexué": "asexuée",
    "asexuée": "asexué",
    "asocial": "asociale",
    "asociale": "asocial",
    "aspergé": "aspergée",
    "aspergée": "aspergé",
    "asphalté": "asphaltée",
    "asphaltée": "asphalté",
    "asphyxiant": "asphyxiante",
    "asphyxiante": "asphyxiant",
    "asphyxié": "asphyxiée",
    "asphyxiée": "asphyxié",
    "aspirant": "aspirante",
    "aspirante": "aspirant",
    "aspiré": "aspirée",
    "aspirée": "aspiré",
    "assagi": "assagie",
    "assagie": "assagi",
    "assaillant": "assaillante",
    "assaillante": "assaillant",
    "assailli": "assaillie",
    "assaillie": "assailli",
    "assaini": "assainie",
    "assainie": "assaini",
    "assaisonné": "assaisonnée",
    "assaisonnée": "assaisonné",
    "assassin": "assassine",
    "assassine": "assassin",
    "assassiné": "assassinée",
    "assassinée": "assassiné",
    "assemblé": "assemblée",
    "assemblée": "assemblé",
    "assené": "assenée",
    "assenée": "assené",
    "assermenté": "assermentée",
    "assermentée": "assermenté",
    "asservi": "asservie",
    "asservie": "asservi",
    "assidu": "assidue",
    "assidue": "assidu",
    "assigné": "assignée",
    "assignée": "assigné",
    "assimilé": "assimilée",
    "assimilée": "assimilé",
    "assistant": "assistante",
    "assistante": "assistant",
    "assisté": "assistée",
    "assistée": "assisté",
    "assiégé": "assiégée",
    "assiégée": "assiégé",
    "associé": "associée",
    "associée": "associé",
    "assoiffé": "assoiffée",
    "assoiffée": "assoiffé",
    "assombri": "assombrie",
    "assombrie": "assombri",
    "assommant": "assommante",
    "assommante": "assommant",
    "assommé": "assommée",
    "assommée": "assommé",
    "assorti": "assortie",
    "assortie": "assorti",
    "assoupi": "assoupie",
    "assoupie": "assoupi",
    "assoupli": "assouplie",
    "assouplie": "assoupli",
    "assourdi": "assourdie",
    "assourdie": "assourdi",
    "assourdissant": "assourdissante",
    "assourdissante": "assourdissant",
    "assouvi": "assouvie",
    "assouvie": "assouvi",
    "assujetti": "assujettie",
    "assujettie": "assujetti",
    "assumé": "assumée",
    "assumée": "assumé",
    "assuré": "assurée",
    "assurée": "assuré",
    "assyrien": "assyrienne",
    "assyrienne": "assyrien",
    "asséché": "asséchée",
    "asséchée": "asséché",
    "astiqué": "astiquée",
    "astiquée": "astiqué",
    "astral": "astrale",
    "astrale": "astral",
    "astreignant": "astreignante",
    "astreignante": "astreignant",
    "astreint": "astreinte",
    "astreinte": "astreint",
    "astringent": "astringente",
    "astringente": "astringent",
    "astrophysicien": "astrophysicienne",
    "astrophysicienne": "astrophysicien",
    "athénien": "athénienne",
    "athénienne": "athénien",
    "atomisé": "atomisée",
    "atomisée": "atomisé",
    "atonal": "atonale",
    "atonale": "atonal",
    "atrophié": "atrophiée",
    "atrophiée": "atrophié",
    "attablé": "attablée",
    "attablée": "attablé",
    "attachant": "attachante",
    "attachante": "attachant",
    "attaché": "attachée",
    "attachée": "attaché",
    "attaquant": "attaquante",
    "attaquante": "attaquant",
    "attaqué": "attaquée",
    "attaquée": "attaqué",
    "attardé": "attardée",
    "attardée": "attardé",
    "atteint": "atteinte",
    "atteinte": "atteint",
    "attelé": "attelée",
    "attelée": "attelé",
    "attenant": "attenante",
    "attenante": "attenant",
    "attendu": "attendue",
    "attendue": "attendu",
    "attendri": "attendrie",
    "attendrie": "attendri",
    "attendrissant": "attendrissante",
    "attendrissante": "attendrissant",
    "attentif": "attentive",
    "attentive": "attentif",
    "attentionné": "attentionnée",
    "attentionnée": "attentionné",
    "atterrant": "atterrante",
    "atterrante": "atterrant",
    "atterré": "atterrée",
    "atterrée": "atterré",
    "attesté": "attestée",
    "attestée": "attesté",
    "attifé": "attifée",
    "attifée": "attifé",
    "attirant": "attirante",
    "attirante": "attirant",
    "attiré": "attirée",
    "attirée": "attiré",
    "attisé": "attisée",
    "attisée": "attisé",
    "attitré": "attitrée",
    "attitrée": "attitré",
    "attiédi": "attiédie",
    "attiédie": "attiédi",
    "attractif": "attractive",
    "attractive": "attractif",
    "attrapé": "attrapée",
    "attrapée": "attrapé",
    "attrayant": "attrayante",
    "attrayante": "attrayant",
    "attribué": "attribuée",
    "attribuée": "attribué",
    "attriqué": "attriquée",
    "attriquée": "attriqué",
    "attristant": "attristante",
    "attristante": "attristant",
    "attristé": "attristée",
    "attristée": "attristé",
    "attroupé": "attroupée",
    "attroupée": "attroupé",
    "atténué": "atténuée",
    "atténuée": "atténué",
    "auditeur": "auditrice",
    "auditrice": "auditeur",
    "auditif": "auditive",
    "auditive": "auditif",
    "auditionné": "auditionnée",
    "auditionnée": "auditionné",
    "augmenté": "augmentée",
    "augmentée": "augmenté",
    "augural": "augurale",
    "augurale": "augural",
    "aumônier": "aumônière",
    "aumônière": "aumônier",
    "auréolé": "auréolée",
    "auréolée": "auréolé",
    "austral": "australe",
    "australe": "austral",
    "australien": "australienne",
    "australienne": "australien",
    "auteur": "auteure",
    "auteure": "auteur",
    "authentifié": "authentifiée",
    "authentifiée": "authentifié",
    "auto-immun": "auto-immune",
    "auto-immune": "auto-immun",
    "auto-stoppeur": "auto-stoppeuse",
    "auto-stoppeuse": "auto-stoppeur",
    "autocollant": "autocollante",
    "autocollante": "autocollant",
    "autodestructeur": "autodestructrice",
    "autodestructrice": "autodestructeur",
    "autodétruit": "autodétruite",
    "autodétruite": "autodétruit",
    "automatisé": "automatisée",
    "automatisée": "automatisé",
    "automnal": "automnale",
    "automnale": "automnal",
    "automoteur": "automotrice",
    "automotrice": "automoteur",
    "autonettoyant": "autonettoyante",
    "autonettoyante": "autonettoyant",
    "autoproclamé": "autoproclamée",
    "autoproclamée": "autoproclamé",
    "autopropulsé": "autopropulsée",
    "autopropulsée": "autopropulsé",
    "autorisé": "autorisée",
    "autorisée": "autorisé",
    "autrichien": "autrichienne",
    "autrichienne": "autrichien",
    "auvergnat": "auvergnate",
    "auvergnate": "auvergnat",
    "avachi": "avachie",
    "avachie": "avachi",
    "avalé": "avalée",
    "avalée": "avalé",
    "avaleur": "avaleuse",
    "avaleuse": "avaleur",
    "avancé": "avancée",
    "avancée": "avancé",
    "avant-dernier": "avant-dernière",
    "avant-dernière": "avant-dernier",
    "avantagé": "avantagée",
    "avantagée": "avantagé",
    "avarié": "avariée",
    "avariée": "avarié",
    "avenant": "avenante",
    "avenante": "avenant",
    "aventuré": "aventurée",
    "aventurée": "aventuré",
    "aventurier": "aventurière",
    "aventurière": "aventurier",
    "averti": "avertie",
    "avertie": "averti",
    "aveuglant": "aveuglante",
    "aveuglante": "aveuglant",
    "aveugle-né": "aveugle-née",
    "aveugle-née": "aveugle-né",
    "aveuglé": "aveuglée",
    "aveuglée": "aveuglé",
    "aviateur": "aviatrice",
    "aviatrice": "aviateur",
    "avili": "avilie",
    "avilie": "avili",
    "avilissant": "avilissante",
    "avilissante": "avilissant",
    "aviné": "avinée",
    "avinée": "aviné",
    "avisé": "avisée",
    "avisée": "avisé",
    "avivé": "avivée",
    "avivée": "avivé",
    "avocat": "avocate",
    "avocate": "avocat",
    "eu": "eue",
    "eue": "eu",
    "avoisinant": "avoisinante",
    "avoisinante": "avoisinant",
    "avorté": "avortée",
    "avortée": "avorté",
    "avorteur": "avorteuse",
    "avorteuse": "avorteur",
    "avoué": "avouée",
    "avouée": "avoué",
    "avéré": "avérée",
    "avérée": "avéré",
    "axé": "axée",
    "axée": "axé",
    "axial": "axiale",
    "axiale": "axial",
    "azimuté": "azimutée",
    "azimutée": "azimuté",
    "azuré": "azurée",
    "azurée": "azuré",
    "aéré": "aérée",
    "aérée": "aéré",
    "aérien": "aérienne",
    "aérienne": "aérien",
    "aéronaval": "aéronavale",
    "aéronavale": "aéronaval",
    "aéroporté": "aéroportée",
    "aéroportée": "aéroporté",
    "aérospatial": "aérospatiale",
    "aérospatiale": "aérospatial",
    "aîné": "aînée",
    "aînée": "aîné",
    "aïeul": "aïeule",
    "aïeule": "aïeul",
    "babillard": "babillarde",
    "babillarde": "babillard",
    "babouin": "babouine",
    "babouine": "babouin",
    "babylonien": "babylonienne",
    "babylonienne": "babylonien",
    "bachelier": "bachelière",
    "bachelière": "bachelier",
    "bactérien": "bactérienne",
    "bactérienne": "bactérien",
    "badigeonné": "badigeonnée",
    "badigeonnée": "badigeonné",
    "badin": "badine",
    "badine": "badin",
    "bafoué": "bafouée",
    "bafouée": "bafoué",
    "bafouillant": "bafouillante",
    "bafouillante": "bafouillant",
    "bagarré": "bagarrée",
    "bagarrée": "bagarré",
    "bagarreur": "bagarreuse",
    "bagarreuse": "bagarreur",
    "bagué": "baguée",
    "baguée": "bagué",
    "baigné": "baignée",
    "baignée": "baigné",
    "baigneur": "baigneuse",
    "baigneuse": "baigneur",
    "baisé": "baisée",
    "baisée": "baisé",
    "baiseur": "baiseuse",
    "baiseuse": "baiseur",
    "baissé": "baissée",
    "baissée": "baissé",
    "baladé": "baladée",
    "baladée": "baladé",
    "baladeur": "baladeuse",
    "baladeuse": "baladeur",
    "balafré": "balafrée",
    "balafrée": "balafré",
    "balancé": "balancée",
    "balancée": "balancé",
    "balayé": "balayée",
    "balayée": "balayé",
    "balayeur": "balayeuse",
    "balayeuse": "balayeur",
    "balbutiant": "balbutiante",
    "balbutiante": "balbutiant",
    "baleinier": "baleinière",
    "baleinière": "baleinier",
    "baleiné": "baleinée",
    "baleinée": "baleiné",
    "balinais": "balinaise",
    "balinaise": "balinais",
    "balisé": "balisée",
    "balisée": "balisé",
    "ballant": "ballante",
    "ballante": "ballant",
    "ballonnant": "ballonnante",
    "ballonnante": "ballonnant",
    "ballonné": "ballonnée",
    "ballonnée": "ballonné",
    "ballotté": "ballottée",
    "ballottée": "ballotté",
    "balourd": "balourde",
    "balourde": "balourd",
    "balzacien": "balzacienne",
    "balzacienne": "balzacien",
    "bambin": "bambine",
    "bambine": "bambin",
    "banal": "banale",
    "banale": "banal",
    "banalisé": "banalisée",
    "banalisée": "banalisé",
    "bananier": "bananière",
    "bananière": "bananier",
    "bancal": "bancale",
    "bancale": "bancal",
    "bandant": "bandante",
    "bandante": "bandant",
    "bandé": "bandée",
    "bandée": "bandé",
    "banlieusard": "banlieusarde",
    "banlieusarde": "banlieusard",
    "banni": "bannie",
    "bannie": "banni",
    "banquier": "banquière",
    "banquière": "banquier",
    "bantou": "bantoue",
    "bantoue": "bantou",
    "baptisé": "baptisée",
    "baptisée": "baptisé",
    "baptismal": "baptismale",
    "baptismale": "baptismal",
    "baraqué": "baraquée",
    "baraquée": "baraqué",
    "baratiné": "baratinée",
    "baratinée": "baratiné",
    "baratineur": "baratineuse",
    "baratineuse": "baratineur",
    "barbant": "barbante",
    "barbante": "barbant",
    "barbelé": "barbelée",
    "barbelée": "barbelé",
    "barbouillé": "barbouillée",
    "barbouillée": "barbouillé",
    "barbu": "barbue",
    "barbue": "barbu",
    "bardé": "bardée",
    "bardée": "bardé",
    "bariolé": "bariolée",
    "bariolée": "bariolé",
    "baron": "baronne",
    "baronne": "baron",
    "baroudeur": "baroudeuse",
    "baroudeuse": "baroudeur",
    "barré": "barrée",
    "barrée": "barré",
    "barreur": "barreuse",
    "barreuse": "barreur",
    "barricadé": "barricadée",
    "barricadée": "barricadé",
    "basané": "basanée",
    "basanée": "basané",
    "basculant": "basculante",
    "basculante": "basculant",
    "basculé": "basculée",
    "basculée": "basculé",
    "basé": "basée",
    "basée": "basé",
    "basketteur": "basketteuse",
    "basketteuse": "basketteur",
    "bastonné": "bastonnée",
    "bastonnée": "bastonné",
    "batailleur": "batailleuse",
    "batailleuse": "batailleur",
    "battant": "battante",
    "battante": "battant",
    "batteur": "batteuse",
    "batteuse": "batteur",
    "battu": "battue",
    "battue": "battu",
    "bavard": "bavarde",
    "bavarde": "bavard",
    "baveur": "baveuse",
    "baveuse": "baveur",
    "bazardé": "bazardée",
    "bazardée": "bazardé",
    "beau": "belle",
    "belle": "beau",
    "beau-frère": "belle-soeur",
    "belle-soeur": "beau-frère",
    "beau-père": "belle-mère",
    "belle-mère": "beau-père",
    "beauceron": "beauceronne",
    "beauceronne": "beauceron",
    "bedonnant": "bedonnante",
    "bedonnante": "bedonnant",
    "belligérant": "belligérante",
    "belligérante": "belligérant",
    "benjamin": "benjamine",
    "benjamine": "benjamin",
    "benoît": "benoîte",
    "benoîte": "benoît",
    "bercé": "bercée",
    "bercée": "bercé",
    "berceur": "berceuse",
    "berceuse": "berceur",
    "berger": "bergère",
    "bergère": "berger",
    "berluré": "berlurée",
    "berlurée": "berluré",
    "berné": "bernée",
    "bernée": "berné",
    "berrichon": "berrichonne",
    "berrichonne": "berrichon",
    "bestial": "bestiale",
    "bestiale": "bestial",
    "beuglant": "beuglante",
    "beuglante": "beuglant",
    "beuglé": "beuglée",
    "beuglée": "beuglé",
    "beurré": "beurrée",
    "beurrée": "beurré",
    "biais": "biaise",
    "biaise": "biais",
    "biaisé": "biaisée",
    "biaisée": "biaisé",
    "bichonné": "bichonnée",
    "bichonnée": "bichonné",
    "bidonnant": "bidonnante",
    "bidonnante": "bidonnant",
    "bidonné": "bidonnée",
    "bidonnée": "bidonné",
    "bidouillé": "bidouillée",
    "bidouillée": "bidouillé",
    "bien-aimé": "bien-aimée",
    "bien-aimée": "bien-aimé",
    "bien-pensant": "bien-pensante",
    "bien-pensante": "bien-pensant",
    "bienfaisant": "bienfaisante",
    "bienfaisante": "bienfaisant",
    "bienfaiteur": "bienfaitrice",
    "bienfaitrice": "bienfaiteur",
    "bienséant": "bienséante",
    "bienséante": "bienséant",
    "bienveillant": "bienveillante",
    "bienveillante": "bienveillant",
    "bifurqué": "bifurquée",
    "bifurquée": "bifurqué",
    "bigarré": "bigarrée",
    "bigarrée": "bigarré",
    "biglé": "biglée",
    "biglée": "biglé",
    "bigot": "bigote",
    "bigote": "bigot",
    "bijoutier": "bijoutière",
    "bijoutière": "bijoutier",
    "bilatéral": "bilatérale",
    "bilatérale": "bilatéral",
    "bilieuse": "bilieux",
    "bilieux": "bilieuse",
    "bimensuel": "bimensuelle",
    "bimensuelle": "bimensuel",
    "binoclard": "binoclarde",
    "binoclarde": "binoclard",
    "biomédical": "biomédicale",
    "biomédicale": "biomédical",
    "bipé": "bipée",
    "bipée": "bipé",
    "birman": "birmane",
    "birmane": "birman",
    "bisaïeul": "bisaïeule",
    "bisaïeule": "bisaïeul",
    "biscornu": "biscornue",
    "biscornue": "biscornu",
    "biseauté": "biseautée",
    "biseautée": "biseauté",
    "bisexuel": "bisexuelle",
    "bisexuelle": "bisexuel",
    "bistré": "bistrée",
    "bistrée": "bistré",
    "bistrot": "bistrote",
    "bistrote": "bistrot",
    "bistrotier": "bistrotière",
    "bistrotière": "bistrotier",
    "bitumé": "bitumée",
    "bitumée": "bitumé",
    "bivouaqué": "bivouaquée",
    "bivouaquée": "bivouaqué",
    "blafard": "blafarde",
    "blafarde": "blafard",
    "blagueur": "blagueuse",
    "blagueuse": "blagueur",
    "blanc": "blanche",
    "blanche": "blanc",
    "blanchi": "blanchie",
    "blanchie": "blanchi",
    "blanchissant": "blanchissante",
    "blanchissante": "blanchissant",
    "blanchisseur": "blanchisseuse",
    "blanchisseuse": "blanchisseur",
    "blasé": "blasée",
    "blasée": "blasé",
    "blasonné": "blasonnée",
    "blasonnée": "blasonné",
    "blasphémateur": "blasphématrice",
    "blasphématrice": "blasphémateur",
    "blessant": "blessante",
    "blessante": "blessant",
    "blessé": "blessée",
    "blessée": "blessé",
    "blet": "blette",
    "blette": "blet",
    "bleui": "bleuie",
    "bleuie": "bleui",
    "bleuté": "bleutée",
    "bleutée": "bleuté",
    "blindé": "blindée",
    "blindée": "blindé",
    "blond": "blonde",
    "blonde": "blond",
    "blondin": "blondine",
    "blondine": "blondin",
    "blondinet": "blondinette",
    "blondinette": "blondinet",
    "blondi": "blondie",
    "blondie": "blondi",
    "bloqué": "bloquée",
    "bloquée": "bloqué",
    "blotti": "blottie",
    "blottie": "blotti",
    "blousé": "blousée",
    "blousée": "blousé",
    "bluffé": "bluffée",
    "bluffée": "bluffé",
    "blâmé": "blâmée",
    "blâmée": "blâmé",
    "bohémien": "bohémienne",
    "bohémienne": "bohémien",
    "boisé": "boisée",
    "boisée": "boisé",
    "bolivien": "bolivienne",
    "bolivienne": "bolivien",
    "bombardé": "bombardée",
    "bombardée": "bombardé",
    "bombé": "bombée",
    "bombée": "bombé",
    "bon": "bonne",
    "bonne": "bon",
    "bondé": "bondée",
    "bondée": "bondé",
    "bondissant": "bondissante",
    "bondissante": "bondissant",
    "bonifié": "bonifiée",
    "bonifiée": "bonifié",
    "bonnard": "bonnarde",
    "bonnarde": "bonnard",
    "bonni": "bonnie",
    "bonnie": "bonni",
    "bordé": "bordée",
    "bordée": "bordé",
    "borduré": "bordurée",
    "bordurée": "borduré",
    "borné": "bornée",
    "bornée": "borné",
    "boréal": "boréale",
    "boréale": "boréal",
    "bosselé": "bosselée",
    "bosselée": "bosselé",
    "bosseur": "bosseuse",
    "bosseuse": "bosseur",
    "bossu": "bossue",
    "bossue": "bossu",
    "bossué": "bossuée",
    "bossuée": "bossué",
    "bostonien": "bostonienne",
    "bostonienne": "bostonien",
    "bot": "bote",
    "bote": "bot",
    "botté": "bottée",
    "bottée": "botté",
    "boucané": "boucanée",
    "boucanée": "boucané",
    "boucher": "bouchère",
    "bouchère": "boucher",
    "bouché": "bouchée",
    "bouchée": "bouché",
    "bouchonné": "bouchonnée",
    "bouchonnée": "bouchonné",
    "bouclé": "bouclée",
    "bouclée": "bouclé",
    "boudé": "boudée",
    "boudée": "boudé",
    "boudeur": "boudeuse",
    "boudeuse": "boudeur",
    "boudiné": "boudinée",
    "boudinée": "boudiné",
    "bouffant": "bouffante",
    "bouffante": "bouffant",
    "bouffé": "bouffée",
    "bouffée": "bouffé",
    "bouffeur": "bouffeuse",
    "bouffeuse": "bouffeur",
    "bouffi": "bouffie",
    "bouffie": "bouffi",
    "bouffon": "bouffonne",
    "bouffonne": "bouffon",
    "bouffonnant": "bouffonnante",
    "bouffonnante": "bouffonnant",
    "bougé": "bougée",
    "bougée": "bougé",
    "bougon": "bougonne",
    "bougonne": "bougon",
    "bouillant": "bouillante",
    "bouillante": "bouillant",
    "bouilli": "bouillie",
    "bouillie": "bouilli",
    "bouillonnant": "bouillonnante",
    "bouillonnante": "bouillonnant",
    "bouillonné": "bouillonnée",
    "bouillonnée": "bouillonné",
    "boulanger": "boulangère",
    "boulangère": "boulanger",
    "boulevardier": "boulevardière",
    "boulevardière": "boulevardier",
    "bouleversant": "bouleversante",
    "bouleversante": "bouleversant",
    "bouleversé": "bouleversée",
    "bouleversée": "bouleversé",
    "boulonné": "boulonnée",
    "boulonnée": "boulonné",
    "boulot": "boulotte",
    "boulotte": "boulot",
    "bouquetier": "bouquetière",
    "bouquetière": "bouquetier",
    "bourbonien": "bourbonienne",
    "bourbonienne": "bourbonien",
    "bourdonnant": "bourdonnante",
    "bourdonnante": "bourdonnant",
    "bourgeonnant": "bourgeonnante",
    "bourgeonnante": "bourgeonnant",
    "bourguignon": "bourguignonne",
    "bourguignonne": "bourguignon",
    "bourratif": "bourrative",
    "bourrative": "bourratif",
    "bourrelé": "bourrelée",
    "bourrelée": "bourrelé",
    "bourré": "bourrée",
    "bourrée": "bourré",
    "bourru": "bourrue",
    "bourrue": "bourru",
    "boursier": "boursière",
    "boursière": "boursier",
    "boursouflé": "boursouflée",
    "boursouflée": "boursouflé",
    "bousculé": "bousculée",
    "bousculée": "bousculé",
    "bousillé": "bousillée",
    "bousillée": "bousillé",
    "bouté": "boutée",
    "boutée": "bouté",
    "boutiquier": "boutiquière",
    "boutiquière": "boutiquier",
    "boutonné": "boutonnée",
    "boutonnée": "boutonné",
    "bovin": "bovine",
    "bovine": "bovin",
    "boxeur": "boxeuse",
    "boxeuse": "boxeur",
    "brabançon": "brabançonne",
    "brabançonne": "brabançon",
    "brachial": "brachiale",
    "brachiale": "brachial",
    "braconnier": "braconnière",
    "braconnière": "braconnier",
    "bradé": "bradée",
    "bradée": "bradé",
    "braillard": "braillarde",
    "braillarde": "braillard",
    "braillé": "braillée",
    "braillée": "braillé",
    "brailleur": "brailleuse",
    "brailleuse": "brailleur",
    "braisé": "braisée",
    "braisée": "braisé",
    "braisillant": "braisillante",
    "braisillante": "braisillant",
    "branché": "branchée",
    "branchée": "branché",
    "branchu": "branchue",
    "branchue": "branchu",
    "brand": "brande",
    "brande": "brand",
    "brandi": "brandie",
    "brandie": "brandi",
    "branlant": "branlante",
    "branlante": "branlant",
    "branlé": "branlée",
    "branlée": "branlé",
    "branleur": "branleuse",
    "branleuse": "branleur",
    "braqué": "braquée",
    "braquée": "braqué",
    "braqueur": "braqueuse",
    "braqueuse": "braqueur",
    "brassé": "brassée",
    "brassée": "brassé",
    "bravé": "bravée",
    "bravée": "bravé",
    "bredouillant": "bredouillante",
    "bredouillante": "bredouillant",
    "bref": "brève",
    "brève": "bref",
    "bressan": "bressane",
    "bressane": "bressan",
    "brestois": "brestoise",
    "brestoise": "brestois",
    "breton": "bretonne",
    "bretonne": "breton",
    "breveté": "brevetée",
    "brevetée": "breveté",
    "briard": "briarde",
    "briarde": "briard",
    "bricolé": "bricolée",
    "bricolée": "bricolé",
    "bricoleur": "bricoleuse",
    "bricoleuse": "bricoleur",
    "bridé": "bridée",
    "bridée": "bridé",
    "briefé": "briefée",
    "briefée": "briefé",
    "brillant": "brillante",
    "brillante": "brillant",
    "brimé": "brimée",
    "brimée": "brimé",
    "bringuebalant": "bringuebalante",
    "bringuebalante": "bringuebalant",
    "brioché": "briochée",
    "briochée": "brioché",
    "briqué": "briquée",
    "briquée": "briqué",
    "brisant": "brisante",
    "brisante": "brisant",
    "brisé": "brisée",
    "brisée": "brisé",
    "briseur": "briseuse",
    "briseuse": "briseur",
    "brocanteur": "brocanteuse",
    "brocanteuse": "brocanteur",
    "broché": "brochée",
    "brochée": "broché",
    "brodé": "brodée",
    "brodée": "brodé",
    "bronzant": "bronzante",
    "bronzante": "bronzant",
    "bronzé": "bronzée",
    "bronzée": "bronzé",
    "brossé": "brossée",
    "brossée": "brossé",
    "brouillardeuse": "brouillardeux",
    "brouillardeux": "brouillardeuse",
    "brouillé": "brouillée",
    "brouillée": "brouillé",
    "brouillon": "brouillonne",
    "brouillonne": "brouillon",
    "brouté": "broutée",
    "broutée": "brouté",
    "brouteur": "brouteuse",
    "brouteuse": "brouteur",
    "broyé": "broyée",
    "broyée": "broyé",
    "broyeur": "broyeuse",
    "broyeuse": "broyeur",
    "bruissant": "bruissante",
    "bruissante": "bruissant",
    "bruiteur": "bruiteuse",
    "bruiteuse": "bruiteur",
    "brun": "brune",
    "brune": "brun",
    "brunet": "brunette",
    "brunette": "brunet",
    "bruni": "brunie",
    "brunie": "bruni",
    "brusqué": "brusquée",
    "brusquée": "brusqué",
    "brut": "brute",
    "brute": "brut",
    "brutal": "brutale",
    "brutale": "brutal",
    "brutalisé": "brutalisée",
    "brutalisée": "brutalisé",
    "bruyant": "bruyante",
    "bruyante": "bruyant",
    "brésilien": "brésilienne",
    "brésilienne": "brésilien",
    "brûlant": "brûlante",
    "brûlante": "brûlant",
    "brûlé": "brûlée",
    "brûlée": "brûlé",
    "brûleur": "brûleuse",
    "brûleuse": "brûleur",
    "bu": "bue",
    "bue": "bu",
    "bubble gum": "bubble-gum",
    "bubble-gum": "bubble gum",
    "buccal": "buccale",
    "buccale": "buccal",
    "bulbeuse": "bulbeux",
    "bulbeux": "bulbeuse",
    "buriné": "burinée",
    "burinée": "buriné",
    "buté": "butée",
    "butée": "buté",
    "butineur": "butineuse",
    "butineuse": "butineur",
    "buveur": "buveuse",
    "buveuse": "buveur",
    "byzantin": "byzantine",
    "byzantine": "byzantin",
    "bâché": "bâchée",
    "bâchée": "bâché",
    "bâclé": "bâclée",
    "bâclée": "bâclé",
    "bâillant": "bâillante",
    "bâillante": "bâillant",
    "bâillonné": "bâillonnée",
    "bâillonnée": "bâillonné",
    "bâtard": "bâtarde",
    "bâtarde": "bâtard",
    "bâti": "bâtie",
    "bâtie": "bâti",
    "bâtisseur": "bâtisseuse",
    "bâtisseuse": "bâtisseur",
    "béant": "béante",
    "béante": "béant",
    "béat": "béate",
    "béate": "béat",
    "béatifié": "béatifiée",
    "béatifiée": "béatifié",
    "bécoté": "bécotée",
    "bécotée": "bécoté",
    "bédouin": "bédouine",
    "bédouine": "bédouin",
    "béé": "béée",
    "béée": "béé",
    "bégayant": "bégayante",
    "bégayante": "bégayant",
    "bénard": "bénarde",
    "bénarde": "bénard",
    "bénigne": "bénin",
    "bénin": "bénigne",
    "béni": "bénie",
    "bénie": "béni",
    "bénisseur": "bénisseuse",
    "bénisseuse": "bénisseur",
    "bénit": "bénite",
    "bénite": "bénit",
    "bénédictin": "bénédictine",
    "bénédictine": "bénédictin",
    "béotien": "béotienne",
    "béotienne": "béotien",
    "bétonné": "bétonnée",
    "bétonnée": "bétonné",
    "bêcheur": "bêcheuse",
    "bêcheuse": "bêcheur",
    "bêta": "bêtasse",
    "bêtasse": "bêta",
    "bûcheur": "bûcheuse",
    "bûcheuse": "bûcheur",
    "cabaretier": "cabaretière",
    "cabaretière": "cabaretier",
    "cabochard": "cabocharde",
    "cabocharde": "cabochard",
    "cabossé": "cabossée",
    "cabossée": "cabossé",
    "cabotin": "cabotine",
    "cabotine": "cabotin",
    "cabré": "cabrée",
    "cabrée": "cabré",
    "caché": "cachée",
    "cachée": "caché",
    "cacheté": "cachetée",
    "cachetée": "cacheté",
    "cachottier": "cachottière",
    "cachottière": "cachottier",
    "cadastral": "cadastrale",
    "cadastrale": "cadastral",
    "cadastré": "cadastrée",
    "cadastrée": "cadastré",
    "cadenassé": "cadenassée",
    "cadenassée": "cadenassé",
    "cadencé": "cadencée",
    "cadencée": "cadencé",
    "cadet": "cadette",
    "cadette": "cadet",
    "cadré": "cadrée",
    "cadrée": "cadré",
    "cafard": "cafarde",
    "cafarde": "cafard",
    "cafardeur": "cafardeuse",
    "cafardeuse": "cafardeur",
    "cafteur": "cafteuse",
    "cafteuse": "cafteur",
    "cagot": "cagote",
    "cagote": "cagot",
    "cahotant": "cahotante",
    "cahotante": "cahotant",
    "cahoté": "cahotée",
    "cahotée": "cahoté",
    "caillé": "caillée",
    "caillée": "caillé",
    "caissier": "caissière",
    "caissière": "caissier",
    "cajolé": "cajolée",
    "cajolée": "cajolé",
    "cajoleur": "cajoleuse",
    "cajoleuse": "cajoleur",
    "calamistré": "calamistrée",
    "calamistrée": "calamistré",
    "calamiteuse": "calamiteux",
    "calamiteux": "calamiteuse",
    "calcifié": "calcifiée",
    "calcifiée": "calcifié",
    "calciné": "calcinée",
    "calcinée": "calciné",
    "calculé": "calculée",
    "calculée": "calculé",
    "calé": "calée",
    "calée": "calé",
    "calfaté": "calfatée",
    "calfatée": "calfaté",
    "calfeutré": "calfeutrée",
    "calfeutrée": "calfeutré",
    "calibré": "calibrée",
    "calibrée": "calibré",
    "californien": "californienne",
    "californienne": "californien",
    "call girl": "call-girl",
    "call-girl": "call girl",
    "calligraphié": "calligraphiée",
    "calligraphiée": "calligraphié",
    "calmant": "calmante",
    "calmante": "calmant",
    "calmé": "calmée",
    "calmée": "calmé",
    "calomniateur": "calomniatrice",
    "calomniatrice": "calomniateur",
    "calomnié": "calomniée",
    "calomniée": "calomnié",
    "calqué": "calquée",
    "calquée": "calqué",
    "calédonien": "calédonienne",
    "calédonienne": "calédonien",
    "camard": "camarde",
    "camarde": "camard",
    "cambodgien": "cambodgienne",
    "cambodgienne": "cambodgien",
    "cambré": "cambrée",
    "cambrée": "cambré",
    "cambriolé": "cambriolée",
    "cambriolée": "cambriolé",
    "cambrioleur": "cambrioleuse",
    "cambrioleuse": "cambrioleur",
    "camé": "camée",
    "camée": "camé",
    "camouflé": "camouflée",
    "camouflée": "camouflé",
    "campagnard": "campagnarde",
    "campagnarde": "campagnard",
    "campé": "campée",
    "campée": "campé",
    "campeur": "campeuse",
    "campeuse": "campeur",
    "camus": "camuse",
    "camuse": "camus",
    "canalisé": "canalisée",
    "canalisée": "canalisé",
    "canard": "cane",
    "cane": "canard",
    "canardé": "canardée",
    "canardée": "canardé",
    "cancanier": "cancanière",
    "cancanière": "cancanier",
    "candidat": "candidate",
    "candidate": "candidat",
    "candi": "candie",
    "candie": "candi",
    "cannelé": "cannelée",
    "cannelée": "cannelé",
    "canné": "cannée",
    "cannée": "canné",
    "canonisé": "canonisée",
    "canonisée": "canonisé",
    "cantinier": "cantinière",
    "cantinière": "cantinier",
    "cantonais": "cantonaise",
    "cantonaise": "cantonais",
    "cantonal": "cantonale",
    "cantonale": "cantonal",
    "cantonné": "cantonnée",
    "cantonnée": "cantonné",
    "caoutchouté": "caoutchoutée",
    "caoutchoutée": "caoutchouté",
    "caparaçonné": "caparaçonnée",
    "caparaçonnée": "caparaçonné",
    "capital": "capitale",
    "capitale": "capital",
    "capitonné": "capitonnée",
    "capitonnée": "capitonné",
    "caporal": "caporale",
    "caporale": "caporal",
    "capoté": "capotée",
    "capotée": "capoté",
    "capté": "captée",
    "captée": "capté",
    "captif": "captive",
    "captive": "captif",
    "captivant": "captivante",
    "captivante": "captivant",
    "captivé": "captivée",
    "captivée": "captivé",
    "capturé": "capturée",
    "capturée": "capturé",
    "capétien": "capétienne",
    "capétienne": "capétien",
    "caquetant": "caquetante",
    "caquetante": "caquetant",
    "carabiné": "carabinée",
    "carabinée": "carabiné",
    "caractériel": "caractérielle",
    "caractérielle": "caractériel",
    "caractérisé": "caractérisée",
    "caractérisée": "caractérisé",
    "caramélisé": "caramélisée",
    "caramélisée": "caramélisé",
    "caravanier": "caravanière",
    "caravanière": "caravanier",
    "carbonisé": "carbonisée",
    "carbonisée": "carbonisé",
    "carcéral": "carcérale",
    "carcérale": "carcéral",
    "cardé": "cardée",
    "cardée": "cardé",
    "cardeur": "cardeuse",
    "cardeuse": "cardeur",
    "cardinal": "cardinale",
    "cardinale": "cardinal",
    "caressant": "caressante",
    "caressante": "caressant",
    "caressé": "caressée",
    "caressée": "caressé",
    "caricatural": "caricaturale",
    "caricaturale": "caricatural",
    "carillonneur": "carillonneuse",
    "carillonneuse": "carillonneur",
    "caritatif": "caritative",
    "caritative": "caritatif",
    "carié": "cariée",
    "cariée": "carié",
    "carmé": "carmée",
    "carmée": "carmé",
    "carminé": "carminée",
    "carminée": "carminé",
    "carnassier": "carnassière",
    "carnassière": "carnassier",
    "carné": "carnée",
    "carnée": "carné",
    "carolingien": "carolingienne",
    "carolingienne": "carolingien",
    "carotidien": "carotidienne",
    "carotidienne": "carotidien",
    "carotteur": "carotteuse",
    "carotteuse": "carotteur",
    "carpien": "carpienne",
    "carpienne": "carpien",
    "carrelé": "carrelée",
    "carrelée": "carrelé",
    "carré": "carrée",
    "carrée": "carré",
    "carthaginois": "carthaginoise",
    "carthaginoise": "carthaginois",
    "cartomancien": "cartomancienne",
    "cartomancienne": "cartomancien",
    "cartonné": "cartonnée",
    "cartonnée": "cartonné",
    "cartésien": "cartésienne",
    "cartésienne": "cartésien",
    "casanier": "casanière",
    "casanière": "casanier",
    "cascadeur": "cascadeuse",
    "cascadeuse": "cascadeur",
    "casé": "casée",
    "casée": "casé",
    "casqué": "casquée",
    "casquée": "casqué",
    "cassant": "cassante",
    "cassante": "cassant",
    "cassé": "cassée",
    "cassée": "cassé",
    "casseur": "casseuse",
    "casseuse": "casseur",
    "castillan": "castillane",
    "castillane": "castillan",
    "castrateur": "castratrice",
    "castratrice": "castrateur",
    "castré": "castrée",
    "castrée": "castré",
    "catalan": "catalane",
    "catalane": "catalan",
    "catalogué": "cataloguée",
    "cataloguée": "catalogué",
    "catapulté": "catapultée",
    "catapultée": "catapulté",
    "catastrophé": "catastrophée",
    "catastrophée": "catastrophé",
    "catcheur": "catcheuse",
    "catcheuse": "catcheur",
    "caucasien": "caucasienne",
    "caucasienne": "caucasien",
    "caudal": "caudale",
    "caudale": "caudal",
    "causant": "causante",
    "causante": "causant",
    "causé": "causée",
    "causée": "causé",
    "causeur": "causeuse",
    "causeuse": "causeur",
    "cauteleuse": "cauteleux",
    "cauteleux": "cauteleuse",
    "cautionné": "cautionnée",
    "cautionnée": "cautionné",
    "cavaleur": "cavaleuse",
    "cavaleuse": "cavaleur",
    "cavalier": "cavalière",
    "cavalière": "cavalier",
    "cavillon": "cavillonne",
    "cavillonne": "cavillon",
    "ceint": "ceinte",
    "ceinte": "ceint",
    "ceinturé": "ceinturée",
    "ceinturée": "ceinturé",
    "cendreuse": "cendreux",
    "cendreux": "cendreuse",
    "cendré": "cendrée",
    "cendrée": "cendré",
    "censuré": "censurée",
    "censurée": "censuré",
    "censé": "censée",
    "censée": "censé",
    "centrafricain": "centrafricaine",
    "centrafricaine": "centrafricain",
    "central": "centrale",
    "centrale": "central",
    "centralisé": "centralisée",
    "centralisée": "centralisé",
    "centré": "centrée",
    "centrée": "centré",
    "centrifugeur": "centrifugeuse",
    "centrifugeuse": "centrifugeur",
    "centuplé": "centuplée",
    "centuplée": "centuplé",
    "cerclé": "cerclée",
    "cerclée": "cerclé",
    "cerné": "cernée",
    "cernée": "cerné",
    "certain": "certaine",
    "certaine": "certain",
    "certifié": "certifiée",
    "certifiée": "certifié",
    "cervical": "cervicale",
    "cervicale": "cervical",
    "cessant": "cessante",
    "cessante": "cessant",
    "cessé": "cessée",
    "cessée": "cessé",
    "chafouin": "chafouine",
    "chafouine": "chafouin",
    "chagrin": "chagrine",
    "chagrine": "chagrin",
    "chagriné": "chagrinée",
    "chagrinée": "chagriné",
    "chahuté": "chahutée",
    "chahutée": "chahuté",
    "chahuteur": "chahuteuse",
    "chahuteuse": "chahuteur",
    "chaloupé": "chaloupée",
    "chaloupée": "chaloupé",
    "chamailleur": "chamailleuse",
    "chamailleuse": "chamailleur",
    "chamarré": "chamarrée",
    "chamarrée": "chamarré",
    "chambardé": "chambardée",
    "chambardée": "chambardé",
    "chamboulé": "chamboulée",
    "chamboulée": "chamboulé",
    "chambré": "chambrée",
    "chambrée": "chambré",
    "chameau": "chamelle",
    "chamelle": "chameau",
    "champ": "champ'",
    "champ'": "champ",
    "champion": "championne",
    "championne": "champion",
    "chancelant": "chancelante",
    "chancelante": "chancelant",
    "changeant": "changeante",
    "changeante": "changeant",
    "changé": "changée",
    "changée": "changé",
    "changeur": "changeuse",
    "changeuse": "changeur",
    "chanstiqué": "chanstiquée",
    "chanstiquée": "chanstiqué",
    "chantant": "chantante",
    "chantante": "chantant",
    "chanté": "chantée",
    "chantée": "chanté",
    "chanteur": "chanteuse",
    "chanteuse": "chanteur",
    "chantonnant": "chantonnante",
    "chantonnante": "chantonnant",
    "chantonné": "chantonnée",
    "chantonnée": "chantonné",
    "chançard": "chançarde",
    "chançarde": "chançard",
    "chapardé": "chapardée",
    "chapardée": "chapardé",
    "chapardeur": "chapardeuse",
    "chapardeuse": "chapardeur",
    "chapeauté": "chapeautée",
    "chapeautée": "chapeauté",
    "chaperonné": "chaperonnée",
    "chaperonnée": "chaperonné",
    "chapitré": "chapitrée",
    "chapitrée": "chapitré",
    "charançonné": "charançonnée",
    "charançonnée": "charançonné",
    "charbonnier": "charbonnière",
    "charbonnière": "charbonnier",
    "charcuté": "charcutée",
    "charcutée": "charcuté",
    "charcutier": "charcutière",
    "charcutière": "charcutier",
    "chargé": "chargée",
    "chargée": "chargé",
    "charmant": "charmante",
    "charmante": "charmant",
    "charmé": "charmée",
    "charmée": "charmé",
    "charmeur": "charmeuse",
    "charmeuse": "charmeur",
    "charnel": "charnelle",
    "charnelle": "charnel",
    "charnu": "charnue",
    "charnue": "charnu",
    "charpenté": "charpentée",
    "charpentée": "charpenté",
    "charpentier": "charpentière",
    "charpentière": "charpentier",
    "charretier": "charretière",
    "charretière": "charretier",
    "charrié": "charriée",
    "charriée": "charrié",
    "chassé": "chassée",
    "chassée": "chassé",
    "chat": "chatte",
    "chatte": "chat",
    "chatouillé": "chatouillée",
    "chatouillée": "chatouillé",
    "chatoyant": "chatoyante",
    "chatoyante": "chatoyant",
    "chaud": "chaude",
    "chaude": "chaud",
    "chauffant": "chauffante",
    "chauffante": "chauffant",
    "chauffé": "chauffée",
    "chauffée": "chauffé",
    "chauffeur": "chauffeuse",
    "chauffeuse": "chauffeur",
    "chaulé": "chaulée",
    "chaulée": "chaulé",
    "chaussé": "chaussée",
    "chaussée": "chaussé",
    "chauvin": "chauvine",
    "chauvine": "chauvin",
    "chavirant": "chavirante",
    "chavirante": "chavirant",
    "chaviré": "chavirée",
    "chavirée": "chaviré",
    "cheminé": "cheminée",
    "cheminée": "cheminé",
    "chenu": "chenue",
    "chenue": "chenu",
    "cher": "chère",
    "chère": "cher",
    "cherché": "cherchée",
    "cherchée": "cherché",
    "chercheur": "chercheuse",
    "chercheuse": "chercheur",
    "chevalin": "chevaline",
    "chevaline": "chevalin",
    "chevauché": "chevauchée",
    "chevauchée": "chevauché",
    "chevelu": "chevelue",
    "chevelue": "chevelu",
    "chevillé": "chevillée",
    "chevillée": "chevillé",
    "chevrier": "chevrière",
    "chevrière": "chevrier",
    "chevronné": "chevronnée",
    "chevronnée": "chevronné",
    "chevroté": "chevrotée",
    "chevrotée": "chevroté",
    "chewing gum": "chewing-gum",
    "chewing-gum": "chewing gum",
    "chiadé": "chiadée",
    "chiadée": "chiadé",
    "chialeur": "chialeuse",
    "chialeuse": "chialeur",
    "chiant": "chiante",
    "chiante": "chiant",
    "chicanier": "chicanière",
    "chicanière": "chicanier",
    "chicot": "chicote",
    "chicote": "chicot",
    "chien": "chienne",
    "chienne": "chien",
    "chié": "chiée",
    "chiée": "chié",
    "chieur": "chieuse",
    "chieuse": "chieur",
    "chiffonné": "chiffonnée",
    "chiffonnée": "chiffonné",
    "chiffonnier": "chiffonnière",
    "chiffonnière": "chiffonnier",
    "chiffré": "chiffrée",
    "chiffrée": "chiffré",
    "chiffreur": "chiffreuse",
    "chiffreuse": "chiffreur",
    "chilien": "chilienne",
    "chilienne": "chilien",
    "chiné": "chinée",
    "chinée": "chiné",
    "chinois": "chinoise",
    "chinoise": "chinois",
    "chipé": "chipée",
    "chipée": "chipé",
    "chipoteur": "chipoteuse",
    "chipoteuse": "chipoteur",
    "chiromancien": "chiromancienne",
    "chiromancienne": "chiromancien",
    "chiropraticien": "chiropraticienne",
    "chiropraticienne": "chiropraticien",
    "chirurgical": "chirurgicale",
    "chirurgicale": "chirurgical",
    "chirurgien": "chirurgienne",
    "chirurgienne": "chirurgien",
    "chloré": "chlorée",
    "chlorée": "chloré",
    "chocolatier": "chocolatière",
    "chocolatière": "chocolatier",
    "chocolaté": "chocolatée",
    "chocolatée": "chocolaté",
    "chu": "chue",
    "chue": "chu",
    "choisi": "choisie",
    "choisie": "choisi",
    "chopé": "chopée",
    "chopée": "chopé",
    "choppé": "choppée",
    "choppée": "choppé",
    "choquant": "choquante",
    "choquante": "choquant",
    "choqué": "choquée",
    "choquée": "choqué",
    "choral": "chorale",
    "chorale": "choral",
    "chou": "choute",
    "choute": "chou",
    "choucard": "choucarde",
    "choucarde": "choucard",
    "chouchouté": "chouchoutée",
    "chouchoutée": "chouchouté",
    "chouravé": "chouravée",
    "chouravée": "chouravé",
    "chouraveur": "chouraveuse",
    "chouraveuse": "chouraveur",
    "chouré": "chourée",
    "chourée": "chouré",
    "choyé": "choyée",
    "choyée": "choyé",
    "chromé": "chromée",
    "chromée": "chromé",
    "chroniqueur": "chroniqueuse",
    "chroniqueuse": "chroniqueur",
    "chronométré": "chronométrée",
    "chronométrée": "chronométré",
    "chrétien": "chrétienne",
    "chrétienne": "chrétien",
    "chtarbé": "chtarbée",
    "chtarbée": "chtarbé",
    "chuchotant": "chuchotante",
    "chuchotante": "chuchotant",
    "chuchoté": "chuchotée",
    "chuchotée": "chuchoté",
    "chuchoteur": "chuchoteuse",
    "chuchoteuse": "chuchoteur",
    "chuintant": "chuintante",
    "chuintante": "chuintant",
    "chuté": "chutée",
    "chutée": "chuté",
    "châtain": "châtaine",
    "châtaine": "châtain",
    "châtelain": "châtelaine",
    "châtelaine": "châtelain",
    "châtié": "châtiée",
    "châtiée": "châtié",
    "chéri": "chérie",
    "chérie": "chéri",
    "chétif": "chétive",
    "chétive": "chétif",
    "chômeur": "chômeuse",
    "chômeuse": "chômeur",
    "chômé": "chômée",
    "chômée": "chômé",
    "ci-joint": "ci-jointe",
    "ci-jointe": "ci-joint",
    "ciblé": "ciblée",
    "ciblée": "ciblé",
    "cicatrisé": "cicatrisée",
    "cicatrisée": "cicatrisé",
    "cimenté": "cimentée",
    "cimentée": "cimenté",
    "cinglant": "cinglante",
    "cinglante": "cinglant",
    "cinglé": "cinglée",
    "cinglée": "cinglé",
    "cintré": "cintrée",
    "cintrée": "cintré",
    "circonscrit": "circonscrite",
    "circonscrite": "circonscrit",
    "circonspect": "circonspecte",
    "circonspecte": "circonspect",
    "circonstanciel": "circonstancielle",
    "circonstancielle": "circonstanciel",
    "circonstancié": "circonstanciée",
    "circonstanciée": "circonstancié",
    "circonvenu": "circonvenue",
    "circonvenue": "circonvenu",
    "ciré": "cirée",
    "cirée": "ciré",
    "cireur": "cireuse",
    "cireuse": "cireur",
    "cisaillé": "cisaillée",
    "cisaillée": "cisaillé",
    "ciselé": "ciselée",
    "ciselée": "ciselé",
    "cistercien": "cistercienne",
    "cistercienne": "cistercien",
    "citadin": "citadine",
    "citadine": "citadin",
    "cité": "citée",
    "citée": "cité",
    "citoyen": "citoyenne",
    "citoyenne": "citoyen",
    "citronné": "citronnée",
    "citronnée": "citronné",
    "civil": "civile",
    "civile": "civil",
    "civilisateur": "civilisatrice",
    "civilisatrice": "civilisateur",
    "civilisé": "civilisée",
    "civilisée": "civilisé",
    "clair": "claire",
    "claire": "clair",
    "claironnant": "claironnante",
    "claironnante": "claironnant",
    "clairsemé": "clairsemée",
    "clairsemée": "clairsemé",
    "clairvoyant": "clairvoyante",
    "clairvoyante": "clairvoyant",
    "clamecé": "clamecée",
    "clamecée": "clamecé",
    "clamé": "clamée",
    "clamée": "clamé",
    "clamsé": "clamsée",
    "clamsée": "clamsé",
    "clandestin": "clandestine",
    "clandestine": "clandestin",
    "clapotant": "clapotante",
    "clapotante": "clapotant",
    "claquant": "claquante",
    "claquante": "claquant",
    "claquemuré": "claquemurée",
    "claquemurée": "claquemuré",
    "claqué": "claquée",
    "claquée": "claqué",
    "clarifié": "clarifiée",
    "clarifiée": "clarifié",
    "classé": "classée",
    "classée": "classé",
    "classificateur": "classificatrice",
    "classificatrice": "classificateur",
    "claudicant": "claudicante",
    "claudicante": "claudicant",
    "cliché": "clichée",
    "clichée": "cliché",
    "client": "cliente",
    "cliente": "client",
    "clignotant": "clignotante",
    "clignotante": "clignotant",
    "climatisé": "climatisée",
    "climatisée": "climatisé",
    "clinicien": "clinicienne",
    "clinicienne": "clinicien",
    "clinquant": "clinquante",
    "clinquante": "clinquant",
    "cliquetant": "cliquetante",
    "cliquetante": "cliquetant",
    "clitoridien": "clitoridienne",
    "clitoridienne": "clitoridien",
    "clochard": "clocharde",
    "clocharde": "clochard",
    "cloisonné": "cloisonnée",
    "cloisonnée": "cloisonné",
    "cloqué": "cloquée",
    "cloquée": "cloqué",
    "cloué": "clouée",
    "clouée": "cloué",
    "cloueur": "cloueuse",
    "cloueuse": "cloueur",
    "clouté": "cloutée",
    "cloutée": "clouté",
    "cloîtré": "cloîtrée",
    "cloîtrée": "cloîtré",
    "clément": "clémente",
    "clémente": "clément",
    "clérical": "cléricale",
    "cléricale": "clérical",
    "clôturé": "clôturée",
    "clôturée": "clôturé",
    "coagulé": "coagulée",
    "coagulée": "coagulé",
    "cocardier": "cocardière",
    "cocardière": "cocardier",
    "coché": "cochée",
    "cochée": "coché",
    "cochon": "cochonne",
    "cochonne": "cochon",
    "cocu": "cocue",
    "cocue": "cocu",
    "codé": "codée",
    "codée": "codé",
    "codifié": "codifiée",
    "codifiée": "codifié",
    "coercitif": "coercitive",
    "coercitive": "coercitif",
    "coffré": "coffrée",
    "coffrée": "coffré",
    "cofondateur": "cofondatrice",
    "cofondatrice": "cofondateur",
    "cogné": "cognée",
    "cognée": "cogné",
    "cogneur": "cogneuse",
    "cogneuse": "cogneur",
    "cognitif": "cognitive",
    "cognitive": "cognitif",
    "cohérent": "cohérente",
    "cohérente": "cohérent",
    "coiffant": "coiffante",
    "coiffante": "coiffant",
    "coiffé": "coiffée",
    "coiffée": "coiffé",
    "coiffeur": "coiffeuse",
    "coiffeuse": "coiffeur",
    "coincé": "coincée",
    "coincée": "coincé",
    "collaborateur": "collaboratrice",
    "collaboratrice": "collaborateur",
    "collant": "collante",
    "collante": "collant",
    "collatéral": "collatérale",
    "collatérale": "collatéral",
    "collecté": "collectée",
    "collectée": "collecté",
    "collectif": "collective",
    "collective": "collectif",
    "collectionneur": "collectionneuse",
    "collectionneuse": "collectionneur",
    "collectivisé": "collectivisée",
    "collectivisée": "collectivisé",
    "collé": "collée",
    "collée": "collé",
    "colloqué": "colloquée",
    "colloquée": "colloqué",
    "collégial": "collégiale",
    "collégiale": "collégial",
    "collégien": "collégienne",
    "collégienne": "collégien",
    "colmaté": "colmatée",
    "colmatée": "colmaté",
    "colombien": "colombienne",
    "colombienne": "colombien",
    "colombin": "colombine",
    "colombine": "colombin",
    "colonel": "colonelle",
    "colonelle": "colonel",
    "colonial": "coloniale",
    "coloniale": "colonial",
    "colonisé": "colonisée",
    "colonisée": "colonisé",
    "coloré": "colorée",
    "colorée": "coloré",
    "colorié": "coloriée",
    "coloriée": "colorié",
    "colossal": "colossale",
    "colossale": "colossal",
    "colporté": "colportée",
    "colportée": "colporté",
    "colporteur": "colporteuse",
    "colporteuse": "colporteur",
    "coltiné": "coltinée",
    "coltinée": "coltiné",
    "combatif": "combative",
    "combative": "combatif",
    "combattant": "combattante",
    "combattante": "combattant",
    "combattu": "combattue",
    "combattue": "combattu",
    "combiné": "combinée",
    "combinée": "combiné",
    "comblé": "comblée",
    "comblée": "comblé",
    "commandant": "commandante",
    "commandante": "commandant",
    "commandé": "commandée",
    "commandée": "commandé",
    "commandité": "commanditée",
    "commanditée": "commandité",
    "commencé": "commencée",
    "commencée": "commencé",
    "commentateur": "commentatrice",
    "commentatrice": "commentateur",
    "commenté": "commentée",
    "commentée": "commenté",
    "commençant": "commençante",
    "commençante": "commençant",
    "commercial": "commerciale",
    "commerciale": "commercial",
    "commerçant": "commerçante",
    "commerçante": "commerçant",
    "commotionné": "commotionnée",
    "commotionnée": "commotionné",
    "commué": "commuée",
    "commuée": "commué",
    "commun": "commune",
    "commune": "commun",
    "communal": "communale",
    "communale": "communal",
    "communiant": "communiante",
    "communiante": "communiant",
    "communicatif": "communicative",
    "communicative": "communicatif",
    "communiqué": "communiquée",
    "communiquée": "communiqué",
    "commémoratif": "commémorative",
    "commémorative": "commémoratif",
    "compact": "compacte",
    "compacte": "compact",
    "compagne": "compagnon",
    "compagnon": "compagne",
    "comparatif": "comparative",
    "comparative": "comparatif",
    "comparé": "comparée",
    "comparée": "comparé",
    "compartimenté": "compartimentée",
    "compartimentée": "compartimenté",
    "compassé": "compassée",
    "compassée": "compassé",
    "compati": "compatie",
    "compatie": "compati",
    "compatissant": "compatissante",
    "compatissante": "compatissant",
    "compensé": "compensée",
    "compensée": "compensé",
    "complaisant": "complaisante",
    "complaisante": "complaisant",
    "complet": "complète",
    "complète": "complet",
    "complexé": "complexée",
    "complexée": "complexé",
    "complimenté": "complimentée",
    "complimentée": "complimenté",
    "compliqué": "compliquée",
    "compliquée": "compliqué",
    "comploteur": "comploteuse",
    "comploteuse": "comploteur",
    "complété": "complétée",
    "complétée": "complété",
    "comportemental": "comportementale",
    "comportementale": "comportemental",
    "comporté": "comportée",
    "comportée": "comporté",
    "composant": "composante",
    "composante": "composant",
    "composé": "composée",
    "composée": "composé",
    "compositeur": "compositrice",
    "compositrice": "compositeur",
    "compressé": "compressée",
    "compressée": "compressé",
    "comprimé": "comprimée",
    "comprimée": "comprimé",
    "compromettant": "compromettante",
    "compromettante": "compromettant",
    "compréhensif": "compréhensive",
    "compréhensive": "compréhensif",
    "compté": "comptée",
    "comptée": "compté",
    "compulsif": "compulsive",
    "compulsive": "compulsif",
    "compétent": "compétente",
    "compétente": "compétent",
    "compétiteur": "compétitrice",
    "compétitrice": "compétiteur",
    "compétitif": "compétitive",
    "compétitive": "compétitif",
    "comtal": "comtale",
    "comtale": "comtal",
    "comte": "comtesse",
    "comtesse": "comte",
    "comédien": "comédienne",
    "comédienne": "comédien",
    "con": "conne",
    "conne": "con",
    "conard": "conarde",
    "conarde": "conard",
    "concassé": "concassée",
    "concassée": "concassé",
    "concentré": "concentrée",
    "concentrée": "concentré",
    "concepteur": "conceptrice",
    "conceptrice": "concepteur",
    "conceptuel": "conceptuelle",
    "conceptuelle": "conceptuel",
    "concerné": "concernée",
    "concernée": "concerné",
    "concerté": "concertée",
    "concertée": "concerté",
    "conçu": "conçue",
    "conçue": "conçu",
    "conciliant": "conciliante",
    "conciliante": "conciliant",
    "conciliateur": "conciliatrice",
    "conciliatrice": "conciliateur",
    "concitoyen": "concitoyenne",
    "concitoyenne": "concitoyen",
    "concluant": "concluante",
    "concluante": "concluant",
    "conclu": "conclue",
    "conclue": "conclu",
    "concocté": "concoctée",
    "concoctée": "concocté",
    "concomitant": "concomitante",
    "concomitante": "concomitant",
    "concordant": "concordante",
    "concordante": "concordant",
    "concret": "concrète",
    "concrète": "concret",
    "concrétisé": "concrétisée",
    "concrétisée": "concrétisé",
    "concubin": "concubine",
    "concubine": "concubin",
    "concupiscent": "concupiscente",
    "concupiscente": "concupiscent",
    "concurrencé": "concurrencée",
    "concurrencée": "concurrencé",
    "concurrent": "concurrente",
    "concurrente": "concurrent",
    "concurrentiel": "concurrentielle",
    "concurrentielle": "concurrentiel",
    "concédé": "concédée",
    "concédée": "concédé",
    "condamné": "condamnée",
    "condamnée": "condamné",
    "condensé": "condensée",
    "condensée": "condensé",
    "condescendant": "condescendante",
    "condescendante": "condescendant",
    "conditionné": "conditionnée",
    "conditionnée": "conditionné",
    "conducteur": "conductrice",
    "conductrice": "conducteur",
    "conduit": "conduite",
    "conduite": "conduit",
    "confectionné": "confectionnée",
    "confectionnée": "confectionné",
    "confectionneur": "confectionneuse",
    "confectionneuse": "confectionneur",
    "confessé": "confessée",
    "confessée": "confessé",
    "confessionnel": "confessionnelle",
    "confessionnelle": "confessionnel",
    "confiant": "confiante",
    "confiante": "confiant",
    "confident": "confidente",
    "confidente": "confident",
    "confidentiel": "confidentielle",
    "confidentielle": "confidentiel",
    "confié": "confiée",
    "confiée": "confié",
    "confiné": "confinée",
    "confinée": "confiné",
    "confit": "confite",
    "confite": "confit",
    "confirmé": "confirmée",
    "confirmée": "confirmé",
    "confiseur": "confiseuse",
    "confiseuse": "confiseur",
    "confisqué": "confisquée",
    "confisquée": "confisqué",
    "conflictuel": "conflictuelle",
    "conflictuelle": "conflictuel",
    "confondant": "confondante",
    "confondante": "confondant",
    "confondu": "confondue",
    "confondue": "confondu",
    "conformé": "conformée",
    "conformée": "conformé",
    "conforté": "confortée",
    "confortée": "conforté",
    "confraternel": "confraternelle",
    "confraternelle": "confraternel",
    "confronté": "confrontée",
    "confrontée": "confronté",
    "confédéré": "confédérée",
    "confédérée": "confédéré",
    "conférencier": "conférencière",
    "conférencière": "conférencier",
    "conféré": "conférée",
    "conférée": "conféré",
    "congelé": "congelée",
    "congelée": "congelé",
    "congestif": "congestive",
    "congestive": "congestif",
    "congestionné": "congestionnée",
    "congestionnée": "congestionné",
    "congédié": "congédiée",
    "congédiée": "congédié",
    "congénital": "congénitale",
    "congénitale": "congénital",
    "conjectural": "conjecturale",
    "conjecturale": "conjectural",
    "conjoint": "conjointe",
    "conjointe": "conjoint",
    "conjonctif": "conjonctive",
    "conjonctive": "conjonctif",
    "conjugal": "conjugale",
    "conjugale": "conjugal",
    "conjugué": "conjuguée",
    "conjuguée": "conjugué",
    "conjuré": "conjurée",
    "conjurée": "conjuré",
    "connaisseur": "connaisseuse",
    "connaisseuse": "connaisseur",
    "connu": "connue",
    "connue": "connu",
    "connecté": "connectée",
    "connectée": "connecté",
    "conquérant": "conquérante",
    "conquérante": "conquérant",
    "consacré": "consacrée",
    "consacrée": "consacré",
    "consanguin": "consanguine",
    "consanguine": "consanguin",
    "conscient": "consciente",
    "consciente": "conscient",
    "conseiller": "conseillère",
    "conseillère": "conseiller",
    "conseillé": "conseillée",
    "conseillée": "conseillé",
    "consensuel": "consensuelle",
    "consensuelle": "consensuel",
    "consentant": "consentante",
    "consentante": "consentant",
    "consenti": "consentie",
    "consentie": "consenti",
    "conservateur": "conservatrice",
    "conservatrice": "conservateur",
    "conservé": "conservée",
    "conservée": "conservé",
    "considéré": "considérée",
    "considérée": "considéré",
    "consigné": "consignée",
    "consignée": "consigné",
    "consistant": "consistante",
    "consistante": "consistant",
    "consolant": "consolante",
    "consolante": "consolant",
    "consolateur": "consolatrice",
    "consolatrice": "consolateur",
    "consolé": "consolée",
    "consolée": "consolé",
    "consolidé": "consolidée",
    "consolidée": "consolidé",
    "consommateur": "consommatrice",
    "consommatrice": "consommateur",
    "consommé": "consommée",
    "consommée": "consommé",
    "conspirateur": "conspiratrice",
    "conspiratrice": "conspirateur",
    "constant": "constante",
    "constante": "constant",
    "constaté": "constatée",
    "constatée": "constaté",
    "constellé": "constellée",
    "constellée": "constellé",
    "consternant": "consternante",
    "consternante": "consternant",
    "consterné": "consternée",
    "consternée": "consterné",
    "constipé": "constipée",
    "constipée": "constipé",
    "constituant": "constituante",
    "constituante": "constituant",
    "constitué": "constituée",
    "constituée": "constitué",
    "constitutionnel": "constitutionnelle",
    "constitutionnelle": "constitutionnel",
    "constructif": "constructive",
    "constructive": "constructif",
    "construit": "construite",
    "construite": "construit",
    "consubstantiel": "consubstantielle",
    "consubstantielle": "consubstantiel",
    "consultant": "consultante",
    "consultante": "consultant",
    "consultatif": "consultative",
    "consultative": "consultatif",
    "consulté": "consultée",
    "consultée": "consulté",
    "consumé": "consumée",
    "consumée": "consumé",
    "consécutif": "consécutive",
    "consécutive": "consécutif",
    "conséquent": "conséquente",
    "conséquente": "conséquent",
    "contacté": "contactée",
    "contactée": "contacté",
    "contaminé": "contaminée",
    "contaminée": "contaminé",
    "contemplateur": "contemplatrice",
    "contemplatrice": "contemplateur",
    "contemplatif": "contemplative",
    "contemplative": "contemplatif",
    "contemplé": "contemplée",
    "contemplée": "contemplé",
    "contemporain": "contemporaine",
    "contemporaine": "contemporain",
    "contenu": "contenue",
    "contenue": "contenu",
    "content": "contente",
    "contente": "content",
    "contenté": "contentée",
    "contentée": "contenté",
    "conté": "contée",
    "contée": "conté",
    "contesté": "contestée",
    "contestée": "contesté",
    "conteur": "conteuse",
    "conteuse": "conteur",
    "contigu": "contiguë",
    "contiguë": "contigu",
    "continental": "continentale",
    "continentale": "continental",
    "contingent": "contingente",
    "contingente": "contingent",
    "continuel": "continuelle",
    "continuelle": "continuel",
    "continué": "continuée",
    "continuée": "continué",
    "contondant": "contondante",
    "contondante": "contondant",
    "contorsionné": "contorsionnée",
    "contorsionnée": "contorsionné",
    "contourné": "contournée",
    "contournée": "contourné",
    "contraceptif": "contraceptive",
    "contraceptive": "contraceptif",
    "contracté": "contractée",
    "contractée": "contracté",
    "contractuel": "contractuelle",
    "contractuelle": "contractuel",
    "contraignant": "contraignante",
    "contraignante": "contraignant",
    "contraint": "contrainte",
    "contrainte": "contraint",
    "contrariant": "contrariante",
    "contrariante": "contrariant",
    "contrarié": "contrariée",
    "contrariée": "contrarié",
    "contrasté": "contrastée",
    "contrastée": "contrasté",
    "contre-productif": "contre-productive",
    "contre-productive": "contre-productif",
    "contrebalancé": "contrebalancée",
    "contrebalancée": "contrebalancé",
    "contrebandier": "contrebandière",
    "contrebandière": "contrebandier",
    "contrecarré": "contrecarrée",
    "contrecarrée": "contrecarré",
    "contredit": "contredite",
    "contredite": "contredit",
    "contrefait": "contrefaite",
    "contrefaite": "contrefait",
    "contré": "contrée",
    "contrée": "contré",
    "contresigné": "contresignée",
    "contresignée": "contresigné",
    "contrit": "contrite",
    "contrite": "contrit",
    "controversé": "controversée",
    "controversée": "controversé",
    "contrôlé": "contrôlée",
    "contrôlée": "contrôlé",
    "contrôleur": "contrôleuse",
    "contrôleuse": "contrôleur",
    "contusionné": "contusionnée",
    "contusionnée": "contusionné",
    "convaincant": "convaincante",
    "convaincante": "convaincant",
    "convaincu": "convaincue",
    "convaincue": "convaincu",
    "convalescent": "convalescente",
    "convalescente": "convalescent",
    "convenu": "convenue",
    "convenue": "convenu",
    "conventionnel": "conventionnelle",
    "conventionnelle": "conventionnel",
    "conventuel": "conventuelle",
    "conventuelle": "conventuel",
    "convergent": "convergente",
    "convergente": "convergent",
    "converti": "convertie",
    "convertie": "converti",
    "convié": "conviée",
    "conviée": "convié",
    "convivial": "conviviale",
    "conviviale": "convivial",
    "convoité": "convoitée",
    "convoitée": "convoité",
    "convoqué": "convoquée",
    "convoquée": "convoqué",
    "convoyé": "convoyée",
    "convoyée": "convoyé",
    "convoyeur": "convoyeuse",
    "convoyeuse": "convoyeur",
    "convulsé": "convulsée",
    "convulsée": "convulsé",
    "convulsif": "convulsive",
    "convulsive": "convulsif",
    "coopérant": "coopérante",
    "coopérante": "coopérant",
    "coopératif": "coopérative",
    "coopérative": "coopératif",
    "coordinateur": "coordinatrice",
    "coordinatrice": "coordinateur",
    "coordonné": "coordonnée",
    "coordonnée": "coordonné",
    "copain": "copine",
    "copine": "copain",
    "copié": "copiée",
    "copiée": "copié",
    "copieur": "copieuse",
    "copieuse": "copieur",
    "copineur": "copineuse",
    "copineuse": "copineur",
    "coquet": "coquette",
    "coquette": "coquet",
    "coquin": "coquine",
    "coquine": "coquin",
    "cordeau": "cordelle",
    "cordelle": "cordeau",
    "cordelier": "cordelière",
    "cordelière": "cordelier",
    "cordial": "cordiale",
    "cordiale": "cordial",
    "cordonnier": "cordonnière",
    "cordonnière": "cordonnier",
    "corné": "cornée",
    "cornée": "corné",
    "cornu": "cornue",
    "cornue": "cornu",
    "cornéen": "cornéenne",
    "cornéenne": "cornéen",
    "cornélien": "cornélienne",
    "cornélienne": "cornélien",
    "coronal": "coronale",
    "coronale": "coronal",
    "coronarien": "coronarienne",
    "coronarienne": "coronarien",
    "corporatif": "corporative",
    "corporative": "corporatif",
    "corporel": "corporelle",
    "corporelle": "corporel",
    "corpulent": "corpulente",
    "corpulente": "corpulent",
    "correct": "correcte",
    "correcte": "correct",
    "correcteur": "correctrice",
    "correctrice": "correcteur",
    "correctif": "corrective",
    "corrective": "correctif",
    "correctionnel": "correctionnelle",
    "correctionnelle": "correctionnel",
    "correspondant": "correspondante",
    "correspondante": "correspondant",
    "corrigé": "corrigée",
    "corrigée": "corrigé",
    "corrodant": "corrodante",
    "corrodante": "corrodant",
    "corrodé": "corrodée",
    "corrodée": "corrodé",
    "corrompu": "corrompue",
    "corrompue": "corrompu",
    "corrosif": "corrosive",
    "corrosive": "corrosif",
    "corrupteur": "corruptrice",
    "corruptrice": "corrupteur",
    "corrélatif": "corrélative",
    "corrélative": "corrélatif",
    "corsé": "corsée",
    "corsée": "corsé",
    "corseté": "corsetée",
    "corsetée": "corseté",
    "cortical": "corticale",
    "corticale": "cortical",
    "coruscant": "coruscante",
    "coruscante": "coruscant",
    "coréen": "coréenne",
    "coréenne": "coréen",
    "cosmétiqué": "cosmétiquée",
    "cosmétiquée": "cosmétiqué",
    "cossard": "cossarde",
    "cossarde": "cossard",
    "cossu": "cossue",
    "cossue": "cossu",
    "costal": "costale",
    "costale": "costal",
    "costaud": "costaude",
    "costaude": "costaud",
    "costumé": "costumée",
    "costumée": "costumé",
    "costumier": "costumière",
    "costumière": "costumier",
    "coté": "cotée",
    "cotée": "coté",
    "couard": "couarde",
    "couarde": "couard",
    "couché": "couchée",
    "couchée": "couché",
    "coucheur": "coucheuse",
    "coucheuse": "coucheur",
    "coudé": "coudée",
    "coudée": "coudé",
    "cousu": "cousue",
    "cousue": "cousu",
    "couillonné": "couillonnée",
    "couillonnée": "couillonné",
    "coulant": "coulante",
    "coulante": "coulant",
    "coulé": "coulée",
    "coulée": "coulé",
    "coulissant": "coulissante",
    "coulissante": "coulissant",
    "coulissé": "coulissée",
    "coulissée": "coulissé",
    "coupant": "coupante",
    "coupante": "coupant",
    "coupé": "coupée",
    "coupée": "coupé",
    "couperosé": "couperosée",
    "couperosée": "couperosé",
    "coupeur": "coupeuse",
    "coupeuse": "coupeur",
    "couplé": "couplée",
    "couplée": "couplé",
    "courant": "courante",
    "courante": "courant",
    "courbatu": "courbatue",
    "courbatue": "courbatu",
    "courbaturé": "courbaturée",
    "courbaturée": "courbaturé",
    "courbé": "courbée",
    "courbée": "courbé",
    "coureur": "coureuse",
    "coureuse": "coureur",
    "couru": "courue",
    "courue": "couru",
    "couronné": "couronnée",
    "couronnée": "couronné",
    "courroucé": "courroucée",
    "courroucée": "courroucé",
    "coursé": "coursée",
    "coursée": "coursé",
    "coursier": "coursière",
    "coursière": "coursier",
    "court": "courte",
    "courte": "court",
    "court-circuité": "court-circuitée",
    "court-circuitée": "court-circuité",
    "courtaud": "courtaude",
    "courtaude": "courtaud",
    "courtier": "courtière",
    "courtière": "courtier",
    "courtisan": "courtisane",
    "courtisane": "courtisan",
    "courtisé": "courtisée",
    "courtisée": "courtisé",
    "cousin": "cousine",
    "cousine": "cousin",
    "coutumier": "coutumière",
    "coutumière": "coutumier",
    "couturé": "couturée",
    "couturée": "couturé",
    "couturier": "couturière",
    "couturière": "couturier",
    "couvé": "couvée",
    "couvée": "couvé",
    "couvert": "couverte",
    "couverte": "couvert",
    "couvrant": "couvrante",
    "couvrante": "couvrant",
    "covalent": "covalente",
    "covalente": "covalent",
    "coûté": "coûtée",
    "coûtée": "coûté",
    "craché": "crachée",
    "crachée": "craché",
    "cracheur": "cracheuse",
    "cracheuse": "cracheur",
    "craint": "crainte",
    "crainte": "craint",
    "craintif": "craintive",
    "craintive": "craintif",
    "cramé": "cramée",
    "cramée": "cramé",
    "cramoisi": "cramoisie",
    "cramoisie": "cramoisi",
    "cramponné": "cramponnée",
    "cramponnée": "cramponné",
    "craquant": "craquante",
    "craquante": "craquant",
    "craquelé": "craquelée",
    "craquelée": "craquelé",
    "craqué": "craquée",
    "craquée": "craqué",
    "crashé": "crashée",
    "crashée": "crashé",
    "cravaté": "cravatée",
    "cravatée": "cravaté",
    "creton": "cretonne",
    "cretonne": "creton",
    "creusé": "creusée",
    "creusée": "creusé",
    "crevant": "crevante",
    "crevante": "crevant",
    "crevard": "crevarde",
    "crevarde": "crevard",
    "crevassé": "crevassée",
    "crevassée": "crevassé",
    "crevé": "crevée",
    "crevée": "crevé",
    "criant": "criante",
    "criante": "criant",
    "criard": "criarde",
    "criarde": "criard",
    "criblé": "criblée",
    "criblée": "criblé",
    "crié": "criée",
    "criée": "crié",
    "crieur": "crieuse",
    "crieuse": "crieur",
    "criminel": "criminelle",
    "criminelle": "criminel",
    "crispant": "crispante",
    "crispante": "crispant",
    "crispé": "crispée",
    "crispée": "crispé",
    "cristallin": "cristalline",
    "cristalline": "cristallin",
    "cristallisé": "cristallisée",
    "cristallisée": "cristallisé",
    "critiqué": "critiquée",
    "critiquée": "critiqué",
    "crochu": "crochue",
    "crochue": "crochu",
    "cru": "crue",
    "crue": "cru",
    "croisé": "croisée",
    "croisée": "croisé",
    "croissant": "croissante",
    "croissante": "croissant",
    "croquant": "croquante",
    "croquante": "croquant",
    "croqué": "croquée",
    "croquée": "croqué",
    "croqueur": "croqueuse",
    "croqueuse": "croqueur",
    "crotté": "crottée",
    "crottée": "crotté",
    "croulant": "croulante",
    "croulante": "croulant",
    "croulé": "croulée",
    "croulée": "croulé",
    "croupi": "croupie",
    "croupie": "croupi",
    "croupissant": "croupissante",
    "croupissante": "croupissant",
    "croustillant": "croustillante",
    "croustillante": "croustillant",
    "croyant": "croyante",
    "croyante": "croyant",
    "crû": "crue",
    "croûté": "croûtée",
    "croûtée": "croûté",
    "crucial": "cruciale",
    "cruciale": "crucial",
    "crucifiant": "crucifiante",
    "crucifiante": "crucifiant",
    "crucifié": "crucifiée",
    "crucifiée": "crucifié",
    "cruel": "cruelle",
    "cruelle": "cruel",
    "crypté": "cryptée",
    "cryptée": "crypté",
    "crâneur": "crâneuse",
    "crâneuse": "crâneur",
    "crânien": "crânienne",
    "crânienne": "crânien",
    "créateur": "créatrice",
    "créatrice": "créateur",
    "créatif": "créative",
    "créative": "créatif",
    "crédité": "créditée",
    "créditée": "crédité",
    "créditeur": "créditrice",
    "créditrice": "créditeur",
    "créé": "créée",
    "créée": "créé",
    "crémier": "crémière",
    "crémière": "crémier",
    "crénelé": "crénelée",
    "crénelée": "crénelé",
    "crépi": "crépie",
    "crépie": "crépi",
    "crépitant": "crépitante",
    "crépitante": "crépitant",
    "crépu": "crépue",
    "crépue": "crépu",
    "crétin": "crétine",
    "crétine": "crétin",
    "crétois": "crétoise",
    "crétoise": "crétois",
    "crêpé": "crêpée",
    "crêpée": "crêpé",
    "crêté": "crêtée",
    "crêtée": "crêté",
    "cubain": "cubaine",
    "cubaine": "cubain",
    "cubital": "cubitale",
    "cubitale": "cubital",
    "cueilleur": "cueilleuse",
    "cueilleuse": "cueilleur",
    "cueilli": "cueillie",
    "cueillie": "cueilli",
    "cuirassé": "cuirassée",
    "cuirassée": "cuirassé",
    "cuisant": "cuisante",
    "cuisante": "cuisant",
    "cuisiné": "cuisinée",
    "cuisinée": "cuisiné",
    "cuisinier": "cuisinière",
    "cuisinière": "cuisinier",
    "cuit": "cuite",
    "cuite": "cuit",
    "cuivré": "cuivrée",
    "cuivrée": "cuivré",
    "culbuté": "culbutée",
    "culbutée": "culbuté",
    "culotté": "culottée",
    "culottée": "culotté",
    "culottier": "culottière",
    "culottière": "culottier",
    "culpabilisé": "culpabilisée",
    "culpabilisée": "culpabilisé",
    "cultivateur": "cultivatrice",
    "cultivatrice": "cultivateur",
    "cultivé": "cultivée",
    "cultivée": "cultivé",
    "culturel": "culturelle",
    "culturelle": "culturel",
    "cumulatif": "cumulative",
    "cumulative": "cumulatif",
    "curateur": "curatrice",
    "curatrice": "curateur",
    "curatif": "curative",
    "curative": "curatif",
    "curé": "curée",
    "curée": "curé",
    "cursif": "cursive",
    "cursive": "cursif",
    "cutané": "cutanée",
    "cutanée": "cutané",
    "cuvé": "cuvée",
    "cuvée": "cuvé",
    "cyanosé": "cyanosée",
    "cyanosée": "cyanosé",
    "cyclopéen": "cyclopéenne",
    "cyclopéenne": "cyclopéen",
    "câblé": "câblée",
    "câblée": "câblé",
    "câlin": "câline",
    "câline": "câlin",
    "câliné": "câlinée",
    "câlinée": "câliné",
    "cédé": "cédée",
    "cédée": "cédé",
    "célinien": "célinienne",
    "célinienne": "célinien",
    "célébré": "célébrée",
    "célébrée": "célébré",
    "céréalier": "céréalière",
    "céréalière": "céréalier",
    "cérébral": "cérébrale",
    "cérébrale": "cérébral",
    "cérémonial": "cérémoniale",
    "cérémoniale": "cérémonial",
    "cérémoniel": "cérémonielle",
    "cérémonielle": "cérémoniel",
    "côtelé": "côtelée",
    "côtelée": "côtelé",
    "côtier": "côtière",
    "côtière": "côtier",
    "côtoyé": "côtoyée",
    "côtoyée": "côtoyé",
    "dactylographié": "dactylographiée",
    "dactylographiée": "dactylographié",
    "dakarois": "dakaroise",
    "dakaroise": "dakarois",
    "dallé": "dallée",
    "dallée": "dallé",
    "dalmatien": "dalmatienne",
    "dalmatienne": "dalmatien",
    "daltonien": "daltonienne",
    "daltonienne": "daltonien",
    "damasquiné": "damasquinée",
    "damasquinée": "damasquiné",
    "damassé": "damassée",
    "damassée": "damassé",
    "damé": "damée",
    "damée": "damé",
    "damné": "damnée",
    "damnée": "damné",
    "damoiseau": "demoiselle",
    "demoiselle": "damoiseau",
    "dansant": "dansante",
    "dansante": "dansant",
    "dansé": "dansée",
    "dansée": "dansé",
    "danseur": "danseuse",
    "danseuse": "danseur",
    "danubien": "danubienne",
    "danubienne": "danubien",
    "dardé": "dardée",
    "dardée": "dardé",
    "daron": "daronne",
    "daronne": "daron",
    "daté": "datée",
    "datée": "daté",
    "dauphin": "dauphine",
    "dauphine": "dauphin",
    "demandé": "demandée",
    "demandée": "demandé",
    "demeuré": "demeurée",
    "demeurée": "demeuré",
    "demi": "demie",
    "demie": "demi",
    "demi-dévêtu": "demi-dévêtue",
    "demi-dévêtue": "demi-dévêtu",
    "demi-mort": "demi-morte",
    "demi-morte": "demi-mort",
    "demi-ton": "demi-tonne",
    "demi-tonne": "demi-ton",
    "dental": "dentale",
    "dentale": "dental",
    "dentelé": "dentelée",
    "dentelée": "dentelé",
    "denté": "dentée",
    "dentée": "denté",
    "dernier": "dernière",
    "dernière": "dernier",
    "descellé": "descellée",
    "descellée": "descellé",
    "descendant": "descendante",
    "descendante": "descendant",
    "descendu": "descendue",
    "descendue": "descendu",
    "descriptif": "descriptive",
    "descriptive": "descriptif",
    "dessalé": "dessalée",
    "dessalée": "dessalé",
    "dessaoulé": "dessaoulée",
    "dessaoulée": "dessaoulé",
    "desserré": "desserrée",
    "desserrée": "desserré",
    "desservi": "desservie",
    "desservie": "desservi",
    "dessinateur": "dessinatrice",
    "dessinatrice": "dessinateur",
    "dessiné": "dessinée",
    "dessinée": "dessiné",
    "dessoudé": "dessoudée",
    "dessoudée": "dessoudé",
    "dessoûlé": "dessoûlée",
    "dessoûlée": "dessoûlé",
    "desséchant": "desséchante",
    "desséchante": "desséchant",
    "desséché": "desséchée",
    "desséchée": "desséché",
    "destiné": "destinée",
    "destinée": "destiné",
    "destitué": "destituée",
    "destituée": "destitué",
    "destructeur": "destructrice",
    "destructrice": "destructeur",
    "destructif": "destructive",
    "destructive": "destructif",
    "devancé": "devancée",
    "devancée": "devancé",
    "devenu": "devenue",
    "devenue": "devenu",
    "deviné": "devinée",
    "devinée": "deviné",
    "devineresse": "devineur",
    "devineur": "devineresse",
    "due": "dû",
    "dû": "due",
    "diabolisé": "diabolisée",
    "diabolisée": "diabolisé",
    "diagonal": "diagonale",
    "diagonale": "diagonal",
    "dialectal": "dialectale",
    "dialectale": "dialectal",
    "dialecticien": "dialecticienne",
    "dialecticienne": "dialecticien",
    "dialogué": "dialoguée",
    "dialoguée": "dialogué",
    "dialysé": "dialysée",
    "dialysée": "dialysé",
    "diamanté": "diamantée",
    "diamantée": "diamanté",
    "diamantin": "diamantine",
    "diamantine": "diamantin",
    "diapré": "diaprée",
    "diaprée": "diapré",
    "dictateur": "dictatrice",
    "dictatrice": "dictateur",
    "dictatorial": "dictatoriale",
    "dictatoriale": "dictatorial",
    "dicté": "dictée",
    "dictée": "dicté",
    "diffamateur": "diffamatrice",
    "diffamatrice": "diffamateur",
    "diffamé": "diffamée",
    "diffamée": "diffamé",
    "diffusé": "diffusée",
    "diffusée": "diffusé",
    "différencié": "différenciée",
    "différenciée": "différencié",
    "différent": "différente",
    "différente": "différent",
    "différentiel": "différentielle",
    "différentielle": "différentiel",
    "différé": "différée",
    "différée": "différé",
    "digestif": "digestive",
    "digestive": "digestif",
    "digital": "digitale",
    "digitale": "digital",
    "digéré": "digérée",
    "digérée": "digéré",
    "dilapidé": "dilapidée",
    "dilapidée": "dilapidé",
    "dilaté": "dilatée",
    "dilatée": "dilaté",
    "diligent": "diligente",
    "diligente": "diligent",
    "dilué": "diluée",
    "diluée": "dilué",
    "diluvien": "diluvienne",
    "diluvienne": "diluvien",
    "dimensionnel": "dimensionnelle",
    "dimensionnelle": "dimensionnel",
    "diminué": "diminuée",
    "diminuée": "diminué",
    "diplômé": "diplômée",
    "diplômée": "diplômé",
    "dit": "dite",
    "dite": "dit",
    "direct": "directe",
    "directe": "direct",
    "directeur": "directrice",
    "directrice": "directeur",
    "directif": "directive",
    "directive": "directif",
    "directionnel": "directionnelle",
    "directionnelle": "directionnel",
    "directorial": "directoriale",
    "directoriale": "directorial",
    "dirigeant": "dirigeante",
    "dirigeante": "dirigeant",
    "dirigé": "dirigée",
    "dirigée": "dirigé",
    "discal": "discale",
    "discale": "discal",
    "discipliné": "disciplinée",
    "disciplinée": "discipliné",
    "discontinu": "discontinue",
    "discontinue": "discontinu",
    "discordant": "discordante",
    "discordante": "discordant",
    "discret": "discrète",
    "discrète": "discret",
    "discrédité": "discréditée",
    "discréditée": "discrédité",
    "discuté": "discutée",
    "discutée": "discuté",
    "disert": "diserte",
    "diserte": "disert",
    "diseur": "diseuse",
    "diseuse": "diseur",
    "disgracié": "disgraciée",
    "disgraciée": "disgracié",
    "disjoint": "disjointe",
    "disjointe": "disjoint",
    "disjoncté": "disjonctée",
    "disjonctée": "disjoncté",
    "disloqué": "disloquée",
    "disloquée": "disloqué",
    "disparaissant": "disparaissante",
    "disparaissante": "disparaissant",
    "disparu": "disparue",
    "disparue": "disparu",
    "dispensateur": "dispensatrice",
    "dispensatrice": "dispensateur",
    "dispensé": "dispensée",
    "dispensée": "dispensé",
    "dispersé": "dispersée",
    "dispersée": "dispersé",
    "disposé": "disposée",
    "disposée": "disposé",
    "disproportionné": "disproportionnée",
    "disproportionnée": "disproportionné",
    "disputé": "disputée",
    "disputée": "disputé",
    "disqualifié": "disqualifiée",
    "disqualifiée": "disqualifié",
    "dissident": "dissidente",
    "dissidente": "dissident",
    "dissimulateur": "dissimulatrice",
    "dissimulatrice": "dissimulateur",
    "dissimulé": "dissimulée",
    "dissimulée": "dissimulé",
    "dissipé": "dissipée",
    "dissipée": "dissipé",
    "dissociatif": "dissociative",
    "dissociative": "dissociatif",
    "dissocié": "dissociée",
    "dissociée": "dissocié",
    "dissolu": "dissolue",
    "dissolue": "dissolu",
    "dissolvant": "dissolvante",
    "dissolvante": "dissolvant",
    "dissuadé": "dissuadée",
    "dissuadée": "dissuadé",
    "dissuasif": "dissuasive",
    "dissuasive": "dissuasif",
    "disséminé": "disséminée",
    "disséminée": "disséminé",
    "disséqué": "disséquée",
    "disséquée": "disséqué",
    "distal": "distale",
    "distale": "distal",
    "distancé": "distancée",
    "distancée": "distancé",
    "distant": "distante",
    "distante": "distant",
    "distendu": "distendue",
    "distendue": "distendu",
    "distillé": "distillée",
    "distillée": "distillé",
    "distinct": "distincte",
    "distincte": "distinct",
    "distinctif": "distinctive",
    "distinctive": "distinctif",
    "distingué": "distinguée",
    "distinguée": "distingué",
    "distordu": "distordue",
    "distordue": "distordu",
    "distrait": "distraite",
    "distraite": "distrait",
    "distrayant": "distrayante",
    "distrayante": "distrayant",
    "distribué": "distribuée",
    "distribuée": "distribué",
    "distributeur": "distributrice",
    "distributrice": "distributeur",
    "divergent": "divergente",
    "divergente": "divergent",
    "diversifié": "diversifiée",
    "diversifiée": "diversifié",
    "diverti": "divertie",
    "divertie": "diverti",
    "divertissant": "divertissante",
    "divertissante": "divertissant",
    "divin": "divine",
    "divine": "divin",
    "divinisé": "divinisée",
    "divinisée": "divinisé",
    "divisé": "divisée",
    "divisée": "divisé",
    "diviseur": "diviseuse",
    "diviseuse": "diviseur",
    "divorcé": "divorcée",
    "divorcée": "divorcé",
    "divulgué": "divulguée",
    "divulguée": "divulgué",
    "diététicien": "diététicienne",
    "diététicienne": "diététicien",
    "docteur": "doctoresse",
    "doctoresse": "docteur",
    "doctoral": "doctorale",
    "doctorale": "doctoral",
    "doctrinal": "doctrinale",
    "doctrinale": "doctrinal",
    "documenté": "documentée",
    "documentée": "documenté",
    "dodelinant": "dodelinante",
    "dodelinante": "dodelinant",
    "dodu": "dodue",
    "dodue": "dodu",
    "dolent": "dolente",
    "dolente": "dolent",
    "domanial": "domaniale",
    "domaniale": "domanial",
    "domestiqué": "domestiquée",
    "domestiquée": "domestiqué",
    "domicilié": "domiciliée",
    "domiciliée": "domicilié",
    "dominant": "dominante",
    "dominante": "dominant",
    "dominateur": "dominatrice",
    "dominatrice": "dominateur",
    "dominé": "dominée",
    "dominée": "dominé",
    "dominicain": "dominicaine",
    "dominicaine": "dominicain",
    "dominical": "dominicale",
    "dominicale": "dominical",
    "dompté": "domptée",
    "domptée": "dompté",
    "dompteur": "dompteuse",
    "dompteuse": "dompteur",
    "donateur": "donatrice",
    "donatrice": "donateur",
    "donné": "donnée",
    "donnée": "donné",
    "donneur": "donneuse",
    "donneuse": "donneur",
    "dopé": "dopée",
    "dopée": "dopé",
    "doré": "dorée",
    "dorée": "doré",
    "dorloté": "dorlotée",
    "dorlotée": "dorloté",
    "dormant": "dormante",
    "dormante": "dormant",
    "dormeur": "dormeuse",
    "dormeuse": "dormeur",
    "dorsal": "dorsale",
    "dorsale": "dorsal",
    "dosé": "dosée",
    "dosée": "dosé",
    "doté": "dotée",
    "dotée": "doté",
    "douanier": "douanière",
    "douanière": "douanier",
    "doublé": "doublée",
    "doublée": "doublé",
    "doubleur": "doubleuse",
    "doubleuse": "doubleur",
    "doucet": "doucette",
    "doucette": "doucet",
    "douché": "douchée",
    "douchée": "douché",
    "doué": "douée",
    "douée": "doué",
    "douillé": "douillée",
    "douillée": "douillé",
    "douillet": "douillette",
    "douillette": "douillet",
    "douté": "doutée",
    "doutée": "douté",
    "doyen": "doyenne",
    "doyenne": "doyen",
    "draconien": "draconienne",
    "draconienne": "draconien",
    "dragon": "dragonne",
    "dragonne": "dragon",
    "dragué": "draguée",
    "draguée": "dragué",
    "dragueur": "dragueuse",
    "dragueuse": "dragueur",
    "dramatisé": "dramatisée",
    "dramatisée": "dramatisé",
    "drapé": "drapée",
    "drapée": "drapé",
    "dressé": "dressée",
    "dressée": "dressé",
    "dresseur": "dresseuse",
    "dresseuse": "dresseur",
    "drivé": "drivée",
    "drivée": "drivé",
    "drogué": "droguée",
    "droguée": "drogué",
    "droit": "droite",
    "droite": "droit",
    "droitier": "droitière",
    "droitière": "droitier",
    "drossé": "drossée",
    "drossée": "drossé",
    "dru": "drue",
    "drue": "dru",
    "drôlet": "drôlette",
    "drôlette": "drôlet",
    "dubitatif": "dubitative",
    "dubitative": "dubitatif",
    "duc": "duchesse",
    "duchesse": "duc",
    "ducal": "ducale",
    "ducale": "ducal",
    "duodénal": "duodénale",
    "duodénale": "duodénal",
    "dupé": "dupée",
    "dupée": "dupé",
    "dur": "dure",
    "dure": "dur",
    "durci": "durcie",
    "durcie": "durci",
    "duré": "durée",
    "durée": "duré",
    "duveteuse": "duveteux",
    "duveteux": "duveteuse",
    "duveté": "duvetée",
    "duvetée": "duveté",
    "dynamisé": "dynamisée",
    "dynamisée": "dynamisé",
    "dynamité": "dynamitée",
    "dynamitée": "dynamité",
    "déambulant": "déambulante",
    "déambulante": "déambulant",
    "déballé": "déballée",
    "déballée": "déballé",
    "débaptisé": "débaptisée",
    "débaptisée": "débaptisé",
    "débarbouillé": "débarbouillée",
    "débarbouillée": "débarbouillé",
    "débarqué": "débarquée",
    "débarquée": "débarqué",
    "débarrassé": "débarrassée",
    "débarrassée": "débarrassé",
    "débattu": "débattue",
    "débattue": "débattu",
    "débauché": "débauchée",
    "débauchée": "débauché",
    "débecté": "débectée",
    "débectée": "débecté",
    "débilitant": "débilitante",
    "débilitante": "débilitant",
    "débilité": "débilitée",
    "débilitée": "débilité",
    "débité": "débitée",
    "débitée": "débité",
    "débiteur": "débitrice",
    "débitrice": "débiteur",
    "déblayé": "déblayée",
    "déblayée": "déblayé",
    "débloqué": "débloquée",
    "débloquée": "débloqué",
    "déboisé": "déboisée",
    "déboisée": "déboisé",
    "débordant": "débordante",
    "débordante": "débordant",
    "débordé": "débordée",
    "débordée": "débordé",
    "débouché": "débouchée",
    "débouchée": "débouché",
    "déboulonné": "déboulonnée",
    "déboulonnée": "déboulonné",
    "débourré": "débourrée",
    "débourrée": "débourré",
    "déboussolé": "déboussolée",
    "déboussolée": "déboussolé",
    "débouté": "déboutée",
    "déboutée": "débouté",
    "déboutonné": "déboutonnée",
    "déboutonnée": "déboutonné",
    "déboîté": "déboîtée",
    "déboîtée": "déboîté",
    "débraillé": "débraillée",
    "débraillée": "débraillé",
    "débranché": "débranchée",
    "débranchée": "débranché",
    "débrayé": "débrayée",
    "débrayée": "débrayé",
    "débridé": "débridée",
    "débridée": "débridé",
    "débriefé": "débriefée",
    "débriefée": "débriefé",
    "débrouillard": "débrouillarde",
    "débrouillarde": "débrouillard",
    "débrouillé": "débrouillée",
    "débrouillée": "débrouillé",
    "débusqué": "débusquée",
    "débusquée": "débusqué",
    "débutant": "débutante",
    "débutante": "débutant",
    "débuté": "débutée",
    "débutée": "débuté",
    "décacheté": "décachetée",
    "décachetée": "décacheté",
    "décadent": "décadente",
    "décadente": "décadent",
    "décaféiné": "décaféinée",
    "décaféinée": "décaféiné",
    "décalcifié": "décalcifiée",
    "décalcifiée": "décalcifié",
    "décalé": "décalée",
    "décalée": "décalé",
    "décalotté": "décalottée",
    "décalottée": "décalotté",
    "décalqué": "décalquée",
    "décalquée": "décalqué",
    "décanté": "décantée",
    "décantée": "décanté",
    "décapant": "décapante",
    "décapante": "décapant",
    "décapé": "décapée",
    "décapée": "décapé",
    "décapité": "décapitée",
    "décapitée": "décapité",
    "décapoté": "décapotée",
    "décapotée": "décapoté",
    "décapsulé": "décapsulée",
    "décapsulée": "décapsulé",
    "décarcassé": "décarcassée",
    "décarcassée": "décarcassé",
    "décati": "décatie",
    "décatie": "décati",
    "décelé": "décelée",
    "décelée": "décelé",
    "décennal": "décennale",
    "décennale": "décennal",
    "décent": "décente",
    "décente": "décent",
    "décentralisé": "décentralisée",
    "décentralisée": "décentralisé",
    "décerné": "décernée",
    "décernée": "décerné",
    "décevant": "décevante",
    "décevante": "décevant",
    "déçu": "déçue",
    "déçue": "déçu",
    "déchargé": "déchargée",
    "déchargée": "déchargé",
    "décharné": "décharnée",
    "décharnée": "décharné",
    "déchaussé": "déchaussée",
    "déchaussée": "déchaussé",
    "déchaîné": "déchaînée",
    "déchaînée": "déchaîné",
    "déchiffré": "déchiffrée",
    "déchiffrée": "déchiffré",
    "déchiqueté": "déchiquetée",
    "déchiquetée": "déchiqueté",
    "déchiqueteur": "déchiqueteuse",
    "déchiqueteuse": "déchiqueteur",
    "déchirant": "déchirante",
    "déchirante": "déchirant",
    "déchiré": "déchirée",
    "déchirée": "déchiré",
    "déchu": "déchue",
    "déchue": "déchu",
    "décidé": "décidée",
    "décidée": "décidé",
    "décimal": "décimale",
    "décimale": "décimal",
    "décimé": "décimée",
    "décimée": "décimé",
    "décisif": "décisive",
    "décisive": "décisif",
    "déclamé": "déclamée",
    "déclamée": "déclamé",
    "déclaré": "déclarée",
    "déclarée": "déclaré",
    "déclassé": "déclassée",
    "déclassée": "déclassé",
    "déclenché": "déclenchée",
    "déclenchée": "déclenché",
    "déclinant": "déclinante",
    "déclinante": "déclinant",
    "décliné": "déclinée",
    "déclinée": "décliné",
    "décloué": "déclouée",
    "déclouée": "décloué",
    "décoché": "décochée",
    "décochée": "décoché",
    "décodé": "décodée",
    "décodée": "décodé",
    "décodeur": "décodeuse",
    "décodeuse": "décodeur",
    "décoiffé": "décoiffée",
    "décoiffée": "décoiffé",
    "décollé": "décollée",
    "décollée": "décollé",
    "décolleté": "décolletée",
    "décolletée": "décolleté",
    "décoloré": "décolorée",
    "décolorée": "décoloré",
    "décommandé": "décommandée",
    "décommandée": "décommandé",
    "décomposé": "décomposée",
    "décomposée": "décomposé",
    "déconcentré": "déconcentrée",
    "déconcentrée": "déconcentré",
    "déconcertant": "déconcertante",
    "déconcertante": "déconcertant",
    "déconcerté": "déconcertée",
    "déconcertée": "déconcerté",
    "déconfit": "déconfite",
    "déconfite": "déconfit",
    "décongelé": "décongelée",
    "décongelée": "décongelé",
    "déconnecté": "déconnectée",
    "déconnectée": "déconnecté",
    "déconneur": "déconneuse",
    "déconneuse": "déconneur",
    "déconseillé": "déconseillée",
    "déconseillée": "déconseillé",
    "déconsidéré": "déconsidérée",
    "déconsidérée": "déconsidéré",
    "décontenancé": "décontenancée",
    "décontenancée": "décontenancé",
    "décontracté": "décontractée",
    "décontractée": "décontracté",
    "décorateur": "décoratrice",
    "décoratrice": "décorateur",
    "décoratif": "décorative",
    "décorative": "décoratif",
    "décoré": "décorée",
    "décorée": "décoré",
    "décortiqué": "décortiquée",
    "décortiquée": "décortiqué",
    "décousu": "décousue",
    "décousue": "décousu",
    "découpé": "découpée",
    "découpée": "découpé",
    "découpeur": "découpeuse",
    "découpeuse": "découpeur",
    "découplé": "découplée",
    "découplée": "découplé",
    "décourageant": "décourageante",
    "décourageante": "décourageant",
    "découragé": "découragée",
    "découragée": "découragé",
    "découvert": "découverte",
    "découverte": "découvert",
    "découvreur": "découvreuse",
    "découvreuse": "découvreur",
    "décrié": "décriée",
    "décriée": "décrié",
    "décrit": "décrite",
    "décrite": "décrit",
    "décroché": "décrochée",
    "décrochée": "décroché",
    "décroissant": "décroissante",
    "décroissante": "décroissant",
    "décrépi": "décrépie",
    "décrépie": "décrépi",
    "décrépit": "décrépite",
    "décrépite": "décrépit",
    "décrété": "décrétée",
    "décrétée": "décrété",
    "déculotté": "déculottée",
    "déculottée": "déculotté",
    "décuplé": "décuplée",
    "décuplée": "décuplé",
    "décédé": "décédée",
    "décédée": "décédé",
    "décérébré": "décérébrée",
    "décérébrée": "décérébré",
    "dédaigné": "dédaignée",
    "dédaignée": "dédaigné",
    "dédicacé": "dédicacée",
    "dédicacée": "dédicacé",
    "dédié": "dédiée",
    "dédiée": "dédié",
    "dédit": "dédite",
    "dédite": "dédit",
    "dédommagé": "dédommagée",
    "dédommagée": "dédommagé",
    "dédoré": "dédorée",
    "dédorée": "dédoré",
    "dédouané": "dédouanée",
    "dédouanée": "dédouané",
    "dédoublé": "dédoublée",
    "dédoublée": "dédoublé",
    "déductif": "déductive",
    "déductive": "déductif",
    "déduit": "déduite",
    "déduite": "déduit",
    "défaillant": "défaillante",
    "défaillante": "défaillant",
    "défait": "défaite",
    "défaite": "défait",
    "défavorisé": "défavorisée",
    "défavorisée": "défavorisé",
    "défenderesse": "défendeur",
    "défendeur": "défenderesse",
    "défendu": "défendue",
    "défendue": "défendu",
    "défenestré": "défenestrée",
    "défenestrée": "défenestré",
    "défensif": "défensive",
    "défensive": "défensif",
    "déferlant": "déferlante",
    "déferlante": "déferlant",
    "défiant": "défiante",
    "défiante": "défiant",
    "déficient": "déficiente",
    "déficiente": "déficient",
    "défié": "défiée",
    "défiée": "défié",
    "défiguré": "défigurée",
    "défigurée": "défiguré",
    "défilé": "défilée",
    "défilée": "défilé",
    "défini": "définie",
    "définie": "défini",
    "définitif": "définitive",
    "définitive": "définitif",
    "défloré": "déflorée",
    "déflorée": "défloré",
    "défoncé": "défoncée",
    "défoncée": "défoncé",
    "déformant": "déformante",
    "déformante": "déformant",
    "déformé": "déformée",
    "déformée": "déformé",
    "défraîchi": "défraîchie",
    "défraîchie": "défraîchi",
    "défriché": "défrichée",
    "défrichée": "défriché",
    "défroqué": "défroquée",
    "défroquée": "défroqué",
    "défunt": "défunte",
    "défunte": "défunt",
    "déférent": "déférente",
    "déférente": "déférent",
    "dégagé": "dégagée",
    "dégagée": "dégagé",
    "déganté": "dégantée",
    "dégantée": "déganté",
    "dégarni": "dégarnie",
    "dégarnie": "dégarni",
    "dégauchi": "dégauchie",
    "dégauchie": "dégauchi",
    "dégelé": "dégelée",
    "dégelée": "dégelé",
    "dégingandé": "dégingandée",
    "dégingandée": "dégingandé",
    "déglingué": "déglinguée",
    "déglinguée": "déglingué",
    "dégluti": "déglutie",
    "déglutie": "dégluti",
    "dégommé": "dégommée",
    "dégommée": "dégommé",
    "dégonflé": "dégonflée",
    "dégonflée": "dégonflé",
    "dégorgé": "dégorgée",
    "dégorgée": "dégorgé",
    "dégoté": "dégotée",
    "dégotée": "dégoté",
    "dégoulinant": "dégoulinante",
    "dégoulinante": "dégoulinant",
    "dégoupillé": "dégoupillée",
    "dégoupillée": "dégoupillé",
    "dégourdi": "dégourdie",
    "dégourdie": "dégourdi",
    "dégouttant": "dégouttante",
    "dégouttante": "dégouttant",
    "dégoûtant": "dégoûtante",
    "dégoûtante": "dégoûtant",
    "dégoûté": "dégoûtée",
    "dégoûtée": "dégoûté",
    "dégradant": "dégradante",
    "dégradante": "dégradant",
    "dégradé": "dégradée",
    "dégradée": "dégradé",
    "dégrafé": "dégrafée",
    "dégrafée": "dégrafé",
    "dégringolé": "dégringolée",
    "dégringolée": "dégringolé",
    "dégrisé": "dégrisée",
    "dégrisée": "dégrisé",
    "dégrossi": "dégrossie",
    "dégrossie": "dégrossi",
    "déguenillé": "déguenillée",
    "déguenillée": "déguenillé",
    "déguisé": "déguisée",
    "déguisée": "déguisé",
    "dégustateur": "dégustatrice",
    "dégustatrice": "dégustateur",
    "dégusté": "dégustée",
    "dégustée": "dégusté",
    "dégénératif": "dégénérative",
    "dégénérative": "dégénératif",
    "dégénéré": "dégénérée",
    "dégénérée": "dégénéré",
    "déhanché": "déhanchée",
    "déhanchée": "déhanché",
    "déhotté": "déhottée",
    "déhottée": "déhotté",
    "déjanté": "déjantée",
    "déjantée": "déjanté",
    "déjeté": "déjetée",
    "déjetée": "déjeté",
    "déjoué": "déjouée",
    "déjouée": "déjoué",
    "délabré": "délabrée",
    "délabrée": "délabré",
    "délaissé": "délaissée",
    "délaissée": "délaissé",
    "délassant": "délassante",
    "délassante": "délassant",
    "délateur": "délatrice",
    "délatrice": "délateur",
    "délavé": "délavée",
    "délavée": "délavé",
    "délayé": "délayée",
    "délayée": "délayé",
    "délesté": "délestée",
    "délestée": "délesté",
    "délibéré": "délibérée",
    "délibérée": "délibéré",
    "délicat": "délicate",
    "délicate": "délicat",
    "délié": "déliée",
    "déliée": "délié",
    "délimité": "délimitée",
    "délimitée": "délimité",
    "délinquant": "délinquante",
    "délinquante": "délinquant",
    "déliquescent": "déliquescente",
    "déliquescente": "déliquescent",
    "délirant": "délirante",
    "délirante": "délirant",
    "délité": "délitée",
    "délitée": "délité",
    "délivré": "délivrée",
    "délivrée": "délivré",
    "délogé": "délogée",
    "délogée": "délogé",
    "déloyal": "déloyale",
    "déloyale": "déloyal",
    "déluré": "délurée",
    "délurée": "déluré",
    "délégué": "déléguée",
    "déléguée": "délégué",
    "démagnétisé": "démagnétisée",
    "démagnétisée": "démagnétisé",
    "démantelé": "démantelée",
    "démantelée": "démantelé",
    "démantibulé": "démantibulée",
    "démantibulée": "démantibulé",
    "démaquillant": "démaquillante",
    "démaquillante": "démaquillant",
    "démaquillé": "démaquillée",
    "démaquillée": "démaquillé",
    "démarcheur": "démarcheuse",
    "démarcheuse": "démarcheur",
    "démarqué": "démarquée",
    "démarquée": "démarqué",
    "démarré": "démarrée",
    "démarrée": "démarré",
    "démasqué": "démasquée",
    "démasquée": "démasqué",
    "démembré": "démembrée",
    "démembrée": "démembré",
    "démené": "démenée",
    "démenée": "démené",
    "dément": "démente",
    "démente": "dément",
    "démenti": "démentie",
    "démentie": "démenti",
    "démentiel": "démentielle",
    "démentielle": "démentiel",
    "démerdé": "démerdée",
    "démerdée": "démerdé",
    "démesuré": "démesurée",
    "démesurée": "démesuré",
    "démeublé": "démeublée",
    "démeublée": "démeublé",
    "déminé": "déminée",
    "déminée": "déminé",
    "démodé": "démodée",
    "démodée": "démodé",
    "démoli": "démolie",
    "démolie": "démoli",
    "démolisseur": "démolisseuse",
    "démolisseuse": "démolisseur",
    "démon": "démone",
    "démone": "démon",
    "démonstrateur": "démonstratrice",
    "démonstratrice": "démonstrateur",
    "démonstratif": "démonstrative",
    "démonstrative": "démonstratif",
    "démonté": "démontée",
    "démontée": "démonté",
    "démontré": "démontrée",
    "démontrée": "démontré",
    "démoralisant": "démoralisante",
    "démoralisante": "démoralisant",
    "démoralisé": "démoralisée",
    "démoralisée": "démoralisé",
    "démultiplié": "démultipliée",
    "démultipliée": "démultiplié",
    "démuni": "démunie",
    "démunie": "démuni",
    "démâté": "démâtée",
    "démâtée": "démâté",
    "déménagé": "déménagée",
    "déménagée": "déménagé",
    "déménageur": "déménageuse",
    "déménageuse": "déménageur",
    "dénaturé": "dénaturée",
    "dénaturée": "dénaturé",
    "déniché": "dénichée",
    "dénichée": "déniché",
    "dénicheur": "dénicheuse",
    "dénicheuse": "dénicheur",
    "dénigrant": "dénigrante",
    "dénigrante": "dénigrant",
    "dénommé": "dénommée",
    "dénommée": "dénommé",
    "dénoncé": "dénoncée",
    "dénoncée": "dénoncé",
    "dénonciateur": "dénonciatrice",
    "dénonciatrice": "dénonciateur",
    "dénoué": "dénouée",
    "dénouée": "dénoué",
    "dénudé": "dénudée",
    "dénudée": "dénudé",
    "dénué": "dénuée",
    "dénuée": "dénué",
    "dépanné": "dépannée",
    "dépannée": "dépanné",
    "dépanneur": "dépanneuse",
    "dépanneuse": "dépanneur",
    "dépareillé": "dépareillée",
    "dépareillée": "dépareillé",
    "déparié": "dépariée",
    "dépariée": "déparié",
    "départemental": "départementale",
    "départementale": "départemental",
    "départi": "départie",
    "départie": "départi",
    "dépassé": "dépassée",
    "dépassée": "dépassé",
    "dépaysant": "dépaysante",
    "dépaysante": "dépaysant",
    "dépaysé": "dépaysée",
    "dépaysée": "dépaysé",
    "dépecé": "dépecée",
    "dépecée": "dépecé",
    "dépeigné": "dépeignée",
    "dépeignée": "dépeigné",
    "dépeint": "dépeinte",
    "dépeinte": "dépeint",
    "dépenaillé": "dépenaillée",
    "dépenaillée": "dépenaillé",
    "dépendant": "dépendante",
    "dépendante": "dépendant",
    "dépensé": "dépensée",
    "dépensée": "dépensé",
    "dépensier": "dépensière",
    "dépensière": "dépensier",
    "dépeuplé": "dépeuplée",
    "dépeuplée": "dépeuplé",
    "déphasé": "déphasée",
    "déphasée": "déphasé",
    "dépité": "dépitée",
    "dépitée": "dépité",
    "déplacé": "déplacée",
    "déplacée": "déplacé",
    "déplaisant": "déplaisante",
    "déplaisante": "déplaisant",
    "dépliant": "dépliante",
    "dépliante": "dépliant",
    "déplié": "dépliée",
    "dépliée": "déplié",
    "déploré": "déplorée",
    "déplorée": "déploré",
    "déployé": "déployée",
    "déployée": "déployé",
    "déplumé": "déplumée",
    "déplumée": "déplumé",
    "dépoilé": "dépoilée",
    "dépoilée": "dépoilé",
    "dépoitraillé": "dépoitraillée",
    "dépoitraillée": "dépoitraillé",
    "dépoli": "dépolie",
    "dépolie": "dépoli",
    "déporté": "déportée",
    "déportée": "déporté",
    "déposé": "déposée",
    "déposée": "déposé",
    "dépossédé": "dépossédée",
    "dépossédée": "dépossédé",
    "dépouillé": "dépouillée",
    "dépouillée": "dépouillé",
    "dépourvu": "dépourvue",
    "dépourvue": "dépourvu",
    "dépravé": "dépravée",
    "dépravée": "dépravé",
    "dépressif": "dépressive",
    "dépressive": "dépressif",
    "déprimant": "déprimante",
    "déprimante": "déprimant",
    "déprimé": "déprimée",
    "déprimée": "déprimé",
    "déprogrammé": "déprogrammée",
    "déprogrammée": "déprogrammé",
    "déprécié": "dépréciée",
    "dépréciée": "déprécié",
    "dépucelé": "dépucelée",
    "dépucelée": "dépucelé",
    "député": "députée",
    "députée": "député",
    "dépêché": "dépêchée",
    "dépêchée": "dépêché",
    "dépêtré": "dépêtrée",
    "dépêtrée": "dépêtré",
    "déraciné": "déracinée",
    "déracinée": "déraciné",
    "dérangeant": "dérangeante",
    "dérangeante": "dérangeant",
    "dérangé": "dérangée",
    "dérangée": "dérangé",
    "dératisé": "dératisée",
    "dératisée": "dératisé",
    "dérivant": "dérivante",
    "dérivante": "dérivant",
    "dérivé": "dérivée",
    "dérivée": "dérivé",
    "dérobé": "dérobée",
    "dérobée": "dérobé",
    "dérouillé": "dérouillée",
    "dérouillée": "dérouillé",
    "déroulé": "déroulée",
    "déroulée": "déroulé",
    "dérouleur": "dérouleuse",
    "dérouleuse": "dérouleur",
    "déroutant": "déroutante",
    "déroutante": "déroutant",
    "dérouté": "déroutée",
    "déroutée": "dérouté",
    "déréglé": "déréglée",
    "déréglée": "déréglé",
    "désabusé": "désabusée",
    "désabusée": "désabusé",
    "désaccordé": "désaccordée",
    "désaccordée": "désaccordé",
    "désacralisé": "désacralisée",
    "désacralisée": "désacralisé",
    "désactivé": "désactivée",
    "désactivée": "désactivé",
    "désaffecté": "désaffectée",
    "désaffectée": "désaffecté",
    "désagrégé": "désagrégée",
    "désagrégée": "désagrégé",
    "désamorcé": "désamorcée",
    "désamorcée": "désamorcé",
    "désappointé": "désappointée",
    "désappointée": "désappointé",
    "désapprobateur": "désapprobatrice",
    "désapprobatrice": "désapprobateur",
    "désapprouvé": "désapprouvée",
    "désapprouvée": "désapprouvé",
    "désarmant": "désarmante",
    "désarmante": "désarmant",
    "désarmé": "désarmée",
    "désarmée": "désarmé",
    "désarticulé": "désarticulée",
    "désarticulée": "désarticulé",
    "désarçonné": "désarçonnée",
    "désarçonnée": "désarçonné",
    "désavoué": "désavouée",
    "désavouée": "désavoué",
    "désaxé": "désaxée",
    "désaxée": "désaxé",
    "désemparé": "désemparée",
    "désemparée": "désemparé",
    "désenchanté": "désenchantée",
    "désenchantée": "désenchanté",
    "désentravé": "désentravée",
    "désentravée": "désentravé",
    "désert": "déserte",
    "déserte": "désert",
    "déserté": "désertée",
    "désertée": "déserté",
    "désespérant": "désespérante",
    "désespérante": "désespérant",
    "désespéré": "désespérée",
    "désespérée": "désespéré",
    "déshabillé": "déshabillée",
    "déshabillée": "déshabillé",
    "déshabité": "déshabitée",
    "déshabitée": "déshabité",
    "désherbé": "désherbée",
    "désherbée": "désherbé",
    "déshonorant": "déshonorante",
    "déshonorante": "déshonorant",
    "déshonoré": "déshonorée",
    "déshonorée": "déshonoré",
    "déshumanisé": "déshumanisée",
    "déshumanisée": "déshumanisé",
    "déshydraté": "déshydratée",
    "déshydratée": "déshydraté",
    "déshérité": "déshéritée",
    "déshéritée": "déshérité",
    "désigné": "désignée",
    "désignée": "désigné",
    "désincarné": "désincarnée",
    "désincarnée": "désincarné",
    "désinfectant": "désinfectante",
    "désinfectante": "désinfectant",
    "désinfecté": "désinfectée",
    "désinfectée": "désinfecté",
    "désintoxiqué": "désintoxiquée",
    "désintoxiquée": "désintoxiqué",
    "désintégré": "désintégrée",
    "désintégrée": "désintégré",
    "désintéressé": "désintéressée",
    "désintéressée": "désintéressé",
    "désirant": "désirante",
    "désirante": "désirant",
    "désiré": "désirée",
    "désirée": "désiré",
    "désisté": "désistée",
    "désistée": "désisté",
    "désobligeant": "désobligeante",
    "désobligeante": "désobligeant",
    "désobéissant": "désobéissante",
    "désobéissante": "désobéissant",
    "désodorisant": "désodorisante",
    "désodorisante": "désodorisant",
    "désoeuvré": "désoeuvrée",
    "désoeuvrée": "désoeuvré",
    "désolant": "désolante",
    "désolante": "désolant",
    "désolé": "désolée",
    "désolée": "désolé",
    "désopilant": "désopilante",
    "désopilante": "désopilant",
    "désordonné": "désordonnée",
    "désordonnée": "désordonné",
    "désorganisé": "désorganisée",
    "désorganisée": "désorganisé",
    "désorienté": "désorientée",
    "désorientée": "désorienté",
    "désossé": "désossée",
    "désossée": "désossé",
    "déstabilisant": "déstabilisante",
    "déstabilisante": "déstabilisant",
    "déstabilisé": "déstabilisée",
    "déstabilisée": "déstabilisé",
    "désuet": "désuète",
    "désuète": "désuet",
    "désuni": "désunie",
    "désunie": "désuni",
    "déséquilibré": "déséquilibrée",
    "déséquilibrée": "déséquilibré",
    "détaché": "détachée",
    "détachée": "détaché",
    "détaillant": "détaillante",
    "détaillante": "détaillant",
    "détaillé": "détaillée",
    "détaillée": "détaillé",
    "détecté": "détectée",
    "détectée": "détecté",
    "déteint": "déteinte",
    "déteinte": "déteint",
    "dételé": "dételée",
    "dételée": "dételé",
    "détendu": "détendue",
    "détendue": "détendu",
    "détenu": "détenue",
    "détenue": "détenu",
    "détenteur": "détentrice",
    "détentrice": "détenteur",
    "déterminant": "déterminante",
    "déterminante": "déterminant",
    "déterminé": "déterminée",
    "déterminée": "déterminé",
    "déterré": "déterrée",
    "déterrée": "déterré",
    "déterreur": "déterreuse",
    "déterreuse": "déterreur",
    "détesté": "détestée",
    "détestée": "détesté",
    "détonant": "détonante",
    "détonante": "détonant",
    "détourné": "détournée",
    "détournée": "détourné",
    "détraqué": "détraquée",
    "détraquée": "détraqué",
    "détrempé": "détrempée",
    "détrempée": "détrempé",
    "détrompé": "détrompée",
    "détrompée": "détrompé",
    "détruit": "détruite",
    "détruite": "détruit",
    "détrôné": "détrônée",
    "détrônée": "détrôné",
    "détérioré": "détériorée",
    "détériorée": "détérioré",
    "dévalé": "dévalée",
    "dévalée": "dévalé",
    "dévalisé": "dévalisée",
    "dévalisée": "dévalisé",
    "dévalorisé": "dévalorisée",
    "dévalorisée": "dévalorisé",
    "dévalué": "dévaluée",
    "dévaluée": "dévalué",
    "dévastateur": "dévastatrice",
    "dévastatrice": "dévastateur",
    "dévasté": "dévastée",
    "dévastée": "dévasté",
    "développé": "développée",
    "développée": "développé",
    "dévergondé": "dévergondée",
    "dévergondée": "dévergondé",
    "déverrouillé": "déverrouillée",
    "déverrouillée": "déverrouillé",
    "déversé": "déversée",
    "déversée": "déversé",
    "déviant": "déviante",
    "déviante": "déviant",
    "dévidé": "dévidée",
    "dévidée": "dévidé",
    "dévié": "déviée",
    "déviée": "dévié",
    "dévisagé": "dévisagée",
    "dévisagée": "dévisagé",
    "dévissé": "dévissée",
    "dévissée": "dévissé",
    "dévoilé": "dévoilée",
    "dévoilée": "dévoilé",
    "dévolu": "dévolue",
    "dévolue": "dévolu",
    "dévorant": "dévorante",
    "dévorante": "dévorant",
    "dévorateur": "dévoratrice",
    "dévoratrice": "dévorateur",
    "dévoré": "dévorée",
    "dévorée": "dévoré",
    "dévoreur": "dévoreuse",
    "dévoreuse": "dévoreur",
    "dévot": "dévote",
    "dévote": "dévot",
    "dévoué": "dévouée",
    "dévouée": "dévoué",
    "dévoyé": "dévoyée",
    "dévoyée": "dévoyé",
    "dévêtu": "dévêtue",
    "dévêtue": "dévêtu",
    "dîneur": "dîneuse",
    "dîneuse": "dîneur",
    "ecchymosé": "ecchymosée",
    "ecchymosée": "ecchymosé",
    "effacé": "effacée",
    "effacée": "effacé",
    "effarant": "effarante",
    "effarante": "effarant",
    "effaré": "effarée",
    "effarée": "effaré",
    "effarouché": "effarouchée",
    "effarouchée": "effarouché",
    "effectif": "effective",
    "effective": "effectif",
    "effectué": "effectuée",
    "effectuée": "effectué",
    "effervescent": "effervescente",
    "effervescente": "effervescent",
    "effeuillé": "effeuillée",
    "effeuillée": "effeuillé",
    "efficient": "efficiente",
    "efficiente": "efficient",
    "effilé": "effilée",
    "effilée": "effilé",
    "effiloché": "effilochée",
    "effilochée": "effiloché",
    "efflanqué": "efflanquée",
    "efflanquée": "efflanqué",
    "effleuré": "effleurée",
    "effleurée": "effleuré",
    "effondré": "effondrée",
    "effondrée": "effondré",
    "efforcé": "efforcée",
    "efforcée": "efforcé",
    "effrangé": "effrangée",
    "effrangée": "effrangé",
    "effrayant": "effrayante",
    "effrayante": "effrayant",
    "effrayé": "effrayée",
    "effrayée": "effrayé",
    "effrité": "effritée",
    "effritée": "effrité",
    "effronté": "effrontée",
    "effrontée": "effronté",
    "effréné": "effrénée",
    "effrénée": "effréné",
    "efféminé": "efféminée",
    "efféminée": "efféminé",
    "emballé": "emballée",
    "emballée": "emballé",
    "emballeur": "emballeuse",
    "emballeuse": "emballeur",
    "embarqué": "embarquée",
    "embarquée": "embarqué",
    "embarrassant": "embarrassante",
    "embarrassante": "embarrassant",
    "embarrassé": "embarrassée",
    "embarrassée": "embarrassé",
    "embastillé": "embastillée",
    "embastillée": "embastillé",
    "embauché": "embauchée",
    "embauchée": "embauché",
    "embaumé": "embaumée",
    "embaumée": "embaumé",
    "embaumeur": "embaumeuse",
    "embaumeuse": "embaumeur",
    "embelli": "embellie",
    "embellie": "embelli",
    "emberlificoté": "emberlificotée",
    "emberlificotée": "emberlificoté",
    "emblavé": "emblavée",
    "emblavée": "emblavé",
    "embobiné": "embobinée",
    "embobinée": "embobiné",
    "embossé": "embossée",
    "embossée": "embossé",
    "embouché": "embouchée",
    "embouchée": "embouché",
    "embourbé": "embourbée",
    "embourbée": "embourbé",
    "embouteillé": "embouteillée",
    "embouteillée": "embouteillé",
    "embouti": "emboutie",
    "emboutie": "embouti",
    "emboutisseur": "emboutisseuse",
    "emboutisseuse": "emboutisseur",
    "emboîté": "emboîtée",
    "emboîtée": "emboîté",
    "embrasé": "embrasée",
    "embrasée": "embrasé",
    "embrassé": "embrassée",
    "embrassée": "embrassé",
    "embrasseur": "embrasseuse",
    "embrasseuse": "embrasseur",
    "embrayé": "embrayée",
    "embrayée": "embrayé",
    "embrigadé": "embrigadée",
    "embrigadée": "embrigadé",
    "embringué": "embringuée",
    "embringuée": "embringué",
    "embroché": "embrochée",
    "embrochée": "embroché",
    "embrouillé": "embrouillée",
    "embrouillée": "embrouillé",
    "embrumé": "embrumée",
    "embrumée": "embrumé",
    "embué": "embuée",
    "embuée": "embué",
    "embusqué": "embusquée",
    "embusquée": "embusqué",
    "embêtant": "embêtante",
    "embêtante": "embêtant",
    "embêté": "embêtée",
    "embêtée": "embêté",
    "emmagasiné": "emmagasinée",
    "emmagasinée": "emmagasiné",
    "emmailloté": "emmaillotée",
    "emmaillotée": "emmailloté",
    "emmanché": "emmanchée",
    "emmanchée": "emmanché",
    "emmené": "emmenée",
    "emmenée": "emmené",
    "emmerdant": "emmerdante",
    "emmerdante": "emmerdant",
    "emmerdé": "emmerdée",
    "emmerdée": "emmerdé",
    "emmerdeur": "emmerdeuse",
    "emmerdeuse": "emmerdeur",
    "emmitouflé": "emmitouflée",
    "emmitouflée": "emmitouflé",
    "emmuré": "emmurée",
    "emmurée": "emmuré",
    "emmêlé": "emmêlée",
    "emmêlée": "emmêlé",
    "empaillé": "empaillée",
    "empaillée": "empaillé",
    "empalé": "empalée",
    "empalée": "empalé",
    "empanaché": "empanachée",
    "empanachée": "empanaché",
    "empaqueté": "empaquetée",
    "empaquetée": "empaqueté",
    "emparé": "emparée",
    "emparée": "emparé",
    "empesé": "empesée",
    "empesée": "empesé",
    "empesté": "empestée",
    "empestée": "empesté",
    "empierré": "empierrée",
    "empierrée": "empierré",
    "empilé": "empilée",
    "empilée": "empilé",
    "empiré": "empirée",
    "empirée": "empiré",
    "emplafonné": "emplafonnée",
    "emplafonnée": "emplafonné",
    "empli": "emplie",
    "emplie": "empli",
    "employé": "employée",
    "employée": "employé",
    "emplumé": "emplumée",
    "emplumée": "emplumé",
    "empoché": "empochée",
    "empochée": "empoché",
    "empoigné": "empoignée",
    "empoignée": "empoigné",
    "empoisonnant": "empoisonnante",
    "empoisonnante": "empoisonnant",
    "empoisonné": "empoisonnée",
    "empoisonnée": "empoisonné",
    "empoisonneur": "empoisonneuse",
    "empoisonneuse": "empoisonneur",
    "emporté": "emportée",
    "emportée": "emporté",
    "empoté": "empotée",
    "empotée": "empoté",
    "empourpré": "empourprée",
    "empourprée": "empourpré",
    "empoussiéré": "empoussiérée",
    "empoussiérée": "empoussiéré",
    "empressé": "empressée",
    "empressée": "empressé",
    "emprisonné": "emprisonnée",
    "emprisonnée": "emprisonné",
    "emprunté": "empruntée",
    "empruntée": "emprunté",
    "empuanti": "empuantie",
    "empuantie": "empuanti",
    "empâté": "empâtée",
    "empâtée": "empâté",
    "empêché": "empêchée",
    "empêchée": "empêché",
    "empêcheur": "empêcheuse",
    "empêcheuse": "empêcheur",
    "empêtré": "empêtrée",
    "empêtrée": "empêtré",
    "enamouré": "enamourée",
    "enamourée": "enamouré",
    "encadré": "encadrée",
    "encadrée": "encadré",
    "encagé": "encagée",
    "encagée": "encagé",
    "encaissé": "encaissée",
    "encaissée": "encaissé",
    "encapuchonné": "encapuchonnée",
    "encapuchonnée": "encapuchonné",
    "encastré": "encastrée",
    "encastrée": "encastré",
    "encaustiqué": "encaustiquée",
    "encaustiquée": "encaustiqué",
    "encerclé": "encerclée",
    "encerclée": "encerclé",
    "enchanté": "enchantée",
    "enchantée": "enchanté",
    "enchanteresse": "enchanteur",
    "enchanteur": "enchanteresse",
    "enchaîné": "enchaînée",
    "enchaînée": "enchaîné",
    "enchevêtré": "enchevêtrée",
    "enchevêtrée": "enchevêtré",
    "enchristé": "enchristée",
    "enchristée": "enchristé",
    "enchâssé": "enchâssée",
    "enchâssée": "enchâssé",
    "enchérisseur": "enchérisseuse",
    "enchérisseuse": "enchérisseur",
    "enclavé": "enclavée",
    "enclavée": "enclavé",
    "enclenché": "enclenchée",
    "enclenchée": "enclenché",
    "enclin": "encline",
    "encline": "enclin",
    "encodé": "encodée",
    "encodée": "encodé",
    "encollé": "encollée",
    "encollée": "encollé",
    "encombrant": "encombrante",
    "encombrante": "encombrant",
    "encombré": "encombrée",
    "encombrée": "encombré",
    "encordé": "encordée",
    "encordée": "encordé",
    "encorné": "encornée",
    "encornée": "encorné",
    "encourageant": "encourageante",
    "encourageante": "encourageant",
    "encouragé": "encouragée",
    "encouragée": "encouragé",
    "encouru": "encourue",
    "encourue": "encouru",
    "encrassé": "encrassée",
    "encrassée": "encrassé",
    "encré": "encrée",
    "encrée": "encré",
    "encroûté": "encroûtée",
    "encroûtée": "encroûté",
    "enculé": "enculée",
    "enculée": "enculé",
    "endetté": "endettée",
    "endettée": "endetté",
    "endeuillé": "endeuillée",
    "endeuillée": "endeuillé",
    "endiablé": "endiablée",
    "endiablée": "endiablé",
    "endiamanté": "endiamantée",
    "endiamantée": "endiamanté",
    "endigué": "endiguée",
    "endiguée": "endigué",
    "endimanché": "endimanchée",
    "endimanchée": "endimanché",
    "endocrinien": "endocrinienne",
    "endocrinienne": "endocrinien",
    "endoctriné": "endoctrinée",
    "endoctrinée": "endoctriné",
    "endolori": "endolorie",
    "endolorie": "endolori",
    "endommagé": "endommagée",
    "endommagée": "endommagé",
    "endormant": "endormante",
    "endormante": "endormant",
    "endormi": "endormie",
    "endormie": "endormi",
    "endossé": "endossée",
    "endossée": "endossé",
    "enduit": "enduite",
    "enduite": "enduit",
    "endurant": "endurante",
    "endurante": "endurant",
    "endurci": "endurcie",
    "endurcie": "endurci",
    "enduré": "endurée",
    "endurée": "enduré",
    "enfanté": "enfantée",
    "enfantée": "enfanté",
    "enfantin": "enfantine",
    "enfantine": "enfantin",
    "enfariné": "enfarinée",
    "enfarinée": "enfariné",
    "enfermé": "enfermée",
    "enfermée": "enfermé",
    "enferré": "enferrée",
    "enferrée": "enferré",
    "enfilé": "enfilée",
    "enfilée": "enfilé",
    "enfiévré": "enfiévrée",
    "enfiévrée": "enfiévré",
    "enflammé": "enflammée",
    "enflammée": "enflammé",
    "enflé": "enflée",
    "enflée": "enflé",
    "enfoiré": "enfoirée",
    "enfoirée": "enfoiré",
    "enfoncé": "enfoncée",
    "enfoncée": "enfoncé",
    "enfoui": "enfouie",
    "enfouie": "enfoui",
    "enfouillé": "enfouillée",
    "enfouillée": "enfouillé",
    "enfouraillé": "enfouraillée",
    "enfouraillée": "enfouraillé",
    "enfourné": "enfournée",
    "enfournée": "enfourné",
    "enfreint": "enfreinte",
    "enfreinte": "enfreint",
    "enfui": "enfuie",
    "enfuie": "enfui",
    "enfumé": "enfumée",
    "enfumée": "enfumé",
    "engageant": "engageante",
    "engageante": "engageant",
    "engagé": "engagée",
    "engagée": "engagé",
    "engendré": "engendrée",
    "engendrée": "engendré",
    "englobé": "englobée",
    "englobée": "englobé",
    "englouti": "engloutie",
    "engloutie": "englouti",
    "englué": "engluée",
    "engluée": "englué",
    "engoncé": "engoncée",
    "engoncée": "engoncé",
    "engorgé": "engorgée",
    "engorgée": "engorgé",
    "engoué": "engouée",
    "engouée": "engoué",
    "engouffré": "engouffrée",
    "engouffrée": "engouffré",
    "engourdi": "engourdie",
    "engourdie": "engourdi",
    "engourdissant": "engourdissante",
    "engourdissante": "engourdissant",
    "engraissé": "engraissée",
    "engraissée": "engraissé",
    "engrangé": "engrangée",
    "engrangée": "engrangé",
    "engrossé": "engrossée",
    "engrossée": "engrossé",
    "engueulé": "engueulée",
    "engueulée": "engueulé",
    "enguirlandé": "enguirlandée",
    "enguirlandée": "enguirlandé",
    "enhardi": "enhardie",
    "enhardie": "enhardi",
    "enivrant": "enivrante",
    "enivrante": "enivrant",
    "enivré": "enivrée",
    "enivrée": "enivré",
    "enjambé": "enjambée",
    "enjambée": "enjambé",
    "enjolivé": "enjolivée",
    "enjolivée": "enjolivé",
    "enjoué": "enjouée",
    "enjouée": "enjoué",
    "enjôlé": "enjôlée",
    "enjôlée": "enjôlé",
    "enjôleur": "enjôleuse",
    "enjôleuse": "enjôleur",
    "enlacé": "enlacée",
    "enlacée": "enlacé",
    "enlaidi": "enlaidie",
    "enlaidie": "enlaidi",
    "enlevé": "enlevée",
    "enlevée": "enlevé",
    "enlisé": "enlisée",
    "enlisée": "enlisé",
    "enluminé": "enluminée",
    "enluminée": "enluminé",
    "enneigé": "enneigée",
    "enneigée": "enneigé",
    "ennemi": "ennemie",
    "ennemie": "ennemi",
    "ennobli": "ennoblie",
    "ennoblie": "ennobli",
    "ennuyant": "ennuyante",
    "ennuyante": "ennuyant",
    "ennuyé": "ennuyée",
    "ennuyée": "ennuyé",
    "enquiquinant": "enquiquinante",
    "enquiquinante": "enquiquinant",
    "enquiquineur": "enquiquineuse",
    "enquiquineuse": "enquiquineur",
    "enquêteur": "enquêteuse",
    "enquêteuse": "enquêteur",
    "enraciné": "enracinée",
    "enracinée": "enraciné",
    "enragé": "enragée",
    "enragée": "enragé",
    "enrayé": "enrayée",
    "enrayée": "enrayé",
    "enregistré": "enregistrée",
    "enregistrée": "enregistré",
    "enregistreur": "enregistreuse",
    "enregistreuse": "enregistreur",
    "enrhumé": "enrhumée",
    "enrhumée": "enrhumé",
    "enrichi": "enrichie",
    "enrichie": "enrichi",
    "enrichissant": "enrichissante",
    "enrichissante": "enrichissant",
    "enrobé": "enrobée",
    "enrobée": "enrobé",
    "enroué": "enrouée",
    "enrouée": "enroué",
    "enroulé": "enroulée",
    "enroulée": "enroulé",
    "enrubanné": "enrubannée",
    "enrubannée": "enrubanné",
    "enrôlé": "enrôlée",
    "enrôlée": "enrôlé",
    "ensablé": "ensablée",
    "ensablée": "ensablé",
    "ensanglanté": "ensanglantée",
    "ensanglantée": "ensanglanté",
    "ensauvagé": "ensauvagée",
    "ensauvagée": "ensauvagé",
    "enseignant": "enseignante",
    "enseignante": "enseignant",
    "enseigné": "enseignée",
    "enseignée": "enseigné",
    "ensemencé": "ensemencée",
    "ensemencée": "ensemencé",
    "enserré": "enserrée",
    "enserrée": "enserré",
    "enseveli": "ensevelie",
    "ensevelie": "enseveli",
    "ensoleillé": "ensoleillée",
    "ensoleillée": "ensoleillé",
    "ensommeillé": "ensommeillée",
    "ensommeillée": "ensommeillé",
    "ensorcelant": "ensorcelante",
    "ensorcelante": "ensorcelant",
    "ensorcelé": "ensorcelée",
    "ensorcelée": "ensorcelé",
    "ensorceleur": "ensorceleuse",
    "ensorceleuse": "ensorceleur",
    "ensuivi": "ensuivie",
    "ensuivie": "ensuivi",
    "ensuqué": "ensuquée",
    "ensuquée": "ensuqué",
    "entaché": "entachée",
    "entachée": "entaché",
    "entaillé": "entaillée",
    "entaillée": "entaillé",
    "entamé": "entamée",
    "entamée": "entamé",
    "entassé": "entassée",
    "entassée": "entassé",
    "entendu": "entendue",
    "entendue": "entendu",
    "enterré": "enterrée",
    "enterrée": "enterré",
    "enthousiasmant": "enthousiasmante",
    "enthousiasmante": "enthousiasmant",
    "enthousiasmé": "enthousiasmée",
    "enthousiasmée": "enthousiasmé",
    "entiché": "entichée",
    "entichée": "entiché",
    "entier": "entière",
    "entière": "entier",
    "entortillé": "entortillée",
    "entortillée": "entortillé",
    "entouré": "entourée",
    "entourée": "entouré",
    "entr'aperçu": "entr'aperçue",
    "entr'aperçue": "entr'aperçu",
    "entr'ouvert": "entr'ouverte",
    "entr'ouverte": "entr'ouvert",
    "entrant": "entrante",
    "entrante": "entrant",
    "entravé": "entravée",
    "entravée": "entravé",
    "entraînant": "entraînante",
    "entraînante": "entraînant",
    "entraîné": "entraînée",
    "entraînée": "entraîné",
    "entraîneur": "entraîneuse",
    "entraîneuse": "entraîneur",
    "entrebâillé": "entrebâillée",
    "entrebâillée": "entrebâillé",
    "entrecoupé": "entrecoupée",
    "entrecoupée": "entrecoupé",
    "entrelacé": "entrelacée",
    "entrelacée": "entrelacé",
    "entremetteur": "entremetteuse",
    "entremetteuse": "entremetteur",
    "entremis": "entremise",
    "entremise": "entremis",
    "entreposé": "entreposée",
    "entreposée": "entreposé",
    "entreprenant": "entreprenante",
    "entreprenante": "entreprenant",
    "entrepreneur": "entrepreneuse",
    "entrepreneuse": "entrepreneur",
    "entré": "entrée",
    "entrée": "entré",
    "entreteneur": "entreteneuse",
    "entreteneuse": "entreteneur",
    "entretenu": "entretenue",
    "entretenue": "entretenu",
    "entrevu": "entrevue",
    "entrevue": "entrevu",
    "entrouvert": "entrouverte",
    "entrouverte": "entrouvert",
    "entubé": "entubée",
    "entubée": "entubé",
    "enturbanné": "enturbannée",
    "enturbannée": "enturbanné",
    "enténébré": "enténébrée",
    "enténébrée": "enténébré",
    "entériné": "entérinée",
    "entérinée": "entériné",
    "entêtant": "entêtante",
    "entêtante": "entêtant",
    "entêté": "entêtée",
    "entêtée": "entêté",
    "entôlé": "entôlée",
    "entôlée": "entôlé",
    "envahi": "envahie",
    "envahie": "envahi",
    "envahissant": "envahissante",
    "envahissante": "envahissant",
    "envasé": "envasée",
    "envasée": "envasé",
    "enveloppant": "enveloppante",
    "enveloppante": "enveloppant",
    "enveloppé": "enveloppée",
    "enveloppée": "enveloppé",
    "envenimé": "envenimée",
    "envenimée": "envenimé",
    "envié": "enviée",
    "enviée": "envié",
    "environnant": "environnante",
    "environnante": "environnant",
    "environnemental": "environnementale",
    "environnementale": "environnemental",
    "environné": "environnée",
    "environnée": "environné",
    "envisagé": "envisagée",
    "envisagée": "envisagé",
    "envolé": "envolée",
    "envolée": "envolé",
    "envoyé": "envoyée",
    "envoyée": "envoyé",
    "envoûtant": "envoûtante",
    "envoûtante": "envoûtant",
    "envoûté": "envoûtée",
    "envoûtée": "envoûté",
    "ergoteur": "ergoteuse",
    "ergoteuse": "ergoteur",
    "errant": "errante",
    "errante": "errant",
    "erroné": "erronée",
    "erronée": "erroné",
    "escaladé": "escaladée",
    "escaladée": "escaladé",
    "escaladeur": "escaladeuse",
    "escaladeuse": "escaladeur",
    "escamoté": "escamotée",
    "escamotée": "escamoté",
    "escarpé": "escarpée",
    "escarpée": "escarpé",
    "esclaffé": "esclaffée",
    "esclaffée": "esclaffé",
    "escompté": "escomptée",
    "escomptée": "escompté",
    "escorté": "escortée",
    "escortée": "escorté",
    "esgourdé": "esgourdée",
    "esgourdée": "esgourdé",
    "espacé": "espacée",
    "espacée": "espacé",
    "espagnol": "espagnole",
    "espagnole": "espagnol",
    "espion": "espionne",
    "espionne": "espion",
    "espionné": "espionnée",
    "espionnée": "espionné",
    "espéré": "espérée",
    "espérée": "espéré",
    "esquinté": "esquintée",
    "esquintée": "esquinté",
    "esquissé": "esquissée",
    "esquissée": "esquissé",
    "esquivé": "esquivée",
    "esquivée": "esquivé",
    "essayé": "essayée",
    "essayée": "essayé",
    "essentiel": "essentielle",
    "essentielle": "essentiel",
    "esseulé": "esseulée",
    "esseulée": "esseulé",
    "essoré": "essorée",
    "essorée": "essoré",
    "essoufflé": "essoufflée",
    "essoufflée": "essoufflé",
    "essuyé": "essuyée",
    "essuyée": "essuyé",
    "est-allemand": "est-allemande",
    "est-allemande": "est-allemand",
    "estampé": "estampée",
    "estampée": "estampé",
    "esthéticien": "esthéticienne",
    "esthéticienne": "esthéticien",
    "estimé": "estimée",
    "estimée": "estimé",
    "estival": "estivale",
    "estivale": "estival",
    "estivant": "estivante",
    "estivante": "estivant",
    "estomaqué": "estomaquée",
    "estomaquée": "estomaqué",
    "estompé": "estompée",
    "estompée": "estompé",
    "estourbi": "estourbie",
    "estourbie": "estourbi",
    "estropié": "estropiée",
    "estropiée": "estropié",
    "estudiantin": "estudiantine",
    "estudiantine": "estudiantin",
    "euclidien": "euclidienne",
    "euclidienne": "euclidien",
    "euphorisant": "euphorisante",
    "euphorisante": "euphorisant",
    "euphorisé": "euphorisée",
    "euphorisée": "euphorisé",
    "eurasien": "eurasienne",
    "eurasienne": "eurasien",
    "européen": "européenne",
    "européenne": "européen",
    "ex-amant": "ex-amante",
    "ex-amante": "ex-amant",
    "ex-assistant": "ex-assistante",
    "ex-assistante": "ex-assistant",
    "ex-champion": "ex-championne",
    "ex-championne": "ex-champion",
    "ex-chanteur": "ex-chanteuse",
    "ex-chanteuse": "ex-chanteur",
    "ex-fiancé": "ex-fiancée",
    "ex-fiancée": "ex-fiancé",
    "ex-petit": "ex-petite",
    "ex-petite": "ex-petit",
    "exacerbé": "exacerbée",
    "exacerbée": "exacerbé",
    "exact": "exacte",
    "exacte": "exact",
    "exagéré": "exagérée",
    "exagérée": "exagéré",
    "exaltant": "exaltante",
    "exaltante": "exaltant",
    "exalté": "exaltée",
    "exaltée": "exalté",
    "examinateur": "examinatrice",
    "examinatrice": "examinateur",
    "examiné": "examinée",
    "examinée": "examiné",
    "exaspérant": "exaspérante",
    "exaspérante": "exaspérant",
    "exaspéré": "exaspérée",
    "exaspérée": "exaspéré",
    "exaucé": "exaucée",
    "exaucée": "exaucé",
    "excavateur": "excavatrice",
    "excavatrice": "excavateur",
    "excellent": "excellente",
    "excellente": "excellent",
    "excentré": "excentrée",
    "excentrée": "excentré",
    "excepté": "exceptée",
    "exceptée": "excepté",
    "exceptionnel": "exceptionnelle",
    "exceptionnelle": "exceptionnel",
    "excessif": "excessive",
    "excessive": "excessif",
    "excisé": "excisée",
    "excisée": "excisé",
    "excitant": "excitante",
    "excitante": "excitant",
    "excité": "excitée",
    "excitée": "excité",
    "exclamatif": "exclamative",
    "exclamative": "exclamatif",
    "exclamé": "exclamée",
    "exclamée": "exclamé",
    "exclu": "exclue",
    "exclue": "exclu",
    "exclusif": "exclusive",
    "exclusive": "exclusif",
    "excommunié": "excommuniée",
    "excommuniée": "excommunié",
    "excrémentiel": "excrémentielle",
    "excrémentielle": "excrémentiel",
    "excusé": "excusée",
    "excusée": "excusé",
    "excédé": "excédée",
    "excédée": "excédé",
    "exempt": "exempte",
    "exempte": "exempt",
    "exempté": "exemptée",
    "exemptée": "exempté",
    "exercé": "exercée",
    "exercée": "exercé",
    "exhalé": "exhalée",
    "exhalée": "exhalé",
    "exhaussé": "exhaussée",
    "exhaussée": "exhaussé",
    "exhaustif": "exhaustive",
    "exhaustive": "exhaustif",
    "exhibé": "exhibée",
    "exhibée": "exhibé",
    "exhumé": "exhumée",
    "exhumée": "exhumé",
    "exigeant": "exigeante",
    "exigeante": "exigeant",
    "exigé": "exigée",
    "exigée": "exigé",
    "exigu": "exiguë",
    "exiguë": "exigu",
    "exilé": "exilée",
    "exilée": "exilé",
    "existant": "existante",
    "existante": "existant",
    "existentiel": "existentielle",
    "existentielle": "existentiel",
    "existé": "existée",
    "existée": "existé",
    "exorbitant": "exorbitante",
    "exorbitante": "exorbitant",
    "exorcisé": "exorcisée",
    "exorcisée": "exorcisé",
    "expansif": "expansive",
    "expansive": "expansif",
    "expatrié": "expatriée",
    "expatriée": "expatrié",
    "expert": "experte",
    "experte": "expert",
    "expié": "expiée",
    "expiée": "expié",
    "expirant": "expirante",
    "expirante": "expirant",
    "expiré": "expirée",
    "expirée": "expiré",
    "explicatif": "explicative",
    "explicative": "explicatif",
    "expliqué": "expliquée",
    "expliquée": "expliqué",
    "exploitant": "exploitante",
    "exploitante": "exploitant",
    "exploité": "exploitée",
    "exploitée": "exploité",
    "explorateur": "exploratrice",
    "exploratrice": "explorateur",
    "exploré": "explorée",
    "explorée": "exploré",
    "explosé": "explosée",
    "explosée": "explosé",
    "explosif": "explosive",
    "explosive": "explosif",
    "exponentiel": "exponentielle",
    "exponentielle": "exponentiel",
    "exporté": "exportée",
    "exportée": "exporté",
    "exposé": "exposée",
    "exposée": "exposé",
    "expressif": "expressive",
    "expressive": "expressif",
    "exprimé": "exprimée",
    "exprimée": "exprimé",
    "exproprié": "expropriée",
    "expropriée": "exproprié",
    "expulsé": "expulsée",
    "expulsée": "expulsé",
    "expurgé": "expurgée",
    "expurgée": "expurgé",
    "expédié": "expédiée",
    "expédiée": "expédié",
    "expéditeur": "expéditrice",
    "expéditrice": "expéditeur",
    "expéditif": "expéditive",
    "expéditive": "expéditif",
    "expérimental": "expérimentale",
    "expérimentale": "expérimental",
    "expérimenté": "expérimentée",
    "expérimentée": "expérimenté",
    "extasié": "extasiée",
    "extasiée": "extasié",
    "exterminateur": "exterminatrice",
    "exterminatrice": "exterminateur",
    "exterminé": "exterminée",
    "exterminée": "exterminé",
    "extirpé": "extirpée",
    "extirpée": "extirpé",
    "extorqué": "extorquée",
    "extorquée": "extorqué",
    "extra-fort": "extra-forte",
    "extra-forte": "extra-fort",
    "extra-sensoriel": "extra-sensorielle",
    "extra-sensorielle": "extra-sensoriel",
    "extra-utérin": "extra-utérine",
    "extra-utérine": "extra-utérin",
    "extraconjugal": "extraconjugale",
    "extraconjugale": "extraconjugal",
    "extradé": "extradée",
    "extradée": "extradé",
    "extrait": "extraite",
    "extraite": "extrait",
    "extrapolé": "extrapolée",
    "extrapolée": "extrapolé",
    "extrasensoriel": "extrasensorielle",
    "extrasensorielle": "extrasensoriel",
    "extravagant": "extravagante",
    "extravagante": "extravagant",
    "extraverti": "extravertie",
    "extravertie": "extraverti",
    "extroverti": "extrovertie",
    "extrovertie": "extroverti",
    "exténuant": "exténuante",
    "exténuante": "exténuant",
    "exténué": "exténuée",
    "exténuée": "exténué",
    "extérieur": "extérieure",
    "extérieure": "extérieur",
    "exubérant": "exubérante",
    "exubérante": "exubérant",
    "exultant": "exultante",
    "exultante": "exultant",
    "exécré": "exécrée",
    "exécrée": "exécré",
    "exécuté": "exécutée",
    "exécutée": "exécuté",
    "exécuteur": "exécutrice",
    "exécutrice": "exécuteur",
    "exécutif": "exécutive",
    "exécutive": "exécutif",
    "fabricant": "fabricante",
    "fabricante": "fabricant",
    "fabriqué": "fabriquée",
    "fabriquée": "fabriqué",
    "facial": "faciale",
    "faciale": "facial",
    "facilité": "facilitée",
    "facilitée": "facilité",
    "facteur": "factrice",
    "factrice": "facteur",
    "facturé": "facturée",
    "facturée": "facturé",
    "facturier": "facturière",
    "facturière": "facturier",
    "facultatif": "facultative",
    "facultative": "facultatif",
    "facétieuse": "facétieux",
    "facétieux": "facétieuse",
    "fadé": "fadée",
    "fadée": "fadé",
    "fagoté": "fagotée",
    "fagotée": "fagoté",
    "faiblard": "faiblarde",
    "faiblarde": "faiblard",
    "fainéant": "fainéante",
    "fainéante": "fainéant",
    "fait": "faite",
    "faite": "fait",
    "faisan": "faisane",
    "faisane": "faisan",
    "faisandé": "faisandée",
    "faisandée": "faisandé",
    "faiseur": "faiseuse",
    "faiseuse": "faiseur",
    "falot": "falote",
    "falote": "falot",
    "falsifié": "falsifiée",
    "falsifiée": "falsifié",
    "familial": "familiale",
    "familiale": "familial",
    "familiarisé": "familiarisée",
    "familiarisée": "familiarisé",
    "familier": "familière",
    "familière": "familier",
    "famé": "famée",
    "famée": "famé",
    "fané": "fanée",
    "fanée": "fané",
    "fanfaron": "fanfaronne",
    "fanfaronne": "fanfaron",
    "faraud": "faraude",
    "faraude": "faraud",
    "farceur": "farceuse",
    "farceuse": "farceur",
    "farci": "farcie",
    "farcie": "farci",
    "fardé": "fardée",
    "fardée": "fardé",
    "farfelu": "farfelue",
    "farfelue": "farfelu",
    "farineuse": "farineux",
    "farineux": "farineuse",
    "fascinant": "fascinante",
    "fascinante": "fascinant",
    "fasciné": "fascinée",
    "fascinée": "fasciné",
    "fast food": "fast-food",
    "fast-food": "fast food",
    "fat": "fate",
    "fate": "fat",
    "fatal": "fatale",
    "fatale": "fatal",
    "fatigant": "fatigante",
    "fatigante": "fatigant",
    "fatigué": "fatiguée",
    "fatiguée": "fatigué",
    "faubourien": "faubourienne",
    "faubourienne": "faubourien",
    "faucardé": "faucardée",
    "faucardée": "faucardé",
    "fauché": "fauchée",
    "fauchée": "fauché",
    "faucheur": "faucheuse",
    "faucheuse": "faucheur",
    "faufilé": "faufilée",
    "faufilée": "faufilé",
    "faussé": "faussée",
    "faussée": "faussé",
    "faustien": "faustienne",
    "faustienne": "faustien",
    "fautif": "fautive",
    "fautive": "fautif",
    "favori": "favorite",
    "favorite": "favori",
    "favorisé": "favorisée",
    "favorisée": "favorisé",
    "façonné": "façonnée",
    "façonnée": "façonné",
    "feignant": "feignante",
    "feignante": "feignant",
    "feint": "feinte",
    "feinte": "feint",
    "fendillé": "fendillée",
    "fendillée": "fendillé",
    "fendu": "fendue",
    "fendue": "fendu",
    "fer": "fère",
    "fère": "fer",
    "fermenté": "fermentée",
    "fermentée": "fermenté",
    "fermé": "fermée",
    "fermée": "fermé",
    "fermier": "fermière",
    "fermière": "fermier",
    "ferraillant": "ferraillante",
    "ferraillante": "ferraillant",
    "ferré": "ferrée",
    "ferrée": "ferré",
    "ferronnier": "ferronnière",
    "ferronnière": "ferronnier",
    "fertilisé": "fertilisée",
    "fertilisée": "fertilisé",
    "fervent": "fervente",
    "fervente": "fervent",
    "fessé": "fessée",
    "fessée": "fessé",
    "fesseur": "fesseuse",
    "fesseuse": "fesseur",
    "fessier": "fessière",
    "fessière": "fessier",
    "fessu": "fessue",
    "fessue": "fessu",
    "festif": "festive",
    "festive": "festif",
    "festonné": "festonnée",
    "festonnée": "festonné",
    "feuilleté": "feuilletée",
    "feuilletée": "feuilleté",
    "feuillu": "feuillue",
    "feuillue": "feuillu",
    "feutré": "feutrée",
    "feutrée": "feutré",
    "fiancé": "fiancée",
    "fiancée": "fiancé",
    "ficelé": "ficelée",
    "ficelée": "ficelé",
    "fiché": "fichée",
    "fichée": "fiché",
    "fichu": "fichue",
    "fichue": "fichu",
    "fictif": "fictive",
    "fictive": "fictif",
    "fieffé": "fieffée",
    "fieffée": "fieffé",
    "fier": "fière",
    "fière": "fier",
    "fié": "fiée",
    "fiée": "fié",
    "figé": "figée",
    "figée": "figé",
    "fignolé": "fignolée",
    "fignolée": "fignolé",
    "figurant": "figurante",
    "figurante": "figurant",
    "figuratif": "figurative",
    "figurative": "figuratif",
    "figuré": "figurée",
    "figurée": "figuré",
    "filant": "filante",
    "filante": "filant",
    "filé": "filée",
    "filée": "filé",
    "fileté": "filetée",
    "filetée": "fileté",
    "fileur": "fileuse",
    "fileuse": "fileur",
    "filial": "filiale",
    "filiale": "filial",
    "filleul": "filleule",
    "filleule": "filleul",
    "filmé": "filmée",
    "filmée": "filmé",
    "filtré": "filtrée",
    "filtrée": "filtré",
    "fin": "fine",
    "fine": "fin",
    "final": "finale",
    "finale": "final",
    "financé": "financée",
    "financée": "financé",
    "financier": "financière",
    "financière": "financier",
    "finaud": "finaude",
    "finaude": "finaud",
    "fini": "finie",
    "finie": "fini",
    "finissant": "finissante",
    "finissante": "finissant",
    "finisseur": "finisseuse",
    "finisseuse": "finisseur",
    "fiscal": "fiscale",
    "fiscale": "fiscal",
    "fissuré": "fissurée",
    "fissurée": "fissuré",
    "fixé": "fixée",
    "fixée": "fixé",
    "fiérot": "fiérote",
    "fiérote": "fiérot",
    "flagellé": "flagellée",
    "flagellée": "flagellé",
    "flageolant": "flageolante",
    "flageolante": "flageolant",
    "flagrant": "flagrante",
    "flagrante": "flagrant",
    "flairé": "flairée",
    "flairée": "flairé",
    "flaireur": "flaireuse",
    "flaireuse": "flaireur",
    "flamand": "flamande",
    "flamande": "flamand",
    "flambant": "flambante",
    "flambante": "flambant",
    "flambé": "flambée",
    "flambée": "flambé",
    "flambeur": "flambeuse",
    "flambeuse": "flambeur",
    "flamboyant": "flamboyante",
    "flamboyante": "flamboyant",
    "flammé": "flammée",
    "flammée": "flammé",
    "flanqué": "flanquée",
    "flanquée": "flanqué",
    "flapi": "flapie",
    "flapie": "flapi",
    "flatté": "flattée",
    "flattée": "flatté",
    "flatteur": "flatteuse",
    "flatteuse": "flatteur",
    "flemmard": "flemmarde",
    "flemmarde": "flemmard",
    "fleurdelisé": "fleurdelisée",
    "fleurdelisée": "fleurdelisé",
    "fleuri": "fleurie",
    "fleurie": "fleuri",
    "flicard": "flicarde",
    "flicarde": "flicard",
    "flingué": "flinguée",
    "flinguée": "flingué",
    "flingueur": "flingueuse",
    "flingueuse": "flingueur",
    "flippé": "flippée",
    "flippée": "flippé",
    "flirteur": "flirteuse",
    "flirteuse": "flirteur",
    "floral": "florale",
    "florale": "floral",
    "florentin": "florentine",
    "florentine": "florentin",
    "florissant": "florissante",
    "florissante": "florissant",
    "flottant": "flottante",
    "flottante": "flottant",
    "flotté": "flottée",
    "flottée": "flotté",
    "flou": "floue",
    "floue": "flou",
    "floué": "flouée",
    "flouée": "floué",
    "fluctuant": "fluctuante",
    "fluctuante": "fluctuant",
    "fluet": "fluette",
    "fluette": "fluet",
    "fluorescent": "fluorescente",
    "fluorescente": "fluorescent",
    "fluvial": "fluviale",
    "fluviale": "fluvial",
    "flâneur": "flâneuse",
    "flâneuse": "flâneur",
    "fléché": "fléchée",
    "fléchée": "fléché",
    "fléchi": "fléchie",
    "fléchie": "fléchi",
    "flétri": "flétrie",
    "flétrie": "flétri",
    "flûté": "flûtée",
    "flûtée": "flûté",
    "focal": "focale",
    "focale": "focal",
    "foetal": "foetale",
    "foetale": "foetal",
    "foiré": "foirée",
    "foirée": "foiré",
    "foisonnant": "foisonnante",
    "foisonnante": "foisonnant",
    "folichon": "folichonne",
    "folichonne": "folichon",
    "follet": "follette",
    "follette": "follet",
    "fomenté": "fomentée",
    "fomentée": "fomenté",
    "foncé": "foncée",
    "foncée": "foncé",
    "fonceur": "fonceuse",
    "fonceuse": "fonceur",
    "foncier": "foncière",
    "foncière": "foncier",
    "fonctionnarisé": "fonctionnarisée",
    "fonctionnarisée": "fonctionnarisé",
    "fonctionnel": "fonctionnelle",
    "fonctionnelle": "fonctionnel",
    "fondamental": "fondamentale",
    "fondamentale": "fondamental",
    "fondant": "fondante",
    "fondante": "fondant",
    "fondateur": "fondatrice",
    "fondatrice": "fondateur",
    "fondé": "fondée",
    "fondée": "fondé",
    "fondu": "fondue",
    "fondue": "fondu",
    "footballeur": "footballeuse",
    "footballeuse": "footballeur",
    "forain": "foraine",
    "foraine": "forain",
    "forcené": "forcenée",
    "forcenée": "forcené",
    "forcé": "forcée",
    "forcée": "forcé",
    "foré": "forée",
    "forée": "foré",
    "forestier": "forestière",
    "forestière": "forestier",
    "foreur": "foreuse",
    "foreuse": "foreur",
    "forgé": "forgée",
    "forgée": "forgé",
    "forgeron": "forgeronne",
    "forgeronne": "forgeron",
    "formalisé": "formalisée",
    "formalisée": "formalisé",
    "formateur": "formatrice",
    "formatrice": "formateur",
    "formel": "formelle",
    "formelle": "formel",
    "formé": "formée",
    "formée": "formé",
    "formulé": "formulée",
    "formulée": "formulé",
    "fornicateur": "fornicatrice",
    "fornicatrice": "fornicateur",
    "fort": "forte",
    "forte": "fort",
    "fortifié": "fortifiée",
    "fortifiée": "fortifié",
    "fortuit": "fortuite",
    "fortuite": "fortuit",
    "fortuné": "fortunée",
    "fortunée": "fortuné",
    "folle": "fou",
    "fou": "folle",
    "fouaillé": "fouaillée",
    "fouaillée": "fouaillé",
    "foudroyant": "foudroyante",
    "foudroyante": "foudroyant",
    "foudroyé": "foudroyée",
    "foudroyée": "foudroyé",
    "fouetté": "fouettée",
    "fouettée": "fouetté",
    "fouetteur": "fouetteuse",
    "fouetteuse": "fouetteur",
    "fouillé": "fouillée",
    "fouillée": "fouillé",
    "fouilleur": "fouilleuse",
    "fouilleuse": "fouilleur",
    "fouineur": "fouineuse",
    "fouineuse": "fouineur",
    "foui": "fouie",
    "fouie": "foui",
    "fouisseur": "fouisseuse",
    "fouisseuse": "fouisseur",
    "foulé": "foulée",
    "foulée": "foulé",
    "fourbi": "fourbie",
    "fourbie": "fourbi",
    "fourbu": "fourbue",
    "fourbue": "fourbu",
    "fourché": "fourchée",
    "fourchée": "fourché",
    "fourchu": "fourchue",
    "fourchue": "fourchu",
    "fourgué": "fourguée",
    "fourguée": "fourgué",
    "fourmilier": "fourmilière",
    "fourmilière": "fourmilier",
    "fourmillant": "fourmillante",
    "fourmillante": "fourmillant",
    "fourni": "fournie",
    "fournie": "fourni",
    "fourrager": "fourragère",
    "fourragère": "fourrager",
    "fourré": "fourrée",
    "fourrée": "fourré",
    "fourvoyé": "fourvoyée",
    "fourvoyée": "fourvoyé",
    "foutral": "foutrale",
    "foutrale": "foutral",
    "foutu": "foutue",
    "foutue": "foutu",
    "fracassant": "fracassante",
    "fracassante": "fracassant",
    "fracassé": "fracassée",
    "fracassée": "fracassé",
    "fractal": "fractale",
    "fractale": "fractal",
    "fractionné": "fractionnée",
    "fractionnée": "fractionné",
    "fracturé": "fracturée",
    "fracturée": "fracturé",
    "fragilisé": "fragilisée",
    "fragilisée": "fragilisé",
    "fragmenté": "fragmentée",
    "fragmentée": "fragmenté",
    "fraiseur": "fraiseuse",
    "fraiseuse": "fraiseur",
    "franc": "franche",
    "franche": "franc",
    "franc-maçon": "franc-maçonne",
    "franc-maçonne": "franc-maçon",
    "franchi": "franchie",
    "franchie": "franchi",
    "franchisé": "franchisée",
    "franchisée": "franchisé",
    "franchouillard": "franchouillarde",
    "franchouillarde": "franchouillard",
    "franciscain": "franciscaine",
    "franciscaine": "franciscain",
    "franco-allemand": "franco-allemande",
    "franco-allemande": "franco-allemand",
    "franco-américain": "franco-américaine",
    "franco-américaine": "franco-américain",
    "franco-anglais": "franco-anglaise",
    "franco-anglaise": "franco-anglais",
    "frangé": "frangée",
    "frangée": "frangé",
    "frangin": "frangine",
    "frangine": "frangin",
    "français": "française",
    "française": "français",
    "frappant": "frappante",
    "frappante": "frappant",
    "frappé": "frappée",
    "frappée": "frappé",
    "fraternel": "fraternelle",
    "fraternelle": "fraternel",
    "fraudeur": "fraudeuse",
    "fraudeuse": "fraudeur",
    "frayé": "frayée",
    "frayée": "frayé",
    "fredonné": "fredonnée",
    "fredonnée": "fredonné",
    "freiné": "freinée",
    "freinée": "freiné",
    "frelaté": "frelatée",
    "frelatée": "frelaté",
    "freudien": "freudienne",
    "freudienne": "freudien",
    "friand": "friande",
    "friande": "friand",
    "frigorifié": "frigorifiée",
    "frigorifiée": "frigorifié",
    "frimé": "frimée",
    "frimée": "frimé",
    "frimeur": "frimeuse",
    "frimeuse": "frimeur",
    "fringant": "fringante",
    "fringante": "fringant",
    "fringué": "fringuée",
    "fringuée": "fringué",
    "fripé": "fripée",
    "fripée": "fripé",
    "fripon": "friponne",
    "friponne": "fripon",
    "friqué": "friquée",
    "friquée": "friqué",
    "frisant": "frisante",
    "frisante": "frisant",
    "frisé": "frisée",
    "frisée": "frisé",
    "frison": "frisonne",
    "frisonne": "frison",
    "frisotté": "frisottée",
    "frisottée": "frisotté",
    "frisquet": "frisquette",
    "frisquette": "frisquet",
    "frissonnant": "frissonnante",
    "frissonnante": "frissonnant",
    "froid": "froide",
    "froide": "froid",
    "froissé": "froissée",
    "froissée": "froissé",
    "fromager": "fromagère",
    "fromagère": "fromager",
    "froncé": "froncée",
    "froncée": "froncé",
    "frondeur": "frondeuse",
    "frondeuse": "frondeur",
    "frontal": "frontale",
    "frontale": "frontal",
    "frontalier": "frontalière",
    "frontalière": "frontalier",
    "frotté": "frottée",
    "frottée": "frotté",
    "frotteur": "frotteuse",
    "frotteuse": "frotteur",
    "froufroutant": "froufroutante",
    "froufroutante": "froufroutant",
    "froussard": "froussarde",
    "froussarde": "froussard",
    "frugal": "frugale",
    "frugale": "frugal",
    "fruité": "fruitée",
    "fruitée": "fruité",
    "fruitier": "fruitière",
    "fruitière": "fruitier",
    "frustrant": "frustrante",
    "frustrante": "frustrant",
    "frustré": "frustrée",
    "frustrée": "frustré",
    "frémissant": "frémissante",
    "frémissante": "frémissant",
    "fréquent": "fréquente",
    "fréquente": "fréquent",
    "fréquenté": "fréquentée",
    "fréquentée": "fréquenté",
    "frété": "frétée",
    "frétée": "frété",
    "frétillant": "frétillante",
    "frétillante": "frétillant",
    "frétillard": "frétillarde",
    "frétillarde": "frétillard",
    "frôlé": "frôlée",
    "frôlée": "frôlé",
    "frôleur": "frôleuse",
    "frôleuse": "frôleur",
    "fugitif": "fugitive",
    "fugitive": "fugitif",
    "fugueur": "fugueuse",
    "fugueuse": "fugueur",
    "fui": "fuie",
    "fuie": "fui",
    "fulgurant": "fulgurante",
    "fulgurante": "fulgurant",
    "fulminant": "fulminante",
    "fulminante": "fulminant",
    "fumant": "fumante",
    "fumante": "fumant",
    "fumé": "fumée",
    "fumée": "fumé",
    "fumeur": "fumeuse",
    "fumeuse": "fumeur",
    "fumier": "fumière",
    "fumière": "fumier",
    "fureteur": "fureteuse",
    "fureteuse": "fureteur",
    "furibard": "furibarde",
    "furibarde": "furibard",
    "furibond": "furibonde",
    "furibonde": "furibond",
    "furtif": "furtive",
    "furtive": "furtif",
    "fusant": "fusante",
    "fusante": "fusant",
    "fuselé": "fuselée",
    "fuselée": "fuselé",
    "fusé": "fusée",
    "fusée": "fusé",
    "fusillé": "fusillée",
    "fusillée": "fusillé",
    "fusionnel": "fusionnelle",
    "fusionnelle": "fusionnel",
    "fustigé": "fustigée",
    "fustigée": "fustigé",
    "futur": "future",
    "future": "futur",
    "futé": "futée",
    "futée": "futé",
    "fuyant": "fuyante",
    "fuyante": "fuyant",
    "fuyard": "fuyarde",
    "fuyarde": "fuyard",
    "fâché": "fâchée",
    "fâchée": "fâché",
    "fécal": "fécale",
    "fécale": "fécal",
    "fécond": "féconde",
    "féconde": "fécond",
    "fécondant": "fécondante",
    "fécondante": "fécondant",
    "fécondé": "fécondée",
    "fécondée": "fécondé",
    "fédéral": "fédérale",
    "fédérale": "fédéral",
    "fédérateur": "fédératrice",
    "fédératrice": "fédérateur",
    "fédéré": "fédérée",
    "fédérée": "fédéré",
    "félicité": "félicitée",
    "félicitée": "félicité",
    "félin": "féline",
    "féline": "félin",
    "félon": "félonne",
    "félonne": "félon",
    "féminin": "féminine",
    "féminine": "féminin",
    "fémoral": "fémorale",
    "fémorale": "fémoral",
    "féodal": "féodale",
    "féodale": "féodal",
    "féru": "férue",
    "férue": "féru",
    "fêlé": "fêlée",
    "fêlée": "fêlé",
    "fêté": "fêtée",
    "fêtée": "fêté",
    "gabonais": "gabonaise",
    "gabonaise": "gabonais",
    "gaffé": "gaffée",
    "gaffée": "gaffé",
    "gaffeur": "gaffeuse",
    "gaffeuse": "gaffeur",
    "gagé": "gagée",
    "gagée": "gagé",
    "gagnant": "gagnante",
    "gagnante": "gagnant",
    "gagné": "gagnée",
    "gagnée": "gagné",
    "gagneur": "gagneuse",
    "gagneuse": "gagneur",
    "gai": "gaie",
    "gaie": "gai",
    "gaillard": "gaillarde",
    "gaillarde": "gaillard",
    "gainé": "gainée",
    "gainée": "gainé",
    "galant": "galante",
    "galante": "galant",
    "galbé": "galbée",
    "galbée": "galbé",
    "galonné": "galonnée",
    "galonnée": "galonné",
    "galopant": "galopante",
    "galopante": "galopant",
    "galopin": "galopine",
    "galopine": "galopin",
    "galvanisé": "galvanisée",
    "galvanisée": "galvanisé",
    "galvaudé": "galvaudée",
    "galvaudée": "galvaudé",
    "gambadant": "gambadante",
    "gambadante": "gambadant",
    "gamin": "gamine",
    "gamine": "gamin",
    "gangrené": "gangrenée",
    "gangrenée": "gangrené",
    "gansé": "gansée",
    "gansée": "gansé",
    "ganté": "gantée",
    "gantée": "ganté",
    "garant": "garante",
    "garante": "garant",
    "garanti": "garantie",
    "garantie": "garanti",
    "gardé": "gardée",
    "gardée": "gardé",
    "gardeur": "gardeuse",
    "gardeuse": "gardeur",
    "gardien": "gardienne",
    "gardienne": "gardien",
    "garé": "garée",
    "garée": "garé",
    "gargouillant": "gargouillante",
    "gargouillante": "gargouillant",
    "garni": "garnie",
    "garnie": "garni",
    "garrotté": "garrottée",
    "garrottée": "garrotté",
    "garçonnier": "garçonnière",
    "garçonnière": "garçonnier",
    "gascon": "gasconne",
    "gasconne": "gascon",
    "gaspillé": "gaspillée",
    "gaspillée": "gaspillé",
    "gaspilleur": "gaspilleuse",
    "gaspilleuse": "gaspilleur",
    "gastro-intestinal": "gastro-intestinale",
    "gastro-intestinale": "gastro-intestinal",
    "gaucher": "gauchère",
    "gauchère": "gaucher",
    "gauchi": "gauchie",
    "gauchie": "gauchi",
    "gaufré": "gaufrée",
    "gaufrée": "gaufré",
    "gaulé": "gaulée",
    "gaulée": "gaulé",
    "gaussé": "gaussée",
    "gaussée": "gaussé",
    "gavé": "gavée",
    "gavée": "gavé",
    "gazé": "gazée",
    "gazée": "gazé",
    "gazonné": "gazonnée",
    "gazonnée": "gazonné",
    "gazouillant": "gazouillante",
    "gazouillante": "gazouillant",
    "geignant": "geignante",
    "geignante": "geignant",
    "geignard": "geignarde",
    "geignarde": "geignard",
    "gelé": "gelée",
    "gelée": "gelé",
    "gentil": "gentille",
    "gentille": "gentil",
    "gentillet": "gentillette",
    "gentillette": "gentillet",
    "gerbé": "gerbée",
    "gerbée": "gerbé",
    "gerbier": "gerbière",
    "gerbière": "gerbier",
    "gercé": "gercée",
    "gercée": "gercé",
    "germain": "germaine",
    "germaine": "germain",
    "germé": "germée",
    "germée": "germé",
    "germinal": "germinale",
    "germinale": "germinal",
    "gesticulant": "gesticulante",
    "gesticulante": "gesticulant",
    "gestuel": "gestuelle",
    "gestuelle": "gestuel",
    "geôlier": "geôlière",
    "geôlière": "geôlier",
    "gibbeuse": "gibbeux",
    "gibbeux": "gibbeuse",
    "giclé": "giclée",
    "giclée": "giclé",
    "gidien": "gidienne",
    "gidienne": "gidien",
    "giflé": "giflée",
    "giflée": "giflé",
    "girond": "gironde",
    "gironde": "girond",
    "girondin": "girondine",
    "girondine": "girondin",
    "gisant": "gisante",
    "gisante": "gisant",
    "gitan": "gitane",
    "gitane": "gitan",
    "givré": "givrée",
    "givrée": "givré",
    "glacé": "glacée",
    "glacée": "glacé",
    "glacial": "glaciale",
    "glaciale": "glacial",
    "glané": "glanée",
    "glanée": "glané",
    "glaneur": "glaneuse",
    "glaneuse": "glaneur",
    "glapi": "glapie",
    "glapie": "glapi",
    "glapissant": "glapissante",
    "glapissante": "glapissant",
    "glaçant": "glaçante",
    "glaçante": "glaçant",
    "glissant": "glissante",
    "glissante": "glissant",
    "glissé": "glissée",
    "glissée": "glissé",
    "global": "globale",
    "globale": "global",
    "gloussant": "gloussante",
    "gloussante": "gloussant",
    "glouton": "gloutonne",
    "gloutonne": "glouton",
    "gluant": "gluante",
    "gluante": "gluant",
    "gobé": "gobée",
    "gobée": "gobé",
    "gobeur": "gobeuse",
    "gobeuse": "gobeur",
    "goguenard": "goguenarde",
    "goguenarde": "goguenard",
    "goinfré": "goinfrée",
    "goinfrée": "goinfré",
    "golfeur": "golfeuse",
    "golfeuse": "golfeur",
    "gominé": "gominée",
    "gominée": "gominé",
    "gommé": "gommée",
    "gommée": "gommé",
    "gommeuse": "gommeux",
    "gommeux": "gommeuse",
    "gondolé": "gondolée",
    "gondolée": "gondolé",
    "gondolier": "gondolière",
    "gondolière": "gondolier",
    "gonflant": "gonflante",
    "gonflante": "gonflant",
    "gonflé": "gonflée",
    "gonflée": "gonflé",
    "gorgé": "gorgée",
    "gorgée": "gorgé",
    "gouailleur": "gouailleuse",
    "gouailleuse": "gouailleur",
    "goudronné": "goudronnée",
    "goudronnée": "goudronné",
    "goulu": "goulue",
    "goulue": "goulu",
    "goupillé": "goupillée",
    "goupillée": "goupillé",
    "gouré": "gourée",
    "gourée": "gouré",
    "gourmand": "gourmande",
    "gourmande": "gourmand",
    "gourmé": "gourmée",
    "gourmée": "gourmé",
    "gouvernemental": "gouvernementale",
    "gouvernementale": "gouvernemental",
    "gouverné": "gouvernée",
    "gouvernée": "gouverné",
    "goûté": "goûtée",
    "goûtée": "goûté",
    "goûteur": "goûteuse",
    "goûteuse": "goûteux",
    "goûteux": "goûteuse",
    "gracié": "graciée",
    "graciée": "gracié",
    "graduel": "graduelle",
    "graduelle": "graduel",
    "gradué": "graduée",
    "graduée": "gradué",
    "gradé": "gradée",
    "gradée": "gradé",
    "graillé": "graillée",
    "graillée": "graillé",
    "graissé": "graissée",
    "graissée": "graissé",
    "grammairien": "grammairienne",
    "grammairienne": "grammairien",
    "grammatical": "grammaticale",
    "grammaticale": "grammatical",
    "grand": "grande",
    "grande": "grand",
    "grand-duc": "grande-duchesse",
    "grande-duchesse": "grand-duc",
    "grand-maman": "grand-papa",
    "grand-papa": "grand-maman",
    "grandiloquent": "grandiloquente",
    "grandiloquente": "grandiloquent",
    "grandi": "grandie",
    "grandie": "grandi",
    "grandissant": "grandissante",
    "grandissante": "grandissant",
    "granuleuse": "granuleux",
    "granuleux": "granuleuse",
    "grasseyant": "grasseyante",
    "grasseyante": "grasseyant",
    "grassouillet": "grassouillette",
    "grassouillette": "grassouillet",
    "gratifiant": "gratifiante",
    "gratifiante": "gratifiant",
    "gratifié": "gratifiée",
    "gratifiée": "gratifié",
    "gratiné": "gratinée",
    "gratinée": "gratiné",
    "gratté": "grattée",
    "grattée": "gratté",
    "gratuit": "gratuite",
    "gratuite": "gratuit",
    "graveleuse": "graveleux",
    "graveleux": "graveleuse",
    "gravé": "gravée",
    "gravée": "gravé",
    "gravi": "gravie",
    "gravie": "gravi",
    "gravitationnel": "gravitationnelle",
    "gravitationnelle": "gravitationnel",
    "grec": "grecque",
    "grecque": "grec",
    "gredin": "gredine",
    "gredine": "gredin",
    "greffé": "greffée",
    "greffée": "greffé",
    "greffier": "greffière",
    "greffière": "greffier",
    "grelottant": "grelottante",
    "grelottante": "grelottant",
    "grelotté": "grelottée",
    "grelottée": "grelotté",
    "grené": "grenée",
    "grenée": "grené",
    "grenu": "grenue",
    "grenue": "grenu",
    "grevé": "grevée",
    "grevée": "grevé",
    "gribouillé": "gribouillée",
    "gribouillée": "gribouillé",
    "griffé": "griffée",
    "griffée": "griffé",
    "griffonné": "griffonnée",
    "griffonnée": "griffonné",
    "griffu": "griffue",
    "griffue": "griffu",
    "grignoté": "grignotée",
    "grignotée": "grignoté",
    "grillagé": "grillagée",
    "grillagée": "grillagé",
    "grillé": "grillée",
    "grillée": "grillé",
    "grimaçant": "grimaçante",
    "grimaçante": "grimaçant",
    "grimé": "grimée",
    "grimée": "grimé",
    "grimpant": "grimpante",
    "grimpante": "grimpant",
    "grimpé": "grimpée",
    "grimpée": "grimpé",
    "grimpeur": "grimpeuse",
    "grimpeuse": "grimpeur",
    "grinçant": "grinçante",
    "grinçante": "grinçant",
    "grippé": "grippée",
    "grippée": "grippé",
    "grisant": "grisante",
    "grisante": "grisant",
    "grisé": "grisée",
    "grisée": "grisé",
    "grisonnant": "grisonnante",
    "grisonnante": "grisonnant",
    "grognon": "grognonne",
    "grognonne": "grognon",
    "grommelé": "grommelée",
    "grommelée": "grommelé",
    "grondant": "grondante",
    "grondante": "grondant",
    "grondé": "grondée",
    "grondée": "grondé",
    "grondeur": "grondeuse",
    "grondeuse": "grondeur",
    "gros": "grosse",
    "grosse": "gros",
    "grossi": "grossie",
    "grossie": "grossi",
    "grossier": "grossière",
    "grossière": "grossier",
    "grossissant": "grossissante",
    "grossissante": "grossissant",
    "grouillant": "grouillante",
    "grouillante": "grouillant",
    "groupé": "groupée",
    "groupée": "groupé",
    "grugé": "grugée",
    "grugée": "grugé",
    "grutier": "grutière",
    "grutière": "grutier",
    "gréco-romain": "gréco-romaine",
    "gréco-romaine": "gréco-romain",
    "gréé": "gréée",
    "gréée": "gréé",
    "grégorien": "grégorienne",
    "grégorienne": "grégorien",
    "grésillant": "grésillante",
    "grésillante": "grésillant",
    "grêlé": "grêlée",
    "grêlée": "grêlé",
    "guerrier": "guerrière",
    "guerrière": "guerrier",
    "guetté": "guettée",
    "guettée": "guetté",
    "guetteur": "guetteuse",
    "guetteuse": "guetteur",
    "gueulard": "gueularde",
    "gueularde": "gueulard",
    "gueulé": "gueulée",
    "gueulée": "gueulé",
    "guichetier": "guichetière",
    "guichetière": "guichetier",
    "guidé": "guidée",
    "guidée": "guidé",
    "guilleret": "guillerette",
    "guillerette": "guilleret",
    "guilloché": "guillochée",
    "guillochée": "guilloché",
    "guillotiné": "guillotinée",
    "guillotinée": "guillotiné",
    "guindé": "guindée",
    "guindée": "guindé",
    "guinéen": "guinéenne",
    "guinéenne": "guinéen",
    "gustatif": "gustative",
    "gustative": "gustatif",
    "guttural": "gutturale",
    "gutturale": "guttural",
    "guéri": "guérie",
    "guérie": "guéri",
    "guérisseur": "guérisseuse",
    "guérisseuse": "guérisseur",
    "gypseuse": "gypseux",
    "gypseux": "gypseuse",
    "gâché": "gâchée",
    "gâchée": "gâché",
    "gâté": "gâtée",
    "gâtée": "gâté",
    "géant": "géante",
    "géante": "géant",
    "gémi": "gémie",
    "gémie": "gémi",
    "gémissant": "gémissante",
    "gémissante": "gémissant",
    "génial": "géniale",
    "géniale": "génial",
    "génital": "génitale",
    "génitale": "génital",
    "géniteur": "génitrice",
    "génitrice": "géniteur",
    "général": "générale",
    "générale": "général",
    "généralisé": "généralisée",
    "généralisée": "généralisé",
    "générateur": "génératrice",
    "génératrice": "générateur",
    "génératif": "générative",
    "générative": "génératif",
    "généré": "générée",
    "générée": "généré",
    "généticien": "généticienne",
    "généticienne": "généticien",
    "géorgien": "géorgienne",
    "géorgienne": "géorgien",
    "gérant": "gérante",
    "gérante": "gérant",
    "géré": "gérée",
    "gérée": "géré",
    "gênant": "gênante",
    "gênante": "gênant",
    "gêné": "gênée",
    "gênée": "gêné",
    "gêneur": "gêneuse",
    "gêneuse": "gêneur",
    "habilité": "habilitée",
    "habilitée": "habilité",
    "habillé": "habillée",
    "habillée": "habillé",
    "habilleur": "habilleuse",
    "habilleuse": "habilleur",
    "habitant": "habitante",
    "habitante": "habitant",
    "habité": "habitée",
    "habitée": "habité",
    "habituel": "habituelle",
    "habituelle": "habituel",
    "habitué": "habituée",
    "habituée": "habitué",
    "haché": "hachée",
    "hachée": "haché",
    "hachuré": "hachurée",
    "hachurée": "hachuré",
    "hagard": "hagarde",
    "hagarde": "hagard",
    "halé": "halée",
    "halée": "halé",
    "haletant": "haletante",
    "haletante": "haletant",
    "hallucinant": "hallucinante",
    "hallucinante": "hallucinant",
    "halluciné": "hallucinée",
    "hallucinée": "halluciné",
    "handicapé": "handicapée",
    "handicapée": "handicapé",
    "hanté": "hantée",
    "hantée": "hanté",
    "happé": "happée",
    "happée": "happé",
    "harangué": "haranguée",
    "haranguée": "harangué",
    "harassant": "harassante",
    "harassante": "harassant",
    "harassé": "harassée",
    "harassée": "harassé",
    "harcelant": "harcelante",
    "harcelante": "harcelant",
    "harcelé": "harcelée",
    "harcelée": "harcelé",
    "harceleur": "harceleuse",
    "harceleuse": "harceleur",
    "hardi": "hardie",
    "hardie": "hardi",
    "harnaché": "harnachée",
    "harnachée": "harnaché",
    "harponné": "harponnée",
    "harponnée": "harponné",
    "hasardé": "hasardée",
    "hasardée": "hasardé",
    "haussé": "haussée",
    "haussée": "haussé",
    "haussier": "haussière",
    "haussière": "haussier",
    "haut": "haute",
    "haute": "haut",
    "hautain": "hautaine",
    "hautaine": "hautain",
    "hawaïen": "hawaïenne",
    "hawaïenne": "hawaïen",
    "haï": "haïe",
    "haïe": "haï",
    "haïtien": "haïtienne",
    "haïtienne": "haïtien",
    "herbu": "herbue",
    "herbue": "herbu",
    "herculéen": "herculéenne",
    "herculéenne": "herculéen",
    "demi-heure": "heur",
    "heur": "demi-heure",
    "heurté": "heurtée",
    "heurtée": "heurté",
    "hexagonal": "hexagonale",
    "hexagonale": "hexagonal",
    "hilarant": "hilarante",
    "hilarante": "hilarant",
    "hindou": "hindoue",
    "hindoue": "hindou",
    "hispano-américain": "hispano-américaine",
    "hispano-américaine": "hispano-américain",
    "hissé": "hissée",
    "hissée": "hissé",
    "historien": "historienne",
    "historienne": "historien",
    "hitlérien": "hitlérienne",
    "hitlérienne": "hitlérien",
    "hivernal": "hivernale",
    "hivernale": "hivernal",
    "hivernant": "hivernante",
    "hivernante": "hivernant",
    "hiérarchisé": "hiérarchisée",
    "hiérarchisée": "hiérarchisé",
    "hollywoodien": "hollywoodienne",
    "hollywoodienne": "hollywoodien",
    "homogénéisé": "homogénéisée",
    "homogénéisée": "homogénéisé",
    "homologué": "homologuée",
    "homologuée": "homologué",
    "homosexuel": "homosexuelle",
    "homosexuelle": "homosexuel",
    "honni": "honnie",
    "honnie": "honni",
    "honoré": "honorée",
    "honorée": "honoré",
    "horizontal": "horizontale",
    "horizontale": "horizontal",
    "horloger": "horlogère",
    "horlogère": "horloger",
    "hormonal": "hormonale",
    "hormonale": "hormonal",
    "horrifiant": "horrifiante",
    "horrifiante": "horrifiant",
    "horrifié": "horrifiée",
    "horrifiée": "horrifié",
    "horripilant": "horripilante",
    "horripilante": "horripilant",
    "horripilé": "horripilée",
    "horripilée": "horripilé",
    "hospitalier": "hospitalière",
    "hospitalière": "hospitalier",
    "hospitalisé": "hospitalisée",
    "hospitalisée": "hospitalisé",
    "houspillé": "houspillée",
    "houspillée": "houspillé",
    "hué": "huée",
    "huée": "hué",
    "huguenot": "huguenote",
    "huguenote": "huguenot",
    "huilé": "huilée",
    "huilée": "huilé",
    "humain": "humaine",
    "humaine": "humain",
    "humanisé": "humanisée",
    "humanisée": "humanisé",
    "humecté": "humectée",
    "humectée": "humecté",
    "humé": "humée",
    "humée": "humé",
    "humiliant": "humiliante",
    "humiliante": "humiliant",
    "humilié": "humiliée",
    "humiliée": "humilié",
    "huppé": "huppée",
    "huppée": "huppé",
    "hurlant": "hurlante",
    "hurlante": "hurlant",
    "hurlé": "hurlée",
    "hurlée": "hurlé",
    "huron": "huronne",
    "huronne": "huron",
    "hussard": "hussarde",
    "hussarde": "hussard",
    "hydratant": "hydratante",
    "hydratante": "hydratant",
    "hydraté": "hydratée",
    "hydratée": "hydraté",
    "hydrogéné": "hydrogénée",
    "hydrogénée": "hydrogéné",
    "hyperactif": "hyperactive",
    "hyperactive": "hyperactif",
    "hyperboréen": "hyperboréenne",
    "hyperboréenne": "hyperboréen",
    "hypertendu": "hypertendue",
    "hypertendue": "hypertendu",
    "hypertrophié": "hypertrophiée",
    "hypertrophiée": "hypertrophié",
    "hypnotisé": "hypnotisée",
    "hypnotisée": "hypnotisé",
    "hypnotiseur": "hypnotiseuse",
    "hypnotiseuse": "hypnotiseur",
    "hypotendu": "hypotendue",
    "hypotendue": "hypotendu",
    "hypotensif": "hypotensive",
    "hypotensive": "hypotensif",
    "hypothéqué": "hypothéquée",
    "hypothéquée": "hypothéqué",
    "hâlé": "hâlée",
    "hâlée": "hâlé",
    "hâté": "hâtée",
    "hâtée": "hâté",
    "hâtif": "hâtive",
    "hâtive": "hâtif",
    "hébergé": "hébergée",
    "hébergée": "hébergé",
    "hébété": "hébétée",
    "hébétée": "hébété",
    "hégélien": "hégélienne",
    "hégélienne": "hégélien",
    "hélicoïdal": "hélicoïdale",
    "hélicoïdale": "hélicoïdal",
    "héliporté": "héliportée",
    "héliportée": "héliporté",
    "héraclitéen": "héraclitéenne",
    "héraclitéenne": "héraclitéen",
    "hérissé": "hérissée",
    "hérissée": "hérissé",
    "hérité": "héritée",
    "héritée": "hérité",
    "héritier": "héritière",
    "héritière": "héritier",
    "hésitant": "hésitante",
    "hésitante": "hésitant",
    "hétérosexuel": "hétérosexuelle",
    "hétérosexuelle": "hétérosexuel",
    "hôtelier": "hôtelière",
    "hôtelière": "hôtelier",
    "ice cream": "ice-cream",
    "ice-cream": "ice cream",
    "identifié": "identifiée",
    "identifiée": "identifié",
    "idiot": "idiote",
    "idiote": "idiot",
    "idolâtré": "idolâtrée",
    "idolâtrée": "idolâtré",
    "idéal": "idéale",
    "idéale": "idéal",
    "idéalisé": "idéalisée",
    "idéalisée": "idéalisé",
    "ignifugé": "ignifugée",
    "ignifugée": "ignifugé",
    "ignominieuse": "ignominieux",
    "ignominieux": "ignominieuse",
    "ignorant": "ignorante",
    "ignorante": "ignorant",
    "ignoré": "ignorée",
    "ignorée": "ignoré",
    "igné": "ignée",
    "ignée": "igné",
    "illettré": "illettrée",
    "illettrée": "illettré",
    "illimité": "illimitée",
    "illimitée": "illimité",
    "illuminé": "illuminée",
    "illuminée": "illuminé",
    "illustrateur": "illustratrice",
    "illustratrice": "illustrateur",
    "illustré": "illustrée",
    "illustrée": "illustré",
    "illégal": "illégale",
    "illégale": "illégal",
    "imagé": "imagée",
    "imagée": "imagé",
    "imaginatif": "imaginative",
    "imaginative": "imaginatif",
    "imaginé": "imaginée",
    "imaginée": "imaginé",
    "imbibé": "imbibée",
    "imbibée": "imbibé",
    "imbu": "imbue",
    "imbue": "imbu",
    "imitateur": "imitatrice",
    "imitatrice": "imitateur",
    "imité": "imitée",
    "imitée": "imité",
    "immaculé": "immaculée",
    "immaculée": "immaculé",
    "immanent": "immanente",
    "immanente": "immanent",
    "immatriculé": "immatriculée",
    "immatriculée": "immatriculé",
    "immatériel": "immatérielle",
    "immatérielle": "immatériel",
    "immergé": "immergée",
    "immergée": "immergé",
    "immigrant": "immigrante",
    "immigrante": "immigrant",
    "immigré": "immigrée",
    "immigrée": "immigré",
    "imminent": "imminente",
    "imminente": "imminent",
    "immiscé": "immiscée",
    "immiscée": "immiscé",
    "immobilier": "immobilière",
    "immobilière": "immobilier",
    "immobilisé": "immobilisée",
    "immobilisée": "immobilisé",
    "immodéré": "immodérée",
    "immodérée": "immodéré",
    "immolé": "immolée",
    "immolée": "immolé",
    "immoral": "immorale",
    "immorale": "immoral",
    "immortalisé": "immortalisée",
    "immortalisée": "immortalisé",
    "immortel": "immortelle",
    "immortelle": "immortel",
    "immunisé": "immunisée",
    "immunisée": "immunisé",
    "immédiat": "immédiate",
    "immédiate": "immédiat",
    "immémorial": "immémoriale",
    "immémoriale": "immémorial",
    "immérité": "imméritée",
    "imméritée": "immérité",
    "impair": "impaire",
    "impaire": "impair",
    "imparfait": "imparfaite",
    "imparfaite": "imparfait",
    "impartial": "impartiale",
    "impartiale": "impartial",
    "imparti": "impartie",
    "impartie": "imparti",
    "impatient": "impatiente",
    "impatiente": "impatient",
    "impatienté": "impatientée",
    "impatientée": "impatienté",
    "impayé": "impayée",
    "impayée": "impayé",
    "impersonnel": "impersonnelle",
    "impersonnelle": "impersonnel",
    "impertinent": "impertinente",
    "impertinente": "impertinent",
    "implanté": "implantée",
    "implantée": "implanté",
    "impliqué": "impliquée",
    "impliquée": "impliqué",
    "implorant": "implorante",
    "implorante": "implorant",
    "imploré": "implorée",
    "implorée": "imploré",
    "impoli": "impolie",
    "impolie": "impoli",
    "impollué": "impolluée",
    "impolluée": "impollué",
    "important": "importante",
    "importante": "important",
    "importé": "importée",
    "importée": "importé",
    "importun": "importune",
    "importune": "importun",
    "importuné": "importunée",
    "importunée": "importuné",
    "imposant": "imposante",
    "imposante": "imposant",
    "imposé": "imposée",
    "imposée": "imposé",
    "impotent": "impotente",
    "impotente": "impotent",
    "impressionnant": "impressionnante",
    "impressionnante": "impressionnant",
    "impressionné": "impressionnée",
    "impressionnée": "impressionné",
    "imprimé": "imprimée",
    "imprimée": "imprimé",
    "improductif": "improductive",
    "improductive": "improductif",
    "impromptu": "impromptue",
    "impromptue": "impromptu",
    "improvisé": "improvisée",
    "improvisée": "improvisé",
    "imprudent": "imprudente",
    "imprudente": "imprudent",
    "imprégné": "imprégnée",
    "imprégnée": "imprégné",
    "imprévoyant": "imprévoyante",
    "imprévoyante": "imprévoyant",
    "imprévu": "imprévue",
    "imprévue": "imprévu",
    "impudent": "impudente",
    "impudente": "impudent",
    "impuissant": "impuissante",
    "impuissante": "impuissant",
    "impulsif": "impulsive",
    "impulsive": "impulsif",
    "impuni": "impunie",
    "impunie": "impuni",
    "impur": "impure",
    "impure": "impur",
    "imputé": "imputée",
    "imputée": "imputé",
    "impénitent": "impénitente",
    "impénitente": "impénitent",
    "impératif": "impérative",
    "impérative": "impératif",
    "impérial": "impériale",
    "impériale": "impérial",
    "impétrant": "impétrante",
    "impétrante": "impétrant",
    "inabouti": "inaboutie",
    "inaboutie": "inabouti",
    "inaccoutumé": "inaccoutumée",
    "inaccoutumée": "inaccoutumé",
    "inachevé": "inachevée",
    "inachevée": "inachevé",
    "inactif": "inactive",
    "inactive": "inactif",
    "inadapté": "inadaptée",
    "inadaptée": "inadapté",
    "inadéquat": "inadéquate",
    "inadéquate": "inadéquat",
    "inaltéré": "inaltérée",
    "inaltérée": "inaltéré",
    "inamical": "inamicale",
    "inamicale": "inamical",
    "inanimé": "inanimée",
    "inanimée": "inanimé",
    "inaperçu": "inaperçue",
    "inaperçue": "inaperçu",
    "inapproprié": "inappropriée",
    "inappropriée": "inapproprié",
    "inarticulé": "inarticulée",
    "inarticulée": "inarticulé",
    "inassouvi": "inassouvie",
    "inassouvie": "inassouvi",
    "inattendu": "inattendue",
    "inattendue": "inattendu",
    "inattentif": "inattentive",
    "inattentive": "inattentif",
    "inaugural": "inaugurale",
    "inaugurale": "inaugural",
    "inauguré": "inaugurée",
    "inaugurée": "inauguré",
    "inavoué": "inavouée",
    "inavouée": "inavoué",
    "incandescent": "incandescente",
    "incandescente": "incandescent",
    "incapacitant": "incapacitante",
    "incapacitante": "incapacitant",
    "incarcéré": "incarcérée",
    "incarcérée": "incarcéré",
    "incarné": "incarnée",
    "incarnée": "incarné",
    "incendié": "incendiée",
    "incendiée": "incendié",
    "incertain": "incertaine",
    "incertaine": "incertain",
    "incessant": "incessante",
    "incessante": "incessant",
    "inchangé": "inchangée",
    "inchangée": "inchangé",
    "incident": "incidente",
    "incidente": "incident",
    "incinéré": "incinérée",
    "incinérée": "incinéré",
    "incisé": "incisée",
    "incisée": "incisé",
    "incisif": "incisive",
    "incisive": "incisif",
    "incitateur": "incitatrice",
    "incitatrice": "incitateur",
    "incité": "incitée",
    "incitée": "incité",
    "incliné": "inclinée",
    "inclinée": "incliné",
    "inclus": "incluse",
    "incluse": "inclus",
    "inclément": "inclémente",
    "inclémente": "inclément",
    "incohérent": "incohérente",
    "incohérente": "incohérent",
    "incommodant": "incommodante",
    "incommodante": "incommodant",
    "incommodé": "incommodée",
    "incommodée": "incommodé",
    "incomplet": "incomplète",
    "incomplète": "incomplet",
    "incompétent": "incompétente",
    "incompétente": "incompétent",
    "inconditionnel": "inconditionnelle",
    "inconditionnelle": "inconditionnel",
    "inconditionné": "inconditionnée",
    "inconditionnée": "inconditionné",
    "incongru": "incongrue",
    "incongrue": "incongru",
    "inconnu": "inconnue",
    "inconnue": "inconnu",
    "inconscient": "inconsciente",
    "inconsciente": "inconscient",
    "inconsidéré": "inconsidérée",
    "inconsidérée": "inconsidéré",
    "inconsistant": "inconsistante",
    "inconsistante": "inconsistant",
    "inconsolé": "inconsolée",
    "inconsolée": "inconsolé",
    "inconstant": "inconstante",
    "inconstante": "inconstant",
    "inconstitutionnel": "inconstitutionnelle",
    "inconstitutionnelle": "inconstitutionnel",
    "inconséquent": "inconséquente",
    "inconséquente": "inconséquent",
    "incontesté": "incontestée",
    "incontestée": "incontesté",
    "incontinent": "incontinente",
    "incontinente": "incontinent",
    "incontrôlé": "incontrôlée",
    "incontrôlée": "incontrôlé",
    "inconvenant": "inconvenante",
    "inconvenante": "inconvenant",
    "incorporel": "incorporelle",
    "incorporelle": "incorporel",
    "incorporé": "incorporée",
    "incorporée": "incorporé",
    "incorrect": "incorrecte",
    "incorrecte": "incorrect",
    "incriminé": "incriminée",
    "incriminée": "incriminé",
    "incroyant": "incroyante",
    "incroyante": "incroyant",
    "incrusté": "incrustée",
    "incrustée": "incrusté",
    "incréé": "incréée",
    "incréée": "incréé",
    "inculpé": "inculpée",
    "inculpée": "inculpé",
    "inculqué": "inculquée",
    "inculquée": "inculqué",
    "incurvé": "incurvée",
    "incurvée": "incurvé",
    "indexé": "indexée",
    "indexée": "indexé",
    "indicateur": "indicatrice",
    "indicatrice": "indicateur",
    "indien": "indienne",
    "indienne": "indien",
    "indifférencié": "indifférenciée",
    "indifférenciée": "indifférencié",
    "indifférent": "indifférente",
    "indifférente": "indifférent",
    "indigent": "indigente",
    "indigente": "indigent",
    "indigné": "indignée",
    "indignée": "indigné",
    "indiqué": "indiquée",
    "indiquée": "indiqué",
    "indirect": "indirecte",
    "indirecte": "indirect",
    "indiscipliné": "indisciplinée",
    "indisciplinée": "indiscipliné",
    "indiscret": "indiscrète",
    "indiscrète": "indiscret",
    "indiscuté": "indiscutée",
    "indiscutée": "indiscuté",
    "indisposé": "indisposée",
    "indisposée": "indisposé",
    "indistinct": "indistincte",
    "indistincte": "indistinct",
    "individuel": "individuelle",
    "individuelle": "individuel",
    "indo-européen": "indo-européenne",
    "indo-européenne": "indo-européen",
    "indolent": "indolente",
    "indolente": "indolent",
    "indompté": "indomptée",
    "indomptée": "indompté",
    "indonésien": "indonésienne",
    "indonésienne": "indonésien",
    "indu": "indue",
    "indue": "indu",
    "inductif": "inductive",
    "inductive": "inductif",
    "induit": "induite",
    "induite": "induit",
    "indulgent": "indulgente",
    "indulgente": "indulgent",
    "industrialisé": "industrialisée",
    "industrialisée": "industrialisé",
    "industriel": "industrielle",
    "industrielle": "industriel",
    "indécent": "indécente",
    "indécente": "indécent",
    "indéfini": "indéfinie",
    "indéfinie": "indéfini",
    "indélicat": "indélicate",
    "indélicate": "indélicat",
    "indépendant": "indépendante",
    "indépendante": "indépendant",
    "indéterminé": "indéterminée",
    "indéterminée": "indéterminé",
    "inemployé": "inemployée",
    "inemployée": "inemployé",
    "inentamé": "inentamée",
    "inentamée": "inentamé",
    "inertiel": "inertielle",
    "inertielle": "inertiel",
    "inespéré": "inespérée",
    "inespérée": "inespéré",
    "inexact": "inexacte",
    "inexacte": "inexact",
    "inexistant": "inexistante",
    "inexistante": "inexistant",
    "inexpliqué": "inexpliquée",
    "inexpliquée": "inexpliqué",
    "inexploité": "inexploitée",
    "inexploitée": "inexploité",
    "inexploré": "inexplorée",
    "inexplorée": "inexploré",
    "inexpressif": "inexpressive",
    "inexpressive": "inexpressif",
    "inexprimé": "inexprimée",
    "inexprimée": "inexprimé",
    "inexpérimenté": "inexpérimentée",
    "inexpérimentée": "inexpérimenté",
    "infamant": "infamante",
    "infamante": "infamant",
    "infant": "infante",
    "infante": "infant",
    "infect": "infecte",
    "infecte": "infect",
    "infecté": "infectée",
    "infectée": "infecté",
    "infernal": "infernale",
    "infernale": "infernal",
    "infesté": "infestée",
    "infestée": "infesté",
    "infiltré": "infiltrée",
    "infiltrée": "infiltré",
    "infini": "infinie",
    "infinie": "infini",
    "infinitésimal": "infinitésimale",
    "infinitésimale": "infinitésimal",
    "infirmier": "infirmière",
    "infirmière": "infirmier",
    "infligé": "infligée",
    "infligée": "infligé",
    "influencé": "influencée",
    "influencée": "influencé",
    "influent": "influente",
    "influente": "influent",
    "infléchi": "infléchie",
    "infléchie": "infléchi",
    "infondé": "infondée",
    "infondée": "infondé",
    "informateur": "informatrice",
    "informatrice": "informateur",
    "informaticien": "informaticienne",
    "informaticienne": "informaticien",
    "informatif": "informative",
    "informative": "informatif",
    "informatisé": "informatisée",
    "informatisée": "informatisé",
    "informel": "informelle",
    "informelle": "informel",
    "informé": "informée",
    "informée": "informé",
    "informulé": "informulée",
    "informulée": "informulé",
    "infortuné": "infortunée",
    "infortunée": "infortuné",
    "infoutu": "infoutue",
    "infoutue": "infoutu",
    "infus": "infuse",
    "infuse": "infus",
    "infusé": "infusée",
    "infusée": "infusé",
    "inférieur": "inférieure",
    "inférieure": "inférieur",
    "ingrat": "ingrate",
    "ingrate": "ingrat",
    "ingurgité": "ingurgitée",
    "ingurgitée": "ingurgité",
    "ingénu": "ingénue",
    "ingénue": "ingénu",
    "ingéré": "ingérée",
    "ingérée": "ingéré",
    "inhabituel": "inhabituelle",
    "inhabituelle": "inhabituel",
    "inhabité": "inhabitée",
    "inhabitée": "inhabité",
    "inhibé": "inhibée",
    "inhibée": "inhibé",
    "inhibiteur": "inhibitrice",
    "inhibitrice": "inhibiteur",
    "inhospitalier": "inhospitalière",
    "inhospitalière": "inhospitalier",
    "inhumain": "inhumaine",
    "inhumaine": "inhumain",
    "inhumé": "inhumée",
    "inhumée": "inhumé",
    "inhérent": "inhérente",
    "inhérente": "inhérent",
    "ininterrompu": "ininterrompue",
    "ininterrompue": "ininterrompu",
    "inintéressant": "inintéressante",
    "inintéressante": "inintéressant",
    "initial": "initiale",
    "initiale": "initial",
    "initialisé": "initialisée",
    "initialisée": "initialisé",
    "initiateur": "initiatrice",
    "initiatrice": "initiateur",
    "initié": "initiée",
    "initiée": "initié",
    "injecté": "injectée",
    "injectée": "injecté",
    "injurié": "injuriée",
    "injuriée": "injurié",
    "injustifié": "injustifiée",
    "injustifiée": "injustifié",
    "innocent": "innocente",
    "innocente": "innocent",
    "innocenté": "innocentée",
    "innocentée": "innocenté",
    "innommé": "innommée",
    "innommée": "innommé",
    "innovant": "innovante",
    "innovante": "innovant",
    "innovateur": "innovatrice",
    "innovatrice": "innovateur",
    "inné": "innée",
    "innée": "inné",
    "inoccupé": "inoccupée",
    "inoccupée": "inoccupé",
    "inoculé": "inoculée",
    "inoculée": "inoculé",
    "inoffensif": "inoffensive",
    "inoffensive": "inoffensif",
    "inondé": "inondée",
    "inondée": "inondé",
    "inopiné": "inopinée",
    "inopinée": "inopiné",
    "inopportun": "inopportune",
    "inopportune": "inopportun",
    "inopérant": "inopérante",
    "inopérante": "inopérant",
    "inorganisé": "inorganisée",
    "inorganisée": "inorganisé",
    "inouï": "inouïe",
    "inouïe": "inouï",
    "inquiet": "inquiète",
    "inquiète": "inquiet",
    "inquisiteur": "inquisitrice",
    "inquisitrice": "inquisiteur",
    "inquisitorial": "inquisitoriale",
    "inquisitoriale": "inquisitorial",
    "inquiétant": "inquiétante",
    "inquiétante": "inquiétant",
    "inquiété": "inquiétée",
    "inquiétée": "inquiété",
    "insatisfaisant": "insatisfaisante",
    "insatisfaisante": "insatisfaisant",
    "insatisfait": "insatisfaite",
    "insatisfaite": "insatisfait",
    "inscrit": "inscrite",
    "inscrite": "inscrit",
    "insensibilisé": "insensibilisée",
    "insensibilisée": "insensibilisé",
    "insensé": "insensée",
    "insensée": "insensé",
    "insignifiant": "insignifiante",
    "insignifiante": "insignifiant",
    "insinuant": "insinuante",
    "insinuante": "insinuant",
    "insinué": "insinuée",
    "insinuée": "insinué",
    "insistant": "insistante",
    "insistante": "insistant",
    "insolent": "insolente",
    "insolente": "insolent",
    "insonorisé": "insonorisée",
    "insonorisée": "insonorisé",
    "insouciant": "insouciante",
    "insouciante": "insouciant",
    "insoumis": "insoumise",
    "insoumise": "insoumis",
    "insoupçonné": "insoupçonnée",
    "insoupçonnée": "insoupçonné",
    "inspecté": "inspectée",
    "inspectée": "inspecté",
    "inspecteur": "inspectrice",
    "inspectrice": "inspecteur",
    "inspirant": "inspirante",
    "inspirante": "inspirant",
    "inspirateur": "inspiratrice",
    "inspiratrice": "inspirateur",
    "inspiré": "inspirée",
    "inspirée": "inspiré",
    "installé": "installée",
    "installée": "installé",
    "instant": "instante",
    "instante": "instant",
    "instantané": "instantanée",
    "instantanée": "instantané",
    "instauré": "instaurée",
    "instaurée": "instauré",
    "instigateur": "instigatrice",
    "instigatrice": "instigateur",
    "instinctif": "instinctive",
    "instinctive": "instinctif",
    "institué": "instituée",
    "instituée": "institué",
    "instituteur": "institutrice",
    "institutrice": "instituteur",
    "institutionnel": "institutionnelle",
    "institutionnelle": "institutionnel",
    "instructif": "instructive",
    "instructive": "instructif",
    "instruit": "instruite",
    "instruite": "instruit",
    "instrumental": "instrumentale",
    "instrumentale": "instrumental",
    "insubordonné": "insubordonnée",
    "insubordonnée": "insubordonné",
    "insuffisant": "insuffisante",
    "insuffisante": "insuffisant",
    "insufflé": "insufflée",
    "insufflée": "insufflé",
    "insultant": "insultante",
    "insultante": "insultant",
    "insulté": "insultée",
    "insultée": "insulté",
    "insurgé": "insurgée",
    "insurgée": "insurgé",
    "insurrectionnel": "insurrectionnelle",
    "insurrectionnelle": "insurrectionnel",
    "inséminé": "inséminée",
    "inséminée": "inséminé",
    "inséré": "insérée",
    "insérée": "inséré",
    "intact": "intacte",
    "intacte": "intact",
    "intellectuel": "intellectuelle",
    "intellectuelle": "intellectuel",
    "intelligent": "intelligente",
    "intelligente": "intelligent",
    "intempestif": "intempestive",
    "intempestive": "intempestif",
    "intemporel": "intemporelle",
    "intemporelle": "intemporel",
    "intendant": "intendante",
    "intendante": "intendant",
    "intensif": "intensive",
    "intensive": "intensif",
    "intensifié": "intensifiée",
    "intensifiée": "intensifié",
    "intenté": "intentée",
    "intentée": "intenté",
    "intentionnel": "intentionnelle",
    "intentionnelle": "intentionnel",
    "intentionné": "intentionnée",
    "intentionnée": "intentionné",
    "interactif": "interactive",
    "interactive": "interactif",
    "interallié": "interalliée",
    "interalliée": "interallié",
    "intercepté": "interceptée",
    "interceptée": "intercepté",
    "interconnecté": "interconnectée",
    "interconnectée": "interconnecté",
    "intercontinental": "intercontinentale",
    "intercontinentale": "intercontinental",
    "intercostal": "intercostale",
    "intercostale": "intercostal",
    "interdit": "interdite",
    "interdite": "interdit",
    "intergouvernemental": "intergouvernementale",
    "intergouvernementale": "intergouvernemental",
    "interlocuteur": "interlocutrice",
    "interlocutrice": "interlocuteur",
    "interloqué": "interloquée",
    "interloquée": "interloqué",
    "intermittent": "intermittente",
    "intermittente": "intermittent",
    "international": "internationale",
    "internationale": "international",
    "interné": "internée",
    "internée": "interné",
    "interpellé": "interpellée",
    "interpellée": "interpellé",
    "interposé": "interposée",
    "interposée": "interposé",
    "interprétatif": "interprétative",
    "interprétative": "interprétatif",
    "interprété": "interprétée",
    "interprétée": "interprété",
    "interracial": "interraciale",
    "interraciale": "interracial",
    "interrogateur": "interrogatrice",
    "interrogatrice": "interrogateur",
    "interrogatif": "interrogative",
    "interrogative": "interrogatif",
    "interrogé": "interrogée",
    "interrogée": "interrogé",
    "interrompu": "interrompue",
    "interrompue": "interrompu",
    "intersidéral": "intersidérale",
    "intersidérale": "intersidéral",
    "interurbain": "interurbaine",
    "interurbaine": "interurbain",
    "intervenant": "intervenante",
    "intervenante": "intervenant",
    "intervenu": "intervenue",
    "intervenue": "intervenu",
    "interviewé": "interviewée",
    "interviewée": "interviewé",
    "intestin": "intestine",
    "intestine": "intestin",
    "intestinal": "intestinale",
    "intestinale": "intestinal",
    "intimidant": "intimidante",
    "intimidante": "intimidant",
    "intimidé": "intimidée",
    "intimidée": "intimidé",
    "intitulé": "intitulée",
    "intitulée": "intitulé",
    "intolérant": "intolérante",
    "intolérante": "intolérant",
    "intouché": "intouchée",
    "intouchée": "intouché",
    "intoxiqué": "intoxiquée",
    "intoxiquée": "intoxiqué",
    "intracrânien": "intracrânienne",
    "intracrânienne": "intracrânien",
    "intransigeant": "intransigeante",
    "intransigeante": "intransigeant",
    "intrigant": "intrigante",
    "intrigante": "intrigant",
    "intrigué": "intriguée",
    "intriguée": "intrigué",
    "introducteur": "introductrice",
    "introductrice": "introducteur",
    "introduit": "introduite",
    "introduite": "introduit",
    "introspectif": "introspective",
    "introspective": "introspectif",
    "introverti": "introvertie",
    "introvertie": "introverti",
    "intrus": "intruse",
    "intruse": "intrus",
    "intubé": "intubée",
    "intubée": "intubé",
    "intuitif": "intuitive",
    "intuitive": "intuitif",
    "intégral": "intégrale",
    "intégrale": "intégral",
    "intégré": "intégrée",
    "intégrée": "intégré",
    "intéressant": "intéressante",
    "intéressante": "intéressant",
    "intéressé": "intéressée",
    "intéressée": "intéressé",
    "intérieur": "intérieure",
    "intérieure": "intérieur",
    "inusité": "inusitée",
    "inusitée": "inusité",
    "inutilisé": "inutilisée",
    "inutilisée": "inutilisé",
    "invaincu": "invaincue",
    "invaincue": "invaincu",
    "invalidant": "invalidante",
    "invalidante": "invalidant",
    "invasif": "invasive",
    "invasive": "invasif",
    "inventé": "inventée",
    "inventée": "inventé",
    "inventif": "inventive",
    "inventive": "inventif",
    "inventorié": "inventoriée",
    "inventoriée": "inventorié",
    "inversé": "inversée",
    "inversée": "inversé",
    "invertébré": "invertébrée",
    "invertébrée": "invertébré",
    "investi": "investie",
    "investie": "investi",
    "investigateur": "investigatrice",
    "investigatrice": "investigateur",
    "inviolé": "inviolée",
    "inviolée": "inviolé",
    "invitant": "invitante",
    "invitante": "invitant",
    "invité": "invitée",
    "invitée": "invité",
    "invoqué": "invoquée",
    "invoquée": "invoqué",
    "invétéré": "invétérée",
    "invétérée": "invétéré",
    "inédit": "inédite",
    "inédite": "inédit",
    "inégal": "inégale",
    "inégale": "inégal",
    "inégalé": "inégalée",
    "inégalée": "inégalé",
    "iodé": "iodée",
    "iodée": "iodé",
    "ionisant": "ionisante",
    "ionisante": "ionisant",
    "ionisé": "ionisée",
    "ionisée": "ionisé",
    "irakien": "irakienne",
    "irakienne": "irakien",
    "iranien": "iranienne",
    "iranienne": "iranien",
    "iraquien": "iraquienne",
    "iraquienne": "iraquien",
    "irisé": "irisée",
    "irisée": "irisé",
    "irradiant": "irradiante",
    "irradiante": "irradiant",
    "irradié": "irradiée",
    "irradiée": "irradié",
    "irraisonné": "irraisonnée",
    "irraisonnée": "irraisonné",
    "irrationnel": "irrationnelle",
    "irrationnelle": "irrationnel",
    "irrigué": "irriguée",
    "irriguée": "irrigué",
    "irritant": "irritante",
    "irritante": "irritant",
    "irrité": "irritée",
    "irritée": "irrité",
    "irréel": "irréelle",
    "irréelle": "irréel",
    "irréfléchi": "irréfléchie",
    "irréfléchie": "irréfléchi",
    "irrégulier": "irrégulière",
    "irrégulière": "irrégulier",
    "irréligieuse": "irréligieux",
    "irréligieux": "irréligieuse",
    "irrésolu": "irrésolue",
    "irrésolue": "irrésolu",
    "irrévérencieuse": "irrévérencieux",
    "irrévérencieux": "irrévérencieuse",
    "islandais": "islandaise",
    "islandaise": "islandais",
    "isolant": "isolante",
    "isolante": "isolant",
    "isolé": "isolée",
    "isolée": "isolé",
    "israélien": "israélienne",
    "israélienne": "israélien",
    "issu": "issue",
    "issue": "issu",
    "italien": "italienne",
    "italienne": "italien",
    "italo-américain": "italo-américaine",
    "italo-américaine": "italo-américain",
    "itinérant": "itinérante",
    "itinérante": "itinérant",
    "jacassant": "jacassante",
    "jacassante": "jacassant",
    "jacasseur": "jacasseuse",
    "jacasseuse": "jacasseur",
    "jacobin": "jacobine",
    "jacobine": "jacobin",
    "jailli": "jaillie",
    "jaillie": "jailli",
    "jaillissant": "jaillissante",
    "jaillissante": "jaillissant",
    "jalonné": "jalonnée",
    "jalonnée": "jalonné",
    "jalousé": "jalousée",
    "jalousée": "jalousé",
    "jamaïcain": "jamaïcaine",
    "jamaïcaine": "jamaïcain",
    "jaugé": "jaugée",
    "jaugée": "jaugé",
    "jauni": "jaunie",
    "jaunie": "jauni",
    "jaunissant": "jaunissante",
    "jaunissante": "jaunissant",
    "javanais": "javanaise",
    "javanaise": "javanais",
    "javellisé": "javellisée",
    "javellisée": "javellisé",
    "jeté": "jetée",
    "jetée": "jeté",
    "jeunet": "jeunette",
    "jeunette": "jeunet",
    "joggeur": "joggeuse",
    "joggeuse": "joggeur",
    "joint": "jointe",
    "jointe": "joint",
    "joli": "jolie",
    "jolie": "joli",
    "joliet": "joliette",
    "joliette": "joliet",
    "jonché": "jonchée",
    "jonchée": "jonché",
    "jongleur": "jongleuse",
    "jongleuse": "jongleur",
    "joué": "jouée",
    "jouée": "joué",
    "joueur": "joueuse",
    "joueuse": "joueur",
    "joufflu": "joufflue",
    "joufflue": "joufflu",
    "jouisseur": "jouisseuse",
    "jouisseuse": "jouisseur",
    "jouissif": "jouissive",
    "jouissive": "jouissif",
    "journalier": "journalière",
    "journalière": "journalier",
    "jouteur": "jouteuse",
    "jouteuse": "jouteur",
    "jouvenceau": "jouvencelle",
    "jouvencelle": "jouvenceau",
    "jovial": "joviale",
    "joviale": "jovial",
    "jubilant": "jubilante",
    "jubilante": "jubilant",
    "jubilé": "jubilée",
    "jubilée": "jubilé",
    "juché": "juchée",
    "juchée": "juché",
    "jugé": "jugée",
    "jugée": "jugé",
    "jugulé": "jugulée",
    "jugulée": "jugulé",
    "juif": "juive",
    "juive": "juif",
    "jumeau": "jumelle",
    "jumelle": "jumeau",
    "jumelé": "jumelée",
    "jumelée": "jumelé",
    "jupitérien": "jupitérienne",
    "jupitérienne": "jupitérien",
    "juré": "jurée",
    "jurée": "juré",
    "juridictionnel": "juridictionnelle",
    "juridictionnelle": "juridictionnel",
    "justicier": "justicière",
    "justicière": "justicier",
    "justificatif": "justificative",
    "justificative": "justificatif",
    "justifié": "justifiée",
    "justifiée": "justifié",
    "khmer": "khmère",
    "khmère": "khmer",
    "kidnappé": "kidnappée",
    "kidnappée": "kidnappé",
    "kolkhozien": "kolkhozienne",
    "kolkhozienne": "kolkhozien",
    "kung fu": "kung-fu",
    "kung-fu": "kung fu",
    "labial": "labiale",
    "labiale": "labial",
    "laborantin": "laborantine",
    "laborantine": "laborantin",
    "labouré": "labourée",
    "labourée": "labouré",
    "lacé": "lacée",
    "lacée": "lacé",
    "lacryma christi": "lacryma-christi",
    "lacryma-christi": "lacryma christi",
    "lacrymal": "lacrymale",
    "lacrymale": "lacrymal",
    "lactescent": "lactescente",
    "lactescente": "lactescent",
    "lacté": "lactée",
    "lactée": "lacté",
    "lacéré": "lacérée",
    "lacérée": "lacéré",
    "laid": "laide",
    "laide": "laid",
    "laineuse": "laineux",
    "laineux": "laineuse",
    "lainier": "lainière",
    "lainière": "lainier",
    "laissé": "laissée",
    "laissée": "laissé",
    "laitier": "laitière",
    "laitière": "laitier",
    "lambrissé": "lambrissée",
    "lambrissée": "lambrissé",
    "laminé": "laminée",
    "laminée": "laminé",
    "lampé": "lampée",
    "lampée": "lampé",
    "lamé": "lamée",
    "lamée": "lamé",
    "lancé": "lancée",
    "lancée": "lancé",
    "lanceur": "lanceuse",
    "lanceuse": "lanceur",
    "lancinant": "lancinante",
    "lancinante": "lancinant",
    "langé": "langée",
    "langée": "langé",
    "languissant": "languissante",
    "languissante": "languissant",
    "laotien": "laotienne",
    "laotienne": "laotien",
    "lapidé": "lapidée",
    "lapidée": "lapidé",
    "lapin": "lapine",
    "lapine": "lapin",
    "lapon": "laponne",
    "laponne": "lapon",
    "laqué": "laquée",
    "laquée": "laqué",
    "lardé": "lardée",
    "lardée": "lardé",
    "largué": "larguée",
    "larguée": "largué",
    "larmoyant": "larmoyante",
    "larmoyante": "larmoyant",
    "larvé": "larvée",
    "larvée": "larvé",
    "laryngé": "laryngée",
    "laryngée": "laryngé",
    "lascif": "lascive",
    "lascive": "lascif",
    "lassant": "lassante",
    "lassante": "lassant",
    "lassé": "lassée",
    "lassée": "lassé",
    "latent": "latente",
    "latente": "latent",
    "latin": "latine",
    "latine": "latin",
    "latinisé": "latinisée",
    "latinisée": "latinisé",
    "latino-américain": "latino-américaine",
    "latino-américaine": "latino-américain",
    "latéral": "latérale",
    "latérale": "latéral",
    "lauréat": "lauréate",
    "lauréate": "lauréat",
    "lavé": "lavée",
    "lavée": "lavé",
    "laveur": "laveuse",
    "laveuse": "laveur",
    "laxatif": "laxative",
    "laxative": "laxatif",
    "lecteur": "lectrice",
    "lectrice": "lecteur",
    "lent": "lente",
    "lente": "lent",
    "lesbien": "lesbienne",
    "lesbienne": "lesbien",
    "lessivé": "lessivée",
    "lessivée": "lessivé",
    "lesté": "lestée",
    "lestée": "lesté",
    "lettré": "lettrée",
    "lettrée": "lettré",
    "levant": "levante",
    "levante": "levant",
    "levé": "levée",
    "levée": "levé",
    "leveur": "leveuse",
    "leveuse": "leveur",
    "liant": "liante",
    "liante": "liant",
    "libellé": "libellée",
    "libellée": "libellé",
    "libertin": "libertine",
    "libertine": "libertin",
    "libidinal": "libidinale",
    "libidinale": "libidinal",
    "libre-penseur": "libre-penseuse",
    "libre-penseuse": "libre-penseur",
    "libyen": "libyenne",
    "libyenne": "libyen",
    "libéral": "libérale",
    "libérale": "libéral",
    "libérateur": "libératrice",
    "libératrice": "libérateur",
    "libéré": "libérée",
    "libérée": "libéré",
    "licencié": "licenciée",
    "licenciée": "licencié",
    "liché": "lichée",
    "lichée": "liché",
    "lié": "liée",
    "liée": "lié",
    "lieur": "lieuse",
    "lieuse": "lieur",
    "liftier": "liftière",
    "liftière": "liftier",
    "lifté": "liftée",
    "liftée": "lifté",
    "ligaturé": "ligaturée",
    "ligaturée": "ligaturé",
    "ligné": "lignée",
    "lignée": "ligné",
    "ligoté": "ligotée",
    "ligotée": "ligoté",
    "lilial": "liliale",
    "liliale": "lilial",
    "lilliputien": "lilliputienne",
    "lilliputienne": "lilliputien",
    "limé": "limée",
    "limée": "limé",
    "limitatif": "limitative",
    "limitative": "limitatif",
    "limité": "limitée",
    "limitée": "limité",
    "limonadier": "limonadière",
    "limonadière": "limonadier",
    "limoneuse": "limoneux",
    "limoneux": "limoneuse",
    "limonier": "limonière",
    "limonière": "limonier",
    "limousin": "limousine",
    "limousine": "limousin",
    "linger": "lingère",
    "lingère": "linger",
    "lingual": "linguale",
    "linguale": "lingual",
    "lion": "lionne",
    "lionne": "lion",
    "lippu": "lippue",
    "lippue": "lippu",
    "liquidé": "liquidée",
    "liquidée": "liquidé",
    "liquéfié": "liquéfiée",
    "liquéfiée": "liquéfié",
    "lu": "lue",
    "lue": "lu",
    "liseré": "liserée",
    "liserée": "liseré",
    "liseur": "liseuse",
    "liseuse": "liseur",
    "lissé": "lissée",
    "lissée": "lissé",
    "listé": "listée",
    "listée": "listé",
    "lithuanien": "lithuanienne",
    "lithuanienne": "lithuanien",
    "litigieuse": "litigieux",
    "litigieux": "litigieuse",
    "littoral": "littorale",
    "littorale": "littoral",
    "littéral": "littérale",
    "littérale": "littéral",
    "lituanien": "lituanienne",
    "lituanienne": "lituanien",
    "livré": "livrée",
    "livrée": "livré",
    "livreur": "livreuse",
    "livreuse": "livreur",
    "lobotomisé": "lobotomisée",
    "lobotomisée": "lobotomisé",
    "local": "locale",
    "locale": "local",
    "localisé": "localisée",
    "localisée": "localisé",
    "locatif": "locative",
    "locative": "locatif",
    "locuteur": "locutrice",
    "locutrice": "locuteur",
    "logé": "logée",
    "logée": "logé",
    "logeur": "logeuse",
    "logeuse": "logeur",
    "logicien": "logicienne",
    "logicienne": "logicien",
    "lointain": "lointaine",
    "lointaine": "lointain",
    "lombard": "lombarde",
    "lombarde": "lombard",
    "londonien": "londonienne",
    "londonienne": "londonien",
    "long": "longue",
    "longue": "long",
    "longé": "longée",
    "longée": "longé",
    "longitudinal": "longitudinale",
    "longitudinale": "longitudinal",
    "longuet": "longuette",
    "longuette": "longuet",
    "lorrain": "lorraine",
    "lorraine": "lorrain",
    "losangé": "losangée",
    "losangée": "losangé",
    "loti": "lotie",
    "lotie": "loti",
    "louangé": "louangée",
    "louangée": "louangé",
    "loubard": "loubarde",
    "loubarde": "loubard",
    "louché": "louchée",
    "louchée": "louché",
    "loucheur": "loucheuse",
    "loucheuse": "loucheur",
    "loué": "louée",
    "louée": "loué",
    "loueur": "loueuse",
    "loueuse": "loueur",
    "louis-philippard": "louis-philipparde",
    "louis-philipparde": "louis-philippard",
    "loup": "louve",
    "louve": "loup",
    "loupé": "loupée",
    "loupée": "loupé",
    "lourd": "lourde",
    "lourde": "lourd",
    "lourdaud": "lourdaude",
    "lourdaude": "lourdaud",
    "lourdé": "lourdée",
    "lourdée": "lourdé",
    "lové": "lovée",
    "lovée": "lové",
    "loyal": "loyale",
    "loyale": "loyal",
    "lubrifiant": "lubrifiante",
    "lubrifiante": "lubrifiant",
    "lubrifié": "lubrifiée",
    "lubrifiée": "lubrifié",
    "luciférien": "luciférienne",
    "luciférienne": "luciférien",
    "lucratif": "lucrative",
    "lucrative": "lucratif",
    "luisant": "luisante",
    "luisante": "luisant",
    "luminescent": "luminescente",
    "luminescente": "luminescent",
    "luné": "lunée",
    "lunée": "luné",
    "luron": "luronne",
    "luronne": "luron",
    "lustral": "lustrale",
    "lustrale": "lustral",
    "lustré": "lustrée",
    "lustrée": "lustré",
    "lustreur": "lustreuse",
    "lustreuse": "lustreur",
    "luthérien": "luthérienne",
    "luthérienne": "luthérien",
    "lutin": "lutine",
    "lutine": "lutin",
    "lutiné": "lutinée",
    "lutinée": "lutiné",
    "lutteur": "lutteuse",
    "lutteuse": "lutteur",
    "luxuriant": "luxuriante",
    "luxuriante": "luxuriant",
    "lycéen": "lycéenne",
    "lycéenne": "lycéen",
    "lynché": "lynchée",
    "lynchée": "lynché",
    "lyophilisé": "lyophilisée",
    "lyophilisée": "lyophilisé",
    "lâché": "lâchée",
    "lâchée": "lâché",
    "lâcheur": "lâcheuse",
    "lâcheuse": "lâcheur",
    "léché": "léchée",
    "léchée": "léché",
    "lécheur": "lécheuse",
    "lécheuse": "lécheur",
    "légal": "légale",
    "légale": "légal",
    "légalisé": "légalisée",
    "légalisée": "légalisé",
    "léger": "légère",
    "légère": "léger",
    "législatif": "législative",
    "législative": "législatif",
    "légitimé": "légitimée",
    "légitimée": "légitimé",
    "légué": "léguée",
    "léguée": "légué",
    "lénifiant": "lénifiante",
    "lénifiante": "lénifiant",
    "léonin": "léonine",
    "léonine": "léonin",
    "lésé": "lésée",
    "lésée": "lésé",
    "létal": "létale",
    "létale": "létal",
    "lézard": "lézarde",
    "lézarde": "lézard",
    "lézardé": "lézardée",
    "lézardée": "lézardé",
    "maboul": "maboule",
    "maboule": "maboul",
    "machinal": "machinale",
    "machinale": "machinal",
    "machiné": "machinée",
    "machinée": "machiné",
    "maculé": "maculée",
    "maculée": "maculé",
    "macédonien": "macédonienne",
    "macédonienne": "macédonien",
    "macéré": "macérée",
    "macérée": "macéré",
    "madré": "madrée",
    "madrée": "madré",
    "mafflu": "mafflue",
    "mafflue": "mafflu",
    "magasinier": "magasinière",
    "magasinière": "magasinier",
    "maghrébin": "maghrébine",
    "maghrébine": "maghrébin",
    "magicien": "magicienne",
    "magicienne": "magicien",
    "magistral": "magistrale",
    "magistrale": "magistral",
    "magistrat": "magistrate",
    "magistrate": "magistrat",
    "magnifié": "magnifiée",
    "magnifiée": "magnifié",
    "magnétisé": "magnétisée",
    "magnétisée": "magnétisé",
    "magyar": "magyare",
    "magyare": "magyar",
    "mahométan": "mahométane",
    "mahométane": "mahométan",
    "maigrelet": "maigrelette",
    "maigrelette": "maigrelet",
    "maigrichon": "maigrichonne",
    "maigrichonne": "maigrichon",
    "maigri": "maigrie",
    "maigrie": "maigri",
    "maintenu": "maintenue",
    "maintenue": "maintenu",
    "maire": "mairesse",
    "mairesse": "maire",
    "majeur": "majeure",
    "majeure": "majeur",
    "mal": "male",
    "male": "mal",
    "mal-aimé": "mal-aimée",
    "mal-aimée": "mal-aimé",
    "mal-baisé": "mal-baisée",
    "mal-baisée": "mal-baisé",
    "maladif": "maladive",
    "maladive": "maladif",
    "maladroit": "maladroite",
    "maladroite": "maladroit",
    "malaisien": "malaisienne",
    "malaisienne": "malaisien",
    "malaisé": "malaisée",
    "malaisée": "malaisé",
    "malappris": "malapprise",
    "malapprise": "malappris",
    "malavisé": "malavisée",
    "malavisée": "malavisé",
    "malaxé": "malaxée",
    "malaxée": "malaxé",
    "malentendant": "malentendante",
    "malentendante": "malentendant",
    "malfaisant": "malfaisante",
    "malfaisante": "malfaisant",
    "malfamé": "malfamée",
    "malfamée": "malfamé",
    "maligne": "malin",
    "malin": "maligne",
    "malintentionné": "malintentionnée",
    "malintentionnée": "malintentionné",
    "malmené": "malmenée",
    "malmenée": "malmené",
    "malodorant": "malodorante",
    "malodorante": "malodorant",
    "malouin": "malouine",
    "malouine": "malouin",
    "malpoli": "malpolie",
    "malpolie": "malpoli",
    "malsain": "malsaine",
    "malsaine": "malsain",
    "malséant": "malséante",
    "malséante": "malséant",
    "malté": "maltée",
    "maltée": "malté",
    "maltraité": "maltraitée",
    "maltraitée": "maltraité",
    "malveillant": "malveillante",
    "malveillante": "malveillant",
    "malvenu": "malvenue",
    "malvenue": "malvenu",
    "mamelu": "mamelue",
    "mamelue": "mamelu",
    "manchot": "manchote",
    "manchote": "manchot",
    "mandaté": "mandatée",
    "mandatée": "mandaté",
    "mandé": "mandée",
    "mandée": "mandé",
    "mangé": "mangée",
    "mangée": "mangé",
    "mangeur": "mangeuse",
    "mangeuse": "mangeur",
    "maniaco-dépressif": "maniaco-dépressive",
    "maniaco-dépressive": "maniaco-dépressif",
    "maniacodépressif": "maniacodépressive",
    "maniacodépressive": "maniacodépressif",
    "manié": "maniée",
    "maniée": "manié",
    "manieur": "manieuse",
    "manieuse": "manieur",
    "manifestant": "manifestante",
    "manifestante": "manifestant",
    "manifesté": "manifestée",
    "manifestée": "manifesté",
    "manigancé": "manigancée",
    "manigancée": "manigancé",
    "manip": "manipe",
    "manipe": "manip",
    "manipulateur": "manipulatrice",
    "manipulatrice": "manipulateur",
    "manipulé": "manipulée",
    "manipulée": "manipulé",
    "maniéré": "maniérée",
    "maniérée": "maniéré",
    "manoeuvré": "manoeuvrée",
    "manoeuvrée": "manoeuvré",
    "manoeuvrier": "manoeuvrière",
    "manoeuvrière": "manoeuvrier",
    "manquant": "manquante",
    "manquante": "manquant",
    "manqué": "manquée",
    "manquée": "manqué",
    "mansardé": "mansardée",
    "mansardée": "mansardé",
    "manucuré": "manucurée",
    "manucurée": "manucuré",
    "manuel": "manuelle",
    "manuelle": "manuel",
    "manufacturé": "manufacturée",
    "manufacturée": "manufacturé",
    "manuscrit": "manuscrite",
    "manuscrite": "manuscrit",
    "maori": "maorie",
    "maorie": "maori",
    "maqué": "maquée",
    "maquée": "maqué",
    "maquereau": "maquerelle",
    "maquerelle": "maquereau",
    "maquillé": "maquillée",
    "maquillée": "maquillé",
    "maquilleur": "maquilleuse",
    "maquilleuse": "maquilleur",
    "maquisard": "maquisarde",
    "maquisarde": "maquisard",
    "marathonien": "marathonienne",
    "marathonienne": "marathonien",
    "maraud": "maraude",
    "maraude": "maraud",
    "maraudeur": "maraudeuse",
    "maraudeuse": "maraudeur",
    "maraîcher": "maraîchère",
    "maraîchère": "maraîcher",
    "marbré": "marbrée",
    "marbrée": "marbré",
    "marchand": "marchande",
    "marchande": "marchand",
    "marchandé": "marchandée",
    "marchandée": "marchandé",
    "marchant": "marchante",
    "marchante": "marchant",
    "marcheur": "marcheuse",
    "marcheuse": "marcheur",
    "mareyeur": "mareyeuse",
    "mareyeuse": "mareyeur",
    "marginal": "marginale",
    "marginale": "marginal",
    "marginalisé": "marginalisée",
    "marginalisée": "marginalisé",
    "marié": "mariée",
    "mariée": "marié",
    "marieur": "marieuse",
    "marieuse": "marieur",
    "mariné": "marinée",
    "marinée": "mariné",
    "marital": "maritale",
    "maritale": "marital",
    "marmonné": "marmonnée",
    "marmonnée": "marmonné",
    "marmoréen": "marmoréenne",
    "marmoréenne": "marmoréen",
    "marocain": "marocaine",
    "marocaine": "marocain",
    "marquant": "marquante",
    "marquante": "marquant",
    "marqué": "marquée",
    "marquée": "marqué",
    "marrant": "marrante",
    "marrante": "marrant",
    "marré": "marrée",
    "marrée": "marré",
    "marri": "marrie",
    "marrie": "marri",
    "martelé": "martelée",
    "martelée": "martelé",
    "martial": "martiale",
    "martiale": "martial",
    "martien": "martienne",
    "martienne": "martien",
    "martiniquais": "martiniquaise",
    "martiniquaise": "martiniquais",
    "martyr": "martyre",
    "martyre": "martyr",
    "martyrisé": "martyrisée",
    "martyrisée": "martyrisé",
    "maréchal": "maréchale",
    "maréchale": "maréchal",
    "masculin": "masculine",
    "masculine": "masculin",
    "masqué": "masquée",
    "masquée": "masqué",
    "massacrant": "massacrante",
    "massacrante": "massacrant",
    "massacré": "massacrée",
    "massacrée": "massacré",
    "massé": "massée",
    "massée": "massé",
    "masseur": "masseuse",
    "masseuse": "masseur",
    "massif": "massive",
    "massive": "massif",
    "masturbé": "masturbée",
    "masturbée": "masturbé",
    "mat": "mat'",
    "mat'": "mat",
    "matelassé": "matelassée",
    "matelassée": "matelassé",
    "matelassier": "matelassière",
    "matelassière": "matelassier",
    "maté": "matée",
    "matée": "maté",
    "maternel": "maternelle",
    "maternelle": "maternel",
    "mateur": "mateuse",
    "mateuse": "mateur",
    "mathématicien": "mathématicienne",
    "mathématicienne": "mathématicien",
    "matinal": "matinale",
    "matinale": "matinal",
    "matois": "matoise",
    "matoise": "matois",
    "maton": "matonne",
    "matonne": "maton",
    "matraqué": "matraquée",
    "matraquée": "matraqué",
    "matriarcal": "matriarcale",
    "matriarcale": "matriarcal",
    "matriciel": "matricielle",
    "matricielle": "matriciel",
    "matrimonial": "matrimoniale",
    "matrimoniale": "matrimonial",
    "matutinal": "matutinale",
    "matutinale": "matutinal",
    "matérialisé": "matérialisée",
    "matérialisée": "matérialisé",
    "matériel": "matérielle",
    "matérielle": "matériel",
    "maudit": "maudite",
    "maudite": "maudit",
    "maximal": "maximale",
    "maximale": "maximal",
    "maçon": "maçonne",
    "maçonne": "maçon",
    "maçonné": "maçonnée",
    "maçonnée": "maçonné",
    "maître": "maîtresse",
    "maîtresse": "maître",
    "maîtrisé": "maîtrisée",
    "maîtrisée": "maîtrisé",
    "meilleur": "meilleure",
    "meilleure": "meilleur",
    "melting pot": "melting-pot",
    "melting-pot": "melting pot",
    "menacé": "menacée",
    "menacée": "menacé",
    "menaçant": "menaçante",
    "menaçante": "menaçant",
    "mendiant": "mendiante",
    "mendiante": "mendiant",
    "mendié": "mendiée",
    "mendiée": "mendié",
    "mendigot": "mendigote",
    "mendigote": "mendigot",
    "mené": "menée",
    "menée": "mené",
    "meneur": "meneuse",
    "meneuse": "meneur",
    "menotté": "menottée",
    "menottée": "menotté",
    "mensonger": "mensongère",
    "mensongère": "mensonger",
    "menstruel": "menstruelle",
    "menstruelle": "menstruel",
    "mensuel": "mensuelle",
    "mensuelle": "mensuel",
    "mental": "mentale",
    "mentale": "mental",
    "menteur": "menteuse",
    "menteuse": "menteur",
    "mentholé": "mentholée",
    "mentholée": "mentholé",
    "mentionné": "mentionnée",
    "mentionnée": "mentionné",
    "mentonnier": "mentonnière",
    "mentonnière": "mentonnier",
    "menu": "menue",
    "menue": "menu",
    "mercier": "mercière",
    "mercière": "mercier",
    "merdé": "merdée",
    "merdée": "merdé",
    "meringué": "meringuée",
    "meringuée": "meringué",
    "mesquin": "mesquine",
    "mesquine": "mesquin",
    "messager": "messagère",
    "messagère": "messager",
    "mesuré": "mesurée",
    "mesurée": "mesuré",
    "metteur": "metteuse",
    "metteuse": "metteur",
    "meublé": "meublée",
    "meublée": "meublé",
    "meunier": "meunière",
    "meunière": "meunier",
    "meurtri": "meurtrie",
    "meurtrie": "meurtri",
    "meurtrier": "meurtrière",
    "meurtrière": "meurtrier",
    "meusien": "meusienne",
    "meusienne": "meusien",
    "mexicain": "mexicaine",
    "mexicaine": "mexicain",
    "mi-souriant": "mi-souriante",
    "mi-souriante": "mi-souriant",
    "mignard": "mignarde",
    "mignarde": "mignard",
    "mignon": "mignonne",
    "mignonne": "mignon",
    "mignonnet": "mignonnette",
    "mignonnette": "mignonnet",
    "migrateur": "migratrice",
    "migratrice": "migrateur",
    "militant": "militante",
    "militante": "militant",
    "militaro-industriel": "militaro-industrielle",
    "militaro-industrielle": "militaro-industriel",
    "milk shake": "milk-shake",
    "milk-shake": "milk shake",
    "millésimé": "millésimée",
    "millésimée": "millésimé",
    "mimé": "mimée",
    "mimée": "mimé",
    "miné": "minée",
    "minée": "miné",
    "minet": "minette",
    "minette": "minet",
    "mineur": "mineure",
    "mineure": "mineur",
    "miniaturisé": "miniaturisée",
    "miniaturisée": "miniaturisé",
    "minier": "minière",
    "minière": "minier",
    "minimal": "minimale",
    "minimale": "minimal",
    "minimisé": "minimisée",
    "minimisée": "minimisé",
    "ministériel": "ministérielle",
    "ministérielle": "ministériel",
    "minoen": "minoenne",
    "minoenne": "minoen",
    "minuté": "minutée",
    "minutée": "minuté",
    "minéral": "minérale",
    "minérale": "minéral",
    "miraculé": "miraculée",
    "miraculée": "miraculé",
    "mirobolant": "mirobolante",
    "mirobolante": "mirobolant",
    "miroitant": "miroitante",
    "miroitante": "miroitant",
    "miséricordieuse": "miséricordieux",
    "miséricordieux": "miséricordieuse",
    "mité": "mitée",
    "mitée": "mité",
    "mitigé": "mitigée",
    "mitigée": "mitigé",
    "mitonné": "mitonnée",
    "mitonnée": "mitonné",
    "mitoyen": "mitoyenne",
    "mitoyenne": "mitoyen",
    "mitraillé": "mitraillée",
    "mitraillée": "mitraillé",
    "mitrailleur": "mitrailleuse",
    "mitrailleuse": "mitrailleur",
    "mitral": "mitrale",
    "mitrale": "mitral",
    "mixé": "mixée",
    "mixée": "mixé",
    "mobilier": "mobilière",
    "mobilière": "mobilier",
    "mobilisé": "mobilisée",
    "mobilisée": "mobilisé",
    "mochard": "mocharde",
    "mocharde": "mochard",
    "modelé": "modelée",
    "modelée": "modelé",
    "modernisé": "modernisée",
    "modernisée": "modernisé",
    "modifié": "modifiée",
    "modifiée": "modifié",
    "modulé": "modulée",
    "modulée": "modulé",
    "modélisé": "modélisée",
    "modélisée": "modélisé",
    "modéré": "modérée",
    "modérée": "modéré",
    "moiré": "moirée",
    "moirée": "moiré",
    "moisi": "moisie",
    "moisie": "moisi",
    "moissonné": "moissonnée",
    "moissonnée": "moissonné",
    "moissonneur": "moissonneuse",
    "moissonneuse": "moissonneur",
    "molesté": "molestée",
    "molestée": "molesté",
    "mollasson": "mollassonne",
    "mollassonne": "mollasson",
    "mollet": "mollette",
    "mollette": "mollet",
    "molletonné": "molletonnée",
    "molletonnée": "molletonné",
    "molli": "mollie",
    "mollie": "molli",
    "momentané": "momentanée",
    "momentanée": "momentané",
    "momifié": "momifiée",
    "momifiée": "momifié",
    "monacal": "monacale",
    "monacale": "monacal",
    "mondain": "mondaine",
    "mondaine": "mondain",
    "mondial": "mondiale",
    "mondiale": "mondial",
    "mongol": "mongole",
    "mongole": "mongol",
    "mongolien": "mongolienne",
    "mongolienne": "mongolien",
    "moniteur": "monitrice",
    "monitrice": "moniteur",
    "monnayé": "monnayée",
    "monnayée": "monnayé",
    "monoparental": "monoparentale",
    "monoparentale": "monoparental",
    "montagnard": "montagnarde",
    "montagnarde": "montagnard",
    "montant": "montante",
    "montante": "montant",
    "monté": "montée",
    "montée": "monté",
    "monteur": "monteuse",
    "monteuse": "monteur",
    "montré": "montrée",
    "montrée": "montré",
    "montreur": "montreuse",
    "montreuse": "montreur",
    "monumental": "monumentale",
    "monumentale": "monumental",
    "moqué": "moquée",
    "moquée": "moqué",
    "moqueur": "moqueuse",
    "moqueuse": "moqueur",
    "moral": "morale",
    "morale": "moral",
    "moralisateur": "moralisatrice",
    "moralisatrice": "moralisateur",
    "morcelé": "morcelée",
    "morcelée": "morcelé",
    "mordant": "mordante",
    "mordante": "mordant",
    "mordeur": "mordeuse",
    "mordeuse": "mordeur",
    "mordillé": "mordillée",
    "mordillée": "mordillé",
    "mordoré": "mordorée",
    "mordorée": "mordoré",
    "mordu": "mordue",
    "mordue": "mordu",
    "morfal": "morfale",
    "morfale": "morfal",
    "morfalé": "morfalée",
    "morfalée": "morfalé",
    "morflé": "morflée",
    "morflée": "morflé",
    "morfondu": "morfondue",
    "morfondue": "morfondu",
    "moribond": "moribonde",
    "moribonde": "moribond",
    "moricaud": "moricaude",
    "moricaude": "moricaud",
    "morigéné": "morigénée",
    "morigénée": "morigéné",
    "mormon": "mormone",
    "mormone": "mormon",
    "mort": "morte",
    "morte": "mort",
    "mort-né": "mort-née",
    "mort-née": "mort-né",
    "mortel": "mortelle",
    "mortelle": "mortel",
    "mortifiant": "mortifiante",
    "mortifiante": "mortifiant",
    "mortifié": "mortifiée",
    "mortifiée": "mortifié",
    "motard": "motarde",
    "motarde": "motard",
    "moteur": "motrice",
    "motrice": "moteur",
    "motivant": "motivante",
    "motivante": "motivant",
    "motivé": "motivée",
    "motivée": "motivé",
    "motorisé": "motorisée",
    "motorisée": "motorisé",
    "mouchard": "moucharde",
    "moucharde": "mouchard",
    "mouché": "mouchée",
    "mouchée": "mouché",
    "moucheté": "mouchetée",
    "mouchetée": "moucheté",
    "moulu": "moulue",
    "moulue": "moulu",
    "mouillant": "mouillante",
    "mouillante": "mouillant",
    "mouillé": "mouillée",
    "mouillée": "mouillé",
    "moulant": "moulante",
    "moulante": "moulant",
    "moulé": "moulée",
    "moulée": "moulé",
    "mourant": "mourante",
    "mourante": "mourant",
    "moussant": "moussante",
    "moussante": "moussant",
    "moussu": "moussue",
    "moussue": "moussu",
    "moustachu": "moustachue",
    "moustachue": "moustachu",
    "moutonnant": "moutonnante",
    "moutonnante": "moutonnant",
    "moutonnier": "moutonnière",
    "moutonnière": "moutonnier",
    "mouvant": "mouvante",
    "mouvante": "mouvant",
    "mouvementé": "mouvementée",
    "mouvementée": "mouvementé",
    "moyen": "moyenne",
    "moyenne": "moyen",
    "mudéjar": "mudéjare",
    "mudéjare": "mudéjar",
    "mué": "muée",
    "muée": "mué",
    "muet": "muette",
    "muette": "muet",
    "mugissant": "mugissante",
    "mugissante": "mugissant",
    "muletier": "muletière",
    "muletière": "muletier",
    "multiculturel": "multiculturelle",
    "multiculturelle": "multiculturel",
    "multidimensionnel": "multidimensionnelle",
    "multidimensionnelle": "multidimensionnel",
    "multinational": "multinationale",
    "multinationale": "multinational",
    "multiplié": "multipliée",
    "multipliée": "multiplié",
    "multiracial": "multiraciale",
    "multiraciale": "multiracial",
    "muni": "munie",
    "munie": "muni",
    "municipal": "municipale",
    "municipale": "municipal",
    "muqueuse": "muqueux",
    "muqueux": "muqueuse",
    "mural": "murale",
    "murale": "mural",
    "muré": "murée",
    "murée": "muré",
    "murmurant": "murmurante",
    "murmurante": "murmurant",
    "murmuré": "murmurée",
    "murmurée": "murmuré",
    "musclé": "musclée",
    "musclée": "musclé",
    "muselé": "muselée",
    "muselée": "muselé",
    "music hall": "music-hall",
    "music-hall": "music hall",
    "musical": "musicale",
    "musicale": "musical",
    "musicien": "musicienne",
    "musicienne": "musicien",
    "musqué": "musquée",
    "musquée": "musqué",
    "mussolinien": "mussolinienne",
    "mussolinienne": "mussolinien",
    "musulman": "musulmane",
    "musulmane": "musulman",
    "mutant": "mutante",
    "mutante": "mutant",
    "muté": "mutée",
    "mutée": "muté",
    "mutilant": "mutilante",
    "mutilante": "mutilant",
    "mutilé": "mutilée",
    "mutilée": "mutilé",
    "mutin": "mutine",
    "mutine": "mutin",
    "mutiné": "mutinée",
    "mutinée": "mutiné",
    "mutuel": "mutuelle",
    "mutuelle": "mutuel",
    "mystificateur": "mystificatrice",
    "mystificatrice": "mystificateur",
    "mystifié": "mystifiée",
    "mystifiée": "mystifié",
    "mâché": "mâchée",
    "mâchée": "mâché",
    "mâcheur": "mâcheuse",
    "mâcheuse": "mâcheur",
    "mâchouillé": "mâchouillée",
    "mâchouillée": "mâchouillé",
    "mâchuré": "mâchurée",
    "mâchurée": "mâchuré",
    "mâtiné": "mâtinée",
    "mâtinée": "mâtiné",
    "mécanicien": "mécanicienne",
    "mécanicienne": "mécanicien",
    "mécanisé": "mécanisée",
    "mécanisée": "mécanisé",
    "méchant": "méchante",
    "méchante": "méchant",
    "méconnu": "méconnue",
    "méconnue": "méconnu",
    "mécontent": "mécontente",
    "mécontente": "mécontent",
    "mécréant": "mécréante",
    "mécréante": "mécréant",
    "médaillé": "médaillée",
    "médaillée": "médaillé",
    "médian": "médiane",
    "médiane": "médian",
    "médiateur": "médiatrice",
    "médiatrice": "médiateur",
    "médiatisé": "médiatisée",
    "médiatisée": "médiatisé",
    "médical": "médicale",
    "médicale": "médical",
    "médicalisé": "médicalisée",
    "médicalisée": "médicalisé",
    "médicinal": "médicinale",
    "médicinale": "médicinal",
    "médico-légal": "médico-légale",
    "médico-légale": "médico-légal",
    "médisant": "médisante",
    "médisante": "médisant",
    "méditatif": "méditative",
    "méditative": "méditatif",
    "médité": "méditée",
    "méditée": "médité",
    "méditerranéen": "méditerranéenne",
    "méditerranéenne": "méditerranéen",
    "médiéval": "médiévale",
    "médiévale": "médiéval",
    "médusé": "médusée",
    "médusée": "médusé",
    "méfiant": "méfiante",
    "méfiante": "méfiant",
    "méfié": "méfiée",
    "méfiée": "méfié",
    "mélangé": "mélangée",
    "mélangée": "mélangé",
    "mémoriel": "mémorielle",
    "mémorielle": "mémoriel",
    "mémorisé": "mémorisée",
    "mémorisée": "mémorisé",
    "ménager": "ménagère",
    "ménagère": "ménager",
    "ménagé": "ménagée",
    "ménagée": "ménagé",
    "ménopausé": "ménopausée",
    "ménopausée": "ménopausé",
    "méprisant": "méprisante",
    "méprisante": "méprisant",
    "méprisé": "méprisée",
    "méprisée": "méprisé",
    "méridien": "méridienne",
    "méridienne": "méridien",
    "méridional": "méridionale",
    "méridionale": "méridional",
    "méritant": "méritante",
    "méritante": "méritant",
    "mérité": "méritée",
    "méritée": "mérité",
    "mérovingien": "mérovingienne",
    "mérovingienne": "mérovingien",
    "métallisé": "métallisée",
    "métallisée": "métallisé",
    "métamorphosé": "métamorphosée",
    "métamorphosée": "métamorphosé",
    "métaphysicien": "métaphysicienne",
    "métaphysicienne": "métaphysicien",
    "métissé": "métissée",
    "métissée": "métissé",
    "métropolitain": "métropolitaine",
    "métropolitaine": "métropolitain",
    "mêlé": "mêlée",
    "mêlée": "mêlé",
    "mûr": "mûre",
    "mûre": "mûr",
    "mûri": "mûrie",
    "mûrie": "mûri",
    "mûrissant": "mûrissante",
    "mûrissante": "mûrissant",
    "nabot": "nabote",
    "nabote": "nabot",
    "nacré": "nacrée",
    "nacrée": "nacré",
    "nageant": "nageante",
    "nageante": "nageant",
    "nageur": "nageuse",
    "nageuse": "nageur",
    "nain": "naine",
    "naine": "nain",
    "naissant": "naissante",
    "naissante": "naissant",
    "nantais": "nantaise",
    "nantaise": "nantais",
    "nanti": "nantie",
    "nantie": "nanti",
    "napolitain": "napolitaine",
    "napolitaine": "napolitain",
    "napoléonien": "napoléonienne",
    "napoléonienne": "napoléonien",
    "nappé": "nappée",
    "nappée": "nappé",
    "narrateur": "narratrice",
    "narratrice": "narrateur",
    "narratif": "narrative",
    "narrative": "narratif",
    "narré": "narrée",
    "narrée": "narré",
    "nasal": "nasale",
    "nasale": "nasal",
    "nasillard": "nasillarde",
    "nasillarde": "nasillard",
    "natal": "natale",
    "natale": "natal",
    "natif": "native",
    "native": "natif",
    "national": "nationale",
    "nationale": "national",
    "nationalisé": "nationalisée",
    "nationalisée": "nationalisé",
    "natté": "nattée",
    "nattée": "natté",
    "naturalisé": "naturalisée",
    "naturalisée": "naturalisé",
    "naturel": "naturelle",
    "naturelle": "naturel",
    "naufragé": "naufragée",
    "naufragée": "naufragé",
    "naufrageur": "naufrageuse",
    "naufrageuse": "naufrageur",
    "nauséabond": "nauséabonde",
    "nauséabonde": "nauséabond",
    "naval": "navale",
    "navale": "naval",
    "navigant": "navigante",
    "navigante": "navigant",
    "navigateur": "navigatrice",
    "navigatrice": "navigateur",
    "navrant": "navrante",
    "navrante": "navrant",
    "navré": "navrée",
    "navrée": "navré",
    "nazi": "nazie",
    "nazie": "nazi",
    "né": "née",
    "née": "né",
    "naïf": "naïve",
    "naïve": "naïf",
    "nervuré": "nervurée",
    "nervurée": "nervuré",
    "nestorien": "nestorienne",
    "nestorienne": "nestorien",
    "net": "nette",
    "nette": "net",
    "nettoyant": "nettoyante",
    "nettoyante": "nettoyant",
    "nettoyé": "nettoyée",
    "nettoyée": "nettoyé",
    "nettoyeur": "nettoyeuse",
    "nettoyeuse": "nettoyeur",
    "neuf": "neuve",
    "neuve": "neuf",
    "neural": "neurale",
    "neurale": "neural",
    "neurochirurgien": "neurochirurgienne",
    "neurochirurgienne": "neurochirurgien",
    "neuronal": "neuronale",
    "neuronale": "neuronal",
    "neutralisant": "neutralisante",
    "neutralisante": "neutralisant",
    "neutralisé": "neutralisée",
    "neutralisée": "neutralisé",
    "niché": "nichée",
    "nichée": "niché",
    "nickelé": "nickelée",
    "nickelée": "nickelé",
    "nictitant": "nictitante",
    "nictitante": "nictitant",
    "nié": "niée",
    "niée": "nié",
    "nietzschéen": "nietzschéenne",
    "nietzschéenne": "nietzschéen",
    "nigaud": "nigaude",
    "nigaude": "nigaud",
    "night club": "night-club",
    "night-club": "night club",
    "nigérian": "nigériane",
    "nigériane": "nigérian",
    "nimbé": "nimbée",
    "nimbée": "nimbé",
    "nippé": "nippée",
    "nippée": "nippé",
    "nippon": "nippone",
    "nippone": "nippon",
    "niqué": "niquée",
    "niquée": "niqué",
    "nitreuse": "nitreux",
    "nitreux": "nitreuse",
    "nivelé": "nivelée",
    "nivelée": "nivelé",
    "niveleur": "niveleuse",
    "niveleuse": "niveleur",
    "noceur": "noceuse",
    "noceuse": "noceur",
    "nocif": "nocive",
    "nocive": "nocif",
    "noir": "noire",
    "noire": "noir",
    "noiraud": "noiraude",
    "noiraude": "noiraud",
    "noirci": "noircie",
    "noircie": "noirci",
    "nominal": "nominale",
    "nominale": "nominal",
    "nominatif": "nominative",
    "nominative": "nominatif",
    "nominé": "nominée",
    "nominée": "nominé",
    "nommé": "nommée",
    "nommée": "nommé",
    "non-violent": "non-violente",
    "non-violente": "non-violent",
    "non-voyant": "non-voyante",
    "non-voyante": "non-voyant",
    "nonchalant": "nonchalante",
    "nonchalante": "nonchalant",
    "nord-africain": "nord-africaine",
    "nord-africaine": "nord-africain",
    "nord-américain": "nord-américaine",
    "nord-américaine": "nord-américain",
    "nord-coréen": "nord-coréenne",
    "nord-coréenne": "nord-coréen",
    "nord-vietnamien": "nord-vietnamienne",
    "nord-vietnamienne": "nord-vietnamien",
    "normal": "normale",
    "normale": "normal",
    "normalien": "normalienne",
    "normalienne": "normalien",
    "normand": "normande",
    "normande": "normand",
    "normatif": "normative",
    "normative": "normatif",
    "norvégien": "norvégienne",
    "norvégienne": "norvégien",
    "notarial": "notariale",
    "notariale": "notarial",
    "notarié": "notariée",
    "notariée": "notarié",
    "noté": "notée",
    "notée": "noté",
    "notifié": "notifiée",
    "notifiée": "notifié",
    "noué": "nouée",
    "nouée": "noué",
    "nourri": "nourrie",
    "nourrie": "nourri",
    "nourricier": "nourricière",
    "nourricière": "nourricier",
    "nourrissant": "nourrissante",
    "nourrissante": "nourrissant",
    "nouveau": "nouvelle",
    "nouvelle": "nouveau",
    "novateur": "novatrice",
    "novatrice": "novateur",
    "noyauté": "noyautée",
    "noyautée": "noyauté",
    "noyé": "noyée",
    "noyée": "noyé",
    "nu": "nue",
    "nue": "nu",
    "nuancé": "nuancée",
    "nuancée": "nuancé",
    "nubien": "nubienne",
    "nubienne": "nubien",
    "nul": "nulle",
    "nulle": "nul",
    "nullard": "nullarde",
    "nullarde": "nullard",
    "numérisé": "numérisée",
    "numérisée": "numérisé",
    "numéroté": "numérotée",
    "numérotée": "numéroté",
    "nuptial": "nuptiale",
    "nuptiale": "nuptial",
    "nutritif": "nutritive",
    "nutritive": "nutritif",
    "nutritionnel": "nutritionnelle",
    "nutritionnelle": "nutritionnel",
    "nègre": "négresse",
    "négresse": "nègre",
    "nécessité": "nécessitée",
    "nécessitée": "nécessité",
    "nécromancien": "nécromancienne",
    "nécromancienne": "nécromancien",
    "nécrosé": "nécrosée",
    "nécrosée": "nécrosé",
    "négatif": "négative",
    "négative": "négatif",
    "négligent": "négligente",
    "négligente": "négligent",
    "négligé": "négligée",
    "négligée": "négligé",
    "négociant": "négociante",
    "négociante": "négociant",
    "négociateur": "négociatrice",
    "négociatrice": "négociateur",
    "négocié": "négociée",
    "négociée": "négocié",
    "négrier": "négrière",
    "négrière": "négrier",
    "néonatal": "néonatale",
    "néonatale": "néonatal",
    "névrosé": "névrosée",
    "névrosée": "névrosé",
    "objectal": "objectale",
    "objectale": "objectal",
    "objectif": "objective",
    "objective": "objectif",
    "oblatif": "oblative",
    "oblative": "oblatif",
    "obligeant": "obligeante",
    "obligeante": "obligeant",
    "obligé": "obligée",
    "obligée": "obligé",
    "oblitéré": "oblitérée",
    "oblitérée": "oblitéré",
    "oblong": "oblongue",
    "oblongue": "oblong",
    "obnubilé": "obnubilée",
    "obnubilée": "obnubilé",
    "obscur": "obscure",
    "obscure": "obscur",
    "obscurci": "obscurcie",
    "obscurcie": "obscurci",
    "observateur": "observatrice",
    "observatrice": "observateur",
    "observé": "observée",
    "observée": "observé",
    "obsessif": "obsessive",
    "obsessive": "obsessif",
    "obsessionnel": "obsessionnelle",
    "obsessionnelle": "obsessionnel",
    "obsidional": "obsidionale",
    "obsidionale": "obsidional",
    "obstiné": "obstinée",
    "obstinée": "obstiné",
    "obstructif": "obstructive",
    "obstructive": "obstructif",
    "obstrué": "obstruée",
    "obstruée": "obstrué",
    "obstétricien": "obstétricienne",
    "obstétricienne": "obstétricien",
    "obsédant": "obsédante",
    "obsédante": "obsédant",
    "obsédé": "obsédée",
    "obsédée": "obsédé",
    "obséquieuse": "obséquieux",
    "obséquieux": "obséquieuse",
    "obtenu": "obtenue",
    "obtenue": "obtenu",
    "obturé": "obturée",
    "obturée": "obturé",
    "obéi": "obéie",
    "obéie": "obéi",
    "obéissant": "obéissante",
    "obéissante": "obéissant",
    "occasionnel": "occasionnelle",
    "occasionnelle": "occasionnel",
    "occasionné": "occasionnée",
    "occasionnée": "occasionné",
    "occidental": "occidentale",
    "occidentale": "occidental",
    "occipital": "occipitale",
    "occipitale": "occipital",
    "occitan": "occitane",
    "occitane": "occitan",
    "occulté": "occultée",
    "occultée": "occulté",
    "occupant": "occupante",
    "occupante": "occupant",
    "occupé": "occupée",
    "occupée": "occupé",
    "ocré": "ocrée",
    "ocrée": "ocré",
    "octogonal": "octogonale",
    "octogonale": "octogonal",
    "octroyé": "octroyée",
    "octroyée": "octroyé",
    "odorant": "odorante",
    "odorante": "odorant",
    "odoriférant": "odoriférante",
    "odoriférante": "odoriférant",
    "oedipien": "oedipienne",
    "oedipienne": "oedipien",
    "oesophagien": "oesophagienne",
    "oesophagienne": "oesophagien",
    "offensant": "offensante",
    "offensante": "offensant",
    "offensé": "offensée",
    "offensée": "offensé",
    "offensif": "offensive",
    "offensive": "offensif",
    "officiel": "officielle",
    "officielle": "officiel",
    "officier": "officière",
    "officière": "officier",
    "offrant": "offrante",
    "offrante": "offrant",
    "offert": "offerte",
    "offerte": "offert",
    "offusqué": "offusquée",
    "offusquée": "offusqué",
    "oint": "ointe",
    "ointe": "oint",
    "oiselier": "oiselière",
    "oiselière": "oiselier",
    "oisif": "oisive",
    "oisive": "oisif",
    "olfactif": "olfactive",
    "olfactive": "olfactif",
    "olympien": "olympienne",
    "olympienne": "olympien",
    "ombilical": "ombilicale",
    "ombilicale": "ombilical",
    "ombragé": "ombragée",
    "ombragée": "ombragé",
    "ombré": "ombrée",
    "ombrée": "ombré",
    "omnidirectionnel": "omnidirectionnelle",
    "omnidirectionnelle": "omnidirectionnel",
    "omnipotent": "omnipotente",
    "omnipotente": "omnipotent",
    "omniprésent": "omniprésente",
    "omniprésente": "omniprésent",
    "omniscient": "omnisciente",
    "omnisciente": "omniscient",
    "ondoyant": "ondoyante",
    "ondoyante": "ondoyant",
    "ondulant": "ondulante",
    "ondulante": "ondulant",
    "ondulé": "ondulée",
    "ondulée": "ondulé",
    "onduleuse": "onduleux",
    "onduleux": "onduleuse",
    "opalescent": "opalescente",
    "opalescente": "opalescent",
    "opalin": "opaline",
    "opaline": "opalin",
    "opportun": "opportune",
    "opportune": "opportun",
    "opposant": "opposante",
    "opposante": "opposant",
    "opposé": "opposée",
    "opposée": "opposé",
    "oppositionnel": "oppositionnelle",
    "oppositionnelle": "oppositionnel",
    "oppressant": "oppressante",
    "oppressante": "oppressant",
    "oppressé": "oppressée",
    "oppressée": "oppressé",
    "oppressif": "oppressive",
    "oppressive": "oppressif",
    "opprimé": "opprimée",
    "opprimée": "opprimé",
    "opticien": "opticienne",
    "opticienne": "opticien",
    "optimal": "optimale",
    "optimale": "optimal",
    "optionnel": "optionnelle",
    "optionnelle": "optionnel",
    "opulent": "opulente",
    "opulente": "opulent",
    "opérateur": "opératrice",
    "opératrice": "opérateur",
    "opérationnel": "opérationnelle",
    "opérationnelle": "opérationnel",
    "opéré": "opérée",
    "opérée": "opéré",
    "oral": "orale",
    "orale": "oral",
    "oranais": "oranaise",
    "oranaise": "oranais",
    "orangé": "orangée",
    "orangée": "orangé",
    "orant": "orante",
    "orante": "orant",
    "orateur": "oratrice",
    "oratrice": "orateur",
    "orbital": "orbitale",
    "orbitale": "orbital",
    "orchestral": "orchestrale",
    "orchestrale": "orchestral",
    "orchestré": "orchestrée",
    "orchestrée": "orchestré",
    "ordonnateur": "ordonnatrice",
    "ordonnatrice": "ordonnateur",
    "ordonné": "ordonnée",
    "ordonnée": "ordonné",
    "ordurier": "ordurière",
    "ordurière": "ordurier",
    "organisateur": "organisatrice",
    "organisatrice": "organisateur",
    "organisé": "organisée",
    "organisée": "organisé",
    "oriental": "orientale",
    "orientale": "oriental",
    "orienté": "orientée",
    "orientée": "orienté",
    "orienteur": "orienteuse",
    "orienteuse": "orienteur",
    "original": "originale",
    "originale": "original",
    "originel": "originelle",
    "originelle": "originel",
    "ornemental": "ornementale",
    "ornementale": "ornemental",
    "ornementé": "ornementée",
    "ornementée": "ornementé",
    "orné": "ornée",
    "ornée": "orné",
    "orphelin": "orpheline",
    "orpheline": "orphelin",
    "oscillant": "oscillante",
    "oscillante": "oscillant",
    "osé": "osée",
    "osée": "osé",
    "ottoman": "ottomane",
    "ottomane": "ottoman",
    "ouaté": "ouatée",
    "ouatée": "ouaté",
    "ouateuse": "ouateux",
    "ouateux": "ouateuse",
    "ouatiné": "ouatinée",
    "ouatinée": "ouatiné",
    "oublié": "oubliée",
    "oubliée": "oublié",
    "ourdi": "ourdie",
    "ourdie": "ourdi",
    "ourlé": "ourlée",
    "ourlée": "ourlé",
    "ours": "ourse",
    "ourse": "ours",
    "ourson": "oursonne",
    "oursonne": "ourson",
    "outillé": "outillée",
    "outillée": "outillé",
    "outrageant": "outrageante",
    "outrageante": "outrageant",
    "outragé": "outragée",
    "outragée": "outragé",
    "outrageuse": "outrageux",
    "outrageux": "outrageuse",
    "outrancier": "outrancière",
    "outrancière": "outrancier",
    "outrecuidant": "outrecuidante",
    "outrecuidante": "outrecuidant",
    "outré": "outrée",
    "outrée": "outré",
    "ouvert": "ouverte",
    "ouverte": "ouvert",
    "ouvragé": "ouvragée",
    "ouvragée": "ouvragé",
    "ouvreur": "ouvreuse",
    "ouvreuse": "ouvreur",
    "ouvrier": "ouvrière",
    "ouvrière": "ouvrier",
    "ouï": "ouïe",
    "ouïe": "ouï",
    "ovarien": "ovarienne",
    "ovarienne": "ovarien",
    "ovoïdal": "ovoïdale",
    "ovoïdale": "ovoïdal",
    "oxydé": "oxydée",
    "oxydée": "oxydé",
    "oxygéné": "oxygénée",
    "oxygénée": "oxygéné",
    "pacificateur": "pacificatrice",
    "pacificatrice": "pacificateur",
    "pacifié": "pacifiée",
    "pacifiée": "pacifié",
    "pagé": "pagée",
    "pagée": "pagé",
    "paillard": "paillarde",
    "paillarde": "paillard",
    "pailleté": "pailletée",
    "pailletée": "pailleté",
    "paillé": "paillée",
    "paillée": "paillé",
    "pair": "paire",
    "paire": "pair",
    "palatin": "palatine",
    "palatine": "palatin",
    "palefrenier": "palefrenière",
    "palefrenière": "palefrenier",
    "palestinien": "palestinienne",
    "palestinienne": "palestinien",
    "palissadé": "palissadée",
    "palissadée": "palissadé",
    "palpitant": "palpitante",
    "palpitante": "palpitant",
    "panaché": "panachée",
    "panachée": "panaché",
    "panaméricain": "panaméricaine",
    "panaméricaine": "panaméricain",
    "pané": "panée",
    "panée": "pané",
    "panetier": "panetière",
    "panetière": "panetier",
    "paniqué": "paniquée",
    "paniquée": "paniqué",
    "pansé": "pansée",
    "pansée": "pansé",
    "pansu": "pansue",
    "pansue": "pansu",
    "pantelant": "pantelante",
    "pantelante": "pantelant",
    "pantouflard": "pantouflarde",
    "pantouflarde": "pantouflard",
    "paon": "paonne",
    "paonne": "paon",
    "papal": "papale",
    "papale": "papal",
    "papelard": "papelarde",
    "papelarde": "papelard",
    "papilionacé": "papilionacée",
    "papilionacée": "papilionacé",
    "papillonnant": "papillonnante",
    "papillonnante": "papillonnant",
    "parachuté": "parachutée",
    "parachutée": "parachuté",
    "paradé": "paradée",
    "paradée": "paradé",
    "paradoxal": "paradoxale",
    "paradoxale": "paradoxal",
    "paraffiné": "paraffinée",
    "paraffinée": "paraffiné",
    "paraguayen": "paraguayenne",
    "paraguayenne": "paraguayen",
    "paralysant": "paralysante",
    "paralysante": "paralysant",
    "paralysé": "paralysée",
    "paralysée": "paralysé",
    "paramédical": "paramédicale",
    "paramédicale": "paramédical",
    "paranormal": "paranormale",
    "paranormale": "paranormal",
    "paraphé": "paraphée",
    "paraphée": "paraphé",
    "parasité": "parasitée",
    "parasitée": "parasité",
    "paru": "parue",
    "parue": "paru",
    "parcheminé": "parcheminée",
    "parcheminée": "parcheminé",
    "parcouru": "parcourue",
    "parcourue": "parcouru",
    "pardonné": "pardonnée",
    "pardonnée": "pardonné",
    "pareil": "pareille",
    "pareille": "pareil",
    "parent": "parente",
    "parente": "parent",
    "parental": "parentale",
    "parentale": "parental",
    "paré": "parée",
    "parée": "paré",
    "parfait": "parfaite",
    "parfaite": "parfait",
    "parfumé": "parfumée",
    "parfumée": "parfumé",
    "parfumeur": "parfumeuse",
    "parfumeuse": "parfumeur",
    "parieur": "parieuse",
    "parieuse": "parieur",
    "parigot": "parigote",
    "parigote": "parigot",
    "parisien": "parisienne",
    "parisienne": "parisien",
    "pariétal": "pariétale",
    "pariétale": "pariétal",
    "parjuré": "parjurée",
    "parjurée": "parjuré",
    "parkinsonien": "parkinsonienne",
    "parkinsonienne": "parkinsonien",
    "parlant": "parlante",
    "parlante": "parlant",
    "parlé": "parlée",
    "parlée": "parlé",
    "parleur": "parleuse",
    "parleuse": "parleur",
    "parmesan": "parmesane",
    "parmesane": "parmesan",
    "parodontal": "parodontale",
    "parodontale": "parodontal",
    "paroissial": "paroissiale",
    "paroissiale": "paroissial",
    "paroissien": "paroissienne",
    "paroissienne": "paroissien",
    "parolier": "parolière",
    "parolière": "parolier",
    "parpaillot": "parpaillote",
    "parpaillote": "parpaillot",
    "parqué": "parquée",
    "parquée": "parqué",
    "parqueté": "parquetée",
    "parquetée": "parqueté",
    "parqueteur": "parqueteuse",
    "parqueteuse": "parqueteur",
    "parrainé": "parrainée",
    "parrainée": "parrainé",
    "parsemé": "parsemée",
    "parsemée": "parsemé",
    "partagé": "partagée",
    "partagée": "partagé",
    "partant": "partante",
    "partante": "partant",
    "partial": "partiale",
    "partiale": "partial",
    "participant": "participante",
    "participante": "participant",
    "particulier": "particulière",
    "particulière": "particulier",
    "partiel": "partielle",
    "partielle": "partiel",
    "parti": "partie",
    "partie": "parti",
    "partisan": "partisane",
    "partisane": "partisan",
    "parvenu": "parvenue",
    "parvenue": "parvenu",
    "pascal": "pascale",
    "pascale": "pascal",
    "passager": "passagère",
    "passagère": "passager",
    "passant": "passante",
    "passante": "passant",
    "passementier": "passementière",
    "passementière": "passementier",
    "passé": "passée",
    "passée": "passé",
    "passeur": "passeuse",
    "passeuse": "passeur",
    "passif": "passive",
    "passive": "passif",
    "passionnant": "passionnante",
    "passionnante": "passionnant",
    "passionnel": "passionnelle",
    "passionnelle": "passionnel",
    "passionné": "passionnée",
    "passionnée": "passionné",
    "pasteurisé": "pasteurisée",
    "pasteurisée": "pasteurisé",
    "pastoral": "pastorale",
    "pastorale": "pastoral",
    "pastoureau": "pastourelle",
    "pastourelle": "pastoureau",
    "pataud": "pataude",
    "pataude": "pataud",
    "patelin": "pateline",
    "pateline": "patelin",
    "patent": "patente",
    "patente": "patent",
    "patenté": "patentée",
    "patentée": "patenté",
    "paternel": "paternelle",
    "paternelle": "paternel",
    "patient": "patiente",
    "patiente": "patient",
    "patiné": "patinée",
    "patinée": "patiné",
    "patineur": "patineuse",
    "patineuse": "patineur",
    "patriarcal": "patriarcale",
    "patriarcale": "patriarcal",
    "patricien": "patricienne",
    "patricienne": "patricien",
    "patron": "patronne",
    "patronne": "patron",
    "patronal": "patronale",
    "patronale": "patronal",
    "patronné": "patronnée",
    "patronnée": "patronné",
    "patrouillé": "patrouillée",
    "patrouillée": "patrouillé",
    "paumé": "paumée",
    "paumée": "paumé",
    "pavlovien": "pavlovienne",
    "pavlovienne": "pavlovien",
    "pavoisé": "pavoisée",
    "pavoisée": "pavoisé",
    "pavé": "pavée",
    "pavée": "pavé",
    "payant": "payante",
    "payante": "payant",
    "payé": "payée",
    "payée": "payé",
    "paysan": "paysanne",
    "paysanne": "paysan",
    "païen": "païenne",
    "païenne": "païen",
    "peaufiné": "peaufinée",
    "peaufinée": "peaufiné",
    "pectoral": "pectorale",
    "pectorale": "pectoral",
    "peinard": "peinarde",
    "peinarde": "peinard",
    "peint": "peinte",
    "peinte": "peint",
    "peiné": "peinée",
    "peinée": "peiné",
    "peinturé": "peinturée",
    "peinturée": "peinturé",
    "peinturluré": "peinturlurée",
    "peinturlurée": "peinturluré",
    "pelé": "pelée",
    "pelée": "pelé",
    "pelleteur": "pelleteuse",
    "pelleteuse": "pelleteur",
    "pelliculé": "pelliculée",
    "pelliculée": "pelliculé",
    "peloté": "pelotée",
    "pelotée": "peloté",
    "pelotonné": "pelotonnée",
    "pelotonnée": "pelotonné",
    "pelvien": "pelvienne",
    "pelvienne": "pelvien",
    "penaud": "penaude",
    "penaude": "penaud",
    "penchant": "penchante",
    "penchante": "penchant",
    "penché": "penchée",
    "penchée": "penché",
    "pendant": "pendante",
    "pendante": "pendant",
    "pendard": "pendarde",
    "pendarde": "pendard",
    "pendu": "pendue",
    "pendue": "pendu",
    "pensant": "pensante",
    "pensante": "pensant",
    "pensé": "pensée",
    "pensée": "pensé",
    "penseur": "penseuse",
    "penseuse": "penseur",
    "pensif": "pensive",
    "pensive": "pensif",
    "pentu": "pentue",
    "pentue": "pentu",
    "percepteur": "perceptrice",
    "perceptrice": "percepteur",
    "perceptif": "perceptive",
    "perceptive": "perceptif",
    "percé": "percée",
    "percée": "percé",
    "perceur": "perceuse",
    "perceuse": "perceur",
    "perçu": "perçue",
    "perçue": "perçu",
    "perché": "perchée",
    "perchée": "perché",
    "percheron": "percheronne",
    "percheronne": "percheron",
    "percutant": "percutante",
    "percutante": "percutant",
    "percuté": "percutée",
    "percutée": "percuté",
    "perdant": "perdante",
    "perdante": "perdant",
    "perdu": "perdue",
    "perdue": "perdu",
    "perfectionné": "perfectionnée",
    "perfectionnée": "perfectionné",
    "perforant": "perforante",
    "perforante": "perforant",
    "perforateur": "perforatrice",
    "perforatrice": "perforateur",
    "perforé": "perforée",
    "perforée": "perforé",
    "performant": "performante",
    "performante": "performant",
    "perlé": "perlée",
    "perlée": "perlé",
    "permanent": "permanente",
    "permanente": "permanent",
    "permanenté": "permanentée",
    "permanentée": "permanenté",
    "perpétré": "perpétrée",
    "perpétrée": "perpétré",
    "perpétuel": "perpétuelle",
    "perpétuelle": "perpétuel",
    "perpétué": "perpétuée",
    "perpétuée": "perpétué",
    "perruquier": "perruquière",
    "perruquière": "perruquier",
    "persan": "persane",
    "persane": "persan",
    "persifleur": "persifleuse",
    "persifleuse": "persifleur",
    "persillé": "persillée",
    "persillée": "persillé",
    "persistant": "persistante",
    "persistante": "persistant",
    "personnalisé": "personnalisée",
    "personnalisée": "personnalisé",
    "personnel": "personnelle",
    "personnelle": "personnel",
    "personnifié": "personnifiée",
    "personnifiée": "personnifié",
    "persuadé": "persuadée",
    "persuadée": "persuadé",
    "persuasif": "persuasive",
    "persuasive": "persuasif",
    "persécuté": "persécutée",
    "persécutée": "persécuté",
    "persévérant": "persévérante",
    "persévérante": "persévérant",
    "pertinent": "pertinente",
    "pertinente": "pertinent",
    "perturbant": "perturbante",
    "perturbante": "perturbant",
    "perturbateur": "perturbatrice",
    "perturbatrice": "perturbateur",
    "perturbé": "perturbée",
    "perturbée": "perturbé",
    "perverti": "pervertie",
    "pervertie": "perverti",
    "perçant": "perçante",
    "perçante": "perçant",
    "pesant": "pesante",
    "pesante": "pesant",
    "pesé": "pesée",
    "pesée": "pesé",
    "pestiféré": "pestiférée",
    "pestiférée": "pestiféré",
    "pestilentiel": "pestilentielle",
    "pestilentielle": "pestilentiel",
    "petiot": "petiote",
    "petiote": "petiot",
    "petit": "petite",
    "petite": "petit",
    "peuplé": "peuplée",
    "peuplée": "peuplé",
    "pharmacien": "pharmacienne",
    "pharmacienne": "pharmacien",
    "philippin": "philippine",
    "philippine": "philippin",
    "philosophal": "philosophale",
    "philosophale": "philosophal",
    "phosphorescent": "phosphorescente",
    "phosphorescente": "phosphorescent",
    "photocopié": "photocopiée",
    "photocopiée": "photocopié",
    "photocopieur": "photocopieuse",
    "photocopieuse": "photocopieur",
    "photographié": "photographiée",
    "photographiée": "photographié",
    "phrasé": "phrasée",
    "phrasée": "phrasé",
    "physicien": "physicienne",
    "physicienne": "physicien",
    "phénicien": "phénicienne",
    "phénicienne": "phénicien",
    "phénoménal": "phénoménale",
    "phénoménale": "phénoménal",
    "piaffant": "piaffante",
    "piaffante": "piaffant",
    "piaillant": "piaillante",
    "piaillante": "piaillant",
    "pianoté": "pianotée",
    "pianotée": "pianoté",
    "piaulant": "piaulante",
    "piaulante": "piaulant",
    "picard": "picarde",
    "picarde": "picard",
    "picoré": "picorée",
    "picorée": "picoré",
    "pictural": "picturale",
    "picturale": "pictural",
    "pieuté": "pieutée",
    "pieutée": "pieuté",
    "pigeon": "pigeonne",
    "pigeonne": "pigeon",
    "pigeonnant": "pigeonnante",
    "pigeonnante": "pigeonnant",
    "pilé": "pilée",
    "pilée": "pilé",
    "pillard": "pillarde",
    "pillarde": "pillard",
    "pillé": "pillée",
    "pillée": "pillé",
    "pilleur": "pilleuse",
    "pilleuse": "pilleur",
    "pilonné": "pilonnée",
    "pilonnée": "pilonné",
    "piloté": "pilotée",
    "pilotée": "piloté",
    "pimenté": "pimentée",
    "pimentée": "pimenté",
    "pimpant": "pimpante",
    "pimpante": "pimpant",
    "pincé": "pincée",
    "pincée": "pincé",
    "pinté": "pintée",
    "pintée": "pinté",
    "pinéal": "pinéale",
    "pinéale": "pinéal",
    "pioché": "piochée",
    "piochée": "pioché",
    "pion": "pionne",
    "pionne": "pion",
    "pionnier": "pionnière",
    "pionnière": "pionnier",
    "pipeur": "pipeuse",
    "pipeuse": "pipeur",
    "piquant": "piquante",
    "piquante": "piquant",
    "piqué": "piquée",
    "piquée": "piqué",
    "piqueté": "piquetée",
    "piquetée": "piqueté",
    "piqueur": "piqueuse",
    "piqueuse": "piqueur",
    "piraté": "piratée",
    "piratée": "piraté",
    "pissé": "pissée",
    "pissée": "pissé",
    "pisseur": "pisseuse",
    "pisseuse": "pisseur",
    "pisté": "pistée",
    "pistée": "pisté",
    "pistonné": "pistonnée",
    "pistonnée": "pistonné",
    "pit bull": "pit-bull",
    "pit-bull": "pit bull",
    "pitchoun": "pitchoune",
    "pitchoune": "pitchoun",
    "pivotant": "pivotante",
    "pivotante": "pivotant",
    "piégé": "piégée",
    "piégée": "piégé",
    "piégeur": "piégeuse",
    "piégeuse": "piégeur",
    "piémontais": "piémontaise",
    "piémontaise": "piémontais",
    "piétinant": "piétinante",
    "piétinante": "piétinant",
    "piétiné": "piétinée",
    "piétinée": "piétiné",
    "piéton": "piétonne",
    "piétonne": "piéton",
    "piétonnier": "piétonnière",
    "piétonnière": "piétonnier",
    "placardé": "placardée",
    "placardée": "placardé",
    "placé": "placée",
    "placée": "placé",
    "placeur": "placeuse",
    "placeuse": "placeur",
    "plafonné": "plafonnée",
    "plafonnée": "plafonné",
    "plafonnier": "plafonnière",
    "plafonnière": "plafonnier",
    "plaidé": "plaidée",
    "plaidée": "plaidé",
    "plaignant": "plaignante",
    "plaignante": "plaignant",
    "plaint": "plainte",
    "plainte": "plaint",
    "plaintif": "plaintive",
    "plaintive": "plaintif",
    "plaisant": "plaisante",
    "plaisante": "plaisant",
    "plan": "plane",
    "plane": "plan",
    "planant": "planante",
    "planante": "planant",
    "planificateur": "planificatrice",
    "planificatrice": "planificateur",
    "planifié": "planifiée",
    "planifiée": "planifié",
    "planqué": "planquée",
    "planquée": "planqué",
    "planté": "plantée",
    "plantée": "planté",
    "plantureuse": "plantureux",
    "plantureux": "plantureuse",
    "plaqué": "plaquée",
    "plaquée": "plaqué",
    "plasticien": "plasticienne",
    "plasticienne": "plasticien",
    "plastifié": "plastifiée",
    "plastifiée": "plastifié",
    "plat": "plate",
    "plate": "plat",
    "platiné": "platinée",
    "platinée": "platiné",
    "platonicien": "platonicienne",
    "platonicienne": "platonicien",
    "plein": "pleine",
    "pleine": "plein",
    "pleural": "pleurale",
    "pleurale": "pleural",
    "pleurant": "pleurante",
    "pleurante": "pleurant",
    "pleurard": "pleurarde",
    "pleurarde": "pleurard",
    "pleuré": "pleurée",
    "pleurée": "pleuré",
    "pleureur": "pleureuse",
    "pleureuse": "pleureur",
    "pleurnichard": "pleurnicharde",
    "pleurnicharde": "pleurnichard",
    "pleurnicheur": "pleurnicheuse",
    "pleurnicheuse": "pleurnicheur",
    "pliant": "pliante",
    "pliante": "pliant",
    "plié": "pliée",
    "pliée": "plié",
    "plieur": "plieuse",
    "plieuse": "plieur",
    "plissé": "plissée",
    "plissée": "plissé",
    "plombé": "plombée",
    "plombée": "plombé",
    "plongeant": "plongeante",
    "plongeante": "plongeant",
    "plongé": "plongée",
    "plongée": "plongé",
    "plongeur": "plongeuse",
    "plongeuse": "plongeur",
    "ployant": "ployante",
    "ployante": "ployant",
    "ployé": "ployée",
    "ployée": "ployé",
    "plum": "plume",
    "plume": "plum",
    "plumé": "plumée",
    "plumée": "plumé",
    "pluriel": "plurielle",
    "plurielle": "pluriel",
    "plutonien": "plutonienne",
    "plutonienne": "plutonien",
    "pluvial": "pluviale",
    "pluviale": "pluvial",
    "plâtré": "plâtrée",
    "plâtrée": "plâtré",
    "plébéien": "plébéienne",
    "plébéienne": "plébéien",
    "plénier": "plénière",
    "plénière": "plénier",
    "pochard": "pocharde",
    "pocharde": "pochard",
    "poché": "pochée",
    "pochée": "poché",
    "poignant": "poignante",
    "poignante": "poignant",
    "poignardé": "poignardée",
    "poignardée": "poignardé",
    "poilu": "poilue",
    "poilue": "poilu",
    "pointé": "pointée",
    "pointée": "pointé",
    "pointeur": "pointeuse",
    "pointeuse": "pointeur",
    "pointillé": "pointillée",
    "pointillée": "pointillé",
    "pointu": "pointue",
    "pointue": "pointu",
    "poinçonné": "poinçonnée",
    "poinçonnée": "poinçonné",
    "poiré": "poirée",
    "poirée": "poiré",
    "poissé": "poissée",
    "poissée": "poissé",
    "poivré": "poivrée",
    "poivrée": "poivré",
    "poivrot": "poivrote",
    "poivrote": "poivrot",
    "polarisé": "polarisée",
    "polarisée": "polarisé",
    "poli": "polie",
    "polie": "poli",
    "policé": "policée",
    "policée": "policé",
    "policier": "policière",
    "policière": "policier",
    "polisseur": "polisseuse",
    "polisseuse": "polisseur",
    "polisson": "polissonne",
    "polissonne": "polisson",
    "politicien": "politicienne",
    "politicienne": "politicien",
    "politisé": "politisée",
    "politisée": "politisé",
    "polluant": "polluante",
    "polluante": "polluant",
    "pollué": "polluée",
    "polluée": "pollué",
    "pollueur": "pollueuse",
    "pollueuse": "pollueur",
    "poltron": "poltronne",
    "poltronne": "poltron",
    "polynésien": "polynésienne",
    "polynésienne": "polynésien",
    "polytechnicien": "polytechnicienne",
    "polytechnicienne": "polytechnicien",
    "polyvalent": "polyvalente",
    "polyvalente": "polyvalent",
    "pommé": "pommée",
    "pommée": "pommé",
    "pompé": "pompée",
    "pompée": "pompé",
    "pomponné": "pomponnée",
    "pomponnée": "pomponné",
    "pompéien": "pompéienne",
    "pompéienne": "pompéien",
    "poncé": "poncée",
    "poncée": "poncé",
    "ponctuel": "ponctuelle",
    "ponctuelle": "ponctuel",
    "ponctué": "ponctuée",
    "ponctuée": "ponctué",
    "pondeur": "pondeuse",
    "pondeuse": "pondeur",
    "pondu": "pondue",
    "pondue": "pondu",
    "pondéré": "pondérée",
    "pondérée": "pondéré",
    "pontifiant": "pontifiante",
    "pontifiante": "pontifiant",
    "pontifical": "pontificale",
    "pontificale": "pontifical",
    "poplité": "poplitée",
    "poplitée": "poplité",
    "populacier": "populacière",
    "populacière": "populacier",
    "porcher": "porchère",
    "porchère": "porcher",
    "porcin": "porcine",
    "porcine": "porcin",
    "portant": "portante",
    "portante": "portant",
    "portatif": "portative",
    "portative": "portatif",
    "porté": "portée",
    "portée": "porté",
    "porteur": "porteuse",
    "porteuse": "porteur",
    "portier": "portière",
    "portière": "portier",
    "portoricain": "portoricaine",
    "portoricaine": "portoricain",
    "posé": "posée",
    "posée": "posé",
    "poseur": "poseuse",
    "poseuse": "poseur",
    "positif": "positive",
    "positive": "positif",
    "positionné": "positionnée",
    "positionnée": "positionné",
    "possessif": "possessive",
    "possessive": "possessif",
    "possédant": "possédante",
    "possédante": "possédant",
    "possédé": "possédée",
    "possédée": "possédé",
    "post-natal": "post-natale",
    "post-natale": "post-natal",
    "postal": "postale",
    "postale": "postal",
    "posté": "postée",
    "postée": "posté",
    "postier": "postière",
    "postière": "postier",
    "postnatal": "postnatale",
    "postnatale": "postnatal",
    "postulant": "postulante",
    "postulante": "postulant",
    "postérieur": "postérieure",
    "postérieure": "postérieur",
    "potager": "potagère",
    "potagère": "potager",
    "potelé": "potelée",
    "potelée": "potelé",
    "potentiel": "potentielle",
    "potentielle": "potentiel",
    "potier": "potière",
    "potière": "potier",
    "poudré": "poudrée",
    "poudrée": "poudré",
    "pouffant": "pouffante",
    "pouffante": "pouffant",
    "poulain": "poulaine",
    "poulaine": "poulain",
    "poupin": "poupine",
    "poupine": "poupin",
    "pourchassé": "pourchassée",
    "pourchassée": "pourchassé",
    "pourpré": "pourprée",
    "pourprée": "pourpré",
    "pourri": "pourrie",
    "pourrie": "pourri",
    "pourrissant": "pourrissante",
    "pourrissante": "pourrissant",
    "poursuivant": "poursuivante",
    "poursuivante": "poursuivant",
    "poursuivi": "poursuivie",
    "poursuivie": "poursuivi",
    "pourvu": "pourvue",
    "pourvue": "pourvu",
    "pourvoyeur": "pourvoyeuse",
    "pourvoyeuse": "pourvoyeur",
    "poussé": "poussée",
    "poussée": "poussé",
    "poussif": "poussive",
    "poussive": "poussif",
    "poète": "poétesse",
    "poétesse": "poète",
    "praticien": "praticienne",
    "praticienne": "praticien",
    "pratiquant": "pratiquante",
    "pratiquante": "pratiquant",
    "pratiqué": "pratiquée",
    "pratiquée": "pratiqué",
    "premier": "première",
    "première": "premier",
    "prenant": "prenante",
    "prenante": "prenant",
    "preneur": "preneuse",
    "preneuse": "preneur",
    "presbytérien": "presbytérienne",
    "presbytérienne": "presbytérien",
    "prescrit": "prescrite",
    "prescrite": "prescrit",
    "pressant": "pressante",
    "pressante": "pressant",
    "pressenti": "pressentie",
    "pressentie": "pressenti",
    "pressurisé": "pressurisée",
    "pressurisée": "pressurisé",
    "pressé": "pressée",
    "pressée": "pressé",
    "prestidigitateur": "prestidigitatrice",
    "prestidigitatrice": "prestidigitateur",
    "prié": "priée",
    "priée": "prié",
    "prieur": "prieure",
    "prieure": "prieur",
    "primal": "primale",
    "primale": "primal",
    "primé": "primée",
    "primée": "primé",
    "primesautier": "primesautière",
    "primesautière": "primesautier",
    "primitif": "primitive",
    "primitive": "primitif",
    "primordial": "primordiale",
    "primordiale": "primordial",
    "prince": "princesse",
    "princesse": "prince",
    "princier": "princière",
    "princière": "princier",
    "principal": "principale",
    "principale": "principal",
    "printanier": "printanière",
    "printanière": "printanier",
    "prisé": "prisée",
    "prisée": "prisé",
    "prisonnier": "prisonnière",
    "prisonnière": "prisonnier",
    "privatif": "privative",
    "privative": "privatif",
    "privatisé": "privatisée",
    "privatisée": "privatisé",
    "privé": "privée",
    "privée": "privé",
    "privilégié": "privilégiée",
    "privilégiée": "privilégié",
    "probant": "probante",
    "probante": "probant",
    "prochain": "prochaine",
    "prochaine": "prochain",
    "proclamé": "proclamée",
    "proclamée": "proclamé",
    "procréateur": "procréatrice",
    "procréatrice": "procréateur",
    "procurateur": "procuratrice",
    "procuratrice": "procurateur",
    "procuré": "procurée",
    "procurée": "procuré",
    "procureur": "procureuse",
    "procureuse": "procureur",
    "procédural": "procédurale",
    "procédurale": "procédural",
    "procédurier": "procédurière",
    "procédurière": "procédurier",
    "prodigué": "prodiguée",
    "prodiguée": "prodigué",
    "producteur": "productrice",
    "productrice": "producteur",
    "productif": "productive",
    "productive": "productif",
    "produit": "produite",
    "produite": "produit",
    "profané": "profanée",
    "profanée": "profané",
    "professionnel": "professionnelle",
    "professionnelle": "professionnel",
    "profilé": "profilée",
    "profilée": "profilé",
    "profileur": "profileuse",
    "profileuse": "profileur",
    "profité": "profitée",
    "profitée": "profité",
    "profiteur": "profiteuse",
    "profiteuse": "profiteur",
    "profond": "profonde",
    "profonde": "profond",
    "profus": "profuse",
    "profuse": "profus",
    "proféré": "proférée",
    "proférée": "proféré",
    "programmateur": "programmatrice",
    "programmatrice": "programmateur",
    "programmé": "programmée",
    "programmée": "programmé",
    "programmeur": "programmeuse",
    "programmeuse": "programmeur",
    "progressé": "progressée",
    "progressée": "progressé",
    "progressif": "progressive",
    "progressive": "progressif",
    "prohibé": "prohibée",
    "prohibée": "prohibé",
    "projeté": "projetée",
    "projetée": "projeté",
    "proliférant": "proliférante",
    "proliférante": "proliférant",
    "prolongé": "prolongée",
    "prolongée": "prolongé",
    "prolétarien": "prolétarienne",
    "prolétarienne": "prolétarien",
    "promené": "promenée",
    "promenée": "promené",
    "promeneur": "promeneuse",
    "promeneuse": "promeneur",
    "prometteur": "prometteuse",
    "prometteuse": "prometteur",
    "promoteur": "promotrice",
    "promotrice": "promoteur",
    "promotionnel": "promotionnelle",
    "promotionnelle": "promotionnel",
    "promu": "promue",
    "promue": "promu",
    "prompt": "prompte",
    "prompte": "prompt",
    "promulgué": "promulguée",
    "promulguée": "promulgué",
    "prométhéen": "prométhéenne",
    "prométhéenne": "prométhéen",
    "pronominal": "pronominale",
    "pronominale": "pronominal",
    "prononcé": "prononcée",
    "prononcée": "prononcé",
    "propagé": "propagée",
    "propagée": "propagé",
    "prophète": "prophétesse",
    "prophétesse": "prophète",
    "proportionnel": "proportionnelle",
    "proportionnelle": "proportionnel",
    "proportionné": "proportionnée",
    "proportionnée": "proportionné",
    "proposé": "proposée",
    "proposée": "proposé",
    "propret": "proprette",
    "proprette": "propret",
    "propulsé": "propulsée",
    "propulsée": "propulsé",
    "proscrit": "proscrite",
    "proscrite": "proscrit",
    "prospectif": "prospective",
    "prospective": "prospectif",
    "prosterné": "prosternée",
    "prosternée": "prosterné",
    "prostitué": "prostituée",
    "prostituée": "prostitué",
    "prostré": "prostrée",
    "prostrée": "prostré",
    "protecteur": "protectrice",
    "protectrice": "protecteur",
    "protestant": "protestante",
    "protestante": "protestant",
    "protubérant": "protubérante",
    "protubérante": "protubérant",
    "protégé": "protégée",
    "protégée": "protégé",
    "protéiné": "protéinée",
    "protéinée": "protéiné",
    "proustien": "proustienne",
    "proustienne": "proustien",
    "prouvé": "prouvée",
    "prouvée": "prouvé",
    "provençal": "provençale",
    "provençale": "provençal",
    "proverbial": "proverbiale",
    "proverbiale": "proverbial",
    "providentiel": "providentielle",
    "providentielle": "providentiel",
    "provincial": "provinciale",
    "provinciale": "provincial",
    "provocant": "provocante",
    "provocante": "provocant",
    "provocateur": "provocatrice",
    "provocatrice": "provocateur",
    "provoqué": "provoquée",
    "provoquée": "provoqué",
    "proximal": "proximale",
    "proximale": "proximal",
    "proéminent": "proéminente",
    "proéminente": "proéminent",
    "prudent": "prudente",
    "prudente": "prudent",
    "prussien": "prussienne",
    "prussienne": "prussien",
    "précambrien": "précambrienne",
    "précambrienne": "précambrien",
    "précepteur": "préceptrice",
    "préceptrice": "précepteur",
    "précipité": "précipitée",
    "précipitée": "précipité",
    "précisé": "précisée",
    "précisée": "précisé",
    "précité": "précitée",
    "précitée": "précité",
    "précolombien": "précolombienne",
    "précolombienne": "précolombien",
    "préconisé": "préconisée",
    "préconisée": "préconisé",
    "préconçu": "préconçue",
    "préconçue": "préconçu",
    "précédent": "précédente",
    "précédente": "précédent",
    "précédé": "précédée",
    "précédée": "précédé",
    "prédateur": "prédatrice",
    "prédatrice": "prédateur",
    "prédestiné": "prédestinée",
    "prédestinée": "prédestiné",
    "prédictif": "prédictive",
    "prédictive": "prédictif",
    "prédit": "prédite",
    "prédite": "prédit",
    "prédisposé": "prédisposée",
    "prédisposée": "prédisposé",
    "prédominant": "prédominante",
    "prédominante": "prédominant",
    "prédéterminé": "prédéterminée",
    "prédéterminée": "prédéterminé",
    "préexistant": "préexistante",
    "préexistante": "préexistant",
    "préfabriqué": "préfabriquée",
    "préfabriquée": "préfabriqué",
    "préfiguré": "préfigurée",
    "préfigurée": "préfiguré",
    "préfrontal": "préfrontale",
    "préfrontale": "préfrontal",
    "préférentiel": "préférentielle",
    "préférentielle": "préférentiel",
    "préféré": "préférée",
    "préférée": "préféré",
    "préjugé": "préjugée",
    "préjugée": "préjugé",
    "prélevé": "prélevée",
    "prélevée": "prélevé",
    "prématuré": "prématurée",
    "prématurée": "prématuré",
    "prémenstruel": "prémenstruelle",
    "prémenstruelle": "prémenstruel",
    "prémuni": "prémunie",
    "prémunie": "prémuni",
    "prémédité": "préméditée",
    "préméditée": "prémédité",
    "prénatal": "prénatale",
    "prénatale": "prénatal",
    "prénommé": "prénommée",
    "prénommée": "prénommé",
    "prénuptial": "prénuptiale",
    "prénuptiale": "prénuptial",
    "préoccupant": "préoccupante",
    "préoccupante": "préoccupant",
    "préoccupé": "préoccupée",
    "préoccupée": "préoccupé",
    "préparateur": "préparatrice",
    "préparatrice": "préparateur",
    "préparé": "préparée",
    "préparée": "préparé",
    "prépondérant": "prépondérante",
    "prépondérante": "prépondérant",
    "préposé": "préposée",
    "préposée": "préposé",
    "préprogrammé": "préprogrammée",
    "préprogrammée": "préprogrammé",
    "présent": "présente",
    "présente": "présent",
    "présentateur": "présentatrice",
    "présentatrice": "présentateur",
    "présenté": "présentée",
    "présentée": "présenté",
    "préservé": "préservée",
    "préservée": "préservé",
    "président": "présidente",
    "présidente": "président",
    "présidentiel": "présidentielle",
    "présidentielle": "présidentiel",
    "présidé": "présidée",
    "présidée": "présidé",
    "présomptif": "présomptive",
    "présomptive": "présomptif",
    "présomptueuse": "présomptueux",
    "présomptueux": "présomptueuse",
    "présumé": "présumée",
    "présumée": "présumé",
    "présélectionné": "présélectionnée",
    "présélectionnée": "présélectionné",
    "prétendant": "prétendante",
    "prétendante": "prétendant",
    "prétendu": "prétendue",
    "prétendue": "prétendu",
    "prétentiard": "prétentiarde",
    "prétentiarde": "prétentiard",
    "prévenant": "prévenante",
    "prévenante": "prévenant",
    "prévenu": "prévenue",
    "prévenue": "prévenu",
    "préventif": "préventive",
    "préventive": "préventif",
    "prévu": "prévue",
    "prévue": "prévu",
    "prévoyant": "prévoyante",
    "prévoyante": "prévoyant",
    "préélectoral": "préélectorale",
    "préélectorale": "préélectoral",
    "préétabli": "préétablie",
    "préétablie": "préétabli",
    "prêché": "prêchée",
    "prêchée": "prêché",
    "prêcheur": "prêcheuse",
    "prêcheuse": "prêcheur",
    "prêt": "prête",
    "prête": "prêt",
    "prêté": "prêtée",
    "prêtée": "prêté",
    "prêteur": "prêteuse",
    "prêteuse": "prêteur",
    "prêtre": "prêtresse",
    "prêtresse": "prêtre",
    "prôné": "prônée",
    "prônée": "prôné",
    "puant": "puante",
    "puante": "puant",
    "pubien": "pubienne",
    "pubienne": "pubien",
    "public": "publique",
    "publique": "public",
    "publié": "publiée",
    "publiée": "publié",
    "puceau": "pucelle",
    "pucelle": "puceau",
    "pudibond": "pudibonde",
    "pudibonde": "pudibond",
    "puisé": "puisée",
    "puisée": "puisé",
    "puissant": "puissante",
    "puissante": "puissant",
    "pulvérisé": "pulvérisée",
    "pulvérisée": "pulvérisé",
    "punaisé": "punaisée",
    "punaisée": "punaisé",
    "punching ball": "punching-ball",
    "punching-ball": "punching ball",
    "puni": "punie",
    "punie": "puni",
    "punitif": "punitive",
    "punitive": "punitif",
    "pur": "pure",
    "pure": "pur",
    "purgatif": "purgative",
    "purgative": "purgatif",
    "purgé": "purgée",
    "purgée": "purgé",
    "purificateur": "purificatrice",
    "purificatrice": "purificateur",
    "purifié": "purifiée",
    "purifiée": "purifié",
    "puritain": "puritaine",
    "puritaine": "puritain",
    "purpurin": "purpurine",
    "purpurine": "purpurin",
    "purulent": "purulente",
    "purulente": "purulent",
    "putassier": "putassière",
    "putassière": "putassier",
    "putatif": "putative",
    "putative": "putatif",
    "putréfié": "putréfiée",
    "putréfiée": "putréfié",
    "puéril": "puérile",
    "puérile": "puéril",
    "pyramidal": "pyramidale",
    "pyramidale": "pyramidal",
    "pâlichon": "pâlichonne",
    "pâlichonne": "pâlichon",
    "pâli": "pâlie",
    "pâlie": "pâli",
    "pâlissant": "pâlissante",
    "pâlissante": "pâlissant",
    "pâlot": "pâlotte",
    "pâlotte": "pâlot",
    "pâmé": "pâmée",
    "pâmée": "pâmé",
    "pât": "pâte",
    "pâte": "pât",
    "pâtissier": "pâtissière",
    "pâtissière": "pâtissier",
    "pèlerin": "pèlerine",
    "pèlerine": "pèlerin",
    "pécheresse": "pécheur",
    "pécheur": "pécheresse",
    "pédant": "pédante",
    "pédante": "pédant",
    "péjoratif": "péjorative",
    "péjorative": "péjoratif",
    "pénal": "pénale",
    "pénale": "pénal",
    "pénalisé": "pénalisée",
    "pénalisée": "pénalisé",
    "pénard": "pénarde",
    "pénarde": "pénard",
    "pénien": "pénienne",
    "pénienne": "pénien",
    "pénitent": "pénitente",
    "pénitente": "pénitent",
    "pénétrant": "pénétrante",
    "pénétrante": "pénétrant",
    "pénétré": "pénétrée",
    "pénétrée": "pénétré",
    "péquenaud": "péquenaude",
    "péquenaude": "péquenaud",
    "péri": "périe",
    "périe": "péri",
    "périgourdin": "périgourdine",
    "périgourdine": "périgourdin",
    "périmé": "périmée",
    "périmée": "périmé",
    "péritonéal": "péritonéale",
    "péritonéale": "péritonéal",
    "péruvien": "péruvienne",
    "péruvienne": "péruvien",
    "pétant": "pétante",
    "pétante": "pétant",
    "pétaradant": "pétaradante",
    "pétaradante": "pétaradant",
    "pétardier": "pétardière",
    "pétardière": "pétardier",
    "pété": "pétée",
    "pétée": "pété",
    "pétillant": "pétillante",
    "pétillante": "pétillant",
    "pétri": "pétrie",
    "pétrie": "pétri",
    "pétrifié": "pétrifiée",
    "pétrifiée": "pétrifié",
    "pétrolier": "pétrolière",
    "pétrolière": "pétrolier",
    "pétulant": "pétulante",
    "pétulante": "pétulant",
    "pêché": "pêchée",
    "pêchée": "pêché",
    "pêcheur": "pêcheuse",
    "pêcheuse": "pêcheur",
    "quadrillé": "quadrillée",
    "quadrillée": "quadrillé",
    "qualifié": "qualifiée",
    "qualifiée": "qualifié",
    "qualitatif": "qualitative",
    "qualitative": "qualitatif",
    "quantitatif": "quantitative",
    "quantitative": "quantitatif",
    "querellé": "querellée",
    "querellée": "querellé",
    "questionné": "questionnée",
    "questionnée": "questionné",
    "questionneur": "questionneuse",
    "questionneuse": "questionneur",
    "queuté": "queutée",
    "queutée": "queuté",
    "quiet": "quiète",
    "quiète": "quiet",
    "quincaillier": "quincaillière",
    "quincaillière": "quincaillier",
    "quintessencié": "quintessenciée",
    "quintessenciée": "quintessencié",
    "quitté": "quittée",
    "quittée": "quitté",
    "quotidien": "quotidienne",
    "quotidienne": "quotidien",
    "québécois": "québécoise",
    "québécoise": "québécois",
    "quémandeur": "quémandeuse",
    "quémandeuse": "quémandeur",
    "quêteur": "quêteuse",
    "quêteuse": "quêteur",
    "rabaissé": "rabaissée",
    "rabaissée": "rabaissé",
    "rabatteur": "rabatteuse",
    "rabatteuse": "rabatteur",
    "rabattu": "rabattue",
    "rabattue": "rabattu",
    "rabelaisien": "rabelaisienne",
    "rabelaisienne": "rabelaisien",
    "raboté": "rabotée",
    "rabotée": "raboté",
    "rabougri": "rabougrie",
    "rabougrie": "rabougri",
    "rabroué": "rabrouée",
    "rabrouée": "rabroué",
    "rabâché": "rabâchée",
    "rabâchée": "rabâché",
    "raccommodé": "raccommodée",
    "raccommodée": "raccommodé",
    "raccompagné": "raccompagnée",
    "raccompagnée": "raccompagné",
    "raccordé": "raccordée",
    "raccordée": "raccordé",
    "raccourci": "raccourcie",
    "raccourcie": "raccourci",
    "raccroché": "raccrochée",
    "raccrochée": "raccroché",
    "racheté": "rachetée",
    "rachetée": "racheté",
    "racial": "raciale",
    "raciale": "racial",
    "raclant": "raclante",
    "raclante": "raclant",
    "raclé": "raclée",
    "raclée": "raclé",
    "racleur": "racleuse",
    "racleuse": "racleur",
    "racoleur": "racoleuse",
    "racoleuse": "racoleur",
    "raconté": "racontée",
    "racontée": "raconté",
    "raconteur": "raconteuse",
    "raconteuse": "raconteur",
    "racorni": "racornie",
    "racornie": "racorni",
    "racé": "racée",
    "racée": "racé",
    "radial": "radiale",
    "radiale": "radial",
    "radiant": "radiante",
    "radiante": "radiant",
    "radical": "radicale",
    "radicale": "radical",
    "radié": "radiée",
    "radiée": "radié",
    "radin": "radine",
    "radine": "radin",
    "radiné": "radinée",
    "radinée": "radiné",
    "radioactif": "radioactive",
    "radioactive": "radioactif",
    "radiodiffusé": "radiodiffusée",
    "radiodiffusée": "radiodiffusé",
    "radiographié": "radiographiée",
    "radiographiée": "radiographié",
    "radouci": "radoucie",
    "radoucie": "radouci",
    "raffermi": "raffermie",
    "raffermie": "raffermi",
    "raffiné": "raffinée",
    "raffinée": "raffiné",
    "rafistolé": "rafistolée",
    "rafistolée": "rafistolé",
    "raflé": "raflée",
    "raflée": "raflé",
    "rafraîchi": "rafraîchie",
    "rafraîchie": "rafraîchi",
    "rafraîchissant": "rafraîchissante",
    "rafraîchissante": "rafraîchissant",
    "ragaillardi": "ragaillardie",
    "ragaillardie": "ragaillardi",
    "rageur": "rageuse",
    "rageuse": "rageur",
    "ragoûtant": "ragoûtante",
    "ragoûtante": "ragoûtant",
    "raidi": "raidie",
    "raidie": "raidi",
    "raillé": "raillée",
    "raillée": "raillé",
    "railleur": "railleuse",
    "railleuse": "railleur",
    "raisonné": "raisonnée",
    "raisonnée": "raisonné",
    "raisonneur": "raisonneuse",
    "raisonneuse": "raisonneur",
    "rajeuni": "rajeunie",
    "rajeunie": "rajeuni",
    "rajeunissant": "rajeunissante",
    "rajeunissante": "rajeunissant",
    "rajouté": "rajoutée",
    "rajoutée": "rajouté",
    "rajusté": "rajustée",
    "rajustée": "rajusté",
    "ralenti": "ralentie",
    "ralentie": "ralenti",
    "rallié": "ralliée",
    "ralliée": "rallié",
    "rallongé": "rallongée",
    "rallongée": "rallongé",
    "rallumé": "rallumée",
    "rallumée": "rallumé",
    "ramassé": "ramassée",
    "ramassée": "ramassé",
    "ramasseur": "ramasseuse",
    "ramasseuse": "ramasseur",
    "ramené": "ramenée",
    "ramenée": "ramené",
    "ramifié": "ramifiée",
    "ramifiée": "ramifié",
    "ramolli": "ramollie",
    "ramollie": "ramolli",
    "ramoné": "ramonée",
    "ramonée": "ramoné",
    "rampant": "rampante",
    "rampante": "rampant",
    "rampé": "rampée",
    "rampée": "rampé",
    "rancardé": "rancardée",
    "rancardée": "rancardé",
    "ranci": "rancie",
    "rancie": "ranci",
    "rancunier": "rancunière",
    "rancunière": "rancunier",
    "randonneur": "randonneuse",
    "randonneuse": "randonneur",
    "rangé": "rangée",
    "rangée": "rangé",
    "ranimé": "ranimée",
    "ranimée": "ranimé",
    "rançonné": "rançonnée",
    "rançonnée": "rançonné",
    "rapatrié": "rapatriée",
    "rapatriée": "rapatrié",
    "rapetissé": "rapetissée",
    "rapetissée": "rapetissé",
    "rapiécé": "rapiécée",
    "rapiécée": "rapiécé",
    "rappelé": "rappelée",
    "rappelée": "rappelé",
    "rappeur": "rappeuse",
    "rappeuse": "rappeur",
    "rapporté": "rapportée",
    "rapportée": "rapporté",
    "rapporteur": "rapporteuse",
    "rapporteuse": "rapporteur",
    "rapproché": "rapprochée",
    "rapprochée": "rapproché",
    "raqué": "raquée",
    "raquée": "raqué",
    "raréfié": "raréfiée",
    "raréfiée": "raréfié",
    "rasant": "rasante",
    "rasante": "rasant",
    "rasé": "rasée",
    "rasée": "rasé",
    "raseur": "raseuse",
    "raseuse": "raseur",
    "rassasié": "rassasiée",
    "rassasiée": "rassasié",
    "rassemblé": "rassemblée",
    "rassemblée": "rassemblé",
    "rassurant": "rassurante",
    "rassurante": "rassurant",
    "rassuré": "rassurée",
    "rassurée": "rassuré",
    "rasséréné": "rassérénée",
    "rassérénée": "rasséréné",
    "ratatiné": "ratatinée",
    "ratatinée": "ratatiné",
    "raté": "ratée",
    "ratée": "raté",
    "ratifié": "ratifiée",
    "ratifiée": "ratifié",
    "rationnel": "rationnelle",
    "rationnelle": "rationnel",
    "rationné": "rationnée",
    "rationnée": "rationné",
    "ratissé": "ratissée",
    "ratissée": "ratissé",
    "rattaché": "rattachée",
    "rattachée": "rattaché",
    "rattrapé": "rattrapée",
    "rattrapée": "rattrapé",
    "raturé": "raturée",
    "raturée": "raturé",
    "ravageant": "ravageante",
    "ravageante": "ravageant",
    "ravagé": "ravagée",
    "ravagée": "ravagé",
    "ravageur": "ravageuse",
    "ravageuse": "ravageur",
    "ravalé": "ravalée",
    "ravalée": "ravalé",
    "ravi": "ravie",
    "ravie": "ravi",
    "raviné": "ravinée",
    "ravinée": "raviné",
    "ravisé": "ravisée",
    "ravisée": "ravisé",
    "ravissant": "ravissante",
    "ravissante": "ravissant",
    "ravisseur": "ravisseuse",
    "ravisseuse": "ravisseur",
    "ravitaillé": "ravitaillée",
    "ravitaillée": "ravitaillé",
    "ravitailleur": "ravitailleuse",
    "ravitailleuse": "ravitailleur",
    "ravivé": "ravivée",
    "ravivée": "ravivé",
    "rayé": "rayée",
    "rayée": "rayé",
    "rayon": "rayonne",
    "rayonne": "rayon",
    "rayonnant": "rayonnante",
    "rayonnante": "rayonnant",
    "rebaptisé": "rebaptisée",
    "rebaptisée": "rebaptisé",
    "rebattu": "rebattue",
    "rebattue": "rebattu",
    "rebellé": "rebellée",
    "rebellée": "rebellé",
    "rebiffé": "rebiffée",
    "rebiffée": "rebiffé",
    "rebondi": "rebondie",
    "rebondie": "rebondi",
    "rebondissant": "rebondissante",
    "rebondissante": "rebondissant",
    "rebouché": "rebouchée",
    "rebouchée": "rebouché",
    "rebouclé": "rebouclée",
    "rebouclée": "rebouclé",
    "reboutonné": "reboutonnée",
    "reboutonnée": "reboutonné",
    "rebranché": "rebranchée",
    "rebranchée": "rebranché",
    "rebroussé": "rebroussée",
    "rebroussée": "rebroussé",
    "rebutant": "rebutante",
    "rebutante": "rebutant",
    "rebuté": "rebutée",
    "rebutée": "rebuté",
    "rebâti": "rebâtie",
    "rebâtie": "rebâti",
    "recalé": "recalée",
    "recalée": "recalé",
    "recelé": "recelée",
    "recelée": "recelé",
    "receleur": "receleuse",
    "receleuse": "receleur",
    "recentré": "recentrée",
    "recentrée": "recentré",
    "receveur": "receveuse",
    "receveuse": "receveur",
    "reçu": "reçue",
    "reçue": "reçu",
    "rechargé": "rechargée",
    "rechargée": "rechargé",
    "recherché": "recherchée",
    "recherchée": "recherché",
    "rechigné": "rechignée",
    "rechignée": "rechigné",
    "reclassé": "reclassée",
    "reclassée": "reclassé",
    "recoiffé": "recoiffée",
    "recoiffée": "recoiffé",
    "recollé": "recollée",
    "recollée": "recollé",
    "recommandé": "recommandée",
    "recommandée": "recommandé",
    "recommencé": "recommencée",
    "recommencée": "recommencé",
    "recomposé": "recomposée",
    "recomposée": "recomposé",
    "reconditionné": "reconditionnée",
    "reconditionnée": "reconditionné",
    "reconduit": "reconduite",
    "reconduite": "reconduit",
    "reconnaissant": "reconnaissante",
    "reconnaissante": "reconnaissant",
    "reconnu": "reconnue",
    "reconnue": "reconnu",
    "reconsidéré": "reconsidérée",
    "reconsidérée": "reconsidéré",
    "reconstituant": "reconstituante",
    "reconstituante": "reconstituant",
    "reconstitué": "reconstituée",
    "reconstituée": "reconstitué",
    "reconstruit": "reconstruite",
    "reconstruite": "reconstruit",
    "reconverti": "reconvertie",
    "reconvertie": "reconverti",
    "recopié": "recopiée",
    "recopiée": "recopié",
    "recouché": "recouchée",
    "recouchée": "recouché",
    "recousu": "recousue",
    "recousue": "recousu",
    "recourbé": "recourbée",
    "recourbée": "recourbé",
    "recouvert": "recouverte",
    "recouverte": "recouvert",
    "recouvré": "recouvrée",
    "recouvrée": "recouvré",
    "recraché": "recrachée",
    "recrachée": "recraché",
    "recroquevillé": "recroquevillée",
    "recroquevillée": "recroquevillé",
    "recru": "recrue",
    "recrue": "recru",
    "recruté": "recrutée",
    "recrutée": "recruté",
    "re-création": "recréation",
    "recréation": "re-création",
    "recrépi": "recrépie",
    "recrépie": "recrépi",
    "rectal": "rectale",
    "rectale": "rectal",
    "rectifié": "rectifiée",
    "rectifiée": "rectifié",
    "recueilli": "recueillie",
    "recueillie": "recueilli",
    "recuit": "recuite",
    "recuite": "recuit",
    "re-cul": "recul",
    "recul": "re-cul",
    "reculé": "reculée",
    "reculée": "reculé",
    "reculotté": "reculottée",
    "reculottée": "reculotté",
    "recyclé": "recyclée",
    "recyclée": "recyclé",
    "redescendu": "redescendue",
    "redescendue": "redescendu",
    "redessiné": "redessinée",
    "redessinée": "redessiné",
    "redevenu": "redevenue",
    "redevenue": "redevenu",
    "redit": "redite",
    "redite": "redit",
    "redondant": "redondante",
    "redondante": "redondant",
    "redoublé": "redoublée",
    "redoublée": "redoublé",
    "redouté": "redoutée",
    "redoutée": "redouté",
    "redressé": "redressée",
    "redressée": "redressé",
    "redresseur": "redresseuse",
    "redresseuse": "redresseur",
    "refait": "refaite",
    "refaite": "refait",
    "refermé": "refermée",
    "refermée": "refermé",
    "refilé": "refilée",
    "refilée": "refilé",
    "reflué": "refluée",
    "refluée": "reflué",
    "reflété": "reflétée",
    "reflétée": "reflété",
    "reforgé": "reforgée",
    "reforgée": "reforgé",
    "reformé": "reformée",
    "reformée": "reformé",
    "reformulé": "reformulée",
    "reformulée": "reformulé",
    "refoulant": "refoulante",
    "refoulante": "refoulant",
    "refoulé": "refoulée",
    "refoulée": "refoulé",
    "refroidi": "refroidie",
    "refroidie": "refroidi",
    "refusé": "refusée",
    "refusée": "refusé",
    "regagné": "regagnée",
    "regagnée": "regagné",
    "regardant": "regardante",
    "regardante": "regardant",
    "regardé": "regardée",
    "regardée": "regardé",
    "regretté": "regrettée",
    "regrettée": "regretté",
    "regroupé": "regroupée",
    "regroupée": "regroupé",
    "rehaussé": "rehaussée",
    "rehaussée": "rehaussé",
    "rejeté": "rejetée",
    "rejetée": "rejeté",
    "rejoint": "rejointe",
    "rejointe": "rejoint",
    "relancé": "relancée",
    "relancée": "relancé",
    "relaps": "relapse",
    "relapse": "relaps",
    "relaté": "relatée",
    "relatée": "relaté",
    "relatif": "relative",
    "relative": "relatif",
    "relationnel": "relationnelle",
    "relationnelle": "relationnel",
    "relaxant": "relaxante",
    "relaxante": "relaxant",
    "relaxé": "relaxée",
    "relaxée": "relaxé",
    "relayé": "relayée",
    "relayée": "relayé",
    "relevé": "relevée",
    "relevée": "relevé",
    "relié": "reliée",
    "reliée": "relié",
    "relu": "relue",
    "relue": "relu",
    "relooké": "relookée",
    "relookée": "relooké",
    "reluisant": "reluisante",
    "reluisante": "reluisant",
    "reluqué": "reluquée",
    "reluquée": "reluqué",
    "relâché": "relâchée",
    "relâchée": "relâché",
    "relégué": "reléguée",
    "reléguée": "relégué",
    "remanié": "remaniée",
    "remaniée": "remanié",
    "remaquillé": "remaquillée",
    "remaquillée": "remaquillé",
    "remarié": "remariée",
    "remariée": "remarié",
    "remarqué": "remarquée",
    "remarquée": "remarqué",
    "rembarré": "rembarrée",
    "rembarrée": "rembarré",
    "remblayé": "remblayée",
    "remblayée": "remblayé",
    "rembourré": "rembourrée",
    "rembourrée": "rembourré",
    "remboursé": "remboursée",
    "remboursée": "remboursé",
    "remercié": "remerciée",
    "remerciée": "remercié",
    "remis": "remise",
    "remise": "remis",
    "remisé": "remisée",
    "remisée": "remisé",
    "remodelé": "remodelée",
    "remodelée": "remodelé",
    "remonté": "remontée",
    "remontée": "remonté",
    "remorqué": "remorquée",
    "remorquée": "remorqué",
    "remorqueur": "remorqueuse",
    "remorqueuse": "remorqueur",
    "remplacé": "remplacée",
    "remplacée": "remplacé",
    "remplaçant": "remplaçante",
    "remplaçante": "remplaçant",
    "rempli": "remplie",
    "remplie": "rempli",
    "remporté": "remportée",
    "remportée": "remporté",
    "remuant": "remuante",
    "remuante": "remuant",
    "remué": "remuée",
    "remuée": "remué",
    "remâché": "remâchée",
    "remâchée": "remâché",
    "renaissant": "renaissante",
    "renaissante": "renaissant",
    "renard": "renarde",
    "renarde": "renard",
    "rené": "renée",
    "renée": "rené",
    "rencardé": "rencardée",
    "rencardée": "rencardé",
    "rencontré": "rencontrée",
    "rencontrée": "rencontré",
    "rendu": "rendue",
    "rendue": "rendu",
    "renfermé": "renfermée",
    "renfermée": "renfermé",
    "renflé": "renflée",
    "renflée": "renflé",
    "renfloué": "renflouée",
    "renflouée": "renfloué",
    "renfoncé": "renfoncée",
    "renfoncée": "renfoncé",
    "renforcé": "renforcée",
    "renforcée": "renforcé",
    "renfrogné": "renfrognée",
    "renfrognée": "renfrogné",
    "renié": "reniée",
    "reniée": "renié",
    "reniflé": "reniflée",
    "reniflée": "reniflé",
    "renifleur": "renifleuse",
    "renifleuse": "renifleur",
    "renommé": "renommée",
    "renommée": "renommé",
    "renoncé": "renoncée",
    "renoncée": "renoncé",
    "renoué": "renouée",
    "renouée": "renoué",
    "renouvelé": "renouvelée",
    "renouvelée": "renouvelé",
    "renseigné": "renseignée",
    "renseignée": "renseigné",
    "rentabilisé": "rentabilisée",
    "rentabilisée": "rentabilisé",
    "rentier": "rentière",
    "rentière": "rentier",
    "rentrant": "rentrante",
    "rentrante": "rentrant",
    "rentré": "rentrée",
    "rentrée": "rentré",
    "renversant": "renversante",
    "renversante": "renversant",
    "renversé": "renversée",
    "renversée": "renversé",
    "renvoyé": "renvoyée",
    "renvoyée": "renvoyé",
    "renégat": "renégate",
    "renégate": "renégat",
    "reparu": "reparue",
    "reparue": "reparu",
    "reparti": "repartie",
    "repartie": "reparti",
    "repassé": "repassée",
    "repassée": "repassé",
    "repasseur": "repasseuse",
    "repasseuse": "repasseur",
    "repeigné": "repeignée",
    "repeignée": "repeigné",
    "repeint": "repeinte",
    "repeinte": "repeint",
    "repensé": "repensée",
    "repensée": "repensé",
    "repentant": "repentante",
    "repentante": "repentant",
    "repenti": "repentie",
    "repentie": "repenti",
    "reperdu": "reperdue",
    "reperdue": "reperdu",
    "replacé": "replacée",
    "replacée": "replacé",
    "replanté": "replantée",
    "replantée": "replanté",
    "replet": "replète",
    "replète": "replet",
    "replié": "repliée",
    "repliée": "replié",
    "replongé": "replongée",
    "replongée": "replongé",
    "replâtré": "replâtrée",
    "replâtrée": "replâtré",
    "reporté": "reportée",
    "reportée": "reporté",
    "reposant": "reposante",
    "reposante": "reposant",
    "reposé": "reposée",
    "reposée": "reposé",
    "repoudré": "repoudrée",
    "repoudrée": "repoudré",
    "repoussant": "repoussante",
    "repoussante": "repoussant",
    "repoussé": "repoussée",
    "repoussée": "repoussé",
    "reprisé": "reprisée",
    "reprisée": "reprisé",
    "reproché": "reprochée",
    "reprochée": "reproché",
    "reproducteur": "reproductrice",
    "reproductrice": "reproducteur",
    "reproductif": "reproductive",
    "reproductive": "reproductif",
    "reproduit": "reproduite",
    "reproduite": "reproduit",
    "représentant": "représentante",
    "représentante": "représentant",
    "représentatif": "représentative",
    "représentative": "représentatif",
    "représenté": "représentée",
    "représentée": "représenté",
    "reptilien": "reptilienne",
    "reptilienne": "reptilien",
    "repu": "repue",
    "repue": "repu",
    "repéré": "repérée",
    "repérée": "repéré",
    "repêché": "repêchée",
    "repêchée": "repêché",
    "requinqué": "requinquée",
    "requinquée": "requinqué",
    "requérant": "requérante",
    "requérante": "requérant",
    "rescapé": "rescapée",
    "rescapée": "rescapé",
    "respecté": "respectée",
    "respectée": "respecté",
    "respectif": "respective",
    "respective": "respectif",
    "respiré": "respirée",
    "respirée": "respiré",
    "resplendissant": "resplendissante",
    "resplendissante": "resplendissant",
    "resquilleur": "resquilleuse",
    "resquilleuse": "resquilleur",
    "ressaisi": "ressaisie",
    "ressaisie": "ressaisi",
    "ressassé": "ressassée",
    "ressassée": "ressassé",
    "ressemblant": "ressemblante",
    "ressemblante": "ressemblant",
    "ressenti": "ressentie",
    "ressentie": "ressenti",
    "resserré": "resserrée",
    "resserrée": "resserré",
    "ressorti": "ressortie",
    "ressortie": "ressorti",
    "ressortissant": "ressortissante",
    "ressortissante": "ressortissant",
    "ressoudé": "ressoudée",
    "ressoudée": "ressoudé",
    "ressurgi": "ressurgie",
    "ressurgie": "ressurgi",
    "ressuscité": "ressuscitée",
    "ressuscitée": "ressuscité",
    "ressuyé": "ressuyée",
    "ressuyée": "ressuyé",
    "restant": "restante",
    "restante": "restant",
    "restaurateur": "restauratrice",
    "restauratrice": "restaurateur",
    "restauré": "restaurée",
    "restaurée": "restauré",
    "resté": "restée",
    "restée": "resté",
    "restitué": "restituée",
    "restituée": "restitué",
    "restreint": "restreinte",
    "restreinte": "restreint",
    "restrictif": "restrictive",
    "restrictive": "restrictif",
    "re-structuration": "restructuration",
    "restructuration": "re-structuration",
    "restructuré": "restructurée",
    "restructurée": "restructuré",
    "retaillé": "retaillée",
    "retaillée": "retaillé",
    "retapé": "retapée",
    "retapée": "retapé",
    "retapissé": "retapissée",
    "retapissée": "retapissé",
    "retardé": "retardée",
    "retardée": "retardé",
    "retenu": "retenue",
    "retenue": "retenu",
    "retentissant": "retentissante",
    "retentissante": "retentissant",
    "retiré": "retirée",
    "retirée": "retiré",
    "retombant": "retombante",
    "retombante": "retombant",
    "retombé": "retombée",
    "retombée": "retombé",
    "retouché": "retouchée",
    "retouchée": "retouché",
    "retourné": "retournée",
    "retournée": "retourné",
    "retracé": "retracée",
    "retracée": "retracé",
    "retraité": "retraitée",
    "retraitée": "retraité",
    "retranché": "retranchée",
    "retranchée": "retranché",
    "retravaillé": "retravaillée",
    "retravaillée": "retravaillé",
    "retraversé": "retraversée",
    "retraversée": "retraversé",
    "retroussé": "retroussée",
    "retroussée": "retroussé",
    "retrouvé": "retrouvée",
    "retrouvée": "retrouvé",
    "revanchard": "revancharde",
    "revancharde": "revanchard",
    "revenant": "revenante",
    "revenante": "revenant",
    "revendeur": "revendeuse",
    "revendeuse": "revendeur",
    "revendicateur": "revendicatrice",
    "revendicatrice": "revendicateur",
    "revendicatif": "revendicative",
    "revendicative": "revendicatif",
    "revendiqué": "revendiquée",
    "revendiquée": "revendiqué",
    "revendu": "revendue",
    "revendue": "revendu",
    "revenu": "revenue",
    "revenue": "revenu",
    "reverdi": "reverdie",
    "reverdie": "reverdi",
    "reverni": "revernie",
    "revernie": "reverni",
    "revigorant": "revigorante",
    "revigorante": "revigorant",
    "revigoré": "revigorée",
    "revigorée": "revigoré",
    "revisité": "revisitée",
    "revisitée": "revisité",
    "revécu": "revécue",
    "revécue": "revécu",
    "revu": "revue",
    "revue": "revu",
    "re-vérifié": "revérifié",
    "revérifié": "re-vérifié",
    "revêtu": "revêtue",
    "revêtue": "revêtu",
    "rhabillé": "rhabillée",
    "rhabillée": "rhabillé",
    "rhumatisant": "rhumatisante",
    "rhumatisante": "rhumatisant",
    "rhumatismal": "rhumatismale",
    "rhumatismale": "rhumatismal",
    "rhénan": "rhénane",
    "rhénane": "rhénan",
    "riant": "riante",
    "riante": "riant",
    "ribot": "ribote",
    "ribote": "ribot",
    "ricain": "ricaine",
    "ricaine": "ricain",
    "ricanant": "ricanante",
    "ricanante": "ricanant",
    "ricaneur": "ricaneuse",
    "ricaneuse": "ricaneur",
    "richard": "richarde",
    "richarde": "richard",
    "rideau": "ridelle",
    "ridelle": "rideau",
    "ridé": "ridée",
    "ridée": "ridé",
    "ridiculisé": "ridiculisée",
    "ridiculisée": "ridiculisé",
    "rieur": "rieuse",
    "rieuse": "rieur",
    "rigolard": "rigolarde",
    "rigolarde": "rigolard",
    "rigolo": "rigolote",
    "rigolote": "rigolo",
    "rimé": "rimée",
    "rimée": "rimé",
    "rimeur": "rimeuse",
    "rimeuse": "rimeur",
    "rincé": "rincée",
    "rincée": "rincé",
    "ringard": "ringarde",
    "ringarde": "ringard",
    "ripoliné": "ripolinée",
    "ripolinée": "ripoliné",
    "risqué": "risquée",
    "risquée": "risqué",
    "rital": "ritale",
    "ritale": "rital",
    "rituel": "rituelle",
    "rituelle": "rituel",
    "rival": "rivale",
    "rivale": "rival",
    "rivé": "rivée",
    "rivée": "rivé",
    "roast beef": "roast-beef",
    "roast-beef": "roast beef",
    "roboratif": "roborative",
    "roborative": "roboratif",
    "robotisé": "robotisée",
    "robotisée": "robotisé",
    "rockeur": "rockeuse",
    "rockeuse": "rockeur",
    "rocking chair": "rocking-chair",
    "rocking-chair": "rocking chair",
    "rodé": "rodée",
    "rodée": "rodé",
    "rogné": "rognée",
    "rognée": "rogné",
    "roidi": "roidie",
    "roidie": "roidi",
    "romain": "romaine",
    "romaine": "romain",
    "roman": "romane",
    "romane": "roman",
    "romancé": "romancée",
    "romancée": "romancé",
    "romancier": "romancière",
    "romancière": "romancier",
    "romanichel": "romanichelle",
    "romanichelle": "romanichel",
    "rompu": "rompue",
    "rompue": "rompu",
    "ronchon": "ronchonne",
    "ronchonne": "ronchon",
    "ronchonneur": "ronchonneuse",
    "ronchonneuse": "ronchonneur",
    "rond": "ronde",
    "ronde": "rond",
    "rondelet": "rondelette",
    "rondelette": "rondelet",
    "rondouillard": "rondouillarde",
    "rondouillarde": "rondouillard",
    "ronflant": "ronflante",
    "ronflante": "ronflant",
    "ronfleur": "ronfleuse",
    "ronfleuse": "ronfleur",
    "rongeant": "rongeante",
    "rongeante": "rongeant",
    "rongé": "rongée",
    "rongée": "rongé",
    "rongeur": "rongeuse",
    "rongeuse": "rongeur",
    "ronronnant": "ronronnante",
    "ronronnante": "ronronnant",
    "rosé": "rosée",
    "rosée": "rosé",
    "rosi": "rosie",
    "rosie": "rosi",
    "rossé": "rossée",
    "rossée": "rossé",
    "rotatif": "rotative",
    "rotative": "rotatif",
    "roteur": "roteuse",
    "roteuse": "roteur",
    "roturier": "roturière",
    "roturière": "roturier",
    "roublard": "roublarde",
    "roublarde": "roublard",
    "roucoulant": "roucoulante",
    "roucoulante": "roucoulant",
    "roué": "rouée",
    "rouée": "roué",
    "rougeaud": "rougeaude",
    "rougeaude": "rougeaud",
    "rougeoyant": "rougeoyante",
    "rougeoyante": "rougeoyant",
    "rougi": "rougie",
    "rougie": "rougi",
    "rougissant": "rougissante",
    "rougissante": "rougissant",
    "rouillé": "rouillée",
    "rouillée": "rouillé",
    "roulant": "roulante",
    "roulante": "roulant",
    "roulé": "roulée",
    "roulée": "roulé",
    "rouleur": "rouleuse",
    "rouleuse": "rouleur",
    "roumain": "roumaine",
    "roumaine": "roumain",
    "rouquin": "rouquine",
    "rouquine": "rouquin",
    "roussi": "roussie",
    "roussie": "roussi",
    "routard": "routarde",
    "routarde": "routard",
    "routier": "routière",
    "routière": "routier",
    "routinier": "routinière",
    "routinière": "routinier",
    "rouvert": "rouverte",
    "rouverte": "rouvert",
    "royal": "royale",
    "royale": "royal",
    "rubicond": "rubiconde",
    "rubiconde": "rubicond",
    "ruché": "ruchée",
    "ruchée": "ruché",
    "rué": "ruée",
    "ruée": "rué",
    "rugissant": "rugissante",
    "rugissante": "rugissant",
    "ruiné": "ruinée",
    "ruinée": "ruiné",
    "ruisselant": "ruisselante",
    "ruisselante": "ruisselant",
    "ruminant": "ruminante",
    "ruminante": "ruminant",
    "ruminé": "ruminée",
    "ruminée": "ruminé",
    "rupin": "rupine",
    "rupine": "rupin",
    "rural": "rurale",
    "rurale": "rural",
    "rusé": "rusée",
    "rusée": "rusé",
    "russifié": "russifiée",
    "russifiée": "russifié",
    "russo-allemand": "russo-allemande",
    "russo-allemande": "russo-allemand",
    "rustaud": "rustaude",
    "rustaude": "rustaud",
    "rutilant": "rutilante",
    "rutilante": "rutilant",
    "rythmé": "rythmée",
    "rythmée": "rythmé",
    "râblé": "râblée",
    "râblée": "râblé",
    "râlant": "râlante",
    "râlante": "râlant",
    "râleur": "râleuse",
    "râleuse": "râleur",
    "râpé": "râpée",
    "râpée": "râpé",
    "réactif": "réactive",
    "réactive": "réactif",
    "réactivé": "réactivée",
    "réactivée": "réactivé",
    "réadmis": "réadmise",
    "réadmise": "réadmis",
    "réaffecté": "réaffectée",
    "réaffectée": "réaffecté",
    "réaffirmé": "réaffirmée",
    "réaffirmée": "réaffirmé",
    "réalisateur": "réalisatrice",
    "réalisatrice": "réalisateur",
    "réalisé": "réalisée",
    "réalisée": "réalisé",
    "réanimé": "réanimée",
    "réanimée": "réanimé",
    "réapparu": "réapparue",
    "réapparue": "réapparu",
    "réapprovisionné": "réapprovisionnée",
    "réapprovisionnée": "réapprovisionné",
    "rébarbatif": "rébarbative",
    "rébarbative": "rébarbatif",
    "récalcitrant": "récalcitrante",
    "récalcitrante": "récalcitrant",
    "récapitulé": "récapitulée",
    "récapitulée": "récapitulé",
    "récent": "récente",
    "récente": "récent",
    "récepteur": "réceptrice",
    "réceptrice": "récepteur",
    "réceptif": "réceptive",
    "réceptive": "réceptif",
    "récessif": "récessive",
    "récessive": "récessif",
    "réchauffé": "réchauffée",
    "réchauffée": "réchauffé",
    "récidivant": "récidivante",
    "récidivante": "récidivant",
    "récitant": "récitante",
    "récitante": "récitant",
    "récité": "récitée",
    "récitée": "récité",
    "réclamé": "réclamée",
    "réclamée": "réclamé",
    "récolté": "récoltée",
    "récoltée": "récolté",
    "récompensé": "récompensée",
    "récompensée": "récompensé",
    "réconcilié": "réconciliée",
    "réconciliée": "réconcilié",
    "réconfortant": "réconfortante",
    "réconfortante": "réconfortant",
    "réconforté": "réconfortée",
    "réconfortée": "réconforté",
    "récrié": "récriée",
    "récriée": "récrié",
    "récréatif": "récréative",
    "récréative": "récréatif",
    "récup": "récupe",
    "récupe": "récup",
    "récupéré": "récupérée",
    "récupérée": "récupéré",
    "récuré": "récurée",
    "récurée": "récuré",
    "récurrent": "récurrente",
    "récurrente": "récurrent",
    "récusé": "récusée",
    "récusée": "récusé",
    "rédacteur": "rédactrice",
    "rédactrice": "rédacteur",
    "rédactionnel": "rédactionnelle",
    "rédactionnelle": "rédactionnel",
    "rédempteur": "rédemptrice",
    "rédemptrice": "rédempteur",
    "rédigé": "rédigée",
    "rédigée": "rédigé",
    "réduit": "réduite",
    "réduite": "réduit",
    "réel": "réelle",
    "réelle": "réel",
    "réexaminé": "réexaminée",
    "réexaminée": "réexaminé",
    "réflexif": "réflexive",
    "réflexive": "réflexif",
    "réfléchi": "réfléchie",
    "réfléchie": "réfléchi",
    "réfléchissant": "réfléchissante",
    "réfléchissante": "réfléchissant",
    "réformé": "réformée",
    "réformée": "réformé",
    "réfrigérant": "réfrigérante",
    "réfrigérante": "réfrigérant",
    "réfrigéré": "réfrigérée",
    "réfrigérée": "réfrigéré",
    "réfugié": "réfugiée",
    "réfugiée": "réfugié",
    "réfuté": "réfutée",
    "réfutée": "réfuté",
    "référencé": "référencée",
    "référencée": "référencé",
    "référé": "référée",
    "référée": "référé",
    "régalé": "régalée",
    "régalée": "régalé",
    "régent": "régente",
    "régente": "régent",
    "régenté": "régentée",
    "régentée": "régenté",
    "régional": "régionale",
    "régionale": "régional",
    "régi": "régie",
    "régie": "régi",
    "régisseur": "régisseuse",
    "régisseuse": "régisseur",
    "réglementé": "réglementée",
    "réglementée": "réglementé",
    "réglé": "réglée",
    "réglée": "réglé",
    "régnant": "régnante",
    "régnante": "régnant",
    "régressif": "régressive",
    "régressive": "régressif",
    "régulateur": "régulatrice",
    "régulatrice": "régulateur",
    "régulé": "régulée",
    "régulée": "régulé",
    "régulier": "régulière",
    "régulière": "régulier",
    "régénérateur": "régénératrice",
    "régénératrice": "régénérateur",
    "régénéré": "régénérée",
    "régénérée": "régénéré",
    "réhabilité": "réhabilitée",
    "réhabilitée": "réhabilité",
    "réimplanté": "réimplantée",
    "réimplantée": "réimplanté",
    "réincarné": "réincarnée",
    "réincarnée": "réincarné",
    "réinstallé": "réinstallée",
    "réinstallée": "réinstallé",
    "réintroduit": "réintroduite",
    "réintroduite": "réintroduit",
    "réintégré": "réintégrée",
    "réintégrée": "réintégré",
    "réitéré": "réitérée",
    "réitérée": "réitéré",
    "réjoui": "réjouie",
    "réjouie": "réjoui",
    "réjouissant": "réjouissante",
    "réjouissante": "réjouissant",
    "rémanent": "rémanente",
    "rémanente": "rémanent",
    "rémunérateur": "rémunératrice",
    "rémunératrice": "rémunérateur",
    "rénal": "rénale",
    "rénale": "rénal",
    "rénové": "rénovée",
    "rénovée": "rénové",
    "répandu": "répandue",
    "répandue": "répandu",
    "réparateur": "réparatrice",
    "réparatrice": "réparateur",
    "réparé": "réparée",
    "réparée": "réparé",
    "réparti": "répartie",
    "répartie": "réparti",
    "répercuté": "répercutée",
    "répercutée": "répercuté",
    "répertorié": "répertoriée",
    "répertoriée": "répertorié",
    "répondeur": "répondeuse",
    "répondeuse": "répondeur",
    "répressif": "répressive",
    "répressive": "répressif",
    "réprimandé": "réprimandée",
    "réprimandée": "réprimandé",
    "réprimé": "réprimée",
    "réprimée": "réprimé",
    "réprobateur": "réprobatrice",
    "réprobatrice": "réprobateur",
    "républicain": "républicaine",
    "républicaine": "républicain",
    "répudié": "répudiée",
    "répudiée": "répudié",
    "répugnant": "répugnante",
    "répugnante": "répugnant",
    "répugné": "répugnée",
    "répugnée": "répugné",
    "répulsif": "répulsive",
    "répulsive": "répulsif",
    "réputé": "réputée",
    "réputée": "réputé",
    "répété": "répétée",
    "répétée": "répété",
    "répétiteur": "répétitrice",
    "répétitrice": "répétiteur",
    "répétitif": "répétitive",
    "répétitive": "répétitif",
    "réquisitionné": "réquisitionnée",
    "réquisitionnée": "réquisitionné",
    "réservé": "réservée",
    "réservée": "réservé",
    "résident": "résidente",
    "résidente": "résident",
    "résidentiel": "résidentielle",
    "résidentielle": "résidentiel",
    "résiduel": "résiduelle",
    "résiduelle": "résiduel",
    "résigné": "résignée",
    "résignée": "résigné",
    "résilié": "résiliée",
    "résiliée": "résilié",
    "résineuse": "résineux",
    "résineux": "résineuse",
    "résistant": "résistante",
    "résistante": "résistant",
    "résolu": "résolue",
    "résolue": "résolu",
    "résonnant": "résonnante",
    "résonnante": "résonnant",
    "résorbé": "résorbée",
    "résorbée": "résorbé",
    "résultant": "résultante",
    "résultante": "résultant",
    "résumé": "résumée",
    "résumée": "résumé",
    "rétabli": "rétablie",
    "rétablie": "rétabli",
    "réticent": "réticente",
    "réticente": "réticent",
    "réticulé": "réticulée",
    "réticulée": "réticulé",
    "rétif": "rétive",
    "rétive": "rétif",
    "rétinien": "rétinienne",
    "rétinienne": "rétinien",
    "rétracté": "rétractée",
    "rétractée": "rétracté",
    "rétribué": "rétribuée",
    "rétribuée": "rétribué",
    "rétroactif": "rétroactive",
    "rétroactive": "rétroactif",
    "rétrogradé": "rétrogradée",
    "rétrogradée": "rétrogradé",
    "rétrospectif": "rétrospective",
    "rétrospective": "rétrospectif",
    "rétréci": "rétrécie",
    "rétrécie": "rétréci",
    "réunifié": "réunifiée",
    "réunifiée": "réunifié",
    "réuni": "réunie",
    "réunie": "réuni",
    "réussi": "réussie",
    "réussie": "réussi",
    "réveillé": "réveillée",
    "réveillée": "réveillé",
    "réverbéré": "réverbérée",
    "réverbérée": "réverbéré",
    "révisé": "révisée",
    "révisée": "révisé",
    "révoltant": "révoltante",
    "révoltante": "révoltant",
    "révolté": "révoltée",
    "révoltée": "révolté",
    "révolu": "révolue",
    "révolue": "révolu",
    "révoqué": "révoquée",
    "révoquée": "révoqué",
    "révulsé": "révulsée",
    "révulsée": "révulsé",
    "révulsif": "révulsive",
    "révulsive": "révulsif",
    "révélateur": "révélatrice",
    "révélatrice": "révélateur",
    "révélé": "révélée",
    "révélée": "révélé",
    "révérend": "révérende",
    "révérende": "révérend",
    "révéré": "révérée",
    "révérée": "révéré",
    "réécrit": "réécrite",
    "réécrite": "réécrit",
    "rééquipé": "rééquipée",
    "rééquipée": "rééquipé",
    "réévalué": "réévaluée",
    "réévaluée": "réévalué",
    "rêvé": "rêvée",
    "rêvée": "rêvé",
    "rêveur": "rêveuse",
    "rêveuse": "rêveur",
    "rôdé": "rôdée",
    "rôdée": "rôdé",
    "rôdeur": "rôdeuse",
    "rôdeuse": "rôdeur",
    "rôti": "rôtie",
    "rôtie": "rôti",
    "rôtisseur": "rôtisseuse",
    "rôtisseuse": "rôtisseur",
    "sablé": "sablée",
    "sablée": "sablé",
    "sabordé": "sabordée",
    "sabordée": "sabordé",
    "saboté": "sabotée",
    "sabotée": "saboté",
    "saboteur": "saboteuse",
    "saboteuse": "saboteur",
    "sabré": "sabrée",
    "sabrée": "sabré",
    "saccadé": "saccadée",
    "saccadée": "saccadé",
    "saccagé": "saccagée",
    "saccagée": "saccagé",
    "sacerdotal": "sacerdotale",
    "sacerdotale": "sacerdotal",
    "sacramentel": "sacramentelle",
    "sacramentelle": "sacramentel",
    "sacré": "sacrée",
    "sacrée": "sacré",
    "sacrificiel": "sacrificielle",
    "sacrificielle": "sacrificiel",
    "sacrifié": "sacrifiée",
    "sacrifiée": "sacrifié",
    "sacristain": "sacristaine",
    "sacristaine": "sacristain",
    "sacro-saint": "sacro-sainte",
    "sacro-sainte": "sacro-saint",
    "sagittal": "sagittale",
    "sagittale": "sagittal",
    "saharien": "saharienne",
    "saharienne": "saharien",
    "saignant": "saignante",
    "saignante": "saignant",
    "saigné": "saignée",
    "saignée": "saigné",
    "saillant": "saillante",
    "saillante": "saillant",
    "sailli": "saillie",
    "saillie": "sailli",
    "saint": "sainte",
    "sainte": "saint",
    "saisi": "saisie",
    "saisie": "saisi",
    "saisissant": "saisissante",
    "saisissante": "saisissant",
    "saisonnier": "saisonnière",
    "saisonnière": "saisonnier",
    "salarié": "salariée",
    "salariée": "salarié",
    "salé": "salée",
    "salée": "salé",
    "salin": "saline",
    "saline": "salin",
    "sali": "salie",
    "salie": "sali",
    "salissant": "salissante",
    "salissante": "salissant",
    "salopé": "salopée",
    "salopée": "salopé",
    "salué": "saluée",
    "saluée": "salué",
    "salvadorien": "salvadorienne",
    "salvadorienne": "salvadorien",
    "salvateur": "salvatrice",
    "salvatrice": "salvateur",
    "sanctifié": "sanctifiée",
    "sanctifiée": "sanctifié",
    "sanctionné": "sanctionnée",
    "sanctionnée": "sanctionné",
    "sanglant": "sanglante",
    "sanglante": "sanglant",
    "sanglé": "sanglée",
    "sanglée": "sanglé",
    "sanglotant": "sanglotante",
    "sanglotante": "sanglotant",
    "sanguin": "sanguine",
    "sanguine": "sanguin",
    "sanguinolent": "sanguinolente",
    "sanguinolente": "sanguinolent",
    "sanscrit": "sanscrite",
    "sanscrite": "sanscrit",
    "saoudien": "saoudienne",
    "saoudienne": "saoudien",
    "saoul": "saoule",
    "saoule": "saoul",
    "saoulé": "saoulée",
    "saoulée": "saoulé",
    "sapé": "sapée",
    "sapée": "sapé",
    "saqué": "saquée",
    "saquée": "saqué",
    "sarrasin": "sarrasine",
    "sarrasine": "sarrasin",
    "satané": "satanée",
    "satanée": "satané",
    "satiné": "satinée",
    "satinée": "satiné",
    "satisfait": "satisfaite",
    "satisfaite": "satisfait",
    "satisfaisant": "satisfaisante",
    "satisfaisante": "satisfaisant",
    "saturé": "saturée",
    "saturée": "saturé",
    "saucissonné": "saucissonnée",
    "saucissonnée": "saucissonné",
    "sauf": "sauve",
    "sauve": "sauf",
    "saugrenu": "saugrenue",
    "saugrenue": "saugrenu",
    "saumoné": "saumonée",
    "saumonée": "saumoné",
    "saupoudré": "saupoudrée",
    "saupoudrée": "saupoudré",
    "sautant": "sautante",
    "sautante": "sautant",
    "sauté": "sautée",
    "sautée": "sauté",
    "sauteur": "sauteuse",
    "sauteuse": "sauteur",
    "sautillant": "sautillante",
    "sautillante": "sautillant",
    "sauvageon": "sauvageonne",
    "sauvageonne": "sauvageon",
    "sauvagin": "sauvagine",
    "sauvagine": "sauvagin",
    "sauvegardé": "sauvegardée",
    "sauvegardée": "sauvegardé",
    "sauvé": "sauvée",
    "sauvée": "sauvé",
    "savant": "savante",
    "savante": "savant",
    "su": "sue",
    "sue": "su",
    "savonné": "savonnée",
    "savonnée": "savonné",
    "savouré": "savourée",
    "savourée": "savouré",
    "savoyard": "savoyarde",
    "savoyarde": "savoyard",
    "saxon": "saxonne",
    "saxonne": "saxon",
    "scandalisé": "scandalisée",
    "scandalisée": "scandalisé",
    "scandé": "scandée",
    "scandée": "scandé",
    "scanné": "scannée",
    "scannée": "scanné",
    "scellé": "scellée",
    "scellée": "scellé",
    "scié": "sciée",
    "sciée": "scié",
    "scindé": "scindée",
    "scindée": "scindé",
    "scintillant": "scintillante",
    "scintillante": "scintillant",
    "sclérosé": "sclérosée",
    "sclérosée": "sclérosé",
    "scotché": "scotchée",
    "scotchée": "scotché",
    "scout": "scoute",
    "scoute": "scout",
    "scruté": "scrutée",
    "scrutée": "scruté",
    "sculpté": "sculptée",
    "sculptée": "sculpté",
    "sculpteur": "sculptrice",
    "sculptrice": "sculpteur",
    "sculptural": "sculpturale",
    "sculpturale": "sculptural",
    "scélérat": "scélérate",
    "scélérate": "scélérat",
    "sec": "sèche",
    "sèche": "sec",
    "second": "seconde",
    "seconde": "second",
    "secondé": "secondée",
    "secondée": "secondé",
    "secoué": "secouée",
    "secouée": "secoué",
    "secouru": "secourue",
    "secourue": "secouru",
    "secret": "secrète",
    "secrète": "secret",
    "secrété": "secrétée",
    "secrétée": "secrété",
    "sectionné": "sectionnée",
    "sectionnée": "sectionné",
    "seigneurial": "seigneuriale",
    "seigneuriale": "seigneurial",
    "seine": "senne",
    "senne": "seine",
    "sellé": "sellée",
    "sellée": "sellé",
    "semé": "semée",
    "semée": "semé",
    "semestriel": "semestrielle",
    "semestrielle": "semestriel",
    "semi-officiel": "semi-officielle",
    "semi-officielle": "semi-officiel",
    "sempiternel": "sempiternelle",
    "sempiternelle": "sempiternel",
    "sensationnel": "sensationnelle",
    "sensationnelle": "sensationnel",
    "sensibilisé": "sensibilisée",
    "sensibilisée": "sensibilisé",
    "sensitif": "sensitive",
    "sensitive": "sensitif",
    "sensoriel": "sensorielle",
    "sensorielle": "sensoriel",
    "sensuel": "sensuelle",
    "sensuelle": "sensuel",
    "sensé": "sensée",
    "sensée": "sensé",
    "senti": "sentie",
    "sentie": "senti",
    "sentimental": "sentimentale",
    "sentimentale": "sentimental",
    "septentrional": "septentrionale",
    "septentrionale": "septentrional",
    "serein": "sereine",
    "sereine": "serein",
    "serin": "serine",
    "serine": "serin",
    "sermonné": "sermonnée",
    "sermonnée": "sermonné",
    "serpentin": "serpentine",
    "serpentine": "serpentin",
    "serré": "serrée",
    "serrée": "serré",
    "serrurier": "serrurière",
    "serrurière": "serrurier",
    "serti": "sertie",
    "sertie": "serti",
    "servant": "servante",
    "servante": "servant",
    "serveur": "serveuse",
    "serveuse": "serveur",
    "servi": "servie",
    "servie": "servi",
    "seul": "seule",
    "seule": "seul",
    "seulet": "seulette",
    "seulette": "seulet",
    "sevré": "sevrée",
    "sevrée": "sevré",
    "sex appeal": "sex-appeal",
    "sex-appeal": "sex appeal",
    "sex shop": "sex-shop",
    "sex-shop": "sex shop",
    "sexuel": "sexuelle",
    "sexuelle": "sexuel",
    "seyant": "seyante",
    "seyante": "seyant",
    "shakespearien": "shakespearienne",
    "shakespearienne": "shakespearien",
    "shampouineur": "shampouineuse",
    "shampouineuse": "shampouineur",
    "shogunal": "shogunale",
    "shogunale": "shogunal",
    "shooté": "shootée",
    "shootée": "shooté",
    "sibyllin": "sibylline",
    "sibylline": "sibyllin",
    "sibérien": "sibérienne",
    "sibérienne": "sibérien",
    "sicilien": "sicilienne",
    "sicilienne": "sicilien",
    "sidéral": "sidérale",
    "sidérale": "sidéral",
    "sidérant": "sidérante",
    "sidérante": "sidérant",
    "sidéré": "sidérée",
    "sidérée": "sidéré",
    "siennois": "siennoise",
    "siennoise": "siennois",
    "sifflé": "sifflée",
    "sifflée": "sifflé",
    "sigillé": "sigillée",
    "sigillée": "sigillé",
    "signalé": "signalée",
    "signalée": "signalé",
    "signé": "signée",
    "signée": "signé",
    "signifiant": "signifiante",
    "signifiante": "signifiant",
    "significatif": "significative",
    "significative": "significatif",
    "signifié": "signifiée",
    "signifiée": "signifié",
    "signor": "signora",
    "signora": "signor",
    "silhouetté": "silhouettée",
    "silhouettée": "silhouetté",
    "siliconé": "siliconée",
    "siliconée": "siliconé",
    "silicosé": "silicosée",
    "silicosée": "silicosé",
    "sillonné": "sillonnée",
    "sillonnée": "sillonné",
    "simien": "simienne",
    "simienne": "simien",
    "simplet": "simplette",
    "simplette": "simplet",
    "simplifié": "simplifiée",
    "simplifiée": "simplifié",
    "simulateur": "simulatrice",
    "simulatrice": "simulateur",
    "simulé": "simulée",
    "simulée": "simulé",
    "simultané": "simultanée",
    "simultanée": "simultané",
    "singulier": "singulière",
    "singulière": "singulier",
    "sinistré": "sinistrée",
    "sinistrée": "sinistré",
    "sinusal": "sinusale",
    "sinusale": "sinusal",
    "sinusoïdal": "sinusoïdale",
    "sinusoïdale": "sinusoïdal",
    "siphonné": "siphonnée",
    "siphonnée": "siphonné",
    "sis": "sise",
    "sise": "sis",
    "situé": "située",
    "située": "situé",
    "skieur": "skieuse",
    "skieuse": "skieur",
    "sniffé": "sniffée",
    "sniffée": "sniffé",
    "snobé": "snobée",
    "snobée": "snobé",
    "snobinard": "snobinarde",
    "snobinarde": "snobinard",
    "soap opéra": "soap-opéra",
    "soap-opéra": "soap opéra",
    "social": "sociale",
    "sociale": "social",
    "sociétal": "sociétale",
    "sociétale": "sociétal",
    "sodomisé": "sodomisée",
    "sodomisée": "sodomisé",
    "soiffard": "soiffarde",
    "soiffarde": "soiffard",
    "soignant": "soignante",
    "soignante": "soignant",
    "soigné": "soignée",
    "soignée": "soigné",
    "soldat": "soldate",
    "soldate": "soldat",
    "soldé": "soldée",
    "soldée": "soldé",
    "solennel": "solennelle",
    "solennelle": "solennel",
    "solidifié": "solidifiée",
    "solidifiée": "solidifié",
    "sollicité": "sollicitée",
    "sollicitée": "sollicité",
    "solliciteur": "solliciteuse",
    "solliciteuse": "solliciteur",
    "solognot": "solognote",
    "solognote": "solognot",
    "somalien": "somalienne",
    "somalienne": "somalien",
    "sombré": "sombrée",
    "sombrée": "sombré",
    "sommeillant": "sommeillante",
    "sommeillante": "sommeillant",
    "sommé": "sommée",
    "sommée": "sommé",
    "somnolent": "somnolente",
    "somnolente": "somnolent",
    "sondé": "sondée",
    "sondée": "sondé",
    "sondeur": "sondeuse",
    "sondeuse": "sondeur",
    "songeur": "songeuse",
    "songeuse": "songeur",
    "sonnant": "sonnante",
    "sonnante": "sonnant",
    "sonné": "sonnée",
    "sonnée": "sonné",
    "sophistiqué": "sophistiquée",
    "sophistiquée": "sophistiqué",
    "sorcier": "sorcière",
    "sorcière": "sorcier",
    "sororal": "sororale",
    "sororale": "sororal",
    "sortant": "sortante",
    "sortante": "sortant",
    "sorti": "sortie",
    "sortie": "sorti",
    "sot": "sotte",
    "sotte": "sot",
    "soucié": "souciée",
    "souciée": "soucié",
    "soudain": "soudaine",
    "soudaine": "soudain",
    "soudé": "soudée",
    "soudée": "soudé",
    "soudeur": "soudeuse",
    "soudeuse": "soudeur",
    "soudoyé": "soudoyée",
    "soudoyée": "soudoyé",
    "soufflant": "soufflante",
    "soufflante": "soufflant",
    "soufflé": "soufflée",
    "soufflée": "soufflé",
    "souffleté": "souffletée",
    "souffletée": "souffleté",
    "souffleur": "souffleuse",
    "souffleuse": "souffleur",
    "souffrant": "souffrante",
    "souffrante": "souffrant",
    "souffert": "soufferte",
    "soufferte": "souffert",
    "soufré": "soufrée",
    "soufrée": "soufré",
    "souhaité": "souhaitée",
    "souhaitée": "souhaité",
    "souillé": "souillée",
    "souillée": "souillé",
    "soulagé": "soulagée",
    "soulagée": "soulagé",
    "soulevé": "soulevée",
    "soulevée": "soulevé",
    "souligné": "soulignée",
    "soulignée": "souligné",
    "soupesé": "soupesée",
    "soupesée": "soupesé",
    "soupçonné": "soupçonnée",
    "soupçonnée": "soupçonné",
    "sourd": "sourde",
    "sourde": "sourd",
    "souriant": "souriante",
    "souriante": "souriant",
    "sous-alimenté": "sous-alimentée",
    "sous-alimentée": "sous-alimenté",
    "sous-cutané": "sous-cutanée",
    "sous-cutanée": "sous-cutané",
    "sous-directeur": "sous-directrice",
    "sous-directrice": "sous-directeur",
    "sous-développé": "sous-développée",
    "sous-développée": "sous-développé",
    "sous-entendu": "sous-entendue",
    "sous-entendue": "sous-entendu",
    "sous-estimé": "sous-estimée",
    "sous-estimée": "sous-estimé",
    "sous-jacent": "sous-jacente",
    "sous-jacente": "sous-jacent",
    "sous-marin": "sous-marine",
    "sous-marine": "sous-marin",
    "sous-qualifié": "sous-qualifiée",
    "sous-qualifiée": "sous-qualifié",
    "sous-tendu": "sous-tendue",
    "sous-tendue": "sous-tendu",
    "sous-titré": "sous-titrée",
    "sous-titrée": "sous-titré",
    "souscrit": "souscrite",
    "souscrite": "souscrit",
    "soussigné": "soussignée",
    "soussignée": "soussigné",
    "soustrait": "soustraite",
    "soustraite": "soustrait",
    "soutaché": "soutachée",
    "soutachée": "soutaché",
    "soutenu": "soutenue",
    "soutenue": "soutenu",
    "souterrain": "souterraine",
    "souterraine": "souterrain",
    "soutiré": "soutirée",
    "soutirée": "soutiré",
    "souvenu": "souvenue",
    "souvenue": "souvenu",
    "souverain": "souveraine",
    "souveraine": "souverain",
    "soûl": "soûle",
    "soûle": "soûl",
    "soûlant": "soûlante",
    "soûlante": "soûlant",
    "soûlard": "soûlarde",
    "soûlarde": "soûlard",
    "soûlé": "soûlée",
    "soûlée": "soûlé",
    "spatial": "spatiale",
    "spatiale": "spatial",
    "spatio-temporel": "spatio-temporelle",
    "spatio-temporelle": "spatio-temporel",
    "speaker": "speakerine",
    "speakerine": "speaker",
    "spectateur": "spectatrice",
    "spectatrice": "spectateur",
    "spectral": "spectrale",
    "spectrale": "spectral",
    "speedé": "speedée",
    "speedée": "speedé",
    "spinal": "spinale",
    "spinale": "spinal",
    "spirituel": "spirituelle",
    "spirituelle": "spirituel",
    "spolié": "spoliée",
    "spoliée": "spolié",
    "sponsorisé": "sponsorisée",
    "sponsorisée": "sponsorisé",
    "spontané": "spontanée",
    "spontanée": "spontané",
    "sportif": "sportive",
    "sportive": "sportif",
    "spécial": "spéciale",
    "spéciale": "spécial",
    "spécialisé": "spécialisée",
    "spécialisée": "spécialisé",
    "spécifié": "spécifiée",
    "spécifiée": "spécifié",
    "spéculateur": "spéculatrice",
    "spéculatrice": "spéculateur",
    "spéculatif": "spéculative",
    "spéculative": "spéculatif",
    "squameuse": "squameux",
    "squameux": "squameuse",
    "squatté": "squattée",
    "squattée": "squatté",
    "stabilisateur": "stabilisatrice",
    "stabilisatrice": "stabilisateur",
    "stabilisé": "stabilisée",
    "stabilisée": "stabilisé",
    "stagnant": "stagnante",
    "stagnante": "stagnant",
    "stalinien": "stalinienne",
    "stalinienne": "stalinien",
    "standardisé": "standardisée",
    "standardisée": "standardisé",
    "stationné": "stationnée",
    "stationnée": "stationné",
    "statisticien": "statisticienne",
    "statisticienne": "statisticien",
    "statufié": "statufiée",
    "statufiée": "statufié",
    "sternal": "sternale",
    "sternale": "sternal",
    "stigmatisé": "stigmatisée",
    "stigmatisée": "stigmatisé",
    "stimulant": "stimulante",
    "stimulante": "stimulant",
    "stimulé": "stimulée",
    "stimulée": "stimulé",
    "stipulé": "stipulée",
    "stipulée": "stipulé",
    "stocké": "stockée",
    "stockée": "stocké",
    "stomacal": "stomacale",
    "stomacale": "stomacal",
    "stoppé": "stoppée",
    "stoppée": "stoppé",
    "stoppeur": "stoppeuse",
    "stoppeuse": "stoppeur",
    "story board": "story-board",
    "story-board": "story board",
    "strasbourgeois": "strasbourgeoise",
    "strasbourgeoise": "strasbourgeois",
    "stratifié": "stratifiée",
    "stratifiée": "stratifié",
    "stressant": "stressante",
    "stressante": "stressant",
    "stressé": "stressée",
    "stressée": "stressé",
    "strict": "stricte",
    "stricte": "strict",
    "strident": "stridente",
    "stridente": "strident",
    "strié": "striée",
    "striée": "strié",
    "strip-teaseur": "strip-teaseuse",
    "strip-teaseuse": "strip-teaseur",
    "structural": "structurale",
    "structurale": "structural",
    "structurel": "structurelle",
    "structurelle": "structurel",
    "structuré": "structurée",
    "structurée": "structuré",
    "stupéfait": "stupéfaite",
    "stupéfaite": "stupéfait",
    "stupéfiant": "stupéfiante",
    "stupéfiante": "stupéfiant",
    "stupéfié": "stupéfiée",
    "stupéfiée": "stupéfié",
    "stylé": "stylée",
    "stylée": "stylé",
    "stylisé": "stylisée",
    "stylisée": "stylisé",
    "stérilisant": "stérilisante",
    "stérilisante": "stérilisant",
    "stérilisé": "stérilisée",
    "stérilisée": "stérilisé",
    "stéréotypé": "stéréotypée",
    "stéréotypée": "stéréotypé",
    "suant": "suante",
    "suante": "suant",
    "subconscient": "subconsciente",
    "subconsciente": "subconscient",
    "subdivisé": "subdivisée",
    "subdivisée": "subdivisé",
    "subi": "subie",
    "subie": "subi",
    "subit": "subite",
    "subite": "subit",
    "subjectif": "subjective",
    "subjective": "subjectif",
    "subjugué": "subjuguée",
    "subjuguée": "subjugué",
    "sublimé": "sublimée",
    "sublimée": "sublimé",
    "subliminal": "subliminale",
    "subliminale": "subliminal",
    "submergé": "submergée",
    "submergée": "submergé",
    "subordonné": "subordonnée",
    "subordonnée": "subordonné",
    "suborné": "subornée",
    "subornée": "suborné",
    "substantiel": "substantielle",
    "substantielle": "substantiel",
    "subséquent": "subséquente",
    "subséquente": "subséquent",
    "subtil": "subtile",
    "subtile": "subtil",
    "subtilisé": "subtilisée",
    "subtilisée": "subtilisé",
    "subtropical": "subtropicale",
    "subtropicale": "subtropical",
    "suburbain": "suburbaine",
    "suburbaine": "suburbain",
    "subventionné": "subventionnée",
    "subventionnée": "subventionné",
    "subversif": "subversive",
    "subversive": "subversif",
    "successif": "successive",
    "successive": "successif",
    "successoral": "successorale",
    "successorale": "successoral",
    "succinct": "succincte",
    "succincte": "succinct",
    "succulent": "succulente",
    "succulente": "succulent",
    "sucé": "sucée",
    "sucée": "sucé",
    "suceur": "suceuse",
    "suceuse": "suceur",
    "sucré": "sucrée",
    "sucrée": "sucré",
    "sucrier": "sucrière",
    "sucrière": "sucrier",
    "sud-africain": "sud-africaine",
    "sud-africaine": "sud-africain",
    "sud-américain": "sud-américaine",
    "sud-américaine": "sud-américain",
    "sud-coréen": "sud-coréenne",
    "sud-coréenne": "sud-coréen",
    "sud-vietnamien": "sud-vietnamienne",
    "sud-vietnamienne": "sud-vietnamien",
    "sué": "suée",
    "suée": "sué",
    "suffisant": "suffisante",
    "suffisante": "suffisant",
    "suffocant": "suffocante",
    "suffocante": "suffocant",
    "suffoqué": "suffoquée",
    "suffoquée": "suffoqué",
    "suggestif": "suggestive",
    "suggestive": "suggestif",
    "suggéré": "suggérée",
    "suggérée": "suggéré",
    "suicidé": "suicidée",
    "suicidée": "suicidé",
    "suiffé": "suiffée",
    "suiffée": "suiffé",
    "suintant": "suintante",
    "suintante": "suintant",
    "suivant": "suivante",
    "suivante": "suivant",
    "suiveur": "suiveuse",
    "suiveuse": "suiveur",
    "suivi": "suivie",
    "suivie": "suivi",
    "sujet": "sujette",
    "sujette": "sujet",
    "sulfuré": "sulfurée",
    "sulfurée": "sulfuré",
    "sulpicien": "sulpicienne",
    "sulpicienne": "sulpicien",
    "superficiel": "superficielle",
    "superficielle": "superficiel",
    "superflu": "superflue",
    "superflue": "superflu",
    "superlatif": "superlative",
    "superlative": "superlatif",
    "superposé": "superposée",
    "superposée": "superposé",
    "superpuissant": "superpuissante",
    "superpuissante": "superpuissant",
    "supervisé": "supervisée",
    "supervisée": "supervisé",
    "supplanté": "supplantée",
    "supplantée": "supplanté",
    "suppliant": "suppliante",
    "suppliante": "suppliant",
    "supplicié": "suppliciée",
    "suppliciée": "supplicié",
    "supplié": "suppliée",
    "suppliée": "supplié",
    "suppléant": "suppléante",
    "suppléante": "suppléant",
    "supporté": "supportée",
    "supportée": "supporté",
    "supporteur": "supportrice",
    "supportrice": "supporteur",
    "supposé": "supposée",
    "supposée": "supposé",
    "supprimé": "supprimée",
    "supprimée": "supprimé",
    "suppurant": "suppurante",
    "suppurante": "suppurant",
    "supérieur": "supérieure",
    "supérieure": "supérieur",
    "sur": "sure",
    "sure": "sur",
    "surabondant": "surabondante",
    "surabondante": "surabondant",
    "suractivé": "suractivée",
    "suractivée": "suractivé",
    "suraigu": "suraiguë",
    "suraiguë": "suraigu",
    "surajouté": "surajoutée",
    "surajoutée": "surajouté",
    "suranné": "surannée",
    "surannée": "suranné",
    "surarmé": "surarmée",
    "surarmée": "surarmé",
    "surbaissé": "surbaissée",
    "surbaissée": "surbaissé",
    "surbooké": "surbookée",
    "surbookée": "surbooké",
    "surchargé": "surchargée",
    "surchargée": "surchargé",
    "surchauffé": "surchauffée",
    "surchauffée": "surchauffé",
    "surdimensionné": "surdimensionnée",
    "surdimensionnée": "surdimensionné",
    "surdoré": "surdorée",
    "surdorée": "surdoré",
    "surdoué": "surdouée",
    "surdouée": "surdoué",
    "surdéveloppé": "surdéveloppée",
    "surdéveloppée": "surdéveloppé",
    "surestimé": "surestimée",
    "surestimée": "surestimé",
    "suret": "surette",
    "surette": "suret",
    "surexcité": "surexcitée",
    "surexcitée": "surexcité",
    "surexposé": "surexposée",
    "surexposée": "surexposé",
    "surfait": "surfaite",
    "surfaite": "surfait",
    "surfeur": "surfeuse",
    "surfeuse": "surfeur",
    "surfin": "surfine",
    "surfine": "surfin",
    "surgelé": "surgelée",
    "surgelée": "surgelé",
    "surgi": "surgie",
    "surgie": "surgi",
    "surhumain": "surhumaine",
    "surhumaine": "surhumain",
    "suri": "surie",
    "surie": "suri",
    "surintendant": "surintendante",
    "surintendante": "surintendant",
    "surmené": "surmenée",
    "surmenée": "surmené",
    "surmonté": "surmontée",
    "surmontée": "surmonté",
    "surnaturel": "surnaturelle",
    "surnaturelle": "surnaturel",
    "surnommé": "surnommée",
    "surnommée": "surnommé",
    "surpassé": "surpassée",
    "surpassée": "surpassé",
    "surpeuplé": "surpeuplée",
    "surpeuplée": "surpeuplé",
    "surplombant": "surplombante",
    "surplombante": "surplombant",
    "surplombé": "surplombée",
    "surplombée": "surplombé",
    "surprenant": "surprenante",
    "surprenante": "surprenant",
    "surpuissant": "surpuissante",
    "surpuissante": "surpuissant",
    "surqualifié": "surqualifiée",
    "surqualifiée": "surqualifié",
    "surréel": "surréelle",
    "surréelle": "surréel",
    "surrénal": "surrénale",
    "surrénale": "surrénal",
    "surveillant": "surveillante",
    "surveillante": "surveillant",
    "surveillé": "surveillée",
    "surveillée": "surveillé",
    "survenu": "survenue",
    "survenue": "survenu",
    "survivant": "survivante",
    "survivante": "survivant",
    "survolé": "survolée",
    "survolée": "survolé",
    "survolté": "survoltée",
    "survoltée": "survolté",
    "surélevé": "surélevée",
    "surélevée": "surélevé",
    "suscité": "suscitée",
    "suscitée": "suscité",
    "susdit": "susdite",
    "susdite": "susdit",
    "susmentionné": "susmentionnée",
    "susmentionnée": "susmentionné",
    "susnommé": "susnommée",
    "susnommée": "susnommé",
    "suspect": "suspecte",
    "suspecte": "suspect",
    "suspecté": "suspectée",
    "suspectée": "suspecté",
    "suspendu": "suspendue",
    "suspendue": "suspendu",
    "suspensif": "suspensive",
    "suspensive": "suspensif",
    "susurré": "susurrée",
    "susurrée": "susurré",
    "suturé": "suturée",
    "suturée": "suturé",
    "suzerain": "suzeraine",
    "suzeraine": "suzerain",
    "symbolisé": "symbolisée",
    "symbolisée": "symbolisé",
    "sympathisant": "sympathisante",
    "sympathisante": "sympathisant",
    "synchronisé": "synchronisée",
    "synchronisée": "synchronisé",
    "syncopé": "syncopée",
    "syncopée": "syncopé",
    "syndical": "syndicale",
    "syndicale": "syndical",
    "syndiqué": "syndiquée",
    "syndiquée": "syndiqué",
    "synthétisé": "synthétisée",
    "synthétisée": "synthétisé",
    "syrien": "syrienne",
    "syrienne": "syrien",
    "systématisé": "systématisée",
    "systématisée": "systématisé",
    "séant": "séante",
    "séante": "séant",
    "sébacé": "sébacée",
    "sébacée": "sébacé",
    "séché": "séchée",
    "séchée": "séché",
    "sécrété": "sécrétée",
    "sécrétée": "sécrété",
    "séculier": "séculière",
    "séculière": "séculier",
    "sécurisant": "sécurisante",
    "sécurisante": "sécurisant",
    "sécurisé": "sécurisée",
    "sécurisée": "sécurisé",
    "sédatif": "sédative",
    "sédative": "sédatif",
    "séducteur": "séductrice",
    "séductrice": "séducteur",
    "séduit": "séduite",
    "séduite": "séduit",
    "séduisant": "séduisante",
    "séduisante": "séduisant",
    "sélect": "sélecte",
    "sélecte": "sélect",
    "sélectif": "sélective",
    "sélective": "sélectif",
    "sélectionné": "sélectionnée",
    "sélectionnée": "sélectionné",
    "sémillant": "sémillante",
    "sémillante": "sémillant",
    "séminal": "séminale",
    "séminale": "séminal",
    "sénateur": "sénatrice",
    "sénatrice": "sénateur",
    "sénatorial": "sénatoriale",
    "sénatoriale": "sénatorial",
    "séparateur": "séparatrice",
    "séparatrice": "séparateur",
    "séparé": "séparée",
    "séparée": "séparé",
    "sépulcral": "sépulcrale",
    "sépulcrale": "sépulcral",
    "séquentiel": "séquentielle",
    "séquentielle": "séquentiel",
    "séquestré": "séquestrée",
    "séquestrée": "séquestré",
    "sériel": "sérielle",
    "sérielle": "sériel",
    "séronégatif": "séronégative",
    "séronégative": "séronégatif",
    "séropositif": "séropositive",
    "séropositive": "séropositif",
    "sévillan": "sévillane",
    "sévillane": "sévillan",
    "sûr": "sûre",
    "sûre": "sûr",
    "tabassé": "tabassée",
    "tabassée": "tabassé",
    "tabou": "taboue",
    "taboue": "tabou",
    "taché": "tachée",
    "tachée": "taché",
    "tacheté": "tachetée",
    "tachetée": "tacheté",
    "tacticien": "tacticienne",
    "tacticienne": "tacticien",
    "tahitien": "tahitienne",
    "tahitienne": "tahitien",
    "tailladé": "tailladée",
    "tailladée": "tailladé",
    "taillé": "taillée",
    "taillée": "taillé",
    "talonné": "talonnée",
    "talonnée": "talonné",
    "tambouriné": "tambourinée",
    "tambourinée": "tambouriné",
    "tamisé": "tamisée",
    "tamisée": "tamisé",
    "tamponné": "tamponnée",
    "tamponnée": "tamponné",
    "tamponneur": "tamponneuse",
    "tamponneuse": "tamponneur",
    "tancé": "tancée",
    "tancée": "tancé",
    "tangent": "tangente",
    "tangente": "tangent",
    "tannant": "tannante",
    "tannante": "tannant",
    "tanné": "tannée",
    "tannée": "tanné",
    "tapageur": "tapageuse",
    "tapageuse": "tapageur",
    "tapant": "tapante",
    "tapante": "tapant",
    "tapé": "tapée",
    "tapée": "tapé",
    "tapi": "tapie",
    "tapie": "tapi",
    "tapiné": "tapinée",
    "tapinée": "tapiné",
    "tapissé": "tapissée",
    "tapissée": "tapissé",
    "tapissier": "tapissière",
    "tapissière": "tapissier",
    "taquin": "taquine",
    "taquine": "taquin",
    "taquiné": "taquinée",
    "taquinée": "taquiné",
    "tarabiscoté": "tarabiscotée",
    "tarabiscotée": "tarabiscoté",
    "taraudé": "taraudée",
    "taraudée": "taraudé",
    "tardé": "tardée",
    "tardée": "tardé",
    "tardif": "tardive",
    "tardive": "tardif",
    "taré": "tarée",
    "tarée": "taré",
    "tari": "tarie",
    "tarie": "tari",
    "tartan": "tartane",
    "tartane": "tartan",
    "tartiné": "tartinée",
    "tartinée": "tartiné",
    "tassé": "tassée",
    "tassée": "tassé",
    "tatillon": "tatillonne",
    "tatillonne": "tatillon",
    "tatoué": "tatouée",
    "tatouée": "tatoué",
    "tatoueur": "tatoueuse",
    "tatoueuse": "tatoueur",
    "taulard": "taularde",
    "taularde": "taulard",
    "taulier": "taulière",
    "taulière": "taulier",
    "taurin": "taurine",
    "taurine": "taurin",
    "tavelé": "tavelée",
    "tavelée": "tavelé",
    "taxé": "taxée",
    "taxée": "taxé",
    "technicien": "technicienne",
    "technicienne": "technicien",
    "teint": "teinte",
    "teinte": "teint",
    "teinté": "teintée",
    "teintée": "teinté",
    "teinturier": "teinturière",
    "teinturière": "teinturier",
    "temporal": "temporale",
    "temporale": "temporal",
    "temporel": "temporelle",
    "temporelle": "temporel",
    "temporisé": "temporisée",
    "temporisée": "temporisé",
    "tempéré": "tempérée",
    "tempérée": "tempéré",
    "tenaillé": "tenaillée",
    "tenaillée": "tenaillé",
    "tenancier": "tenancière",
    "tenancière": "tenancier",
    "tenant": "tenante",
    "tenante": "tenant",
    "tendu": "tendue",
    "tendue": "tendu",
    "tenu": "tenue",
    "tenue": "tenu",
    "tentant": "tentante",
    "tentante": "tentant",
    "tentateur": "tentatrice",
    "tentatrice": "tentateur",
    "tenté": "tentée",
    "tentée": "tenté",
    "terminal": "terminale",
    "terminale": "terminal",
    "terminé": "terminée",
    "terminée": "terminé",
    "terni": "ternie",
    "ternie": "terni",
    "terrassé": "terrassée",
    "terrassée": "terrassé",
    "terré": "terrée",
    "terrée": "terré",
    "terrien": "terrienne",
    "terrienne": "terrien",
    "terrifiant": "terrifiante",
    "terrifiante": "terrifiant",
    "terrifié": "terrifiée",
    "terrifiée": "terrifié",
    "territorial": "territoriale",
    "territoriale": "territorial",
    "terrorisant": "terrorisante",
    "terrorisante": "terrorisant",
    "terrorisé": "terrorisée",
    "terrorisée": "terrorisé",
    "testé": "testée",
    "testée": "testé",
    "testeur": "testeuse",
    "testeuse": "testeur",
    "teuton": "teutonne",
    "teutonne": "teuton",
    "texan": "texane",
    "texane": "texan",
    "textuel": "textuelle",
    "textuelle": "textuel",
    "thaï": "thaïe",
    "thaïe": "thaï",
    "thermal": "thermale",
    "thermale": "thermal",
    "thyroïdien": "thyroïdienne",
    "thyroïdienne": "thyroïdien",
    "théoricien": "théoricienne",
    "théoricienne": "théoricien",
    "théâtral": "théâtrale",
    "théâtrale": "théâtral",
    "théâtralisé": "théâtralisée",
    "théâtralisée": "théâtralisé",
    "tibial": "tibiale",
    "tibiale": "tibial",
    "tibétain": "tibétaine",
    "tibétaine": "tibétain",
    "tigre": "tigresse",
    "tigresse": "tigre",
    "tigré": "tigrée",
    "tigrée": "tigré",
    "timbré": "timbrée",
    "timbrée": "timbré",
    "timoré": "timorée",
    "timorée": "timoré",
    "tintinnabulant": "tintinnabulante",
    "tintinnabulante": "tintinnabulant",
    "tiraillé": "tiraillée",
    "tiraillée": "tiraillé",
    "tirebouchonné": "tirebouchonnée",
    "tirebouchonnée": "tirebouchonné",
    "tiré": "tirée",
    "tirée": "tiré",
    "tireur": "tireuse",
    "tireuse": "tireur",
    "tissé": "tissée",
    "tissée": "tissé",
    "tisserand": "tisserande",
    "tisserande": "tisserand",
    "tisseur": "tisseuse",
    "tisseuse": "tisseur",
    "tissu": "tissue",
    "tissue": "tissu",
    "titré": "titrée",
    "titrée": "titré",
    "titubant": "titubante",
    "titubante": "titubant",
    "titularisé": "titularisée",
    "titularisée": "titularisé",
    "tiédi": "tiédie",
    "tiédie": "tiédi",
    "tocard": "tocarde",
    "tocarde": "tocard",
    "toilé": "toilée",
    "toilée": "toilé",
    "toisé": "toisée",
    "toisée": "toisé",
    "tolérant": "tolérante",
    "tolérante": "tolérant",
    "toléré": "tolérée",
    "tolérée": "toléré",
    "tombal": "tombale",
    "tombale": "tombal",
    "tombant": "tombante",
    "tombante": "tombant",
    "tombé": "tombée",
    "tombée": "tombé",
    "tombeur": "tombeuse",
    "tombeuse": "tombeur",
    "tonal": "tonale",
    "tonale": "tonal",
    "tondeur": "tondeuse",
    "tondeuse": "tondeur",
    "tondu": "tondue",
    "tondue": "tondu",
    "tonifiant": "tonifiante",
    "tonifiante": "tonifiant",
    "tonitruant": "tonitruante",
    "tonitruante": "tonitruant",
    "tonnant": "tonnante",
    "tonnante": "tonnant",
    "toquard": "toquarde",
    "toquarde": "toquard",
    "toqué": "toquée",
    "toquée": "toqué",
    "torché": "torchée",
    "torchée": "torché",
    "tordant": "tordante",
    "tordante": "tordant",
    "tordu": "tordue",
    "tordue": "tordu",
    "torpillé": "torpillée",
    "torpillée": "torpillé",
    "torrentiel": "torrentielle",
    "torrentielle": "torrentiel",
    "torréfié": "torréfiée",
    "torréfiée": "torréfié",
    "tors": "torse",
    "torse": "tors",
    "torsadé": "torsadée",
    "torsadée": "torsadé",
    "tortillé": "tortillée",
    "tortillée": "tortillé",
    "tortu": "tortue",
    "tortue": "tortu",
    "torturant": "torturante",
    "torturante": "torturant",
    "torturé": "torturée",
    "torturée": "torturé",
    "toscan": "toscane",
    "toscane": "toscan",
    "total": "totale",
    "totale": "total",
    "touchant": "touchante",
    "touchante": "touchant",
    "touché": "touchée",
    "touchée": "touché",
    "touffu": "touffue",
    "touffue": "touffu",
    "touillé": "touillée",
    "touillée": "touillé",
    "toulousain": "toulousaine",
    "toulousaine": "toulousain",
    "tourangeau": "tourangelle",
    "tourangelle": "tourangeau",
    "tourbillonnant": "tourbillonnante",
    "tourbillonnante": "tourbillonnant",
    "tourmentant": "tourmentante",
    "tourmentante": "tourmentant",
    "tourmenté": "tourmentée",
    "tourmentée": "tourmenté",
    "tournant": "tournante",
    "tournante": "tournant",
    "tourneboulé": "tourneboulée",
    "tourneboulée": "tourneboulé",
    "tourné": "tournée",
    "tournée": "tourné",
    "tournoyant": "tournoyante",
    "tournoyante": "tournoyant",
    "tourtereau": "tourterelle",
    "tourterelle": "tourtereau",
    "tout": "toute",
    "toute": "tout",
    "tout-puissant": "toute-puissante",
    "toute-puissante": "tout-puissant",
    "tracassé": "tracassée",
    "tracassée": "tracassé",
    "tracé": "tracée",
    "tracée": "tracé",
    "traceur": "traceuse",
    "traceuse": "traceur",
    "trachéal": "trachéale",
    "trachéale": "trachéal",
    "tracté": "tractée",
    "tractée": "tracté",
    "traditionnel": "traditionnelle",
    "traditionnelle": "traditionnel",
    "traducteur": "traductrice",
    "traductrice": "traducteur",
    "traduit": "traduite",
    "traduite": "traduit",
    "trafiquant": "trafiquante",
    "trafiquante": "trafiquant",
    "trafiqué": "trafiquée",
    "trafiquée": "trafiqué",
    "tragédien": "tragédienne",
    "tragédienne": "tragédien",
    "trahi": "trahie",
    "trahie": "trahi",
    "trait": "traite",
    "traite": "trait",
    "traitant": "traitante",
    "traitante": "traitant",
    "traité": "traitée",
    "traitée": "traité",
    "tranchant": "tranchante",
    "tranchante": "tranchant",
    "tranché": "tranchée",
    "tranchée": "tranché",
    "trancheur": "trancheuse",
    "trancheuse": "trancheur",
    "tranquillisant": "tranquillisante",
    "tranquillisante": "tranquillisant",
    "tranquillisé": "tranquillisée",
    "tranquillisée": "tranquillisé",
    "transbahuté": "transbahutée",
    "transbahutée": "transbahuté",
    "transcendant": "transcendante",
    "transcendante": "transcendant",
    "transcendantal": "transcendantale",
    "transcendantale": "transcendantal",
    "transcendé": "transcendée",
    "transcendée": "transcendé",
    "transcontinental": "transcontinentale",
    "transcontinentale": "transcontinental",
    "transcrit": "transcrite",
    "transcrite": "transcrit",
    "transfiguré": "transfigurée",
    "transfigurée": "transfiguré",
    "transformateur": "transformatrice",
    "transformatrice": "transformateur",
    "transformé": "transformée",
    "transformée": "transformé",
    "transféré": "transférée",
    "transférée": "transféré",
    "transgressé": "transgressée",
    "transgressée": "transgressé",
    "transi": "transie",
    "transie": "transi",
    "transitionnel": "transitionnelle",
    "transitionnelle": "transitionnel",
    "transmis": "transmise",
    "transmise": "transmis",
    "transmué": "transmuée",
    "transmuée": "transmué",
    "transparent": "transparente",
    "transparente": "transparent",
    "transpercé": "transpercée",
    "transpercée": "transpercé",
    "transpirant": "transpirante",
    "transpirante": "transpirant",
    "transplanté": "transplantée",
    "transplantée": "transplanté",
    "transporté": "transportée",
    "transportée": "transporté",
    "transporteur": "transporteuse",
    "transporteuse": "transporteur",
    "transposé": "transposée",
    "transposée": "transposé",
    "transsexuel": "transsexuelle",
    "transsexuelle": "transsexuel",
    "transversal": "transversale",
    "transversale": "transversal",
    "trapu": "trapue",
    "trapue": "trapu",
    "trapézoïdal": "trapézoïdale",
    "trapézoïdale": "trapézoïdal",
    "traqué": "traquée",
    "traquée": "traqué",
    "traqueur": "traqueuse",
    "traqueuse": "traqueur",
    "traumatisant": "traumatisante",
    "traumatisante": "traumatisant",
    "traumatisé": "traumatisée",
    "traumatisée": "traumatisé",
    "travaillé": "travaillée",
    "travaillée": "travaillé",
    "travailleur": "travailleuse",
    "travailleuse": "travailleur",
    "traversé": "traversée",
    "traversée": "traversé",
    "traversier": "traversière",
    "traversière": "traversier",
    "travesti": "travestie",
    "travestie": "travesti",
    "traînant": "traînante",
    "traînante": "traînant",
    "traînard": "traînarde",
    "traînarde": "traînard",
    "traîné": "traînée",
    "traînée": "traîné",
    "traître": "traîtresse",
    "traîtresse": "traître",
    "treillissé": "treillissée",
    "treillissée": "treillissé",
    "tremblant": "tremblante",
    "tremblante": "tremblant",
    "tremblé": "tremblée",
    "tremblée": "tremblé",
    "tremblotant": "tremblotante",
    "tremblotante": "tremblotant",
    "trempé": "trempée",
    "trempée": "trempé",
    "tressautant": "tressautante",
    "tressautante": "tressautant",
    "tressé": "tressée",
    "tressée": "tressé",
    "tribal": "tribale",
    "tribale": "tribal",
    "tricard": "tricarde",
    "tricarde": "tricard",
    "triché": "trichée",
    "trichée": "triché",
    "tricheur": "tricheuse",
    "tricheuse": "tricheur",
    "tricoté": "tricotée",
    "tricotée": "tricoté",
    "tricoteur": "tricoteuse",
    "tricoteuse": "tricoteur",
    "tridimensionnel": "tridimensionnelle",
    "tridimensionnelle": "tridimensionnel",
    "trié": "triée",
    "triée": "trié",
    "trieur": "trieuse",
    "trieuse": "trieur",
    "trimbalé": "trimbalée",
    "trimbalée": "trimbalé",
    "trimballé": "trimballée",
    "trimballée": "trimballé",
    "trimestriel": "trimestrielle",
    "trimestrielle": "trimestriel",
    "tringlé": "tringlée",
    "tringlée": "tringlé",
    "triomphal": "triomphale",
    "triomphale": "triomphal",
    "triomphant": "triomphante",
    "triomphante": "triomphant",
    "triomphateur": "triomphatrice",
    "triomphatrice": "triomphateur",
    "tripatouillé": "tripatouillée",
    "tripatouillée": "tripatouillé",
    "tripier": "tripière",
    "tripière": "tripier",
    "triplé": "triplée",
    "triplée": "triplé",
    "tripoté": "tripotée",
    "tripotée": "tripoté",
    "tripoteur": "tripoteuse",
    "tripoteuse": "tripoteur",
    "trisaïeul": "trisaïeule",
    "trisaïeule": "trisaïeul",
    "tristouillard": "tristouillarde",
    "tristouillarde": "tristouillard",
    "tristounet": "tristounette",
    "tristounette": "tristounet",
    "trituré": "triturée",
    "triturée": "trituré",
    "trivial": "triviale",
    "triviale": "trivial",
    "trompé": "trompée",
    "trompée": "trompé",
    "trompeur": "trompeuse",
    "trompeuse": "trompeur",
    "tronché": "tronchée",
    "tronchée": "tronché",
    "tronqué": "tronquée",
    "tronquée": "tronqué",
    "tronçonné": "tronçonnée",
    "tronçonnée": "tronçonné",
    "tropical": "tropicale",
    "tropicale": "tropical",
    "troqué": "troquée",
    "troquée": "troqué",
    "trotteur": "trotteuse",
    "trotteuse": "trotteur",
    "troublant": "troublante",
    "troublante": "troublant",
    "troublé": "troublée",
    "troublée": "troublé",
    "troué": "trouée",
    "trouée": "troué",
    "trouillard": "trouillarde",
    "trouillarde": "trouillard",
    "troussé": "troussée",
    "troussée": "troussé",
    "trouvé": "trouvée",
    "trouvée": "trouvé",
    "troyen": "troyenne",
    "troyenne": "troyen",
    "truand": "truande",
    "truande": "truand",
    "trucidé": "trucidée",
    "trucidée": "trucidé",
    "truculent": "truculente",
    "truculente": "truculent",
    "truffé": "truffée",
    "truffée": "truffé",
    "truqué": "truquée",
    "truquée": "truqué",
    "truqueur": "truqueuse",
    "truqueuse": "truqueur",
    "trébuchant": "trébuchante",
    "trébuchante": "trébuchant",
    "trémulant": "trémulante",
    "trémulante": "trémulant",
    "trépané": "trépanée",
    "trépanée": "trépané",
    "trépassé": "trépassée",
    "trépassée": "trépassé",
    "trépidant": "trépidante",
    "trépidante": "trépidant",
    "trépignant": "trépignante",
    "trépignante": "trépignant",
    "trépigné": "trépignée",
    "trépignée": "trépigné",
    "trésorier": "trésorière",
    "trésorière": "trésorier",
    "tsar": "tsarine",
    "tsarine": "tsar",
    "tu": "tue",
    "tue": "tu",
    "tuant": "tuante",
    "tuante": "tuant",
    "tubard": "tubarde",
    "tubarde": "tubard",
    "tué": "tuée",
    "tuée": "tué",
    "tuméfié": "tuméfiée",
    "tuméfiée": "tuméfié",
    "tunisien": "tunisienne",
    "tunisienne": "tunisien",
    "turbulent": "turbulente",
    "turbulente": "turbulent",
    "turc": "turque",
    "turque": "turc",
    "turgescent": "turgescente",
    "turgescente": "turgescent",
    "turlupiné": "turlupinée",
    "turlupinée": "turlupiné",
    "tuteur": "tutrice",
    "tutrice": "tuteur",
    "tutoyé": "tutoyée",
    "tutoyée": "tutoyé",
    "tuyauté": "tuyautée",
    "tuyautée": "tuyauté",
    "typé": "typée",
    "typée": "typé",
    "tyrannisé": "tyrannisée",
    "tyrannisée": "tyrannisé",
    "tyrolien": "tyrolienne",
    "tyrolienne": "tyrolien",
    "tzar": "tzarine",
    "tzarine": "tzar",
    "tâché": "tâchée",
    "tâchée": "tâché",
    "tâtonnant": "tâtonnante",
    "tâtonnante": "tâtonnant",
    "tâtonné": "tâtonnée",
    "tâtonnée": "tâtonné",
    "télescopé": "télescopée",
    "télescopée": "télescopé",
    "téléchargé": "téléchargée",
    "téléchargée": "téléchargé",
    "télécommandé": "télécommandée",
    "télécommandée": "télécommandé",
    "télégraphié": "télégraphiée",
    "télégraphiée": "télégraphié",
    "téléguidé": "téléguidée",
    "téléguidée": "téléguidé",
    "téléphoné": "téléphonée",
    "téléphonée": "téléphoné",
    "téléspectateur": "téléspectatrice",
    "téléspectatrice": "téléspectateur",
    "télévisé": "télévisée",
    "télévisée": "télévisé",
    "télévisuel": "télévisuelle",
    "télévisuelle": "télévisuel",
    "témoigné": "témoignée",
    "témoignée": "témoigné",
    "ténu": "ténue",
    "ténue": "ténu",
    "tétanisé": "tétanisée",
    "tétanisée": "tétanisé",
    "tété": "tétée",
    "tétée": "tété",
    "têtu": "têtue",
    "têtue": "têtu",
    "tôlier": "tôlière",
    "tôlière": "tôlier",
    "ukrainien": "ukrainienne",
    "ukrainienne": "ukrainien",
    "ulcéré": "ulcérée",
    "ulcérée": "ulcéré",
    "ulcéreuse": "ulcéreux",
    "ulcéreux": "ulcéreuse",
    "ultrasecret": "ultrasecrète",
    "ultrasecrète": "ultrasecret",
    "ultraviolet": "ultraviolette",
    "ultraviolette": "ultraviolet",
    "ultérieur": "ultérieure",
    "ultérieure": "ultérieur",
    "uni": "unie",
    "unie": "uni",
    "unidimensionnel": "unidimensionnelle",
    "unidimensionnelle": "unidimensionnel",
    "unidirectionnel": "unidirectionnelle",
    "unidirectionnelle": "unidirectionnel",
    "unifié": "unifiée",
    "unifiée": "unifié",
    "unilatéral": "unilatérale",
    "unilatérale": "unilatéral",
    "unitarien": "unitarienne",
    "unitarienne": "unitarien",
    "universel": "universelle",
    "universelle": "universel",
    "urbain": "urbaine",
    "urbaine": "urbain",
    "urbanisé": "urbanisée",
    "urbanisée": "urbanisé",
    "urgent": "urgente",
    "urgente": "urgent",
    "usagé": "usagée",
    "usagée": "usagé",
    "usant": "usante",
    "usante": "usant",
    "usé": "usée",
    "usée": "usé",
    "usiné": "usinée",
    "usinée": "usiné",
    "usinier": "usinière",
    "usinière": "usinier",
    "usuel": "usuelle",
    "usuelle": "usuel",
    "usurier": "usurière",
    "usurière": "usurier",
    "usurpateur": "usurpatrice",
    "usurpatrice": "usurpateur",
    "usurpé": "usurpée",
    "usurpée": "usurpé",
    "utilisateur": "utilisatrice",
    "utilisatrice": "utilisateur",
    "utilisé": "utilisée",
    "utilisée": "utilisé",
    "utérin": "utérine",
    "utérine": "utérin",
    "vacancier": "vacancière",
    "vacancière": "vacancier",
    "vacant": "vacante",
    "vacante": "vacant",
    "vacciné": "vaccinée",
    "vaccinée": "vacciné",
    "vachard": "vacharde",
    "vacharde": "vachard",
    "vacillant": "vacillante",
    "vacillante": "vacillant",
    "vadrouilleur": "vadrouilleuse",
    "vadrouilleuse": "vadrouilleur",
    "vagabond": "vagabonde",
    "vagabonde": "vagabond",
    "vaginal": "vaginale",
    "vaginale": "vaginal",
    "vaillant": "vaillante",
    "vaillante": "vaillant",
    "vain": "vaine",
    "vaine": "vain",
    "vaincu": "vaincue",
    "vaincue": "vaincu",
    "valencien": "valencienne",
    "valencienne": "valencien",
    "valentin": "valentine",
    "valentine": "valentin",
    "validé": "validée",
    "validée": "validé",
    "vallonné": "vallonnée",
    "vallonnée": "vallonné",
    "valu": "value",
    "value": "valu",
    "valorisant": "valorisante",
    "valorisante": "valorisant",
    "valorisé": "valorisée",
    "valorisée": "valorisé",
    "valseur": "valseuse",
    "valseuse": "valseur",
    "vampirisé": "vampirisée",
    "vampirisée": "vampirisé",
    "vanillé": "vanillée",
    "vanillée": "vanillé",
    "vanné": "vannée",
    "vannée": "vanné",
    "vantard": "vantarde",
    "vantarde": "vantard",
    "vanté": "vantée",
    "vantée": "vanté",
    "vaporisé": "vaporisée",
    "vaporisée": "vaporisé",
    "variant": "variante",
    "variante": "variant",
    "varié": "variée",
    "variée": "varié",
    "vasouillard": "vasouillarde",
    "vasouillarde": "vasouillard",
    "vassal": "vassale",
    "vassale": "vassal",
    "vaudou": "vaudoue",
    "vaudoue": "vaudou",
    "vaurien": "vaurienne",
    "vaurienne": "vaurien",
    "vautré": "vautrée",
    "vautrée": "vautré",
    "vectoriel": "vectorielle",
    "vectorielle": "vectoriel",
    "veillé": "veillée",
    "veillée": "veillé",
    "veinard": "veinarde",
    "veinarde": "veinard",
    "veiné": "veinée",
    "veinée": "veiné",
    "veineuse": "veineux",
    "veineux": "veineuse",
    "velouté": "veloutée",
    "veloutée": "velouté",
    "velouteuse": "velouteux",
    "velouteux": "velouteuse",
    "velu": "velue",
    "velue": "velu",
    "vendeur": "vendeuse",
    "vendeuse": "vendeur",
    "vendu": "vendue",
    "vendue": "vendu",
    "vendéen": "vendéenne",
    "vendéenne": "vendéen",
    "vengé": "vengée",
    "vengée": "vengé",
    "vengeresse": "vengeur",
    "vengeur": "vengeresse",
    "venu": "venue",
    "venue": "venu",
    "ventilé": "ventilée",
    "ventilée": "ventilé",
    "ventousé": "ventousée",
    "ventousée": "ventousé",
    "ventral": "ventrale",
    "ventrale": "ventral",
    "ventru": "ventrue",
    "ventrue": "ventru",
    "venté": "ventée",
    "ventée": "venté",
    "verbal": "verbale",
    "verbale": "verbal",
    "verbalisé": "verbalisée",
    "verbalisée": "verbalisé",
    "verdi": "verdie",
    "verdie": "verdi",
    "verdissant": "verdissante",
    "verdissante": "verdissant",
    "verdoyant": "verdoyante",
    "verdoyante": "verdoyant",
    "verglacé": "verglacée",
    "verglacée": "verglacé",
    "vermeil": "vermeille",
    "vermeille": "vermeil",
    "vermoulu": "vermoulue",
    "vermoulue": "vermoulu",
    "verni": "vernie",
    "vernie": "verni",
    "vernissé": "vernissée",
    "vernissée": "vernissé",
    "verrouillé": "verrouillée",
    "verrouillée": "verrouillé",
    "versé": "versée",
    "versée": "versé",
    "vert": "verte",
    "verte": "vert",
    "vert-de-grisé": "vert-de-grisée",
    "vert-de-grisée": "vert-de-grisé",
    "vertical": "verticale",
    "verticale": "vertical",
    "vertébral": "vertébrale",
    "vertébrale": "vertébral",
    "vespéral": "vespérale",
    "vespérale": "vespéral",
    "veuf": "veuve",
    "veuve": "veuf",
    "vexant": "vexante",
    "vexante": "vexant",
    "vexé": "vexée",
    "vexée": "vexé",
    "vibrant": "vibrante",
    "vibrante": "vibrant",
    "vibré": "vibrée",
    "vibrée": "vibré",
    "vice-président": "vice-présidente",
    "vice-présidente": "vice-président",
    "vicelard": "vicelarde",
    "vicelarde": "vicelard",
    "vicié": "viciée",
    "viciée": "vicié",
    "vicomte": "vicomtesse",
    "vicomtesse": "vicomte",
    "victorien": "victorienne",
    "victorienne": "victorien",
    "vidangé": "vidangée",
    "vidangée": "vidangé",
    "vidé": "vidée",
    "vidée": "vidé",
    "vieillard": "vieillarde",
    "vieillarde": "vieillard",
    "vieilli": "vieillie",
    "vieillie": "vieilli",
    "vieillissant": "vieillissante",
    "vieillissante": "vieillissant",
    "vieillot": "vieillotte",
    "vieillotte": "vieillot",
    "vietnamien": "vietnamienne",
    "vietnamienne": "vietnamien",
    "vif": "vive",
    "vive": "vif",
    "vigil": "vigile",
    "vigile": "vigil",
    "vigilant": "vigilante",
    "vigilante": "vigilant",
    "vigneron": "vigneronne",
    "vigneronne": "vigneron",
    "vil": "vile",
    "vile": "vil",
    "vilain": "vilaine",
    "vilaine": "vilain",
    "vinaigré": "vinaigrée",
    "vinaigrée": "vinaigré",
    "vindicatif": "vindicative",
    "vindicative": "vindicatif",
    "violacé": "violacée",
    "violacée": "violacé",
    "violent": "violente",
    "violente": "violent",
    "violenté": "violentée",
    "violentée": "violenté",
    "violé": "violée",
    "violée": "violé",
    "violet": "violette",
    "violette": "violet",
    "viral": "virale",
    "virale": "viral",
    "viré": "virée",
    "virée": "viré",
    "virevoltant": "virevoltante",
    "virevoltante": "virevoltant",
    "virgilien": "virgilienne",
    "virgilienne": "virgilien",
    "virginal": "virginale",
    "virginale": "virginal",
    "viril": "virile",
    "virile": "viril",
    "virtuel": "virtuelle",
    "virtuelle": "virtuel",
    "virulent": "virulente",
    "virulente": "virulent",
    "viscéral": "viscérale",
    "viscérale": "viscéral",
    "visé": "visée",
    "visée": "visé",
    "visité": "visitée",
    "visitée": "visité",
    "visiteur": "visiteuse",
    "visiteuse": "visiteur",
    "vissé": "vissée",
    "vissée": "vissé",
    "visualisé": "visualisée",
    "visualisée": "visualisé",
    "visuel": "visuelle",
    "visuelle": "visuel",
    "vital": "vitale",
    "vitale": "vital",
    "vitaminé": "vitaminée",
    "vitaminée": "vitaminé",
    "viticulteur": "viticultrice",
    "viticultrice": "viticulteur",
    "vitré": "vitrée",
    "vitrée": "vitré",
    "vitrifié": "vitrifiée",
    "vitrifiée": "vitrifié",
    "vivandier": "vivandière",
    "vivandière": "vivandier",
    "vivant": "vivante",
    "vivante": "vivant",
    "vivifiant": "vivifiante",
    "vivifiante": "vivifiant",
    "vivifié": "vivifiée",
    "vivifiée": "vivifié",
    "vécu": "vécue",
    "vécue": "vécu",
    "vocal": "vocale",
    "vocale": "vocal",
    "vociférant": "vociférante",
    "vociférante": "vociférant",
    "voilé": "voilée",
    "voilée": "voilé",
    "vu": "vue",
    "vue": "vu",
    "voisin": "voisine",
    "voisine": "voisin",
    "volant": "volante",
    "volante": "volant",
    "volatil": "volatile",
    "volatile": "volatil",
    "volatilisé": "volatilisée",
    "volatilisée": "volatilisé",
    "volé": "volée",
    "volée": "volé",
    "voletant": "voletante",
    "voletante": "voletant",
    "voleur": "voleuse",
    "voleuse": "voleur",
    "vomi": "vomie",
    "vomie": "vomi",
    "voté": "votée",
    "votée": "voté",
    "votif": "votive",
    "votive": "votif",
    "voué": "vouée",
    "vouée": "voué",
    "voulu": "voulue",
    "voulue": "voulu",
    "voyageur": "voyageuse",
    "voyageuse": "voyageur",
    "voyant": "voyante",
    "voyante": "voyant",
    "voyeur": "voyeuse",
    "voyeuse": "voyeur",
    "voyou": "voyoute",
    "voyoute": "voyou",
    "voûté": "voûtée",
    "voûtée": "voûté",
    "vrai": "vraie",
    "vraie": "vrai",
    "vrombissant": "vrombissante",
    "vrombissante": "vrombissant",
    "végétal": "végétale",
    "végétale": "végétal",
    "végétalien": "végétalienne",
    "végétalienne": "végétalien",
    "végétarien": "végétarienne",
    "végétarienne": "végétarien",
    "végétatif": "végétative",
    "végétative": "végétatif",
    "véhiculé": "véhiculée",
    "véhiculée": "véhiculé",
    "véhément": "véhémente",
    "véhémente": "véhément",
    "vénal": "vénale",
    "vénale": "vénal",
    "véniel": "vénielle",
    "vénielle": "véniel",
    "vénitien": "vénitienne",
    "vénitienne": "vénitien",
    "vénusien": "vénusienne",
    "vénusienne": "vénusien",
    "vénéré": "vénérée",
    "vénérée": "vénéré",
    "vénérien": "vénérienne",
    "vénérienne": "vénérien",
    "vénézuélien": "vénézuélienne",
    "vénézuélienne": "vénézuélien",
    "vérificateur": "vérificatrice",
    "vérificatrice": "vérificateur",
    "vérifié": "vérifiée",
    "vérifiée": "vérifié",
    "vérolé": "vérolée",
    "vérolée": "vérolé",
    "vêtu": "vêtue",
    "vêtue": "vêtu",
    "wagnérien": "wagnérienne",
    "wagnérienne": "wagnérien",
    "wallon": "wallonne",
    "wallonne": "wallon",
    "week end": "week-end",
    "week-end": "week end",
    "youpin": "youpine",
    "youpine": "youpin",
    "zigzagant": "zigzagante",
    "zigzagante": "zigzagant",
    "zigzagué": "zigzaguée",
    "zigzaguée": "zigzagué",
    "zodiacal": "zodiacale",
    "zodiacale": "zodiacal",
    "zonard": "zonarde",
    "zonarde": "zonard",
    "zoulou": "zouloue",
    "zouloue": "zoulou",
    "zébré": "zébrée",
    "zébrée": "zébré",
    "zélateur": "zélatrice",
    "zélatrice": "zélateur",
    "zélé": "zélée",
    "zélée": "zélé",
    "âgé": "âgée",
    "âgée": "âgé",
    "ébahi": "ébahie",
    "ébahie": "ébahi",
    "ébaubi": "ébaubie",
    "ébaubie": "ébaubi",
    "ébauché": "ébauchée",
    "ébauchée": "ébauché",
    "éberlué": "éberluée",
    "éberluée": "éberlué",
    "ébloui": "éblouie",
    "éblouie": "ébloui",
    "éblouissant": "éblouissante",
    "éblouissante": "éblouissant",
    "ébouillanté": "ébouillantée",
    "ébouillantée": "ébouillanté",
    "éboulé": "éboulée",
    "éboulée": "éboulé",
    "ébouriffant": "ébouriffante",
    "ébouriffante": "ébouriffant",
    "ébouriffé": "ébouriffée",
    "ébouriffée": "ébouriffé",
    "ébranlé": "ébranlée",
    "ébranlée": "ébranlé",
    "ébruité": "ébruitée",
    "ébruitée": "ébruité",
    "ébréché": "ébréchée",
    "ébréchée": "ébréché",
    "écailler": "écaillère",
    "écaillère": "écailler",
    "écaillé": "écaillée",
    "écaillée": "écaillé",
    "écarquillé": "écarquillée",
    "écarquillée": "écarquillé",
    "écartelé": "écartelée",
    "écartelée": "écartelé",
    "écarté": "écartée",
    "écartée": "écarté",
    "écervelé": "écervelée",
    "écervelée": "écervelé",
    "échancré": "échancrée",
    "échancrée": "échancré",
    "échangé": "échangée",
    "échangée": "échangé",
    "échappé": "échappée",
    "échappée": "échappé",
    "écharpé": "écharpée",
    "écharpée": "écharpé",
    "échaudé": "échaudée",
    "échaudée": "échaudé",
    "échauffé": "échauffée",
    "échauffée": "échauffé",
    "échelonné": "échelonnée",
    "échelonnée": "échelonné",
    "échevelé": "échevelée",
    "échevelée": "échevelé",
    "échiné": "échinée",
    "échinée": "échiné",
    "échu": "échue",
    "échue": "échu",
    "échotier": "échotière",
    "échotière": "échotier",
    "échoué": "échouée",
    "échouée": "échoué",
    "éclaboussant": "éclaboussante",
    "éclaboussante": "éclaboussant",
    "éclaboussé": "éclaboussée",
    "éclaboussée": "éclaboussé",
    "éclairant": "éclairante",
    "éclairante": "éclairant",
    "éclairci": "éclaircie",
    "éclaircie": "éclairci",
    "éclairé": "éclairée",
    "éclairée": "éclairé",
    "éclaireur": "éclaireuse",
    "éclaireuse": "éclaireur",
    "éclatant": "éclatante",
    "éclatante": "éclatant",
    "éclaté": "éclatée",
    "éclatée": "éclaté",
    "éclipsé": "éclipsée",
    "éclipsée": "éclipsé",
    "éclopé": "éclopée",
    "éclopée": "éclopé",
    "éclusé": "éclusée",
    "éclusée": "éclusé",
    "écoeurant": "écoeurante",
    "écoeurante": "écoeurant",
    "écoeuré": "écoeurée",
    "écoeurée": "écoeuré",
    "écolier": "écolière",
    "écolière": "écolier",
    "éconduit": "éconduite",
    "éconduite": "éconduit",
    "économisé": "économisée",
    "économisée": "économisé",
    "écopé": "écopée",
    "écopée": "écopé",
    "écorché": "écorchée",
    "écorchée": "écorché",
    "écorné": "écornée",
    "écornée": "écorné",
    "écoulé": "écoulée",
    "écoulée": "écoulé",
    "écourté": "écourtée",
    "écourtée": "écourté",
    "écouté": "écoutée",
    "écoutée": "écouté",
    "écouteur": "écouteuse",
    "écouteuse": "écouteur",
    "écrabouillé": "écrabouillée",
    "écrabouillée": "écrabouillé",
    "écrasant": "écrasante",
    "écrasante": "écrasant",
    "écrasé": "écrasée",
    "écrasée": "écrasé",
    "écrié": "écriée",
    "écriée": "écrié",
    "écrit": "écrite",
    "écrite": "écrit",
    "écrivain": "écrivaine",
    "écrivaine": "écrivain",
    "écroué": "écrouée",
    "écrouée": "écroué",
    "écroulé": "écroulée",
    "écroulée": "écroulé",
    "écru": "écrue",
    "écrue": "écru",
    "éculé": "éculée",
    "éculée": "éculé",
    "écumant": "écumante",
    "écumante": "écumant",
    "écumé": "écumée",
    "écumée": "écumé",
    "écuyer": "écuyère",
    "écuyère": "écuyer",
    "édenté": "édentée",
    "édentée": "édenté",
    "édicté": "édictée",
    "édictée": "édicté",
    "édifiant": "édifiante",
    "édifiante": "édifiant",
    "édifié": "édifiée",
    "édifiée": "édifié",
    "édité": "éditée",
    "éditée": "édité",
    "éditeur": "éditrice",
    "éditrice": "éditeur",
    "éditorial": "éditoriale",
    "éditoriale": "éditorial",
    "éducateur": "éducatrice",
    "éducatrice": "éducateur",
    "éducatif": "éducative",
    "éducative": "éducatif",
    "éducationnel": "éducationnelle",
    "éducationnelle": "éducationnel",
    "édulcoré": "édulcorée",
    "édulcorée": "édulcoré",
    "éduqué": "éduquée",
    "éduquée": "éduqué",
    "égal": "égale",
    "égale": "égal",
    "égalé": "égalée",
    "égalée": "égalé",
    "égalisateur": "égalisatrice",
    "égalisatrice": "égalisateur",
    "égaré": "égarée",
    "égarée": "égaré",
    "égayé": "égayée",
    "égayée": "égayé",
    "égorgé": "égorgée",
    "égorgée": "égorgé",
    "égratigné": "égratignée",
    "égratignée": "égratigné",
    "égrené": "égrenée",
    "égrenée": "égrené",
    "égrillard": "égrillarde",
    "égrillarde": "égrillard",
    "égyptien": "égyptienne",
    "égyptienne": "égyptien",
    "éhonté": "éhontée",
    "éhontée": "éhonté",
    "éjecté": "éjectée",
    "éjectée": "éjecté",
    "élaboré": "élaborée",
    "élaborée": "élaboré",
    "élancé": "élancée",
    "élancée": "élancé",
    "élargi": "élargie",
    "élargie": "élargi",
    "électeur": "électrice",
    "électrice": "électeur",
    "électif": "élective",
    "élective": "électif",
    "électoral": "électorale",
    "électorale": "électoral",
    "électricien": "électricienne",
    "électricienne": "électricien",
    "électrifié": "électrifiée",
    "électrifiée": "électrifié",
    "électrisant": "électrisante",
    "électrisante": "électrisant",
    "électrisé": "électrisée",
    "électrisée": "électrisé",
    "électrocuté": "électrocutée",
    "électrocutée": "électrocuté",
    "élevé": "élevée",
    "élevée": "élevé",
    "éleveur": "éleveuse",
    "éleveuse": "éleveur",
    "élimé": "élimée",
    "élimée": "élimé",
    "éliminé": "éliminée",
    "éliminée": "éliminé",
    "élisabéthain": "élisabéthaine",
    "élisabéthaine": "élisabéthain",
    "éloigné": "éloignée",
    "éloignée": "éloigné",
    "éloquent": "éloquente",
    "éloquente": "éloquent",
    "élu": "élue",
    "élue": "élu",
    "élucidé": "élucidée",
    "élucidée": "élucidé",
    "éludé": "éludée",
    "éludée": "éludé",
    "élyséen": "élyséenne",
    "élyséenne": "élyséen",
    "élégant": "élégante",
    "élégante": "élégant",
    "éléphant": "éléphante",
    "éléphante": "éléphant",
    "élévateur": "élévatrice",
    "élévatrice": "élévateur",
    "émacié": "émaciée",
    "émaciée": "émacié",
    "émaillé": "émaillée",
    "émaillée": "émaillé",
    "émancipateur": "émancipatrice",
    "émancipatrice": "émancipateur",
    "émancipé": "émancipée",
    "émancipée": "émancipé",
    "émané": "émanée",
    "émanée": "émané",
    "émasculé": "émasculée",
    "émasculée": "émasculé",
    "émergent": "émergente",
    "émergente": "émergent",
    "émergé": "émergée",
    "émergée": "émergé",
    "émerveillé": "émerveillée",
    "émerveillée": "émerveillé",
    "émetteur": "émettrice",
    "émettrice": "émetteur",
    "émietté": "émiettée",
    "émiettée": "émietté",
    "émigrant": "émigrante",
    "émigrante": "émigrant",
    "émigré": "émigrée",
    "émigrée": "émigré",
    "émilien": "émilienne",
    "émilienne": "émilien",
    "émincé": "émincée",
    "émincée": "émincé",
    "éminent": "éminente",
    "éminente": "éminent",
    "émollient": "émolliente",
    "émolliente": "émollient",
    "émondé": "émondée",
    "émondée": "émondé",
    "émotif": "émotive",
    "émotive": "émotif",
    "émotionnel": "émotionnelle",
    "émotionnelle": "émotionnel",
    "émoulu": "émoulue",
    "émoulue": "émoulu",
    "émoussé": "émoussée",
    "émoussée": "émoussé",
    "émoustillant": "émoustillante",
    "émoustillante": "émoustillant",
    "émoustillé": "émoustillée",
    "émoustillée": "émoustillé",
    "émouvant": "émouvante",
    "émouvante": "émouvant",
    "ému": "émue",
    "émue": "ému",
    "éméché": "éméchée",
    "éméchée": "éméché",
    "énamouré": "énamourée",
    "énamourée": "énamouré",
    "énergisant": "énergisante",
    "énergisante": "énergisant",
    "énervant": "énervante",
    "énervante": "énervant",
    "énervé": "énervée",
    "énervée": "énervé",
    "énoncé": "énoncée",
    "énoncée": "énoncé",
    "éolien": "éolienne",
    "éolienne": "éolien",
    "épagneul": "épagneule",
    "épagneule": "épagneul",
    "épaissi": "épaissie",
    "épaissie": "épaissi",
    "épandu": "épandue",
    "épandue": "épandu",
    "épanoui": "épanouie",
    "épanouie": "épanoui",
    "épanouissant": "épanouissante",
    "épanouissante": "épanouissant",
    "épargné": "épargnée",
    "épargnée": "épargné",
    "éparpillé": "éparpillée",
    "éparpillée": "éparpillé",
    "épatant": "épatante",
    "épatante": "épatant",
    "épaté": "épatée",
    "épatée": "épaté",
    "épaulé": "épaulée",
    "épaulée": "épaulé",
    "éperdu": "éperdue",
    "éperdue": "éperdu",
    "éperonné": "éperonnée",
    "éperonnée": "éperonné",
    "épeuré": "épeurée",
    "épeurée": "épeuré",
    "épicé": "épicée",
    "épicée": "épicé",
    "épicier": "épicière",
    "épicière": "épicier",
    "épicurien": "épicurienne",
    "épicurienne": "épicurien",
    "épidural": "épidurale",
    "épidurale": "épidural",
    "épié": "épiée",
    "épiée": "épié",
    "épilé": "épilée",
    "épilée": "épilé",
    "épinglé": "épinglée",
    "épinglée": "épinglé",
    "épiscopal": "épiscopale",
    "épiscopale": "épiscopal",
    "épiscopalien": "épiscopalienne",
    "épiscopalienne": "épiscopalien",
    "épistolier": "épistolière",
    "épistolière": "épistolier",
    "épithélial": "épithéliale",
    "épithéliale": "épithélial",
    "éploré": "éplorée",
    "éplorée": "éploré",
    "éployé": "éployée",
    "éployée": "éployé",
    "épluché": "épluchée",
    "épluchée": "épluché",
    "épongé": "épongée",
    "épongée": "épongé",
    "épousé": "épousée",
    "épousée": "épousé",
    "épousseté": "époussetée",
    "époussetée": "épousseté",
    "époustouflant": "époustouflante",
    "époustouflante": "époustouflant",
    "époustouflé": "époustouflée",
    "époustouflée": "époustouflé",
    "épouvanté": "épouvantée",
    "épouvantée": "épouvanté",
    "épris": "éprise",
    "éprise": "épris",
    "éprouvant": "éprouvante",
    "éprouvante": "éprouvant",
    "éprouvé": "éprouvée",
    "éprouvée": "éprouvé",
    "épuisant": "épuisante",
    "épuisante": "épuisant",
    "épuisé": "épuisée",
    "épuisée": "épuisé",
    "épuré": "épurée",
    "épurée": "épuré",
    "équarri": "équarrie",
    "équarrie": "équarri",
    "équatorial": "équatoriale",
    "équatoriale": "équatorial",
    "équidistant": "équidistante",
    "équidistante": "équidistant",
    "équilibré": "équilibrée",
    "équilibrée": "équilibré",
    "équin": "équine",
    "équine": "équin",
    "équipé": "équipée",
    "équipée": "équipé",
    "équipier": "équipière",
    "équipière": "équipier",
    "équivalent": "équivalente",
    "équivalente": "équivalent",
    "éradiqué": "éradiquée",
    "éradiquée": "éradiqué",
    "éraflé": "éraflée",
    "éraflée": "éraflé",
    "éraillé": "éraillée",
    "éraillée": "éraillé",
    "éreintant": "éreintante",
    "éreintante": "éreintant",
    "éreinté": "éreintée",
    "éreintée": "éreinté",
    "érigé": "érigée",
    "érigée": "érigé",
    "érodé": "érodée",
    "érodée": "érodé",
    "érotisé": "érotisée",
    "érotisée": "érotisé",
    "érudit": "érudite",
    "érudite": "érudit",
    "établi": "établie",
    "établie": "établi",
    "étagé": "étagée",
    "étagée": "étagé",
    "étalé": "étalée",
    "étalée": "étalé",
    "étanché": "étanchée",
    "étanchée": "étanché",
    "étayé": "étayée",
    "étayée": "étayé",
    "éteint": "éteinte",
    "éteinte": "éteint",
    "étendu": "étendue",
    "étendue": "étendu",
    "éternel": "éternelle",
    "éternelle": "éternel",
    "éternisé": "éternisée",
    "éternisée": "éternisé",
    "éthiopien": "éthiopienne",
    "éthiopienne": "éthiopien",
    "éthéré": "éthérée",
    "éthérée": "éthéré",
    "étincelant": "étincelante",
    "étincelante": "étincelant",
    "étiolé": "étiolée",
    "étiolée": "étiolé",
    "étiqueté": "étiquetée",
    "étiquetée": "étiqueté",
    "étiré": "étirée",
    "étirée": "étiré",
    "étoffé": "étoffée",
    "étoffée": "étoffé",
    "étoilé": "étoilée",
    "étoilée": "étoilé",
    "étonnant": "étonnante",
    "étonnante": "étonnant",
    "étonné": "étonnée",
    "étonnée": "étonné",
    "étouffant": "étouffante",
    "étouffante": "étouffant",
    "étouffé": "étouffée",
    "étouffée": "étouffé",
    "étourdi": "étourdie",
    "étourdie": "étourdi",
    "étourdissant": "étourdissante",
    "étourdissante": "étourdissant",
    "étranger": "étrangère",
    "étrangère": "étranger",
    "étranglé": "étranglée",
    "étranglée": "étranglé",
    "étrangleur": "étrangleuse",
    "étrangleuse": "étrangleur",
    "étreint": "étreinte",
    "étreinte": "étreint",
    "étrenné": "étrennée",
    "étrennée": "étrenné",
    "étrillé": "étrillée",
    "étrillée": "étrillé",
    "étripé": "étripée",
    "étripée": "étripé",
    "étriqué": "étriquée",
    "étriquée": "étriqué",
    "étroit": "étroite",
    "étroite": "étroit",
    "étréci": "étrécie",
    "étrécie": "étréci",
    "étudiant": "étudiante",
    "étudiante": "étudiant",
    "étudié": "étudiée",
    "étudiée": "étudié",
    "évacué": "évacuée",
    "évacuée": "évacué",
    "évadé": "évadée",
    "évadée": "évadé",
    "évalué": "évaluée",
    "évaluée": "évalué",
    "évanescent": "évanescente",
    "évanescente": "évanescent",
    "évanoui": "évanouie",
    "évanouie": "évanoui",
    "évaporé": "évaporée",
    "évaporée": "évaporé",
    "évasé": "évasée",
    "évasée": "évasé",
    "évasif": "évasive",
    "évasive": "évasif",
    "éveillé": "éveillée",
    "éveillée": "éveillé",
    "éventé": "éventée",
    "éventée": "éventé",
    "éventré": "éventrée",
    "éventrée": "éventré",
    "éventuel": "éventuelle",
    "éventuelle": "éventuel",
    "évident": "évidente",
    "évidente": "évident",
    "évidé": "évidée",
    "évidée": "évidé",
    "évincé": "évincée",
    "évincée": "évincé",
    "évité": "évitée",
    "évitée": "évité",
    "évocateur": "évocatrice",
    "évocatrice": "évocateur",
    "évolué": "évoluée",
    "évoluée": "évolué",
    "évolutif": "évolutive",
    "évolutive": "évolutif",
    "évoqué": "évoquée",
    "évoquée": "évoqué",
    "ôté": "ôtée",
    "ôtée": "ôté",
    "aucun": "aucune",
    "aucune": "aucun",
    "auxquelles": "auxquels",
    "auxquels": "auxquelles",
    "ce": "celle",
    "celle": "ce",
    "celle-ci": "celui-ci",
    "celui-ci": "celle-ci",
    "celle-là": "celui-là",
    "celui-là": "celle-là",
    "celles": "ceux",
    "ceux": "celles",
    "celles-ci": "ceux-ci",
    "ceux-ci": "celles-ci",
    "celles-là": "ceux-là",
    "ceux-là": "celles-là",
    "chacune": "chacun",
    "chacun": "chacune",
    "desquelles": "desquels",
    "desquels": "desquelles",
    "elles": "ils",
    "ils": "elles",
    "mienne": "mien",
    "mien": "mienne",
    "la sienne": "sien",
    "sien": "la sienne",
    "la tienne": "tien",
    "tien": "la tienne",
    "laquelle": "lequel",
    "lequel": "laquelle",
    "le": "la",
    "la": "le",
    "les miennes": "les miens",
    "les miens": "les miennes",
    "les siennes": "les siens",
    "les siens": "les siennes",
    "les tiennes": "les tiens",
    "les tiens": "les tiennes",
    "lesquelles": "lesquels",
    "lesquels": "lesquelles",
    "quelle": "quel",
    "quel": "quelle",
    "quelles": "quels",
    "quels": "quelles",
    "quelques-unes": "quelques-uns",
    "quelques-uns": "quelques-unes",
    "ton": "ta",
    "ta": "ton",
    "à laquelle": "auquel",
    "auquel": "à laquelle",
    "il": "elle",
    "elle": "il",
}
